@@ -541,9 +541,9 @@ int game_menu_render() {
 	}
 	
 	else if (menu_state == 3) {
+		draw_poly_strf(32.0f, info_y, 100.0f, 1.0f, 1.0f, 1.0f, 1.0f,
+			"Starting game, please wait...");
 		if (load_queued < 4) {
-			draw_poly_strf(32.0f, info_y, 100.0f, 1.0f, 1.0f, 1.0f, 1.0f,
-				"Starting game, please wait...");
 			load_queued++;
 			return 0;
 		} else {
@@ -666,9 +666,11 @@ int game_menu_render() {
 int dc_write_config_file(char *fn) {
 	FILE *cfile;
 	if ((cfile = fopen(fn, "w"))) {
+		fprintf(cfile, "[game]\n");
+		fprintf(cfile, "resource_dir = %s\n", games[*selected].dir);
 		fprintf(cfile, "gfx.dc.render_mode = %s\n", options_nr[0]? "pvr" : "vram");
 		fprintf(cfile, "gfx.dc.refresh_rate = %s\n", sel_freq? "60Hz" : "50Hz");
-		fprintf(cfile, "pic0_dither_mode = ");
+		fprintf(cfile, "pic0_dither_mode = ");		
 		switch (options_nr[1]) {
 		case 0:
 			fprintf(cfile, "dither256\n");
@@ -679,50 +681,18 @@ int dc_write_config_file(char *fn) {
 		case 2:
 			fprintf(cfile, "dither\n");
 		}
-
+					
 		if (options_nr[2])
-			fprintf(cfile, "pic_antialiasing = simple\n");
+			fprintf(cfile, "pic_antialiasing = simple");
 
-		switch (options_nr[3]) {
-		case 0:
-			fprintf(cfile, "midi_device = adlibemu\n");
-			fprintf(cfile, "pcmout_driver = dc\n");
-			fprintf(cfile, "midiout_driver = null\n");
-			break;
-		case 1:
-			fprintf(cfile, "midi_device = mt32gm\n");
-			fprintf(cfile, "pcmout_driver = null\n");
-			fprintf(cfile, "midiout_driver = dcraw\n");
-			break;
-		case 2:
-			fprintf(cfile, "midi_device = mt32\n");
-			fprintf(cfile, "pcmout_driver = null\n");
-			fprintf(cfile, "midiout_driver = dcraw\n");
-			break;
-		case 3:
-			fprintf(cfile, "midi_device = null\n");
-			fprintf(cfile, "pcmout_driver = null\n");
-			fprintf(cfile, "midiout_driver = null\n");
-		}
+		if (options_nr[3])
+			fprintf(cfile, "version = %s\n", options_str[3]);
 
 		if (options_nr[4])
-			fprintf(cfile, "pcmout_stereo = 1\n");
-		else
-			fprintf(cfile, "pcmout_stereo = 0\n");
+			fprintf(cfile, "resource_version = %s\n", options_str[4]);
 
-		switch (options_nr[5]) {
-		case 0:
-			fprintf(cfile, "pcmout_rate = 16000\n");
-			fprintf(cfile, "pcmout_buffer_size = 192\n");
-			break;
-		case 1:
-			fprintf(cfile, "pcmout_rate = 22050\n");
-			fprintf(cfile, "pcmout_buffer_size = 256\n");
-			break;
-		case 2:
-			fprintf(cfile, "pcmout_rate = 11025\n");
-			fprintf(cfile, "pcmout_buffer_size = 128\n");
-		}
+		if (options_nr[5])
+			fprintf(cfile, "pic_port_bounds = \"0, 0, 320, 200\"\n");
 
 		fclose(cfile);
 		return 0;
