@@ -25,8 +25,9 @@
 
 ***************************************************************************/
 
+
 /* Attempt to guess if recent version of Platform SDK */
-#ifdef _WIN32
+#ifdef _MSC_VER
 #	pragma message("******************** IMPORTANT MESSAGE ********************")
 #	pragma message("You must have installed a recent Platform and DirectX SDK")
 #	pragma message("for this build to be successful. Download MS SDKs from:")
@@ -43,8 +44,9 @@
 #include <versions.h>
 #include <kernel.h>
 #include <kdebug.h>
+#include <kernel_types.h>
 
-#if !defined (_WIN32) && !defined (_DREAMCAST)
+#if !defined (_WIN32)
 #include <sys/resource.h>
 #endif
 
@@ -58,11 +60,12 @@ extern int bp_flag;
 static int
 _init_vocabulary(state_t *s) /* initialize vocabulary and related resources */
 {
-	sciprintf("Initializing vocabulary\n");
-
 	s->parser_lastmatch_word = SAID_NO_MATCH;
+	s->parser_rules = NULL;
 
-	if ((s->parser_words = vocab_get_words(s->resmgr, &(s->parser_words_nr)))) {
+	sciprintf("Initializing vocabulary\n");	
+	
+	if ((s->resmgr->sci_version < SCI_VERSION_01_VGA)&&(s->parser_words = vocab_get_words(s->resmgr, &(s->parser_words_nr)))) {
 		s->parser_suffices = vocab_get_suffices(s->resmgr, &(s->parser_suffices_nr));
 		if ((s->parser_branches = vocab_get_branches(s->resmgr, &(s->parser_branches_nr))))
 		    /* Now build a GNF grammar out of this */
@@ -123,14 +126,33 @@ _init_graphics_input(state_t *s)
 	return 0;
 }
 
+static void
+_sci1_alloc_system_colors(state_t *s)
+{
+	gfx_color_t white;
+	gfx_color_t black;
 
-static int
+	white.visual.global_index = 255;
+	white.visual.r = white.visual.g = white.visual.b = 255;
+	white.alpha = 0;
+	white.priority = white.control = 0;
+	white.mask = GFX_MASK_VISUAL;
+	gfxop_set_system_color(s->gfx_state, &white);
+
+	black.visual.global_index = 0;
+	black.visual.r = black.visual.g = black.visual.b = 0;
+	black.alpha = 0;
+	black.priority = black.control = 0;
+	black.mask = GFX_MASK_VISUAL;
+	gfxop_set_system_color(s->gfx_state, &black);
+}
+
+int
 _reset_graphics_input(state_t *s)
 {
 	resource_t *resource;
 	int font_nr;
-	gfx_color_t transparent	= {0};
-
+	gfx_color_t transparent;
 	sciprintf("Initializing graphics\n");
 
 	if (s->resmgr->sci_version <= SCI_VERSION_01) {
@@ -146,21 +168,46 @@ _reset_graphics_input(state_t *s)
 			gfxop_set_system_color(s->gfx_state, &(s->ega_colors[i]));
 		}
 	} else
-		if (gfxop_set_color(s->gfx_state, &(s->ega_colors[0]), 0, 0, 0, 0, -1, -1)) return 1; /* We usually need black */
-
+	{
+		/* Check for Amiga palette file. */
+		FILE *f = sci_fopen("spal", "rb");
+		if (f) {
+			s->gfx_state->resstate->static_palette = 
+			  gfxr_read_pal1_amiga(&s->gfx_state->resstate->static_palette_entries, f);
+			fclose(f);
+			_sci1_alloc_system_colors(s);
+		} else {
+			resource = scir_find_resource(s->resmgr, sci_palette, 999, 1);
+			if (resource) {
+				if (s->version < SCI_VERSION(1,001,000))
+					s->gfx_state->resstate->static_palette = 
+					  gfxr_read_pal1(999, &s->gfx_state->resstate->static_palette_entries, 
+							 resource->data, resource->size); 
+			    	else
+					s->gfx_state->resstate->static_palette = 
+					  gfxr_read_pal11(999, &s->gfx_state->resstate->static_palette_entries, 
+							    resource->data, resource->size); 
+				_sci1_alloc_system_colors(s);
+				scir_unlock_resource(s->resmgr, resource, sci_palette, 999);
+			} else
+				sciprintf("Couldn't find the default palette!\n");
+		}
+	}
 	transparent.mask = 0;
 
 	gfxop_fill_box(s->gfx_state, gfx_rect(0, 0, 320, 200), s->ega_colors[0]); /* Fill screen black */
 	gfxop_update(s->gfx_state);
 
-	s->mouse_pointer_nr = -1; /* No mouse pointer resource */
+	s->mouse_pointer_view = s->mouse_pointer_loop = s->mouse_pointer_cel = -1; /* No mouse pointer resource */
+	s->save_mouse_pointer_view = s->save_mouse_pointer_loop = s->save_mouse_pointer_cel = -1; /* No mouse pointer resource */
 	gfxop_set_pointer_position(s->gfx_state, gfx_point(160, 150));
 
-	s->mouse_pointer_nr = -1;
+	s->mouse_pointer_view = s->mouse_pointer_loop = s->mouse_pointer_cel = -1; /* No mouse pointer resource */
+	s->save_mouse_pointer_view = s->save_mouse_pointer_loop = s->save_mouse_pointer_cel = -1; /* No mouse pointer resource */
+
 
-	s->pic_not_valid = 1; /* Picture is invalid */
 	s->pic_is_new = 0;
-	s->pic_visible_map = 0; /* Other values only make sense for debugging */
+	s->pic_visible_map = GFX_MASK_NONE; /* Other values only make sense for debugging */
 	s->dyn_views = NULL; /* no DynViews */
 	s->drop_views = NULL; /* And, consequently, no list for dropped views */
 
@@ -173,7 +220,7 @@ _reset_graphics_input(state_t *s)
 	font_nr = -1;
 	do {
 		resource = scir_test_resource(s->resmgr, sci_font, ++font_nr);
-	} while ((!resource) && (font_nr < 999));
+	} while ((!resource) && (font_nr < sci_max_resource_nr[s->resmgr->sci_version]));
 
 	if (!resource) {
 		sciprintf("No text font was found.\n");
@@ -182,24 +229,44 @@ _reset_graphics_input(state_t *s)
 
 	s->visual = gfxw_new_visual(s->gfx_state, font_nr);
 
-	if (s->version < SCI_VERSION_FTU_NORMAL_PORTS)
-		s->wm_port = gfxw_new_port(s->visual, NULL, gfx_rect(0, 0, 320, 200), s->ega_colors[0], transparent); else
-			s->wm_port = gfxw_new_port(s->visual, NULL, gfx_rect(0, 10, 320, 190), s->ega_colors[0], transparent);
-	s->titlebar_port = gfxw_new_port(s->visual, NULL, gfx_rect(0, 0, 320, 10), s->ega_colors[0], s->ega_colors[15]);
-	s->picture_port = gfxw_new_port(s->visual, NULL, gfx_rect(0, 10, 320, 190), s->ega_colors[0], transparent);
+	s->wm_port = gfxw_new_port(s->visual, NULL, s->gfx_state->options->pic_port_bounds, s->ega_colors[0], transparent);
+	s->iconbar_port = gfxw_new_port(s->visual, NULL, gfx_rect(0, 0, 320, 200), s->ega_colors[0], transparent);
+	s->iconbar_port->flags |= GFXW_FLAG_NO_IMPLICIT_SWITCH;
+
+	if (s->resmgr->sci_version >= SCI_VERSION_01_VGA)
+	{
+		gfx_color_t fgcolor;
+		gfx_color_t bgcolor;
+
+#if 0
+		fgcolor.visual = s->gfx_state->resstate->static_palette[0];
+		fgcolor.mask = GFX_MASK_VISUAL;
+		bgcolor.visual = s->gfx_state->resstate->static_palette[255];
+		bgcolor.mask = GFX_MASK_VISUAL;
+#endif
+		s->titlebar_port = gfxw_new_port(s->visual, NULL, gfx_rect(0, 0, 320, 10), 
+						 fgcolor, bgcolor);
+	} else
+		s->titlebar_port = gfxw_new_port(s->visual, NULL, gfx_rect(0, 0, 320, 10), 
+						 s->ega_colors[0], s->ega_colors[15]);
 	s->titlebar_port->color.mask |= GFX_MASK_PRIORITY;
 	s->titlebar_port->color.priority = 11;
 	s->titlebar_port->bgcolor.mask |= GFX_MASK_PRIORITY;
 	s->titlebar_port->bgcolor.priority = 11;
+	s->titlebar_port->flags |= GFXW_FLAG_NO_IMPLICIT_SWITCH;
+
+	/* but this is correct */
+	s->picture_port = gfxw_new_port(s->visual, NULL, s->gfx_state->options->pic_port_bounds, s->ega_colors[0], transparent);
 
 	s->pics_drawn_nr = 0;
 
-	s->visual->add(GFXWC(s->visual), GFXW(s->picture_port));
 	s->visual->add(GFXWC(s->visual), GFXW(s->wm_port));
 	s->visual->add(GFXWC(s->visual), GFXW(s->titlebar_port));
+	s->visual->add(GFXWC(s->visual), GFXW(s->picture_port));
+	s->visual->add(GFXWC(s->visual), GFXW(s->iconbar_port));
 	/* Add ports to visual */
 
-	s->port = s->wm_port; /* Currently using the window manager port */
+	s->port = s->picture_port; /* Currently using the picture port */
 
 #if 0
 	s->titlebar_port->bgcolor.mask |= GFX_MASK_PRIORITY;
@@ -239,7 +306,21 @@ _free_graphics_input(state_t *s)
 	s->pics = NULL;
 }
 
+/*------------------------------------------------------------*/
 
+int
+game_init_sound(state_t *s, int sound_flags)
+{
+	if (s->resmgr->sci_version >= SCI_VERSION_01)
+		sound_flags |= SFX_STATE_FLAG_MULTIPLAY;
+
+	s->sfx_init_flags = sound_flags;
+	sfx_init(&s->sound, s->resmgr, sound_flags);
+	return 0;
+}
+
+
+/* Maps a class ID to the script the corresponding class is contained in */
 /* Returns the script number suggested by vocab.996, or -1 if there's none */
 static int
 suggested_script(resource_t *res, unsigned int class)
@@ -254,39 +335,95 @@ suggested_script(resource_t *res, unsigned int class)
 }
 
 
-/* Architectural stuff: Init/Unintialize engine */
+int 
+test_cursor_style(state_t *s)
+{
+	int resource_nr = 0;
+	int ok = 0;
+
+	do {
+		ok |= scir_test_resource(s->resmgr, sci_cursor, resource_nr++) != NULL;
+	} while (resource_nr < 1000 && !ok);
+
+	return ok;
+}
+
 int
-script_init_engine(state_t *s, sci_version_t version)
+create_class_table_sci11(state_t *s)
 {
-	resource_t *vocab996 = scir_find_resource(s->resmgr, sci_vocab, 996, 1);
-	int i;
 	int scriptnr;
-	unsigned int seeker;
+	unsigned int seeker_offset;
+	char *seeker_ptr;
 	int classnr;
-	int size;
-	int magic_offset; /* For strange scripts in older SCI versions */
 
-	s->max_version = SCI_VERSION(9,999,999); /* :-) */
-	s->min_version = 0; /* Set no real limits */
-	s->version = SCI_VERSION_DEFAULT_SCI0;
-	s->kernel_opt_flags = 0;
-
-
-	if (!version) {
-		s->version_lock_flag = 0;
-	} else {
-		s->version = version;
-		s->version_lock_flag = 1; /* Lock version */
-	}
-
-	script_detect_early_versions(s);
+	resource_t *vocab996 = scir_find_resource(s->resmgr, sci_vocab, 996, 1);
 
 	if (!vocab996)
 		s->classtable_size = 20;
 	else
 		s->classtable_size = vocab996->size >> 2;
 
-	s->classtable = sci_calloc(sizeof(class_t), s->classtable_size);
+	s->classtable = (class_t*)sci_calloc(sizeof(class_t), s->classtable_size);
+
+	for (scriptnr = 0; scriptnr < 1000; scriptnr++) {
+		resource_t *heap = scir_find_resource(s->resmgr, sci_heap,
+						      scriptnr, 0);
+
+		if (heap) {
+			int global_vars = getUInt16(heap->data + 2);
+
+			seeker_ptr = (char*)heap->data + 4 + global_vars*2;
+			seeker_offset = 4 + global_vars*2;
+
+			while (getUInt16((byte*)seeker_ptr) == SCRIPT_OBJECT_MAGIC_NUMBER)
+			{
+				if (getUInt16((byte*)seeker_ptr + 14) & SCRIPT_INFO_CLASS)
+				{
+					classnr = getUInt16((byte*)seeker_ptr + 10);
+					if (classnr >= s->classtable_size) {
+
+						if (classnr >= SCRIPT_MAX_CLASSTABLE_SIZE) {
+							fprintf(stderr,"Invalid class number 0x%x in script.%d(0x%x), offset %04x\n",
+								classnr, scriptnr, scriptnr, seeker_offset);
+							return 1;
+						}
+
+						s->classtable = (class_t*)sci_realloc(s->classtable, sizeof(class_t) * (classnr + 1));
+						memset(&(s->classtable[s->classtable_size]), 0,
+						       sizeof(class_t) * (1 + classnr - s->classtable_size)); /* Clear after resize */
+
+						s->classtable_size = classnr + 1; /* Adjust maximum number of entries */
+					}
+					
+					s->classtable[classnr].reg.offset = seeker_offset;
+					s->classtable[classnr].reg.segment = 0;
+					s->classtable[classnr].script = scriptnr;
+				}
+
+				seeker_ptr += getUInt16((byte*)seeker_ptr + 2) * 2;
+				seeker_offset += getUInt16((byte*)seeker_ptr + 2) * 2;
+			}
+		}
+	}
+
+	return 0;
+}
+static int
+create_class_table_sci0(state_t *s)
+{
+	int scriptnr;
+	unsigned int seeker;
+	int classnr;
+	int magic_offset; /* For strange scripts in older SCI versions */
+
+	resource_t *vocab996 = scir_find_resource(s->resmgr, sci_vocab, 996, 1);
+
+	if (!vocab996)
+		s->classtable_size = 20;
+	else
+		s->classtable_size = vocab996->size >> 2;
+
+	s->classtable = (class_t*)sci_calloc(sizeof(class_t), s->classtable_size);
 
 	for (scriptnr = 0; scriptnr < 1000; scriptnr++) {
 		int objtype = 0;
@@ -294,7 +431,6 @@ script_init_engine(state_t *s, sci_version_t version)
 							scriptnr, 0);
 
 		if (script) {
-			size = getInt16(script->data);
 			if (s->version < SCI_VERSION_FTU_NEW_SCRIPT_HEADER)
 				magic_offset = seeker = 2;
 			else
@@ -329,7 +465,7 @@ script_init_engine(state_t *s, sci_version_t version)
 							return 1;
 						}
 
-						s->classtable = sci_realloc(s->classtable, sizeof(class_t) * (classnr + 1));
+						s->classtable = (class_t*)sci_realloc(s->classtable, sizeof(class_t) * (classnr + 1));
 						memset(&(s->classtable[s->classtable_size]), 0,
 						       sizeof(class_t) * (1 + classnr - s->classtable_size)); /* Clear after resize */
 
@@ -339,11 +475,12 @@ script_init_engine(state_t *s, sci_version_t version)
 					sugg_script = suggested_script(vocab996, classnr);
 
 					/* First, test whether the script hasn't been claimed, or if it's been claimed by the wrong script */
-					if (sugg_script == -1 || scriptnr == sugg_script || !s->classtable[classnr].scriptposp) {
+
+					if (sugg_script == -1 || scriptnr == sugg_script /*|| !s->classtable[classnr].reg.segment*/)  {
 						/* Now set the home script of the class */
-						s->classtable[classnr].class_offset = seeker + 4 - magic_offset;
+						s->classtable[classnr].reg.offset = seeker + 4 - magic_offset;
+						s->classtable[classnr].reg.segment = 0;
 						s->classtable[classnr].script = scriptnr;
-						s->classtable[classnr].scriptposp = &(s->scripttable[scriptnr].heappos);
 					}
 
 					seeker += SCRIPT_OBJECT_MAGIC_OFFSET; /* Re-adjust position */
@@ -357,51 +494,96 @@ script_init_engine(state_t *s, sci_version_t version)
 	}
 	scir_unlock_resource(s->resmgr, vocab996, sci_vocab, 996);
 	vocab996 = NULL;
+	return 0;
+}
 
-	s->_heap = heap_new();
-	s->heap = s->_heap->start;
+/* Architectural stuff: Init/Unintialize engine */
+int
+script_init_engine(state_t *s, sci_version_t version)
+{
+	int result;
 
+	s->max_version = SCI_VERSION(9,999,999); /* :-) */
+	s->min_version = 0; /* Set no real limits */
+	s->version = SCI_VERSION_DEFAULT_SCI0;
+	s->kernel_opt_flags = 0;
+	
+	if (!version) {
+		s->version_lock_flag = 0;
+	} else {
+		s->version = version;
+		s->version_lock_flag = 1; /* Lock version */
+	}
+
+	script_detect_versions(s);
+
+        if (s->version >= SCI_VERSION(1,001,000))
+		result = create_class_table_sci11(s); 
+	else
+		result = create_class_table_sci0(s);
+	
+	sm_init(&s->seg_manager, s->version >= SCI_VERSION(1,001,000));
+	s->gc_countdown = GC_INTERVAL - 1;
+
+	if (result)
+	{
+		sciprintf("Failed to initialize class table\n");
+		return 1;
+	}
+
+	s->script_000_segment = script_get_segment(s, 0, SCRIPT_GET_LOCK);
+
+	if (s->script_000_segment <= 0) {
+		sciprintf("Failed to instantiate script.000\n");
+		return 1;
+	}
+
+	s->script_000 = &(s->seg_manager.heap[s->script_000_segment]->data.script);
+
+
+	s->sys_strings = sm_allocate_sys_strings(&s->seg_manager,
+						 &s->sys_strings_segment);
 	/* Allocate static buffer for savegame and CWD directories */
-	s->save_dir = heap_allocate(s->_heap, MAX_SAVE_DIR_SIZE);
-	s->save_dir_copy = 0xffff;
+	sys_string_acquire(s->sys_strings, SYS_STRING_SAVEDIR, "savedir", MAX_SAVE_DIR_SIZE);
+
+	s->save_dir_copy = make_reg(s->sys_strings_segment, SYS_STRING_SAVEDIR);
 	s->save_dir_edit_offset = 0;
-	s->save_dir_copy_buf = sci_malloc(MAX_SAVE_DIR_SIZE);
-	s->save_dir_copy_buf[0] = 0; /* Terminate string */
 
-	save_ff(s->_heap); /* Save heap state */
-
-	s->acc = s->amp_rest = s->prev = 0;
+	s->r_acc = s->r_prev = NULL_REG;
+	s->r_amp_rest = 0;
 
 	s->execution_stack = NULL;    /* Start without any execution stack */
 	s->execution_stack_base = -1; /* No vm is running yet */
 	s->execution_stack_pos = -1;   /* Start at execution stack position 0 */
 
-	s->global_vars = 0; /* Set during launch time */
-
 
 	s->kernel_names = vocabulary_get_knames(s->resmgr, &s->kernel_names_nr);
 	script_map_kernel(s);
 	/* Maps the kernel functions */
 
 	if (_init_vocabulary(s)) return 1;
+	if (s->selector_map.cantBeHere != -1)
+		version_require_later_than(s, SCI_VERSION_FTU_INVERSE_CANBEHERE);
 
 	s->restarting_flags = SCI_GAME_IS_NOT_RESTARTING;
 
-
-	for (i = 0; i < 1000; i++)
-		s->scripttable[i].heappos = 0; /* Mark all scripts as 'not installed' */
-
 	s->bp_list = NULL; /* No breakpoints defined */
 	s->have_bp = 0;
 
 	s->file_handles_nr = 5;
-	s->file_handles = sci_calloc(sizeof(FILE *), s->file_handles_nr);
+	s->file_handles = (FILE**)sci_calloc(sizeof(FILE *), s->file_handles_nr);
 	/* Allocate memory for file handles */
 
 	sci_init_dir(&(s->dirseeker));
-	s->dirseeker_outbuffer = 0;
+	s->dirseeker_outbuffer = NULL_REG;
 	/* Those two are used by FileIO for FIND_FIRST, FIND_NEXT */
 
+	if (s->version >= SCI_VERSION_FTU_LOFS_ABSOLUTE &&
+	    s->version < SCI_VERSION(1,001,000))
+		sm_set_export_width(&s->seg_manager, 1);
+	else
+		sm_set_export_width(&s->seg_manager, 0);
+
 	sciprintf("Engine initialized\n");
 
 	if (_init_graphics_input(s))
@@ -412,16 +594,9 @@ script_init_engine(state_t *s, sci_version_t version)
 
 
 void
-script_set_gamestate_save_dir(state_t *s)
+script_set_gamestate_save_dir(state_t *s, const char* path)
 {
-	char *cwd = sci_getcwd();
-	if (strlen(cwd) > MAX_SAVE_DIR_SIZE)
-		sciprintf("Warning: cwd '%s' is longer than the"
-			  " MAX_SAVE_DIR_SIZE %d\n",
-			  cwd, MAX_SAVE_DIR_SIZE);
-	else
-		strcpy((char *)(s->heap + s->save_dir + 2), cwd);
-	sci_free(cwd);
+	sys_string_set(s->sys_strings, SYS_STRING_SAVEDIR, path);
 }
 
 void
@@ -430,23 +605,8 @@ script_free_vm_memory(state_t *s)
 	int i;
 
 	sciprintf("Freeing VM memory\n");
-	heap_free(s->_heap, s->save_dir);
-	sci_free(s->save_dir_copy_buf);
 	s->save_dir_copy_buf = NULL;
 
-	for (i = 0; i < MAX_HUNK_BLOCKS; i++)
-		if (s->hunk[i].size) {
-			if (s->hunk[i].type == HUNK_TYPE_GFXBUFFER) {
-				gfxw_snapshot_t *snapshot = *((gfxw_snapshot_t **) s->hunk[i].data);
-				sci_free(snapshot);
-			}
-			sci_free(s->hunk[i].data);
-			s->hunk[i].size = 0;
-		}
-
-
-	heap_del(s->_heap);
-	s->_heap = NULL;
 	sci_free(s->classtable);
 	s->classtable = NULL;
 
@@ -464,20 +624,28 @@ script_free_vm_memory(state_t *s)
 	   exception fault whenever you try to close a never-opened file */
 }
 
+extern void
+free_kfunct_tables(state_t *s);
+/* From kernel.c */
+
 void
 script_free_engine(state_t *s)
 {
-	breakpoint_t *bp, *bp_next;
-
 	script_free_vm_memory(s);
 
 	sciprintf("Freeing state-dependant data\n");
 
-	sci_free(s->kfunct_table);
-	s->kfunct_table = NULL;
+	free_kfunct_tables(s);
 
 	_free_vocabulary(s);
 
+}
+
+void
+script_free_breakpoints(state_t *s)
+{
+	breakpoint_t *bp, *bp_next;
+
 	/* Free breakpoint list */
 	bp = s->bp_list;
 	while (bp) {
@@ -490,7 +658,6 @@ script_free_engine(state_t *s)
 	s->bp_list = NULL;
 }
 
-
 /*************************************************************/
 /* Game instance stuff: Init/Unitialize state-dependant data */
 /*************************************************************/
@@ -499,52 +666,32 @@ script_free_engine(state_t *s)
 int
 game_init(state_t *s)
 {
-	heap_ptr stack_handle;
-	heap_ptr parser_handle;
-	heap_ptr script0;
-	heap_ptr game_obj; /* Address of the game object */
-	int i;
+#ifdef __GNUC__
+#  warning "Fixme: Use new VM instantiation code all over the place"
+#endif
+	reg_t game_obj; /* Address of the game object */
+	dstack_t *stack;
 
-	if (!script_instantiate(s, 0, 1)) {
+	stack = sm_allocate_stack(&s->seg_manager, VM_STACK_SIZE,
+					      &s->stack_segment);
+	s->stack_base = stack->entries;
+	s->stack_top = s->stack_base + VM_STACK_SIZE;
+
+	if (!script_instantiate(s, 0)) {
 		sciprintf("game_init(): Could not instantiate script 0\n");
 		return 1;
 	}
 
 	s->parser_valid = 0; /* Invalidate parser */
-	s->parser_event = 0; /* Invalidate parser event */
-
-	stack_handle = heap_allocate(s->_heap, VM_STACK_SIZE);
-	parser_handle = heap_allocate(s->_heap, PARSE_HEAP_SIZE);
-
-	script0 = s->scripttable[0].heappos; /* Get script 0 position */
-
-	if (!script0) {
-		sciprintf("Game initialization requested, but script.000 not loaded\n");
-		return 1;
-	}
+	s->parser_event = NULL_REG; /* Invalidate parser event */
 
 	s->synonyms = NULL;
 	s->synonyms_nr = 0; /* No synonyms */
 
-	/* Initialize hunk data */
-	for (i = 0; i < MAX_HUNK_BLOCKS; i++)
-		s->hunk[i].size = 0;
-	/* Initialize clone list */
-	memset(&(s->clone_list), 0, sizeof(heap_ptr) * SCRIPT_MAX_CLONES);
 	/* Initialize send_calls buffer */
 
 	if (!send_calls_allocated)
-		send_calls = sci_calloc(sizeof(calls_struct_t), send_calls_allocated = 16);
-
-	if (!stack_handle) {
-		sciprintf("game_init(): Insufficient heap space for stack\n");
-		return 1;
-	}
-
-	if (!parser_handle) {
-		sciprintf("game_init(): Insufficient heap space for parser word error block\n");
-		return 1;
-	}
+		send_calls = (calls_struct_t*)sci_calloc(sizeof(calls_struct_t), send_calls_allocated = 16);
 
 	if (s->gfx_state && _reset_graphics_input(s))
 		return 1;
@@ -552,20 +699,12 @@ game_init(state_t *s)
 	s->successor = NULL; /* No successor */
 	s->status_bar_text = NULL; /* Status bar is blank */
 	s->status_bar_foreground = 0;
-	s->status_bar_background = 15;
+	s->status_bar_background = s->resmgr->sci_version >= SCI_VERSION_01_VGA ? 255 : 15;
 
-	fprintf(stderr," Script 0 at %04x\n", script0);
-
-	s->stack_base = stack_handle + 2;
-	s->parser_base = parser_handle + 2;
-	s->global_vars = s->scripttable[0].localvar_offset;
-	/* Global variables are script 0's local variables */
+	sys_string_acquire(s->sys_strings, SYS_STRING_PARSER_BASE, "parser-base", MAX_PARSER_BASE);
+	s->parser_base = make_reg(s->sys_strings_segment, SYS_STRING_PARSER_BASE);
 
 	sci_get_current_time(&(s->game_start_time)); /* Get start time */
-#ifdef ARM_WINCE	
-	s->game_start_time.tv_usec = 0;
-	s->game_start_time.tv_sec = 0;
-#endif	
 	memcpy(&(s->last_wait_time), &(s->game_start_time), sizeof(GTimeVal));
 	/* Use start time as last_wait_time */
 
@@ -574,24 +713,20 @@ game_init(state_t *s)
 
 	srand(time(NULL)); /* Initialize random number generator */
 
-	memset(s->hunk, 0, sizeof(s->hunk)); /* Sets hunk to be unused */
-	memset(s->clone_list, 0, sizeof(s->clone_list)); /* No clones */
-
 	/*	script_dissect(0, s->selector_names, s->selector_names_nr); */
-	game_obj = script0 + GET_HEAP(s->scripttable[0].export_table_offset + 2);
+	game_obj = script_lookup_export(s, 0, 0);
 	/* The first entry in the export table of script 0 points to the game object */
 
-	if (s->version < SCI_VERSION_FTU_NEW_SCRIPT_HEADER)
-		game_obj -= 2; /* Adjust for alternative header */
+	s->game_name = sci_strdup(obj_get_name(s, game_obj));
 
-	if (GET_HEAP(game_obj + SCRIPT_OBJECT_MAGIC_OFFSET) != SCRIPT_OBJECT_MAGIC_NUMBER) {
-		sciprintf("game_init(): Game object is not at 0x%x\n", game_obj);
+	if (!s->game_name) {
+		sciprintf("Error: script.000, export 0 ("PREG") does not\n"
+			  "  yield an object with a name -> sanity check failed\n",
+			  PRINT_REG(game_obj));
 		return 1;
 	}
 
-	s->game_name = s->heap + GET_HEAP(game_obj + SCRIPT_NAME_OFFSET);
-
-	sciprintf(" Game designation is \"%s\"\n", s->game_name);
+	sciprintf(" \"%s\" at "PREG"\n", s->game_name, PRINT_REG(game_obj));
 
 	if (strlen((char *) s->game_name) >= MAX_GAMEDIR_SIZE) {
 
@@ -600,7 +735,6 @@ game_init(state_t *s)
 	}
 
 	s->game_obj = game_obj;
-	s->stack_handle = stack_handle;
 
 	/* Mark parse tree as unused */
 	s->parser_nodes[0].type = PARSE_TREE_NODE_LEAF;
@@ -614,12 +748,19 @@ game_init(state_t *s)
 int
 game_exit(state_t *s)
 {
-	int i;
-
 	if (s->execution_stack) {
 		sci_free(s->execution_stack);
 	}
 
+#if 0
+	sfx_exit(&s->sound);
+/* Reinit because some other code depends on having a valid state */
+	game_init_sound(s, SFX_STATE_FLAG_NOSOUND); 
+#else
+#endif
+
+	sm_destroy(&s->seg_manager);
+	
 	if (s->synonyms_nr) {
 		sci_free(s->synonyms);
 		s->synonyms = NULL;
@@ -628,30 +769,24 @@ game_exit(state_t *s)
 
 	sciprintf("Freeing miscellaneous data...\n");
 
-	/* HACK WARNING: This frees all scripts that were allocated prior to the stack, i.e. those
-	** that won't survive a stack restauration.
-	*/
-	for (i = 1; i < 1000; i++)
-		if (s->scripttable[i].heappos > s->stack_handle)
-			s->scripttable[i].heappos = 0;
-
-	heap_free(s->_heap, s->stack_handle);
-	heap_free(s->_heap, s->parser_base - 2);
-	restore_ff(s->_heap); /* Restore former heap state */
-
+#ifdef __GNUC__
+#warning "Free parser segment here"
+#endif
 	if (send_calls_allocated) {
 		sci_free(send_calls);
 		send_calls_allocated = 0;
 	}
 
-	for (i = 0; i < 1000; i++)
-		/*  if (s->scripttable[i].heappos > s->_heap->old_ff)*/
-		s->scripttable[i].heappos = 0; /* Mark all non-high scripts as 'not installed', except for 0 */
+#ifdef __GNUC__
+#warning "Free scripts here"
+#endif
 
 	menubar_free(s->menubar);
 
 	_free_graphics_input(s);
 
+	sci_free(s->game_name);
+
 	return 0;
 }
 
