@@ -33,22 +33,15 @@
 #  endif
 #  define WIN32_LEAN_AND_MEAN
 #  include <windows.h>
-#elif defined (__MORPHOS__)
+#  include <sys/types.h>
 #  include <sys/stat.h>
+#  define stat _stat
 #elif defined (_DREAMCAST)
 #  include <dc.h>
 #endif
 
-#ifdef HAVE_UNISTD_H
-#  include <unistd.h>
-#endif
-
-#ifdef HAVE_DIRENT_H
-#  include <dirent.h>
-#endif
-
 #ifndef O_BINARY
-#  define O_BINARY 0
+#define O_BINARY 0
 #endif
 
 static int _savegame_indices_nr = -1; /* means 'uninitialized' */
@@ -81,7 +74,7 @@ f_open_mirrored(state_t *s, char *fname)
 
 	fsize = sci_fd_size(fd);
 	if (fsize > 0) {
-		buf = sci_malloc(fsize);
+		buf = (char*)sci_malloc(fsize);
 		read(fd, buf, fsize);
 	}
 
@@ -89,8 +82,12 @@ f_open_mirrored(state_t *s, char *fname)
 
 	chdir(s->work_dir);
 
-	fd = sci_create_file(fname);
-
+	/* Visual C++ doesn't allow to specify O_BINARY with creat() */
+#ifdef _MSC_VER
+	fd = _open(fname, O_CREAT | O_BINARY | O_RDWR, S_IREAD | S_IWRITE);
+#else
+	fd = creat(fname, 0600);
+#endif
 
 	if (!IS_VALID_FD(fd) && buf) {
 		free(buf);
@@ -99,7 +96,7 @@ f_open_mirrored(state_t *s, char *fname)
 		return NULL;
 	}
 
-	if (fsize > 0) {
+	if (fsize) {
 		int ret;
 		ret = write(fd, buf, fsize);
 		if (ret < fsize) {
@@ -147,7 +144,7 @@ file_open(state_t *s, char *filename, int mode)
 	}
 	if (!file) { /* Failed */
 		SCIkdebug(SCIkFILE, "file_open() failed\n");
-		s->acc = 0xffff;
+		s->r_acc = make_reg(0, 0xffff);
 		return;
 	}
 
@@ -155,17 +152,21 @@ file_open(state_t *s, char *filename, int mode)
 		retval++;
 
 	if (retval == s->file_handles_nr) /* Hit size limit => Allocate more space */
-		s->file_handles = sci_realloc(s->file_handles, sizeof(FILE *) * ++(s->file_handles_nr));
+		s->file_handles = (FILE**)sci_realloc(s->file_handles, sizeof(FILE *) * ++(s->file_handles_nr));
 
 	s->file_handles[retval] = file;
 
-	s->acc = retval;
+	s->r_acc = make_reg(0, retval);
 }
 
-void
-kFOpen(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kFOpen(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-  file_open(s, (char *) s->heap + UPARAM(0), UPARAM(1));
+  char *name = kernel_dereference_char_pointer(s, argv[0], 0);
+  int mode = UKPV(1);
+
+  file_open(s, name, mode);
+  return s->r_acc;
 }
 
 void file_close(state_t *s, int handle)
@@ -187,27 +188,28 @@ void file_close(state_t *s, int handle)
   s->file_handles[handle] = NULL;
 }
 
-void
-kFClose(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kFClose(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-  file_close(s, UPARAM(0));
+  file_close(s, UKPV(0));
+  return s->r_acc;
 }
 
-void fputs_wrapper(state_t *s, int handle, char *data)
+void fputs_wrapper(state_t *s, int handle, int size, char *data)
 {
-  SCIkdebug(SCIkFILE, "FPuts'ing \"%s\" to handle %d\n", data, handle);
+	SCIkdebug(SCIkFILE, "FPuts'ing \"%s\" to handle %d\n", data, handle);
 
-  if (handle == 0) {
-    SCIkwarn(SCIkERROR, "Attempt to write to file handle 0\n");
-    return;
-  }
+	if (handle == 0) {
+		SCIkwarn(SCIkERROR, "Attempt to write to file handle 0\n");
+		return;
+	}
 
-  if ((handle >= s->file_handles_nr) || (s->file_handles[handle] == NULL)) {
-    SCIkwarn(SCIkERROR, "Attempt to write to invalid/unused file handle %d\n", handle);
-    return;
-  }
+	if ((handle >= s->file_handles_nr) || (s->file_handles[handle] == NULL)) {
+		SCIkwarn(SCIkERROR, "Attempt to write to invalid/unused file handle %d\n", handle);
+		return;
+	}
 
-  fputs(data, s->file_handles[handle]);
+	fwrite(data, 1, size, s->file_handles[handle]);
 }
 
 void fwrite_wrapper(state_t *s, int handle, char *data, int length)
@@ -228,12 +230,13 @@ void fwrite_wrapper(state_t *s, int handle, char *data, int length)
 }
 
 
-void kFPuts(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t kFPuts(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-  int handle = UPARAM(0);
-  char *data = (char *) (UPARAM(1) + s->heap);
+	int handle = UKPV(0);
+	char *data = kernel_dereference_char_pointer(s, argv[1], 0);
 
-  fputs_wrapper(s, handle, data);
+	fputs_wrapper(s, handle, strlen(data), data);
+	return s->r_acc;
 }
 
 static void
@@ -273,7 +276,7 @@ fread_wrapper(state_t *s, char *dest, int bytes, int handle)
     return;
   }
 
-  s->acc=fread(dest, 1, bytes, s->file_handles[handle]);
+  s->r_acc=make_reg(0,fread(dest, 1, bytes, s->file_handles[handle]));
 }
 
 
@@ -291,7 +294,7 @@ fseek_wrapper(state_t *s, int handle, int offset, int whence)
     return;
   }
 
-  s->acc=fseek(s->file_handles[handle], offset, whence);
+  s->r_acc=make_reg(0, fseek(s->file_handles[handle], offset, whence));
 }
 
 
@@ -299,7 +302,8 @@ static char *
 _chdir_savedir(state_t *s)
 {
 	char *cwd = sci_getcwd();
-	char *save_dir = (char *) s->heap + s->save_dir + 2;
+	char *save_dir = kernel_dereference_char_pointer(s, 
+			make_reg(s->sys_strings_segment, SYS_STRING_SAVEDIR), 0);
 
 	if (chdir(save_dir) && sci_mkpath(save_dir)) {
 
@@ -320,223 +324,263 @@ static void
 _chdir_restoredir(char *dir)
 {
 	if (chdir(dir)) {
-		sciprintf(__FILE__": Can't seemt to return to previous homedir '%s'\n",
+		sciprintf(__FILE__": Can't seem to return to previous homedir '%s'\n",
 			  dir);
 	}
 	free(dir);
 }
 
-#define TEST_DIR_OR_QUIT(dir) if (!dir) { s->acc = 0; return; }
+#define TEST_DIR_OR_QUIT(dir) if (!dir) { return NULL_REG; }
 
-
-void
-kFGets(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kFGets(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-  char *dest = (char *) (UPARAM(0) + s->heap);
-  int maxsize = UPARAM(1);
-  int handle = UPARAM(2);
+  char *dest = kernel_dereference_char_pointer(s, argv[0], 0);
+  int maxsize = UKPV(1);
+  int handle = UKPV(2);
 
   fgets_wrapper(s, dest, maxsize, handle);
-  s->acc = UPARAM(0);
+  return argv[0];
 }
 
 
 /* kGetCWD(address):
 ** Writes the cwd to the supplied address and returns the address in acc.
 */
-void
-kGetCWD(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kGetCWD(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
 	char *wd = sci_getcwd();
-	heap_ptr offset = UPARAM(0);
-	char *targetaddr = (char *) s->heap + offset;
+	char *targetaddr = kernel_dereference_char_pointer(s, argv[0], 0);
 
-	s->acc = offset;
 	strncpy(targetaddr, wd, MAX_SAVE_DIR_SIZE - 1);
 	targetaddr[MAX_SAVE_DIR_SIZE - 1] = 0; /* Terminate */
 
-	SCIkdebug(SCIkFILE, "Copying cwd='%s'(%d chars) to %p"
-		  " (heap starts at %p, offset=%04x)\n",
-		  wd, strlen(wd), targetaddr, s->heap, offset);
+	SCIkdebug(SCIkFILE, "Copying cwd='%s'(%d chars) to %p",
+		  wd, strlen(wd), targetaddr);
 
 	free(wd);
+	return argv[0];
 }
 
+/* Returns a dynamically allocated pointer to the name of the requested save dir */
+char *
+_k_get_savedir_name(int nr)
+{
+	char suffices[] = "0123456789abcdefghijklmnopqrstuvwxyz";
+	char *savedir_name = (char*)sci_malloc(strlen(FREESCI_SAVEDIR_PREFIX) + 2);
+	assert(nr >= 0);
+	assert(nr < MAX_SAVEGAME_NR);
+	strcpy(savedir_name, FREESCI_SAVEDIR_PREFIX);
+	savedir_name[strlen(FREESCI_SAVEDIR_PREFIX)] = suffices[nr];
+	savedir_name[strlen(FREESCI_SAVEDIR_PREFIX) + 1] = 0;
+
+	return savedir_name;
+}
+
+void
+delete_savegame(state_t *s, int savedir_nr)
+{
+	char *workdir = _chdir_savedir(s);
+	char *savedir = _k_get_savedir_name(savedir_nr);
+	char buffer[256];
+
+	sciprintf("Deleting savegame '%s'\n", savedir);
+
+	sprintf(buffer, "%s/%s.id", savedir, s->game_name);
+	sci_unlink(buffer);
+
+	sprintf(buffer, "%s/state", savedir);
+	sci_unlink(buffer);
+
+	sci_rmdir(savedir);
+
+	free(savedir);
+	_chdir_restoredir(workdir);
+}
+	
 #define K_DEVICE_INFO_GET_DEVICE 0
 #define K_DEVICE_INFO_GET_CURRENT_DEVICE 1
 #define K_DEVICE_INFO_PATHS_EQUAL 2
 #define K_DEVICE_INFO_IS_FLOPPY 3
+#define K_DEVICE_INFO_GET_SAVECAT_NAME 7
+#define K_DEVICE_INFO_GET_SAVEFILE_NAME 8
 
 #ifdef _WIN32
 
-void
-kDeviceInfo_Win32(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kDeviceInfo_Win32(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-  char dir_buffer [MAX_PATH], dir_buffer2 [MAX_PATH];
-  int mode = UPARAM(0);
+	char dir_buffer [MAX_PATH], dir_buffer2 [MAX_PATH];
+	int mode = UKPV(0);
 
 
-  switch(mode) {
+	switch(mode) {
 
-  case K_DEVICE_INFO_GET_DEVICE: {
-    heap_ptr input = UPARAM(1);
-    heap_ptr output = UPARAM(2);
-    char *input_s = s->heap + output;
-    char *output_s = s->heap + output;
+	case K_DEVICE_INFO_GET_DEVICE: {
+		char *input_s = (char*)kernel_dereference_bulk_pointer(s, argv[1], 0);
+		char *output_s = (char*)kernel_dereference_bulk_pointer(s, argv[2], 0);
 
-    SCIkASSERT(input >= HEAP_MIN);
-    SCIkASSERT(output >= HEAP_MIN);
+		GetFullPathName (input_s, sizeof (dir_buffer)-1, dir_buffer, NULL);
+		strncpy(output_s, dir_buffer, 2);
+		output_s [2] = 0;
+	}
+		break;
 
-    GetFullPathName (input_s, sizeof (dir_buffer)-1, dir_buffer, NULL);
-    strncpy(output_s, dir_buffer, 2);
-    output_s [2] = 0;
-  }
-  break;
+	case K_DEVICE_INFO_GET_CURRENT_DEVICE: {
+		char *output_s = (char*)kernel_dereference_bulk_pointer(s, argv[1], 0);
 
-  case K_DEVICE_INFO_GET_CURRENT_DEVICE: {
-    heap_ptr output = UPARAM(2);
-    char *output_s = s->heap + output;
+		_getcwd (dir_buffer, sizeof (dir_buffer)-1);
+		strncpy(output_s, dir_buffer, 2);
+		output_s [2] = 0;
+	}
+		break;
 
-    SCIkASSERT(output >= HEAP_MIN);
+	case K_DEVICE_INFO_PATHS_EQUAL: {
+		char *path1_s = (char*)kernel_dereference_bulk_pointer(s, argv[1], 0);
+		char *path2_s = (char*)kernel_dereference_bulk_pointer(s, argv[2], 0);
 
-    _getcwd (dir_buffer, sizeof (dir_buffer)-1);
-    strncpy(output_s, dir_buffer, 2);
-    output_s [2] = 0;
-  }
-  break;
-
-  case K_DEVICE_INFO_PATHS_EQUAL: {
-    heap_ptr path1 = UPARAM(1);
-    heap_ptr path2 = UPARAM(2);
-    char *path1_s = s->heap + path1;
-    char *path2_s = s->heap + path2;
-
-    SCIkASSERT(path1 >= HEAP_MIN);
-    SCIkASSERT(path2 >= HEAP_MIN);
-
-    GetFullPathName (path1_s, sizeof (dir_buffer)-1, dir_buffer, NULL);
-    GetFullPathName (path2_s, sizeof (dir_buffer2)-1, dir_buffer2, NULL);
+		GetFullPathName (path1_s, sizeof (dir_buffer)-1, dir_buffer, NULL);
+		GetFullPathName (path2_s, sizeof (dir_buffer2)-1, dir_buffer2, NULL);
 
 #ifdef _MSC_VER
-    s->acc = !stricmp (path1_s, path2_s);
+		return make_reg(0, !stricmp (path1_s, path2_s));
 #else
-    s->acc = !strcasecmp (path1_s, path2_s);
+		return make_reg(0, !strcasecmp (path1_s, path2_s));
 #endif
-  }
-  break;
+	}
+		break;
 
-  case K_DEVICE_INFO_IS_FLOPPY: {
-    heap_ptr input = UPARAM(1);
-    char *input_s = s->heap + input;
+	case K_DEVICE_INFO_IS_FLOPPY: {
+		char *input_s = (char*)kernel_dereference_bulk_pointer(s, argv[1], 0);
 
-    GetFullPathName (input_s, sizeof (dir_buffer)-1, dir_buffer, NULL);
-    dir_buffer [3] = 0;  /* leave X:\ */
+		GetFullPathName (input_s, sizeof (dir_buffer)-1, dir_buffer, NULL);
+		dir_buffer [3] = 0;  /* leave X:\ */
 
-    s->acc = (GetDriveType (dir_buffer) == DRIVE_REMOVABLE);
-  }
-  break;
+		return make_reg(0, GetDriveType (dir_buffer) == DRIVE_REMOVABLE);
+	}
+		break;
 
-  default: {
-    SCIkwarn(SCIkERROR, "Unknown DeviceInfo() sub-command: %d\n", mode);
-  }
-  }
+/* SCI uses these in a less-than-portable way to delete savegames.
+** Read http://www-plan.cs.colorado.edu/creichen/freesci-logs/2005.10/log20051019.html
+** for more information on our workaround for this.
+*/
+	case K_DEVICE_INFO_GET_SAVECAT_NAME: {
+		char *output_buffer = kernel_dereference_char_pointer(s, argv[1], 0);
+		char *game_prefix = kernel_dereference_char_pointer(s, argv[2], 0);
+
+		sprintf(output_buffer, "%s/__throwaway", s->work_dir);
+	}
+
+		break;
+	case K_DEVICE_INFO_GET_SAVEFILE_NAME: {
+		char *output_buffer = kernel_dereference_char_pointer(s, argv[1], 0);
+		char *game_prefix = kernel_dereference_char_pointer(s, argv[2], 0);
+		int savegame_id = UKPV(3);
+		sprintf(output_buffer, "%s/__throwaway", s->work_dir);
+		delete_savegame(s, savegame_id);
+	}
+		break;
+	default: {
+		SCIkwarn(SCIkERROR, "Unknown DeviceInfo() sub-command: %d\n", mode);
+	}
+	}
+	return s->r_acc;
 }
 
 #else /* !_WIN32 */
 
-void
-kDeviceInfo_Unix(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kDeviceInfo_Unix(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-  int mode = UPARAM(0);
+	int mode = UKPV(0);
 
+	switch(mode) {
 
-  switch(mode) {
+	case K_DEVICE_INFO_GET_DEVICE: {
+		char *output_s = kernel_dereference_char_pointer(s, argv[2], 0);
 
-  case K_DEVICE_INFO_GET_DEVICE: {
-    heap_ptr output = UPARAM(2);
-    char *output_s = (char *) s->heap + output;
+		strcpy(output_s, "/");
+	}
+		break;
 
-    SCIkASSERT(output >= HEAP_MIN);
+	case K_DEVICE_INFO_GET_CURRENT_DEVICE: {
+		char *output_s = kernel_dereference_char_pointer(s, argv[1], 0);
 
-    strcpy(output_s, "/");
-  }
-  break;
+		strcpy(output_s, "/");
+	}
+		break;
 
-  case K_DEVICE_INFO_GET_CURRENT_DEVICE: {
-    heap_ptr output = UPARAM(1);
-    char *output_s = (char *) s->heap + output;
-
-    SCIkASSERT(output >= HEAP_MIN);
-
-    strcpy(output_s, "/");
-  }
-  break;
-
-  case K_DEVICE_INFO_PATHS_EQUAL: {
-    heap_ptr path1 = UPARAM(1);
-    heap_ptr path2 = UPARAM(2);
-    char *path1_s = (char *) s->heap + path1;
-    char *path2_s = (char *) s->heap + path2;
-
-    SCIkASSERT(path1 >= HEAP_MIN);
-    SCIkASSERT(path2 >= HEAP_MIN);
+	case K_DEVICE_INFO_PATHS_EQUAL: {
+		char *path1_s = kernel_dereference_char_pointer(s, argv[1], 0);
+		char *path2_s = kernel_dereference_char_pointer(s, argv[2], 0);
 
 #ifndef HAVE_FNMATCH_H
 #ifndef _DOS
 #  warning "File matches will be unprecise!"
 #endif
-    s->acc = !strcmp(path1_s, path2_s);
+		return make_reg(0, !strcmp(path1_s, path2_s));
 #else
-    s->acc = fnmatch(path1_s, path2_s, FNM_PATHNAME); /* POSIX.2 */
+		return make_reg(0, fnmatch(path1_s, path2_s, FNM_PATHNAME) /* POSIX.2 */);
 #endif
   }
-  break;
+		break;
 
-  case K_DEVICE_INFO_IS_FLOPPY: {
+	case K_DEVICE_INFO_IS_FLOPPY: {
 
-    s->acc = 0; /* Never */
-  }
-  break;
+		return NULL_REG; /* Never */
+	}
+		break;
 
-  default: {
-    SCIkwarn(SCIkERROR, "Unknown DeviceInfo() sub-command: %d\n", mode);
-  }
-  }
+/* SCI uses these in a less-than-portable way to delete savegames.
+** Read http://www-plan.cs.colorado.edu/creichen/freesci-logs/2005.10/log20051019.html
+** for more information on our workaround for this.
+*/
+	case K_DEVICE_INFO_GET_SAVECAT_NAME: {
+		char *output_buffer = kernel_dereference_char_pointer(s, argv[1], 0);
+/*		char *game_prefix = kernel_dereference_char_pointer(s, argv[2], 0);*/
+
+		sprintf(output_buffer, "%s/__throwaway", s->work_dir);
+	}
+
+		break;
+	case K_DEVICE_INFO_GET_SAVEFILE_NAME: {
+		char *output_buffer = kernel_dereference_char_pointer(s, argv[1], 0);
+/*		char *game_prefix = kernel_dereference_char_pointer(s, argv[2], 0);*/
+		int savegame_id = UKPV(3);
+		sprintf(output_buffer, "%s/__throwaway", s->work_dir);
+		delete_savegame(s, savegame_id);
+	}
+		break;
+	default: {
+		SCIkwarn(SCIkERROR, "Unknown DeviceInfo() sub-command: %d\n", mode);
+	}
+	}
+
+	return s->r_acc;
 }
 
 #endif /* !_WIN32 */
 
 
-void
-kGetSaveDir(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kGetSaveDir(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-  s->acc = s->save_dir + 2; /* +2 to step over heap block size */
+	return make_reg(s->sys_strings_segment, SYS_STRING_SAVEDIR);
 }
 
-#if defined(_DREAMCAST) || defined(ARM_WINCE) || defined(_GP32)
 
-void
-kCheckFreeSpace(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kCheckFreeSpace(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-	/* There is no practical way to detect enough memory card space,
-	so we just attempt to save no matter what. */
-	s->acc = 1;
-}
-
-#else /* ! defined(_DREAMCAST) && !defined(ARM_WINCE) && !defined(_GP32) */
-
-void
-kCheckFreeSpace(state_t *s, int funct_nr, int argc, heap_ptr argp)
-{
-  char *path = (char *) s->heap + UPARAM(0);
-  char *testpath = sci_malloc(strlen(path) + 15);
+  char *path = kernel_dereference_char_pointer(s, argv[0], 0);
+  char *testpath = (char*)sci_malloc(strlen(path) + 15);
   char buf[1024];
   int i;
   int fd;
   int failed = 0;
   int pathlen;
 
-  memset(buf, 0, sizeof(buf));
   strcpy(testpath, path);
   strcat(testpath, "freesci.foo");
   pathlen = strlen(testpath);
@@ -546,8 +590,7 @@ kCheckFreeSpace(state_t *s, int funct_nr, int argc, heap_ptr argp)
     if (testpath[pathlen - 2] == 'z') { /* Failed. */
       SCIkwarn(SCIkWARNING, "Failed to find non-existing file for free space test\n");
       free(testpath);
-      s->acc = 0;
-      return;
+      return NULL_REG;
     }
 
     /* If this file couldn't be created, try freesci.fop, freesci.foq etc.,
@@ -562,16 +605,16 @@ kCheckFreeSpace(state_t *s, int funct_nr, int argc, heap_ptr argp)
       ++testpath[pathlen - 1];
   }
 
-  fd = sci_create_file(testpath);
+  fd = creat(testpath, 0600);
 
   if (!IS_VALID_FD(fd)) {
     SCIkwarn(SCIkWARNING,"Could not test for disk space: %s\n", strerror(errno));
     SCIkwarn(SCIkWARNING,"Test path was '%s'\n", testpath);
     free(testpath);
-    s->acc = 0;
-    return;
+    return NULL_REG;
   }
 
+  memset(buf, 0, sizeof(buf));
   for (i = 0; i < 1024; i++) /* Check for 1 MB */
     if (write(fd, buf, 1024) < 1024)
       failed = 1;
@@ -580,39 +623,12 @@ kCheckFreeSpace(state_t *s, int funct_nr, int argc, heap_ptr argp)
 
   remove(testpath);
 
-  s->acc = !failed;
+  sci_free(testpath);
 
-  free(testpath);
+  return make_reg(0, !failed);
 }
 
-#endif
 
-/* Returns a dynamically allocated pointer to the name of the requested save dir */
-#ifdef _DREAMCAST
-
-char *
-_k_get_savedir_name(int nr)
-{
-	return strdup("/ram");
-}
-
-#else /* !_DREAMCAST */
-
-char *
-_k_get_savedir_name(int nr)
-{
-	char suffices[] = "0123456789abcdefghijklmnopqrstuvwxyz";
-	char *savedir_name = sci_malloc(strlen(FREESCI_SAVEDIR_PREFIX) + 2);
-	assert(nr >= 0);
-	assert(nr < MAX_SAVEGAME_NR);
-	strcpy(savedir_name, FREESCI_SAVEDIR_PREFIX);
-	savedir_name[strlen(FREESCI_SAVEDIR_PREFIX)] = suffices[nr];
-	savedir_name[strlen(FREESCI_SAVEDIR_PREFIX) + 1] = 0;
-
-	return savedir_name;
-}
-
-#endif
 
 int
 _k_check_file(char *filename, int minfilesize)
@@ -647,92 +663,13 @@ _k_find_savegame_by_name(char *game_id_file, char *name)
 				fclose(idfile);
 			}
 
-			chdir(G_DIR_PARENT_S);
+			chdir("..");
 		}
 		free(buf);
 	}
 	return 0;
 }
 
-
-int
-test_savegame(state_t *s, char *savegame_id, char *savegame_name, int savegame_name_length)
-{
-	int retval = 1;
-	char *game_id = (char *) s->game_name;
-	char *game_id_file = (char *) sci_malloc(strlen(game_id) + strlen(FREESCI_ID_SUFFIX) + 1);
-
-	strcpy(game_id_file, game_id);
-	strcat(game_id_file, FREESCI_ID_SUFFIX);
-
-	if (chdir(savegame_id)) {
-		retval = 0; /* Couldn't enter savedir */
-	}  else {
-
-		if (_k_check_file(FREESCI_FILE_HEAP, SCI_HEAP_SIZE))
-			retval = 0;
-		if (_k_check_file(FREESCI_FILE_STATE, 1))
-			retval = 0;
-		if (savegame_name) {
-			if (_k_check_file(game_id_file, 1)) {
-				*savegame_name = 0;
-			} else {
-				int foo = open(game_id_file, O_RDONLY);
-				int bytes_read = read(foo, savegame_name, savegame_name_length);
-				savegame_name[MIN(bytes_read, savegame_name_length)] = 0; /* terminate */
-				close(foo);
-			}
-		} else if (_k_check_file(game_id_file, 1))
-			retval = 0;
-
-		chdir (G_DIR_PARENT_S);
-	}
-	free(game_id_file);
-	return retval;
-}
-
-void
-kCheckSaveGame(state_t *s, int funct_nr, int argc, heap_ptr argp)
-{
-	int savedir_nr = UPARAM(1);
-	char *buf = NULL;
-	char *workdir = _chdir_savedir(s);
-	TEST_DIR_OR_QUIT(workdir);
-
-
-
-	if (soundserver_dead) {
-		sciprintf("Soundserver is dead- cannot save game state!");
-		_chdir_restoredir(workdir);
-		s->acc = 0;
-		return;
-	}
-
-	if ((savedir_nr < 0) || (savedir_nr > MAX_SAVEGAME_NR-1)) {
-		_chdir_restoredir(workdir);
-		s->acc = 0;
-		return;
-	}
-
-	savedir_nr = _savegame_indices[savedir_nr].id;
-
-	if ((savedir_nr < 0) || (savedir_nr > MAX_SAVEGAME_NR-1)) {
-		_chdir_restoredir(workdir);
-		s->acc = 0;
-		return;
-	}
-
-#ifdef _DREAMCAST
-	dc_retrieve_savegame((char *) s->game_name, savedir_nr);
-#endif
-
-	s->acc = test_savegame(s, (buf = _k_get_savedir_name(savedir_nr)), NULL, 0);
-
-	_chdir_restoredir(workdir);
-	free(buf);
-}
-
-
 #ifdef _DREAMCAST
 static long
 get_file_mtime(int fd)
@@ -772,56 +709,10 @@ _savegame_index_struct_compare(const void *a, const void *b)
 		- ((struct _savegame_index_struct *)a)->timestamp;
 }
 
-#ifdef _DREAMCAST
-
 static void
-update_savegame_indices(char *game_id)
+update_savegame_indices(char *gfname)
 {
 	int i;
-	char *vmu;
-
-	_savegame_indices_nr = 0;
-
-	if (!(vmu = dc_get_first_vmu())) {
-		sciprintf("%s, L%d: No VMU found!\n", __FILE__, __LINE__);
-		return;
-	}
-
-	fs_chdir(vmu);
-	sci_free(vmu);
-
-	for (i = 0; i < MAX_SAVEGAME_NR; i++) {
-		char *catname = dc_get_cat_name(game_id, i);
-		sci_dir_t dirent;
-		sci_init_dir(&dirent);
-
-		if (sci_find_first(&dirent, catname)) {
-			_savegame_indices[_savegame_indices_nr].id = i;
-			_savegame_indices[_savegame_indices_nr++].timestamp = 0;
-		}
-
-		sci_finish_find(&dirent);
-
-		sci_free(catname);
-	}
-
-	fs_chdir("/ram");
-
-	qsort(_savegame_indices, _savegame_indices_nr, sizeof(struct _savegame_index_struct),
-	      _savegame_index_struct_compare);
-
-}
-
-#else /* !_DREAMCAST */
-
-static void
-update_savegame_indices(char *game_id)
-{
-	int i;
-	char *gfname = sci_malloc(strlen(game_id) + strlen(FREESCI_ID_SUFFIX) + 1);
-
-	strcpy(gfname, game_id);
-	strcat(gfname, FREESCI_ID_SUFFIX);
 
 	_savegame_indices_nr = 0;
 
@@ -836,29 +727,89 @@ update_savegame_indices(char *game_id)
 				_savegame_indices[_savegame_indices_nr++].timestamp = get_file_mtime(fd);
 				close(fd);
 			}
-			chdir(G_DIR_PARENT_S);
+			chdir("..");
 		}
 
 		free(dirname);
 	}
 
-	sci_free(gfname);
-
 	qsort(_savegame_indices, _savegame_indices_nr, sizeof(struct _savegame_index_struct),
 	      _savegame_index_struct_compare);
 
 }
 
-#endif
-
-void
-kGetSaveFiles(state_t *s, int funct_nr, int argc, heap_ptr argp)
+int
+test_savegame(state_t *s, char *savegame_id, char *savegame_name, int savegame_name_length)
 {
-	char *game_id = (char *) (UPARAM(0) + s->heap);
-	heap_ptr nametarget = UPARAM(1);
-	heap_ptr nameoffsets = UPARAM(2);
+	FILE *f;
+	char buffer[80];
+	int version = -1;
+
+	chdir(savegame_id);
+	f = fopen("state", "r");
+
+	if (!f) return 0;
+	while (!feof(f))
+	{
+		char *seeker;
+		fgets(buffer, sizeof(buffer), f);
+		if ((seeker = strstr(buffer, "savegame_version = ")) != NULL)
+		{
+			seeker += strlen("savegame_version = ");
+			version = strtol(seeker, NULL, 10);
+			break;
+		}
+	}
+
+	fclose(f);
+	return version >= FREESCI_MINIMUM_SAVEGAME_VERSION &&
+	       version <= FREESCI_CURRENT_SAVEGAME_VERSION;
+}
+
+reg_t
+kCheckSaveGame(state_t *s, int funct_nr, int argc, reg_t *argv)
+{
+	char *game_id = kernel_dereference_char_pointer(s, argv[0], 0);
+	int savedir_nr = UKPV(1);
+	char *buf = NULL;
+	char *workdir = _chdir_savedir(s);
+	TEST_DIR_OR_QUIT(workdir);
+
+	if (_savegame_indices_nr < 0) {
+		char *game_id_file_name = (char*)sci_malloc(strlen(game_id) + strlen(FREESCI_ID_SUFFIX) + 1);
+
+		strcpy(game_id_file_name, game_id);
+		strcat(game_id_file_name, FREESCI_ID_SUFFIX);
+		SCIkwarn(SCIkWARNING, "Savegame index list not initialized!\n");
+		update_savegame_indices(game_id_file_name);
+	}
+
+	savedir_nr = _savegame_indices[savedir_nr].id;
+
+
+	if (savedir_nr > MAX_SAVEGAME_NR-1) {
+		_chdir_restoredir(workdir);
+		return NULL_REG;
+	}
+
+	s->r_acc = make_reg(0, test_savegame(s, (buf = _k_get_savedir_name(savedir_nr)), NULL, 0));
+
+	_chdir_restoredir(workdir);
+	free(buf);
+
+	return s->r_acc;
+}
+
+
+reg_t
+kGetSaveFiles(state_t *s, int funct_nr, int argc, reg_t *argv)
+{
+	char *game_id = kernel_dereference_char_pointer(s, argv[0], 0);
+	char *nametarget = kernel_dereference_char_pointer(s, argv[1], 0);
+	reg_t nametarget_base = argv[1];
+	reg_t *nameoffsets = kernel_dereference_reg_pointer(s, argv[2], 0);
 	int gfname_len = strlen(game_id) + strlen(FREESCI_ID_SUFFIX) + 1;
-	char *gfname = sci_malloc(gfname_len);
+	char *gfname = (char*)sci_malloc(gfname_len);
 	int i;
 	char *workdir = _chdir_savedir(s);
 	TEST_DIR_OR_QUIT(workdir);
@@ -866,21 +817,14 @@ kGetSaveFiles(state_t *s, int funct_nr, int argc, heap_ptr argp)
 	strcpy(gfname, game_id);
 	strcat(gfname, FREESCI_ID_SUFFIX); /* This file is used to identify in-game savegames */
 
-	update_savegame_indices(game_id);
+	update_savegame_indices(gfname);
 
-	SCIkASSERT(UPARAM(0) >= 800);
-	SCIkASSERT(nametarget >= 800);
-	SCIkASSERT(nameoffsets >= 800);
-
-	s->acc = 0;
+	s->r_acc = NULL_REG;
 
 	for (i = 0; i < _savegame_indices_nr; i++) {
 		char *savedir_name = _k_get_savedir_name(_savegame_indices[i].id);
 		FILE *idfile;
 
-#ifdef _DREAMCAST
-		dc_retrieve_savegame(game_id, _savegame_indices[i].id);
-#endif
 		if (!chdir(savedir_name)) {
 
 
@@ -892,53 +836,49 @@ kGetSaveFiles(state_t *s, int funct_nr, int argc, heap_ptr argp)
 					if (namebuf[strlen(namebuf) - 1] == '\n')
 						namebuf[strlen(namebuf) - 1] = 0; /* Remove trailing newline */
 
-					++s->acc; /* Increase number of files found */
+					*nameoffsets = s->r_acc; /* Store savegame ID */
+					++s->r_acc.offset; /* Increase number of files found */
 
-					PUT_HEAP(nameoffsets, i); /* Write down the savegame number */
-					nameoffsets += 2; /* Make sure the next ID string address is written to the next pointer */
-					strncpy((char *) s->heap + nametarget, namebuf, SCI_MAX_SAVENAME_LENGTH); /* Copy identifier string */
-					s->heap[nametarget + SCI_MAX_SAVENAME_LENGTH - 1] = 0; /* Make sure it's terminated */
+					nameoffsets++; /* Make sure the next ID string address is written to the next pointer */
+					strncpy(nametarget, namebuf, SCI_MAX_SAVENAME_LENGTH); /* Copy identifier string */
+					*(nametarget + SCI_MAX_SAVENAME_LENGTH - 1) = 0; /* Make sure it's terminated */
 					nametarget += SCI_MAX_SAVENAME_LENGTH; /* Increase name offset pointer accordingly */
+					nametarget_base.offset += SCI_MAX_SAVENAME_LENGTH;
 
 					fclose(idfile);
 				}
 			}
-			chdir(G_DIR_PARENT_S);
+			chdir("..");
 		}
 		free(savedir_name);
 	}
 
 	free(gfname);
-	s->heap[nametarget] = 0; /* Terminate list */
+	*nametarget = 0; /* Terminate list */
 
 	_chdir_restoredir(workdir);
+	return s->r_acc;
 }
 
-
-void
-kSaveGame(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kSaveGame(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-	char *game_id = (char *) (UPARAM(0) + s->heap);
+	char *game_id = (char*)kernel_dereference_bulk_pointer(s, argv[0], 0);
 	char *savegame_dir;
-	int savedir_nr = UPARAM(1);
+	int savedir_nr = UKPV(1);
 	int savedir_id; /* Savegame ID, derived from savedir_nr and the savegame ID list */
-	char *game_id_file_name = sci_malloc(strlen(game_id) + strlen(FREESCI_ID_SUFFIX) + 1);
-	char *game_description = (char *) (UPARAM(2) + s->heap);
+	char *game_id_file_name = (char*)sci_malloc(strlen(game_id) + strlen(FREESCI_ID_SUFFIX) + 1);
+	char *game_description = (char*)kernel_dereference_bulk_pointer(s, argv[2], 0);
 	char *workdir = _chdir_savedir(s);
+	char *version = argc > 3 ? strdup((char*)kernel_dereference_bulk_pointer(s, argv[3], 0)) : NULL;
 	TEST_DIR_OR_QUIT(workdir);
 
-	if (soundserver_dead) {
-		sciprintf("Soundserver is dead- cannot save game state!");
-		s->acc = 0;
-		_chdir_restoredir(workdir);
-		return;
-	}
-
+	s->game_version = version;
 
 	strcpy(game_id_file_name, game_id);
 	strcat(game_id_file_name, FREESCI_ID_SUFFIX);
 
-	update_savegame_indices(game_id);
+	update_savegame_indices(game_id_file_name);
 
 	if (savedir_nr >= 0 && savedir_nr < _savegame_indices_nr)
 		/* Overwrite */
@@ -958,29 +898,20 @@ kSaveGame(state_t *s, int funct_nr, int argc, heap_ptr argp)
 		if (savedir_id >= MAX_SAVEGAME_NR) {
 			sciprintf("Internal error: Free savegame ID is %d, shouldn't happen!\n",
 				  savedir_id);
-			s->acc = 0;
-			return;
+			return NULL_REG;
 		}
 
 		/* This loop terminates when savedir_id is not in [x | ex. n. _savegame_indices[n].id = x] */
 	} else {
 		sciprintf("Savegame ID %d is not allowed!\n", savedir_nr);
-		s->acc = 0;
-		return;
+		return NULL_REG;
 	}
 
 	savegame_dir = _k_get_savedir_name(savedir_id);
 
-	s->acc = 1;
-
-#ifdef _GP32
-	/* Write save files without updating FAT to speed up the process. */
-	smNoFATUpdate("dev0:\\");
-#endif
-
 	if (gamestate_save(s, savegame_dir)) {
 		sciprintf("Saving the game failed.\n");
-		s->acc = 0;
+		s->r_acc = NULL_REG;
 	} else {
 		FILE *idfile;
 
@@ -994,37 +925,36 @@ kSaveGame(state_t *s, int funct_nr, int argc, heap_ptr argp)
 		} else {
 			sciprintf("Creating the game ID file failed.\n");
 			sciprintf("You can still restore from inside the debugger with \"restore_game %s\"\n", savegame_dir);
-			s->acc = 0;
+			s->r_acc = NULL_REG;
 		}
 
-		chdir (G_DIR_PARENT_S);
+		chdir ("..");
+		s->r_acc = make_reg(0, 1);
 	}
 	free(game_id_file_name);
-
-#ifdef _DREAMCAST
-	if (dc_store_savegame(game_id, game_description, savedir_id) < 0)
-		s->acc = 0;
-#elif defined(_GP32)
-	/* Update FAT. */
-	smFATUpdate("dev0:\\");
-#endif
-
-	sci_free(savegame_dir);
 	_chdir_restoredir(workdir);
+
+	free(s->game_version);
+	s->game_version = NULL;
+	return s->r_acc;
 }
 
 
-void
-kRestoreGame(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kRestoreGame(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-	char *game_id = (char *) (UPARAM(0) + s->heap);
-	int savedir_nr = UPARAM(1);
+	char *game_id = (char*)kernel_dereference_bulk_pointer(s, argv[0], 0);
+	int savedir_nr = UKPV(1);
 	char *workdir = _chdir_savedir(s);
 	TEST_DIR_OR_QUIT(workdir);
 
 	if (_savegame_indices_nr < 0) {
+		char *game_id_file_name = (char*)sci_malloc(strlen(game_id) + strlen(FREESCI_ID_SUFFIX) + 1);
+
+		strcpy(game_id_file_name, game_id);
+		strcat(game_id_file_name, FREESCI_ID_SUFFIX);
 		SCIkwarn(SCIkWARNING, "Savegame index list not initialized!\n");
-		update_savegame_indices(game_id);
+		update_savegame_indices(game_id_file_name);
 	}
 
 	savedir_nr = _savegame_indices[savedir_nr].id;
@@ -1043,37 +973,35 @@ kRestoreGame(state_t *s, int funct_nr, int argc, heap_ptr argp)
 			s->execution_stack_pos = s->execution_stack_base;
 
 		} else {
+			s->r_acc = make_reg(0, 1);
 			sciprintf("Restoring failed (game_id = '%s').\n", game_id);
 		}
 
 	} else {
-		s->acc = 1;
+		s->r_acc = make_reg(0, 1);
 		sciprintf("Savegame #%d not found!\n", savedir_nr);
 	}
 
 	_chdir_restoredir(workdir);
+	return s->r_acc;
 }
 
 
-void
-kValidPath(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kValidPath(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-	heap_ptr offset = UPARAM(0);
-	char *pathname = (char *) s->heap + offset;
+	char *pathname = kernel_dereference_char_pointer(s, argv[0], 0);
 	char cpath[PATH_MAX + 1];
 	getcwd(cpath, PATH_MAX + 1);
 
-	if (offset == s->save_dir_copy) {
-		pathname = s->save_dir_copy_buf;
-		SCIkdebug(SCIkFILE, "Magic: ValidPath(savedir_tmp)\n");
-	}
-
-	s->acc = !chdir(pathname); /* Try to go there. If it works, return 1, 0 otherwise. */
+	s->r_acc = make_reg(0, !chdir(pathname)); /* Try to go there. If it works, return 1, 0 otherwise. */
 
 	chdir(cpath);
+
+	return s->r_acc;
 }
 
-#define K_FILEIO_OPEN		0
+#define K_FILEIO_OPEN 		0
 #define K_FILEIO_CLOSE		1
 #define K_FILEIO_READ_RAW	2
 #define K_FILEIO_WRITE_RAW	3
@@ -1087,11 +1015,13 @@ kValidPath(state_t *s, int funct_nr, int argc, heap_ptr argp)
 
 
 char *
-write_filename_to_mem(state_t *s, heap_ptr address, char *string)
+write_filename_to_mem(state_t *s, reg_t address, char *string)
 {
+	char *mem = kernel_dereference_char_pointer(s, address, 0);
+
 	if (string) {
-		memset(s->heap + address, 0, 12);
-		strncpy((char *) s->heap + address, string, 11);
+		memset(mem, 0, 13);
+		strncpy(mem, string, 12);
 	}
 
 	return string;
@@ -1102,125 +1032,126 @@ next_file(state_t *s)
 {
 	if (write_filename_to_mem(s, s->dirseeker_outbuffer,
 				  sci_find_next(&(s->dirseeker))))
-		s->acc = s->dirseeker_outbuffer;
+		s->r_acc = s->dirseeker_outbuffer;
 	else
-		s->acc = 0;
+		s->r_acc = NULL_REG;
 }
 
 void
-first_file(state_t *s, char *dir, char *mask, heap_ptr buffer)
+first_file(state_t *s, const char *dir, char *mask, reg_t buffer)
 {
-	if (buffer == 0) {
+	if (!buffer.segment) {
 		sciprintf("Warning: first_file(state,\"%s\",\"%s\", 0) invoked!\n",
 			  dir, mask);
-		s->acc = 0;
+		s->r_acc = NULL_REG;
 		return;
 	}
 
-	if (strcmp(dir, G_DIR_CURRENT_S)) {
+	if (strcmp(dir, ".")) {
 		sciprintf("%s L%d: Non-local first_file: Not implemented yet\n",
 			  __FILE__, __LINE__);
-		s->acc = 0;
+		s->r_acc = NULL_REG;
 		return;
 	}
 
-	if (s->dirseeker_outbuffer)
+	/* Get rid of the old find structure */
+	if (s->dirseeker_outbuffer.segment)
 		sci_finish_find(&(s->dirseeker));
 
 	s->dirseeker_outbuffer = buffer;
 
 	if (write_filename_to_mem(s, s->dirseeker_outbuffer,
 				  sci_find_first(&(s->dirseeker), mask)))
-		s->acc = s->dirseeker_outbuffer;
+		s->r_acc = s->dirseeker_outbuffer;
 	else
-		s->acc = 0;
+		s->r_acc = NULL_REG;
 }
 
-void
-kFileIO(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kFileIO(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-  int func_nr = UPARAM(0);
-
+  int func_nr = UKPV(0);
 
   switch (func_nr) {
 
     case K_FILEIO_OPEN :
     {
-	char *name = (char *) s->heap + UPARAM(1);
-	int mode = UPARAM(2);
+	char *name = kernel_dereference_char_pointer(s, argv[1], 0);
+	int mode = UKPV(2);
 
 	file_open(s, name, mode);
 	break;
     }
     case K_FILEIO_CLOSE :
     {
-	int handle = UPARAM(1);
+	int handle = UKPV(1);
 
 	file_close(s, handle);
 	break;
     }
     case K_FILEIO_READ_RAW :
     {
-	char *dest = (char *) s->heap + UPARAM(2);
-	int size = UPARAM(3);
-	int handle = UPARAM(1);
+	int handle = UKPV(1);
+	char *dest = kernel_dereference_char_pointer(s, argv[2], 0);
+	int size = UKPV(3);
 
 	fread_wrapper(s, dest, size, handle);
 	break;
     }
     case K_FILEIO_WRITE_RAW :
     {
-	char *buf = (char *) s->heap + UPARAM(2);
-	int size = UPARAM(3);
-	int handle = UPARAM(1);
+	int handle = UKPV(1);
+	char *buf = kernel_dereference_char_pointer(s, argv[2], 0);
+	int size = UKPV(3);
 
 	fwrite_wrapper(s, handle, buf, size);
 	break;
     }
     case K_FILEIO_UNLINK :
     {
-	char *name = (char *) (s->heap + UPARAM(1));
+	char *name = kernel_dereference_char_pointer(s, argv[1], 0);
 
 	unlink(name);
 	break;
     }
     case K_FILEIO_READ_STRING :
     {
-	char *dest = (char *) (s->heap + UPARAM(1));
-	int size = UPARAM(2);
-	int handle = UPARAM(3);
+	char *dest = kernel_dereference_char_pointer(s, argv[1], 0);
+	int size = UKPV(2);
+	int handle = UKPV(3);
 
 	fgets_wrapper(s, dest, size, handle);
-	break;
+	return argv[1];
     }
     case K_FILEIO_WRITE_STRING :
     {
-	char *buf = (char *) (s->heap + UPARAM(1));
-	int size = UPARAM(2);
-	int handle = UPARAM(3);
+	int handle = UKPV(1);
+	int size = UKPV(3);
+	char *buf = kernel_dereference_char_pointer(s, argv[2], size);
 
-	fputs_wrapper(s, handle, buf);
+	if (buf)
+		fputs_wrapper(s, handle, size, buf);
 	break;
     }
     case K_FILEIO_SEEK :
     {
-	int handle = UPARAM(1);
-	int offset = UPARAM(2);
-	int whence = UPARAM(3);
+	int handle = UKPV(1);
+	int offset = UKPV(2);
+	int whence = UKPV(3);
 
 	fseek_wrapper(s, handle, offset, whence);
 	break;
     }
     case K_FILEIO_FIND_FIRST :
     {
-	char *mask = (char *) (s->heap + UPARAM(1));
-	heap_ptr buf = UPARAM(2);
-	int attr = UPARAM(3); /* We won't use this, Win32 might, though... */
+	char *mask = kernel_dereference_char_pointer(s, argv[1], 0);
+	reg_t buf = argv[2];
+	/* int attr = UKPV(3); */ /* We won't use this, Win32 might, though... */
 
 #ifndef _WIN32
 	if (strcmp(mask, "*.*")==0) strcpy(mask, "*"); /* For UNIX */
 #endif
-	first_file(s, G_DIR_CURRENT_S, mask, buf);
+	first_file(s, ".", mask, buf);
 
 	break;
     }
@@ -1231,11 +1162,13 @@ kFileIO(state_t *s, int funct_nr, int argc, heap_ptr argp)
     }
     case K_FILEIO_STAT :
     {
-	char *name = (char *) (s->heap + UPARAM(1));
-	s->acc=1-_k_check_file(name, 0);
+	char *name = kernel_dereference_char_pointer(s, argv[1], 0);
+	s->r_acc=make_reg(0, 1-_k_check_file(name, 0));
 	break;
     }
     default :
         SCIkwarn(SCIkERROR, "Unknown FileIO() sub-command: %d\n", func_nr);
   }
+
+  return s->r_acc;
 }
