@@ -30,134 +30,181 @@
 #include <sci_widgets.h>
 
 
-void
-kAddMenu(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kAddMenu(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-	menubar_add_menu(s->gfx_state, s->menubar, (char *) (s->heap + UPARAM(0)),
-			 (char *) (s->heap + UPARAM(1)), s->titlebar_port->font_nr, s->heap);
+	char *name = kernel_dereference_char_pointer(s, argv[0], 0);
+	char *contents = kernel_dereference_char_pointer(s, argv[1], 0);
+
+	menubar_add_menu(s->gfx_state, s->menubar, name,
+			 contents, s->titlebar_port->font_nr, argv[1]);
+
+	return s->r_acc;
+
 }
 
 
-void
-kSetMenu(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kSetMenu(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-	int index = UPARAM(0);
+	int index = UKPV(0);
 	int i = 2;
 
 	while (i < argc) {
-		menubar_set_attribute(s, (index >> 8) - 1, (index & 0xff) - 1, PARAM(i - 1), UPARAM(i));
+		menubar_set_attribute(s, (index >> 8) - 1, (index & 0xff) - 1, UKPV(i - 1), argv[i]);
 		i += 2;
 	}
+
+	return s->r_acc;
 }
 
-void
-kGetMenu(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kGetMenu(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-	int index = UPARAM(0);
+	int index = UKPV(0);
 
-	s->acc = menubar_get_attribute(s, (index >> 8) - 1, (index & 0xff) - 1, PARAM(1));
+	return menubar_get_attribute(s, (index >> 8) - 1, (index & 0xff) - 1, UKPV(1));
 }
 
 
-void
-kDrawStatus(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kDrawStatus(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-	heap_ptr text = PARAM(0);
-	int fgcolor = UPARAM_OR_ALT(1, s->status_bar_foreground);
-	int bgcolor = UPARAM_OR_ALT(2, s->status_bar_background);
+	reg_t text = argv[0];
+	int fgcolor = SKPV_OR_ALT(1, s->status_bar_foreground);
+	int bgcolor = SKPV_OR_ALT(2, s->status_bar_background);
 
-	s->titlebar_port->color=s->ega_colors[fgcolor];
-	s->titlebar_port->bgcolor=s->ega_colors[bgcolor];
+	s->titlebar_port->color.visual = *(get_pic_color(s, fgcolor));
+	s->titlebar_port->color.mask = GFX_MASK_VISUAL;
+	s->titlebar_port->bgcolor.visual = *(get_pic_color(s, bgcolor));
+	s->titlebar_port->bgcolor.mask = GFX_MASK_VISUAL;
 
 	s->status_bar_foreground=fgcolor;
 	s->status_bar_background=bgcolor;
 
-	if (s->status_bar_text)
-		free(s->status_bar_text);
+	if (NULL != s->status_bar_text) {
+		sci_free(s->status_bar_text);
+		s->status_bar_text = NULL;
+	}
 
-	s->status_bar_text = NULL;
-
-	if (text)
-		s->status_bar_text = sci_strdup((char *) s->heap + text);
+	if (text.segment)
+		s->status_bar_text = sci_strdup(kernel_dereference_char_pointer(s, text, 0));
 
 	sciw_set_status_bar(s, s->titlebar_port, s->status_bar_text, fgcolor, bgcolor);
 
 	gfxop_update(s->gfx_state);
+
+	return s->r_acc;
 }
 
 
-void
-kDrawMenuBar(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kDrawMenuBar(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
 
-	if (PARAM(0))
+	if (SKPV(0))
 		sciw_set_menubar(s, s->titlebar_port, s->menubar, -1);
 	else
 		sciw_set_status_bar(s, s->titlebar_port, NULL, 0, 0);
 
 	s->titlebar_port->draw(GFXW(s->titlebar_port), gfx_point(0, 0));
 	gfxop_update(s->gfx_state);
+
+	return s->r_acc;
 }
 
 
 #define ABOUT_FREESCI_PAGES_NR 7
 
-struct {
-	char *title;
-	char *body;
+static struct {
+	const char *title;
+	const char *body;
 	int fgcolor, bgcolor;
 } _about_freesci_pages[ABOUT_FREESCI_PAGES_NR] = {
 	{"FreeSCI hackers and contributors",
-	 "Alexander R. Angas\nDirectX 8 driver, Event sound server, Generic memory functions\n\n"
-	 "Anders Baden Nielsen\nPPC testing\n\n"
-	 "Bas Zoetekouw\nMan pages, debian package management, CVS maintenance\n\n"
-	 "Carl Muckenhoupt\nSources to the SCI resource viewer tools that started it all\n\n"
-	 "Chris Kehler\nMakefile enhancements\n\n"
-	 "Christoph Reichenbach\nProject & Website maintenance, UN*X code\n\n"
-	 "Christopher T. Lansdown\nOriginal CVS maintainer, Alpha compatibility fixes",
-	 0, 15},
+	 "Alexander R. Angas\nDirectX 8 driver, Event sound server, Generic memory functions"
+	 "\n\n"
+	 "Anders Baden Nielsen\nPPC testing"
+	 "\n\n"
+	 "Bas Zoetekouw\nMan pages, debian package management, CVS maintenance"
+	 "\n\n"
+	 "Carl Muckenhoupt\nSources to the SCI resource viewer tools that started it all"
+	 "\n\n"
+	 "Chris Kehler\nMakefile enhancements"
+	 "\n\n"
+	 "Christoph Reichenbach\nUN*X code, VM/Graphics/Sound/other infrastructure"
+	 "\n\n"
+	 "Christopher T. Lansdown\nOriginal CVS maintainer, Alpha compatibility fixes"
+	 ,0, 15},
 	{"More FreeSCI hackers and contributors",
-	 "Claudio Matsuoka\nCVS snapshots, daily builds, BeOS and cygwin ports\n\n"
-	 "Dark Minister\nSCI research (bytecode and parser)\n\n"
-	 "Dmitry Jemerov\nPort to the Win32 platform, numerous bugfixes\n\n"
-	 "Emmanuel Jeandel\nBugfixes and bug reports\n\n"
-	 "Francois-R Boyer\nMT-32 information and mapping code\n\n"
-	 "George Reid\nFreeBSD package management\n\n"
-	 "Hugues Valois\nGame selection menu",
-	 0, 15},
+	 "Claudio Matsuoka\nCVS snapshots, daily builds, BeOS and cygwin ports"
+	 "\n\n"
+	 "Dark Minister\nSCI research (bytecode and parser)"
+	 "\n\n"
+	 "Dmitry Jemerov\nPort to the Win32 platform, numerous bugfixes"
+	 "\n\n"
+	 "Emmanuel Jeandel\nBugfixes and bug reports"
+	 "\n\n"
+	 "Francois-R Boyer\nMT-32 information and mapping code"
+	 "\n\n"
+	 "George Reid\nFreeBSD package management"
+	 "\n\n"
+	 "Hubert Maier\nAmigaOS 4 port"
+	 ,0, 15},
 	{"Even more FreeSCI hackers & contributors",
-	 "Ismail Khatib\nPort to the PocketPC platform\n\n"
-	 "Lars Skovlund\nMost of the relevant documentation, several bugfixes\n\n"
-	 "Magnus Reftel\nHeap implementation, Python class viewer, bugfixes\n\n"
-	 "Matt Hargett\nClean-ups, bugfixes, Hardcore QA, Win32\n\n"
-	 "Paul David Doherty\nGame version information\n\n"
-	 "Petr Vyhnak\nThe DCL-INFLATE algorithm, many Win32 improvements\n\n"
-	 "Rainer Canavan\nIRIX MIDI driver and bug fixes",
-	 0, 15},
+	 "Hugues Valois\nGame selection menu"
+	 "\n\n"
+	 "Johannes Manhave\nDocument format translation"
+	 "\n\n"
+	 "Jordi Vilalta\nNumerous code and website clean-up patches"
+	 "\n\n"
+	 "Lars Skovlund\nProject maintenance, most documentation, bugfixes, SCI1 support"
+	 "\n\n"
+	 "Magnus Reftel\nHeap implementation, Python class viewer, bugfixes"
+	 "\n\n"
+	 "Matt Hargett\nClean-ups, bugfixes, Hardcore QA, Win32"
+	 "\n\n"
+	 "Max Horn\nSetJump implementation"
+	 ,0, 15},
 	{"Still more of them",
-	 "Rainer De Temple\nSCI research\n\n"
-	 "Ravi I.\nSCI0 sound resource specification\n\n"
-	 "Ruediger Hanke\nPort to the MorphOS platform\n\n"
-	 "Rune Orsval\nConfiguration file editor\n\n"
-	 "Rickard Lind\nMT32->GM MIDI mapping magic, sound research\n\n"
-	 "Rink Springer\nPort to the DOS platform, several bug fixes\n\n"
-	 "Robey Pointer\nBug tracking system hosting",
-	 0, 15},
-	{"And even more",
-	 "Sergey Lapin\nPort of Carl's type 2 decompression code\n\n"
-	 "Solomon Peachy\nSDL ports and much of the sound subsystem\n\n"
-	 "Vyacheslav Dikonov\nConfig script improvements\n\n"
+	 "Paul David Doherty\nGame version information"
+	 "\n\n"
+	 "Petr Vyhnak\nThe DCL-INFLATE algorithm, many Win32 improvements"
+	 "\n\n"
+	 "Rainer Canavan\nIRIX MIDI driver and bug fixes"
+	 "\n\n"
+	 "Rainer De Temple\nSCI research"
+	 "\n\n"
+	 "Ravi I.\nSCI0 sound resource specification"
+	 "\n\n"
+	 "Ruediger Hanke\nPort to the MorphOS platform"
+	 "\n\n"
+	 "Rune Orsval\nConfiguration file editor"
+	 ,0, 15},
+	{"Is there no end to these contributors?",
+	 "Rickard Lind\nMT32->GM MIDI mapping magic, sound research"
+	 "\n\n"
+	 "Rink Springer\nPort to the DOS platform, several bug fixes"
+	 "\n\n"
+	 "Robey Pointer\nBug tracking system hosting"
+	 "\n\n"
+	 "Sergey Lapin\nPort of Carl's type 2 decompression code"
+	 "\n\n"
+	 "Solomon Peachy\nSDL ports and much of the sound subsystem"
+	 "\n\n"
+	 "Vyacheslav Dikonov\nConfig script improvements"
+	 "\n\n"
 	 "Walter van Niftrik\nPorts to the Dreamcast and GP32 platforms",
 	 0, 15},
 	{"The CSCI5573 Team at CU Boulder",
 	 "Xiaojun Chen\nSean Terrell\nChristoph Reichenbach\n\n"
-	 "Special thanks to Prof. Dr. Gary Nutt\n\nfor allowing the FreeSCI VM extension as a\ncourse project in his Advanced OS course",
-	 0, 15},
+	 "Special thanks to Prof. Dr. Gary Nutt\n\nfor allowing the FreeSCI VM extension as a\ncourse project in his Advanced OS course"
+	 ,0, 15},
 	{"Special Thanks",
-	 "\nSpecial Thanks as well\n\n\nto the linuxgames.com and telefragged.com crew\nfor hosting us\n\n"
+	 "Special Thanks as well\n\n\nto the linuxgames.com and telefragged.com crew\nfor hosting us\n\n"
 	 "To the savannah.gnu.org staff\nfor hosting our mailing list\n\n"
-	 "To Bob Heitman and Corey Cole for their support",
-	 0, 15}
+	 "To Bob Heitman and Corey Cole for their support"
+	 ,0, 15}
 };
 
 
@@ -190,17 +237,19 @@ about_freesci(state_t *s)
 		_about_freesci_pages[page].fgcolor = 0;
 		_about_freesci_pages[page].bgcolor = 15;
 
-		gfxop_get_text_params(s->gfx_state, bodyfont, _about_freesci_pages[page].body, 300, &width, &height, 0);
-		gfxop_get_text_params(s->gfx_state, titlefont, _about_freesci_pages[page].title, 300, &width2, &foo, 0);
+		gfxop_get_text_params(s->gfx_state, bodyfont, _about_freesci_pages[page].body, 300, &width, &height, 0,
+				      NULL, NULL, NULL);
+		gfxop_get_text_params(s->gfx_state, titlefont, _about_freesci_pages[page].title, 300, &width2, &foo, 0,
+				      NULL, NULL, NULL);
 
 		width += 4;
 		width2 += 4;
-		height += 4;
+		height += 12;
 
 		if (width2 > width)
 			width = width2;
 
-		port = sciw_new_window(s, gfx_rect(156 - (width >> 1), 105 - (height >> 1), width, height),
+		port = sciw_new_window(s, gfx_rect(156 - (width >> 1), 100 - (height >> 1), width, height),
 				       bodyfont, s->ega_colors[_about_freesci_pages[page].fgcolor],
 				       s->ega_colors[_about_freesci_pages[page].bgcolor],
 				       titlefont, s->ega_colors[15], s->ega_colors[0],
@@ -263,14 +312,15 @@ _menu_go_down(state_t *s, int menu_nr, int item_nr)
   gfxop_update(s->gfx_state);
 
 
-void
-kMenuSelect(state_t *s, int funct_nr, int argc, heap_ptr argp)
+reg_t
+kMenuSelect(state_t *s, int funct_nr, int argc, reg_t *argv)
 {
-	heap_ptr event = UPARAM(0);
+	reg_t event = argv[0];
+	/*int pause_sound = UKPV_OR_ALT(1, 1);*/ /* FIXME: Do this eventually */
 	int claimed = 0;
-	int type = GET_SELECTOR(event, type);
-	int message = GET_SELECTOR(event, message);
-	int modifiers = GET_SELECTOR(event, modifiers);
+	int type = GET_SEL32V(event, type);
+	int message = GET_SEL32V(event, message);
+	int modifiers = GET_SEL32V(event, modifiers);
 	int menu_nr = -1, item_nr;
 	menu_item_t *item;
 	int menu_mode = 0; /* Menu is active */
@@ -350,7 +400,7 @@ kMenuSelect(state_t *s, int funct_nr, int argc, heap_ptr argp)
 			switch (event.type) {
 			case SCI_EVT_QUIT:
 				quit_vm();
-				return;
+				return NULL_REG;
 
 			case SCI_EVT_KEYBOARD:
 				switch (event.data) {
@@ -463,19 +513,21 @@ kMenuSelect(state_t *s, int funct_nr, int argc, heap_ptr argp)
 	}
 
 	if (claimed) {
-		PUT_SELECTOR(event, claimed, 1);
+		PUT_SEL32(event, claimed, make_reg(0, 1));
 
 		if (menu_nr > -1) {
-			s->acc = ((menu_nr + 1) << 8) | (item_nr + 1);
+			s->r_acc = make_reg(0, ((menu_nr + 1) << 8) | (item_nr + 1));
 #ifdef MENU_FREESCI_BLATANT_PLUG
 			if (s->menubar->menus[menu_nr].items[item_nr].flags == MENU_FREESCI_BLATANT_PLUG)
 				about_freesci(s);
 #endif
 
 		} else
-			s->acc = 0;
+			s->r_acc = NULL_REG;
 
-		SCIkdebug(SCIkMENU, "Menu: Claim -> %04x\n", s->acc);
+		SCIkdebug(SCIkMENU, "Menu: Claim -> %04x\n", s->r_acc.offset);
 	}
-	else s->acc = 0x0; /* Not claimed */
+	else s->r_acc = NULL_REG; /* Not claimed */
+
+	return s->r_acc;
 }
