@@ -1,5 +1,5 @@
 /***************************************************************************
- dc_driver.c Copyright (C) 2002-2004 Walter van Niftrik
+ dc_driver.c Copyright (C) 2002-2005 Walter van Niftrik
 
 
  This program may be modified and copied freely according to the terms of
@@ -125,8 +125,10 @@ pvr_init_gfx(struct _gfx_driver *drv, int xfact, int yfact, int bytespp)
 
 	/* Initialize PVR to defaults and set background color to black */
 
-	if (flags & SCI_DC_REFRESH_50HZ) vid_set_mode(DM_640x480_PAL_IL, PM_RGB565);
-	else vid_set_mode(DM_640x480, PM_RGB565);
+	if (flags & SCI_DC_REFRESH_50HZ)
+		vid_set_mode(DM_640x480_PAL_IL, PM_RGB565);
+	else
+		vid_set_mode(DM_640x480, PM_RGB565);
 	
 	pvr_init_defaults();
 	pvr_set_bg_color(0,0,0);
@@ -365,14 +367,12 @@ dc_map_key(int *keystate, uint8 key)
 ** Returns   : (int) Converted freesci keycode on success, 0 on error
 */
 {
-	if ((key >= KBD_KEY_A) && (key <= KBD_KEY_Z)) {
-		return ('a' + (key - KBD_KEY_A));
-	}
-	
-	if ((key >= KBD_KEY_1) && (key <= KBD_KEY_9)) {
-		return ('1' + (key - KBD_KEY_1));
-	}
-	
+	if ((key >= KBD_KEY_A) && (key <= KBD_KEY_Z))
+		return 'a' + (key - KBD_KEY_A);
+
+	if ((key >= KBD_KEY_1) && (key <= KBD_KEY_9))
+		return '1' + (key - KBD_KEY_1);
+
 	switch (key) {
 		case KBD_KEY_0:			return '0';
 		case KBD_KEY_BACKSPACE:		return SCI_K_BACKSPACE;
@@ -475,7 +475,8 @@ dc_input_thread(struct _gfx_driver *drv)
 
 		/* Keyboard handling */
 		/* Experimental workaround for the Mad Catz adapter problem */
-		if (!kaddr) kaddr = maple_enum_type(0, MAPLE_FUNC_KEYBOARD);
+		if (!kaddr)
+			kaddr = maple_enum_type(0, MAPLE_FUNC_KEYBOARD);
 		if (kaddr && (kbd = maple_dev_status(kaddr))) {
 			key = kbd->cond.keys[0];
 			skeys = kbd->shift_keys;
@@ -668,7 +669,8 @@ dc_input_thread(struct _gfx_driver *drv)
 					event.data = SCI_K_ESC;
 				event.type = SCI_EVT_KEYBOARD;
 				event.buckybits = bucky | vkbd_bucky;
-				if (event.data) dc_add_event(drv, &event);
+				if (event.data)
+					dc_add_event(drv, &event);
 			}
 		}
 		else drv->capabilities &= ~GFX_CAPABILITY_MOUSE_SUPPORT;
@@ -751,7 +753,8 @@ dc_draw_line_buffer(byte *buf, int line, int bytespp, int x1,
 				*pixel = color;
 				y += dy;
 				if (y >= dx) {
-					y -= dx; pixel += pixy;
+					y -= dx;
+					pixel += pixy;
 				}
 			}
 			break;
@@ -855,12 +858,12 @@ dc_copy_rect_buffer(byte *src, byte *dest, int srcline, int destline,
 static int
 dc_set_parameter(struct _gfx_driver *drv, char *attribute, char *value)
 {
-	if (!stricmp(attribute, "render_mode")) {
-		if (!stricmp(value, "vram")) {
+	if (!strcasecmp(attribute, "render_mode")) {
+		if (!strcasecmp(value, "vram")) {
 			flags &= ~SCI_DC_RENDER_PVR;
 			return GFX_OK;
 		}
-		else if (!stricmp(value, "pvr")) {
+		else if (!strcasecmp(value, "pvr")) {
 			flags |= SCI_DC_RENDER_PVR;
 			return GFX_OK;
 		}
@@ -869,12 +872,12 @@ dc_set_parameter(struct _gfx_driver *drv, char *attribute, char *value)
 			return GFX_FATAL;
 		}
 	}
-	if (!stricmp(attribute, "refresh_rate")) {
-		if (!stricmp(value, "60Hz")) {
+	if (!strcasecmp(attribute, "refresh_rate")) {
+		if (!strcasecmp(value, "60Hz")) {
 			flags &= ~SCI_DC_REFRESH_50HZ;
 			return GFX_OK;
 		}
-		else if (!stricmp(value, "50Hz")) {
+		else if (!strcasecmp(value, "50Hz")) {
 			flags |= SCI_DC_REFRESH_50HZ;
 			return GFX_OK;
 		}
@@ -900,9 +903,11 @@ dc_init_specific(struct _gfx_driver *drv, int xfact, int yfact, int bytespp)
 
 	pvr_shutdown();
 
-	if (!S)	S = sci_malloc(sizeof(struct _dc_state));
-	if (!S) return GFX_FATAL;
-	
+	if (!drv->state /* = S */)
+		drv->state = sci_malloc(sizeof(struct _dc_state));
+	if (!S)
+		return GFX_FATAL;
+
 	if ((flags & SCI_DC_RENDER_PVR) && ((xfact != 1 && xfact != 2)
 	  || bytespp != 2 || xfact != yfact)) {
 		sciprintf("Error: PVR rendering mode does not support "
@@ -983,7 +988,8 @@ dc_init_specific(struct _gfx_driver *drv, int xfact, int yfact, int bytespp)
 static int
 dc_init(struct _gfx_driver *drv)
 {
-	if (dc_init_specific(drv, 1, 1, 2) != GFX_OK) return GFX_FATAL;
+	if (dc_init_specific(drv, 1, 1, 2) != GFX_OK)
+		return GFX_FATAL;
 	
 	return GFX_OK;
 }
@@ -1018,8 +1024,9 @@ dc_exit(struct _gfx_driver *drv)
 	/*** Drawing operations ***/
 	
 static int
-dc_draw_line(struct _gfx_driver *drv, rect_t line, gfx_color_t color,
-	      gfx_line_mode_t line_mode, gfx_line_style_t line_style)
+dc_draw_line(struct _gfx_driver *drv, point_t start, point_t end,
+	      gfx_color_t color, gfx_line_mode_t line_mode,
+	      gfx_line_style_t line_style)
 {
 	uint32 scolor;
 	int xfact = (line_mode == GFX_LINE_MODE_FINE)? 1: XFACT;
@@ -1032,26 +1039,31 @@ dc_draw_line(struct _gfx_driver *drv, rect_t line, gfx_color_t color,
 
 	scolor = dc_get_color(drv, color);
 
-	x1 = line.x;
-	y1 = line.y;
-		
 	for (xc = 0; xc < xfact; xc++)
 		for (yc = 0; yc < yfact; yc++) {
-			x1 = line.x + xc;
-			y1 = line.y + yc;
-			x2 = x1 + line.xl;
-			y2 = y1 + line.yl;
-				
-			if (x1 < 0) x1 = 0;
-			if (x2 < 0) x2 = 0;
-			if (y1 < 0) y1 = 0;
-			if (y2 < 0) y2 = 0;
-				
-			if (x1 > xsize) x1 = xsize;
-			if (x2 >= xsize) x2 = xsize - 1;
-			if (y1 > ysize) y1 = ysize;
-			if (y2 >= ysize) y2 = ysize - 1;
+			x1 = start.x + xc;
+			y1 = start.y + yc;
+			x2 = end.x + xc;
+			y2 = end.y + yc;
+
+			if (x1 < 0)
+				x1 = 0;
+			if (x2 < 0)
+				x2 = 0;
+			if (y1 < 0)
+				y1 = 0;
+			if (y2 < 0)
+				y2 = 0;
 				
+			if (x1 > xsize)
+				x1 = xsize;
+			if (x2 >= xsize)
+				x2 = xsize - 1;
+			if (y1 > ysize)
+				y1 = ysize;
+			if (y2 >= ysize)
+				y2 = ysize - 1;
+
 			if (color.mask & GFX_MASK_VISUAL)
 				dc_draw_line_buffer(S->visual[1],
 				  XFACT*320*BYTESPP, BYTESPP, x1, y1, x2, y2,
@@ -1185,12 +1197,19 @@ dc_get_event(struct _gfx_driver *drv)
 	S->pointer_dx = 0;
 	S->pointer_dy = 0;
 	sem_signal(S->sem_pointer);
-	if (drv->pointer_x < 0) drv->pointer_x = 0;
-	if (drv->pointer_x >= 320*XFACT) drv->pointer_x = 320*XFACT-1;
-	if (drv->pointer_y < 0) drv->pointer_y = 0;
-	if (drv->pointer_y >= 200*YFACT) drv->pointer_y = 200*YFACT-1;
+
+	if (drv->pointer_x < 0)
+		drv->pointer_x = 0;
+	if (drv->pointer_x >= 320*XFACT)
+		drv->pointer_x = 320*XFACT-1;
+	if (drv->pointer_y < 0)
+		drv->pointer_y = 0;
+	if (drv->pointer_y >= 200*YFACT)
+		drv->pointer_y = 200*YFACT-1;
+
 	sem_wait(S->sem_event);
 	first = S->first_event;
+
 	if (first) {
 		event = first->event;
 		S->first_event = first->next;
@@ -1199,6 +1218,7 @@ dc_get_event(struct _gfx_driver *drv)
 		sem_signal(S->sem_event);
 		return event;
 	}
+
 	sem_signal(S->sem_event);
 	event.type = SCI_EVT_NONE;
 	event.buckybits = S->buckystate;
@@ -1211,20 +1231,21 @@ dc_usec_sleep(struct _gfx_driver *drv, long usecs)
 {
 	/* TODO: wake up on mouse move */
 	int ms = usecs/1000;
-	if (ms) thd_sleep(ms);
+	if (ms)
+		thd_sleep(ms);
 	return GFX_OK;
 }
 
 gfx_driver_t
 gfx_driver_dc = {
 	"dc",
-	"0.2",
+	"0.2b",
 	SCI_GFX_DRIVER_MAGIC,
 	SCI_GFX_DRIVER_VERSION,
 	NULL,
 	0,
 	0,
-	GFX_CAPABILITY_FINE_LINES | GFX_CAPABILITY_PIXMAP_GRABBING,
+	GFX_CAPABILITY_FINE_LINES,
 	0,
 	dc_set_parameter,
 	dc_init_specific,
