@@ -38,7 +38,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <directfb.h>
+#include <directfb/directfb.h>
 #include <unistd.h>
 
 #define SCIDFB_DRIVER_VERSION "0.4"
@@ -147,11 +147,9 @@ scidfb_mode_callback(unsigned int width, unsigned int height, unsigned int bpp,
 
 
 static DFBEnumerationResult
-scidfb_layer_callback(DFBDisplayLayerID layer_id, DFBDisplayLayerDescription descr,
-		      void *resultp)
+scidfb_layer_callback(unsigned int layer_id, DFBDisplayLayerCapabilities caps, void *resultp)
 {
 	unsigned int *results = (unsigned int *) resultp;
-	DFBDisplayLayerCapabilities caps = descr.caps;
 
 	if (!(caps & DLCAPS_SURFACE))
 		return DFENUM_OK; /* We need a surface */
@@ -207,14 +205,14 @@ scidfb_find_layer()
 
 	results[1] = 0;
 	SCIDFB_CHECKED(scidfb_framebuffer->EnumDisplayLayers(scidfb_framebuffer,
-							     scidfb_layer_callback, &results));
+						      scidfb_layer_callback, &results));
 
 	if (!results[1])
 		return GFX_FATAL; /* No decent layer! */
 	else {
 		SCIDFB_CHECKED(scidfb_framebuffer->GetDisplayLayer(scidfb_framebuffer,
-								   results[0],
-								   &scidfb_layer));
+							    results[0],
+							    &scidfb_layer));
 		
 		return GFX_OK;
 	}
@@ -253,12 +251,11 @@ _scidfb_init_input(int *found_keyboard, int *found_mouse)
 	}
 
 	
-	SCIDFB_CHECKED(scidfb_framebuffer->CreateInputEventBuffer(scidfb_framebuffer,
-								  DICAPS_KEYS
-								  | DICAPS_AXES
-								  | DICAPS_BUTTONS,
-								  DFB_TRUE,
-								  &scidfb_input_buffer));
+	SCIDFB_CHECKED(scidfb_framebuffer->CreateEventBuffer(scidfb_framebuffer,
+							     DICAPS_KEYS
+							     | DICAPS_AXES
+							     | DICAPS_BUTTONS,
+							     &scidfb_input_buffer));
 }
 
 static int
@@ -424,7 +421,7 @@ scidfb_xlate_key(DFBInputDeviceKeyIdentifier key, int keysym)
 	case DIKI_KP_9: return SCI_K_PGUP;
 
 	default:
-		break;
+
 	}
 	return 0; /* Could not map key */
 }
@@ -611,7 +608,7 @@ _scidfb_decode_pixel_format(DFBSurfacePixelFormat pixel_format,
 
 	switch (pixel_format) {
 
-	case DSPF_ARGB1555:
+	case DSPF_RGB15:
 		*bytespp = 2;
 		*rm = 5; _rs = 10;
 		*gm = 5; _gs = 5;
@@ -786,7 +783,7 @@ scidfb_exit(gfx_driver_t *drv)
 }
 
 static int
-scidfb_draw_line(gfx_driver_t *drv, rect_t line, gfx_color_t color,
+scidfb_draw_line(gfx_driver_t *drv, point_t start, point_t end, gfx_color_t color,
 		 gfx_line_mode_t line_mode, gfx_line_style_t line_style)
 {
 	int xc, yc;
@@ -795,15 +792,16 @@ scidfb_draw_line(gfx_driver_t *drv, rect_t line, gfx_color_t color,
 
 	if (line_mode == GFX_LINE_MODE_FINE) {
 		SCIDFB_CHECKED(scidfb_visual->DrawLine(scidfb_visual,
-						       line.x, line.y,
-						       line.x + line.xl, line.y + line.yl));
+						       start.x, start.y,
+						       end.x, end.y));
 	} else /* "Correct" lines */
 		for (xc = 0; xc < drv->mode->xfact; xc++)
 			for (yc = 0; yc < drv->mode->yfact; yc++)
 				SCIDFB_CHECKED(scidfb_visual->DrawLine(scidfb_visual,
-								       line.x + xc, line.y + yc,
-								       line.x + line.xl + xc,
-								       line.y + line.yl + yc));
+								       start.x + xc,
+								       start.y + yc,
+								       end.x + xc,
+								       end.y + yc));
 	
 	return GFX_OK;
 }
@@ -1011,7 +1009,6 @@ gfx_driver_t gfx_driver_dfb = {
 	NULL,
 	0, 0,
 	GFX_CAPABILITY_MOUSE_SUPPORT
-	| GFX_CAPABILITY_PIXMAP_GRABBING
 	| GFX_CAPABILITY_FINE_LINES,
 	0,
 	scidfb_set_parameter,
