@@ -1,5 +1,5 @@
 /***************************************************************************
- dx_driver.cpp Copyright (C) 2002,2003,2005-6 Alexander R Angas,
+ dx_driver.cpp Copyright (C) 2008 Alexander R Angas,
                Some portions Copyright (C) 1999 Dmitry Jemerov
 
  This program may be modified and copied freely according to the terms of
@@ -23,19 +23,28 @@
 		Alexander R Angas (Alex) <arangas AT internode dot on dot net>
 
  History:
-
 	20051106 (AAngas) - Rewrite
 	20060201 (AAngas) - Fixed wrong format for texture priority maps
 	20060205 (AAngas) - Changed pointer to use D3DXSprite
 	20060208 (AAngas) - Fixed pointer alpha blending bug
+	20060307 (AAngas) - Fixed crash on exit
+	20080118 (AAngas) - Fixed pointer scaling and window size
 
+ Notes:
+    DirectX handles all scaling. All drawing functions assume an unscaled
+	resolution.
 
 TODO:
-	Full screen
-	Lost devices
-	Moving and activating window
-	Allow user to specify hardware or software vertex processing
-	Add fancies
+	P1 - Corrupt cursor at different resolutions
+	P1 - Lost devices
+	P1 - Problems moving and activating window
+	P1 - Occasional pause when starting FreeSCI and poor sound resulting (?)
+	P1 - Mouse pointer corruption at bottom and why need to scale vertical by 4?
+	P2 - Full screen
+	P3 - Draw lines as Direct3D vertices
+	P3 - Fine line mode
+	P3 - Allow user to specify hardware or software vertex processing
+	P3 - Fancies
 
 ***************************************************************************/
 
@@ -47,12 +56,18 @@ TODO:
 
 #include <graphics_directx.h>
 
+#define TO_STRING2(x) #x
+#define TO_STRING(x) TO_STRING2(x)
 #if (DIRECT3D_VERSION < 0x0800)
-#error ERROR: DirectX 8 or higher is required for this driver.
+#	error The DirectX 8 SDK (or higher) is required for this driver.
+#elif (DIRECT3D_VERSION > 0x0800)
+#	pragma message (" ")
+#	pragma message ("*** This driver has been developed against version 8 of the DirectX SDK and may not work against your version " TO_STRING(DIRECT3D_VERSION))
+#	pragma message (" ")
 #endif
 
 
-// Store driver flags
+// Stores driver flags
 static int flags = 0;
 
 
@@ -71,7 +86,7 @@ RenderD3D(struct _gfx_driver *drv)
 {
 	HRESULT hr;
 
-	// Check we haven't lost the device
+	// Check we haven't lost the device (i.e. window focus)
 	if (CheckDevice(drv))
 	{
 		// Begin scene
@@ -138,9 +153,14 @@ CheckDevice(struct _gfx_driver *drv)
 
 // Create window to draw to
 static gfx_return_value_t
-InitWindow(struct _gfx_driver *drv)
+InitWindow(struct _gfx_driver *drv, UINT width, UINT height)
 {
-	// Register the window class.
+	DWORD dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;
+	DWORD dwStyle = WS_OVERLAPPEDWINDOW;
+	RECT clientSize = { 0, 0, width, height };
+	sciprintf("Window %d x %d\n", width, height);
+
+	// Register the window class
 	ZeroMemory( &(dx_state->wc), sizeof(WNDCLASSEX) );
 	dx_state->wc.cbSize = sizeof(WNDCLASSEX);
 	dx_state->wc.style = CS_HREDRAW | CS_VREDRAW | CS_NOCLOSE;
@@ -149,20 +169,29 @@ InitWindow(struct _gfx_driver *drv)
 	dx_state->wc.lpszClassName = DX_CLASS_NAME;
 	if ( RegisterClassEx( &dx_state->wc ) == 0 )
 	{
-		sciprintf("dx_init_specific(): RegisterClassEx failed (%u)\n", GetLastError());
+		sciprintf("InitWindow(): RegisterClassEx failed (%u)\n", GetLastError());
 		return GFX_FATAL;
 	}
 
-	// Create the application's window.
-	dx_state->hWnd = CreateWindow(
+	// Get correct size of window to pass to CreateWindow (considering window decorations)
+	if ( AdjustWindowRectEx( &clientSize, dwStyle, false, dwExStyle ) == 0 )
+	{
+		sciprintf("InitWindow(): Window size calculation failed (%u)\n", GetLastError());
+		return GFX_FATAL;
+	}
+
+	// Create the application's window
+	dx_state->hWnd = CreateWindowEx(
+		dwExStyle,
 		DX_CLASS_NAME, DX_APP_NAME,
-		0,
-		CW_USEDEFAULT, CW_USEDEFAULT, dx_state->xsize, dx_state->ysize,
-		GetDesktopWindow(), NULL, dx_state->wc.hInstance, NULL );
+		dwStyle | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
+		CW_USEDEFAULT, CW_USEDEFAULT,
+		clientSize.right - clientSize.left, clientSize.bottom - clientSize.top,
+		NULL/*GetDesktopWindow()*/, NULL, dx_state->wc.hInstance, NULL );
 
 	if ( dx_state->hWnd == NULL )
 	{
-		sciprintf("dx_init_specific(): CreateWindow failed (%u)\n", GetLastError());
+		sciprintf("InitWindow(): CreateWindow failed (%u)\n", GetLastError());
 		return GFX_FATAL;
 	}
 
@@ -180,7 +209,7 @@ InitD3D(struct _gfx_driver *drv)
 {
 	HRESULT hr;
 
-	sciprintf("Initializing Direct3D...\n");
+	sciprintf("Initializing Direct3D\n");
 
 	// Set our colour format
 	dx_state->d3dFormat = D3DFMT_A8R8G8B8;
@@ -202,7 +231,7 @@ InitD3D(struct _gfx_driver *drv)
 			break;
 		if (adapterId.Driver[0] == '\0')
 			break;
-		sciprintf(" Adapter %u: %s\n", adapterLoop++, adapterId.Description);
+		sciprintf("   Adapter %u: %s\n", adapterLoop++, adapterId.Description);
 	}
 	if (dx_state->adapterId == -1)
 		dx_state->adapterId = D3DADAPTER_DEFAULT;
@@ -219,8 +248,8 @@ InitD3D(struct _gfx_driver *drv)
 	dx_state->presParams.Windowed = TRUE;							// We want windowed by default
 	dx_state->presParams.SwapEffect = D3DSWAPEFFECT_DISCARD;		// Throw away last frame
 	dx_state->presParams.hDeviceWindow = dx_state->hWnd;			// Window handle
-	dx_state->presParams.BackBufferWidth  = dx_state->xsize;		// Back buffer dimensions
-	dx_state->presParams.BackBufferHeight = dx_state->ysize;		// 
+	dx_state->presParams.BackBufferWidth  = 320;					// Back buffer dimensions
+	dx_state->presParams.BackBufferHeight = 200;					// 
 	dx_state->presParams.BackBufferFormat = dx_state->d3dFormat;	// Colour format
 
 	// Check if user requested full screen
@@ -228,17 +257,15 @@ InitD3D(struct _gfx_driver *drv)
 	{
 		if (dx_state->deviceCaps.Caps2 & D3DCAPS2_CANRENDERWINDOWED) {
 			dx_state->presParams.Windowed = FALSE;
+			sciprintf("Full screen mode");
 		} else {
 			sciprintf("Sorry, DirectX will not render in full screen with your video card\n");
 		}
 	}
 
-	// Get current display mode and optimise for this screen
+	// Get current display mode
 	DODX( (dx_state->pD3d->GetAdapterDisplayMode( dx_state->adapterId, &dx_state->displayMode )), InitD3D );
-	sciprintf("   Display %lu x %lu\n", dx_state->displayMode.Width, dx_state->displayMode.Height);
-	if (dx_state->xfact == 0)
-		dx_state->xfact = (int)(dx_state->displayMode.Width / 320);
-	dx_state->yfact = dx_state->xfact;
+	sciprintf("Chosen adapter %u\n", dx_state->adapterId);
 
 	// Turn off Windows mouse pointer
 	ShowCursor(FALSE);
@@ -253,7 +280,7 @@ InitScene(struct _gfx_driver *drv)
 {
 	HRESULT hr;
 
-	sciprintf("Creating scene...\n");
+	sciprintf("Creating scene\n");
 
 	// Describe how scene will be rendered
 	DODX((dx_state->pDevice->SetRenderState( D3DRS_AMBIENT, RGB(255,255,255) )), InitScene);	// Maximum ambient light
@@ -303,28 +330,31 @@ dx_init_specific(struct _gfx_driver *drv,
 	int alpha_mask = 0x00000000, red_mask = 0x00ff0000, green_mask = 0x0000ff00, blue_mask = 0x000000ff;
 	gfx_return_value_t d3dret;
 
+	// Error checking
+	if (xfact < 1 || yfact < 1 || bytespp < 1 || bytespp > 4) {
+		sciprintf("Attempt to open window with invalid scale factors (%d,%d) and bpp=%d!\n",
+		 xfact, yfact, bytespp);
+		return GFX_ERROR;
+	}
+
+	// Prepare memory for gfx_dx_struct_t
 	drv->state = (struct gfx_dx_struct_t *) sci_malloc(sizeof(gfx_dx_struct_t));
 	ZeroMemory(drv->state, sizeof(gfx_dx_struct_t));
 	dx_state->adapterId = -1;	// we will set this later
 
-	// Check for scaling
-	if (xfact != 0)
-		dx_state->xfact = xfact;
-	if (yfact != 0)
-		dx_state->yfact = yfact;
+	// Set window size factor
+	dx_state->xfact = xfact;
+	dx_state->yfact = yfact;
+	dx_state->bpp = bytespp;
+	sciprintf("Window scaling %d x %d @ %d bpp\n", dx_state->xfact, dx_state->yfact, dx_state->bpp << 3);
 
 	// Set up Direct3D
 	d3dret = InitD3D(drv);
 	if (d3dret != GFX_OK)
 		return d3dret;
 
-	// Set window size factor (now that InitD3D has told us how big the window is)
-	dx_state->xsize = dx_state->xfact * 320;
-	dx_state->ysize = dx_state->yfact * 200;
-	dx_state->bpp = bytespp;
-
 	// Create window
-	InitWindow(drv);
+	InitWindow(drv, dx_state->xfact * 320, dx_state->yfact * 200);
 
 	// Create D3D device
 	DODX( (dx_state->pD3d->CreateDevice(dx_state->adapterId, D3DDEVTYPE_HAL, dx_state->hWnd,
@@ -340,9 +370,9 @@ dx_init_specific(struct _gfx_driver *drv,
 												D3DPOOL_MANAGED, &dx_state->pVertBuff )), dx_init_specific);
 
 	dx_state->pvData[0].p = D3DXVECTOR4(                  0.0f,                   0.0f, 0.0f, 1.0f);
-    dx_state->pvData[1].p = D3DXVECTOR4((float)dx_state->xsize,                   0.0f, 0.0f, 1.0f);
-    dx_state->pvData[2].p = D3DXVECTOR4(                  0.0f, (float)dx_state->ysize, 0.0f, 1.0f);
-    dx_state->pvData[3].p = D3DXVECTOR4((float)dx_state->xsize, (float)dx_state->ysize, 0.0f, 1.0f);
+    dx_state->pvData[1].p = D3DXVECTOR4(320.0f,                   0.0f, 0.0f, 1.0f);
+    dx_state->pvData[2].p = D3DXVECTOR4(                  0.0f, 200.0f, 0.0f, 1.0f);
+    dx_state->pvData[3].p = D3DXVECTOR4(320.0f, 200.0f, 0.0f, 1.0f);
 	dx_state->pvData[0].colour = dx_state->pvData[1].colour = dx_state->pvData[2].colour = dx_state->pvData[3].colour = 0xffffffff;
 	dx_state->pvData[0].t = D3DXVECTOR2(0.0f, 0.0f);
 	dx_state->pvData[1].t = D3DXVECTOR2(1.0f, 0.0f);
@@ -358,7 +388,7 @@ dx_init_specific(struct _gfx_driver *drv,
 	int i;
 	for (i = 0; i < NUM_VISUAL_BUFFERS; i++)
 	{
-		DODX((dx_state->pDevice->CreateTexture(dx_state->xsize, dx_state->ysize,
+		DODX((dx_state->pDevice->CreateTexture(320, 200,
 			1, 0,
 			dx_state->d3dFormat,
 			D3DPOOL_MANAGED,
@@ -366,7 +396,7 @@ dx_init_specific(struct _gfx_driver *drv,
 	}
 	for (i = 0; i < NUM_PRIORITY_BUFFERS; i++)
 	{
-		DODX((dx_state->pDevice->CreateTexture(dx_state->xsize, dx_state->ysize,
+		DODX((dx_state->pDevice->CreateTexture(320, 200,
 			1, 0,
 			dx_state->d3dFormat,
 			D3DPOOL_MANAGED,
@@ -379,9 +409,9 @@ dx_init_specific(struct _gfx_driver *drv,
 	// Allocate priority maps
 	for (int i = 0; i < NUM_PRIORITY_BUFFERS; i++)
 	{
-		dx_state->priority_maps[i] = gfx_pixmap_alloc_index_data(gfx_new_pixmap(dx_state->xsize, dx_state->ysize, GFX_RESID_NONE, -i, -777));
+		dx_state->priority_maps[i] = gfx_pixmap_alloc_index_data(gfx_new_pixmap(320, 200, GFX_RESID_NONE, -i, -777));
 		if (!dx_state->priority_maps[i]) {
-			GFXERROR("Out of memory: Could not allocate priority maps! (%dx%d)\n", dx_state->xsize, dx_state->ysize);
+			GFXERROR("Out of memory: Could not allocate priority maps! (%dx%d)\n", 320, 200);
 			return GFX_FATAL;
 		}
 		dx_state->priority_maps[i]->flags |= GFX_PIXMAP_FLAG_SCALED_INDEX;
@@ -394,17 +424,16 @@ dx_init_specific(struct _gfx_driver *drv,
 	dx_state->event_queue = (sci_event_t *) sci_malloc (dx_state->queue_size * sizeof (sci_event_t));
 
 	// Set up graphics mode for primary vis
-	drv->mode = gfx_new_mode(dx_state->xfact, dx_state->yfact, dx_state->bpp,
+	drv->mode = gfx_new_mode(1, 1, dx_state->bpp,
 			   red_mask, green_mask, blue_mask, alpha_mask,
 			   red_shift, green_shift, blue_shift, alpha_shift,
 			   (dx_state->bpp == 1) ? 256 : 0, 0);
 
 	// Set up graphics mode for pointer
-	gfx_mode_t *mode = gfx_new_mode(dx_state->xfact, dx_state->yfact, dx_state->bpp,
+	dx_state->pointerMode = gfx_new_mode(1, 4, dx_state->bpp,
     									0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000,
 										        24,         16,          8,          0,
 										0, GFX_MODE_FLAG_REVERSE_ALPHA);
-	dx_state->pointerMode = *mode;
 
 	return GFX_OK;
 }
@@ -414,7 +443,7 @@ dx_init_specific(struct _gfx_driver *drv,
 static int
 dx_init(struct _gfx_driver *drv)
 {
-	return dx_init_specific(drv, NULL, NULL, 4);
+	return dx_init_specific(drv, 2, 2, 4);
 }
 
 
@@ -437,7 +466,10 @@ dx_exit(struct _gfx_driver *drv)
     SAFE_RELEASE( dx_state->pDevice );
     SAFE_RELEASE( dx_state->pD3d );
 
-	gfx_free_mode(&dx_state->pointerMode);
+	if ( dx_state->pointerMode )
+		gfx_free_mode(dx_state->pointerMode);
+	if ( drv->mode )
+		gfx_free_mode(drv->mode);
 
 	if ( dx_state->event_queue )
 		sci_free(dx_state->event_queue);
@@ -476,12 +508,12 @@ dx_draw_filled_rect(struct _gfx_driver *drv, rect_t box,
 			lr.right++;
 		if (lr.top == lr.bottom)
 			lr.bottom++;
-		if ((UINT)r.right > dx_state->xsize)
-			lr.right = r.right = dx_state->xsize;
-		if ((UINT)r.bottom > dx_state->ysize)
-			lr.bottom = r.bottom = dx_state->ysize;
+		if ((UINT)r.right > 320)
+			lr.right = r.right = 320;
+		if ((UINT)r.bottom > 200)
+			lr.bottom = r.bottom = 200;
 
-//sciprintf("%08X  %i,%i -> %i,%i\n", lineColor, r.left, r.top, r.right, r.bottom);
+		sciprintf("dx_draw_filled_rect(): %08X  %i,%i -> %i,%i\n", lineColor, r.left, r.top, r.right, r.bottom);
 
 		DODX( (dx_state->pTexVisuals[BACK_VIS]->LockRect(0, &lockedRect, &lr, 0)), dx_draw_filled_rect );
 		UINT *rectPtr = (UINT*)lockedRect.pBits;
@@ -496,7 +528,7 @@ dx_draw_filled_rect(struct _gfx_driver *drv, rect_t box,
 				rectPtr++;
 			}
 			rectPtr = startLine;
-			rectPtr += dx_state->xsize;
+			rectPtr += 320;
 		}
 
 		DODX( (dx_state->pTexVisuals[BACK_VIS]->UnlockRect(0)), dx_draw_filled_rect );
@@ -505,11 +537,11 @@ dx_draw_filled_rect(struct _gfx_driver *drv, rect_t box,
 	if (color1.mask & GFX_MASK_PRIORITY)
 	{
 		byte *pri;
-		pri = dx_state->priority_maps[BACK_PRI]->index_data + box.x + box.y*(dx_state->xsize);
+		pri = dx_state->priority_maps[BACK_PRI]->index_data + box.x + box.y*(320);
 		for(int i=0; i<box.yl; i++)
 		{
 			memset(pri,color1.priority,box.xl);
-			pri += dx_state->xsize;
+			pri += 320;
 		}
 	}
 
@@ -519,20 +551,16 @@ dx_draw_filled_rect(struct _gfx_driver *drv, rect_t box,
 
 // Draws a single line to the back buffer
 static int
-dx_draw_line(struct _gfx_driver *drv, rect_t line, gfx_color_t color,
-		    gfx_line_mode_t line_mode, gfx_line_style_t line_style)
+dx_draw_line(struct _gfx_driver *drv,
+			point_t start, point_t end,
+			gfx_color_t color,
+			gfx_line_mode_t line_mode, gfx_line_style_t line_style)
 {
 	if (color.mask & GFX_MASK_VISUAL) {
 
-		// Calculate line thickness
-		int xf = (line_mode == GFX_LINE_MODE_FINE) ? 1 : dx_state->xfact;
-		int yf = (line_mode == GFX_LINE_MODE_FINE) ? 1 : dx_state->yfact;
-		
-		rect_t line_rect = { line.x, line.y, line.xl, line.yl };
-		if (line_rect.xl == 0)	// Line parallel to x axis
-			line_rect.xl += xf;
-		if (line_rect.yl == 0)	// Line parallel to y axis
-			line_rect.yl += yf;
+		rect_t line_rect = { start.x, start.y, end.x - start.x + 1, end.y - start.y + 1 };
+
+		sciprintf("dx_draw_line(): %i,%i -> %i,%i\n", line_rect.x, line_rect.y, line_rect.xl + line_rect.x, line_rect.yl + line_rect.y);
 
 		// Make sure priorities are not updated in dx_draw_filled_rect()
 		gfx_color_t col = color;
@@ -542,18 +570,7 @@ dx_draw_line(struct _gfx_driver *drv, rect_t line, gfx_color_t color,
 	}
 
 	if (color.mask & GFX_MASK_PRIORITY) {
-		unsigned int xc, yc;
-		rect_t newline;
-
-		newline.xl = line.xl;
-		newline.yl = line.yl;
-
-		for (xc = 0; xc < dx_state->xfact; xc++)
-			for (yc = 0; yc < dx_state->yfact; yc++) {
-				newline.x = line.x + xc;
-				newline.y = line.y + yc;
-				gfx_draw_line_pixmap_i(dx_state->priority_maps[BACK_PRI], newline, color.priority);
-			}
+		gfx_draw_line_pixmap_i(dx_state->priority_maps[BACK_PRI], start, end, color.priority);
 	}
 
 	return GFX_OK;
@@ -821,7 +838,7 @@ dx_set_static_buffer(struct _gfx_driver *drv, gfx_pixmap_t *pic,
 	SAFE_RELEASE(vbf);
 
 	// Copy priority map
-	gfx_copy_pixmap_box_i(dx_state->priority_maps[STATIC_PRI], priority, gfx_rect(0, 0, dx_state->xsize, dx_state->ysize));
+	gfx_copy_pixmap_box_i(dx_state->priority_maps[STATIC_PRI], priority, gfx_rect(0, 0, 320, 200));
 
 	return GFX_OK;
 }
@@ -837,6 +854,7 @@ dx_set_pointer(struct _gfx_driver *drv, gfx_pixmap_t *pointer)
 
 	if (pointer->data == NULL)
 		return GFX_OK;
+	pointer->yl /= 2;	// Scale the pointer to compensate for mode scale
 
 	LPDIRECT3DTEXTURE8 pntTex;
 	LPDIRECT3DSURFACE8 pntSrf;
@@ -848,7 +866,7 @@ dx_set_pointer(struct _gfx_driver *drv, gfx_pixmap_t *pointer)
 
 	// Recreate pointer data according to the graphics mode we need
 	gfx_pixmap_free_data(pointer);
-	gfx_xlate_pixmap(pointer, &dx_state->pointerMode, GFX_XLATE_FILTER_NONE);
+	gfx_xlate_pixmap(pointer, dx_state->pointerMode, GFX_XLATE_FILTER_NONE);
 
 	// Create texture and fill with pointer data
 	DODX( (dx_state->pDevice->CreateTexture(pointer->xl, pointer->yl, 1, 0, dx_state->d3dFormat, D3DPOOL_MANAGED, &pntTex )), dx_set_pointer );
@@ -1032,12 +1050,14 @@ ProcessMessages(struct _gfx_driver *drv)
 					MAP_KEY (VK_INSERT,		SCI_K_INSERT);
 					MAP_KEY (VK_DELETE,		SCI_K_DELETE);
 					MAP_KEY (VK_DECIMAL,	SCI_K_DELETE);
+					/* TODO: Glutton no longer had SCI_K etc... declared
 					MAP_KEY (VK_ADD,		SCI_K_PLUS);
 					MAP_KEY (VK_OEM_PLUS,	SCI_K_EQUALS);
 					MAP_KEY (VK_SUBTRACT,	SCI_K_MINUS);
 					MAP_KEY (VK_OEM_MINUS,	SCI_K_MINUS);
 					MAP_KEY (VK_MULTIPLY,	SCI_K_MULTIPLY);
 					MAP_KEY (VK_DIVIDE,		SCI_K_DIVIDE);
+					*/
 					MAP_KEY (VK_OEM_COMMA,	',');
 					MAP_KEY (VK_OEM_PERIOD,	'.');
 					MAP_KEY (VK_OEM_1,		';');	// US keyboards only
@@ -1130,12 +1150,12 @@ ProcessMessages(struct _gfx_driver *drv)
 extern "C"
 gfx_driver_t gfx_driver_dx = {
 	"directx",
-	"0.4.1",
+	"0.4.2",
 	SCI_GFX_DRIVER_MAGIC,
 	SCI_GFX_DRIVER_VERSION,
 	NULL,	/* mode */
 	0, 0,	/* mouse pointer position */
-	GFX_CAPABILITY_MOUSE_SUPPORT | GFX_CAPABILITY_MOUSE_POINTER | GFX_CAPABILITY_COLOR_MOUSE_POINTER | GFX_CAPABILITY_PIXMAP_REGISTRY | GFX_CAPABILITY_PIXMAP_GRABBING | GFX_CAPABILITY_FINE_LINES | GFX_CAPABILITY_WINDOWED,
+	GFX_CAPABILITY_MOUSE_SUPPORT | GFX_CAPABILITY_MOUSE_POINTER | GFX_CAPABILITY_COLOR_MOUSE_POINTER | GFX_CAPABILITY_PIXMAP_REGISTRY | GFX_CAPABILITY_WINDOWED,
 	0,
 	dx_set_param,
 	dx_init_specific,
