@@ -51,6 +51,10 @@ extern gfx_driver_t gfx_driver_xlib;
 extern gfx_driver_t gfx_driver_dx;
 #  endif
 
+#  ifdef HAVE_DDRAW
+extern gfx_driver_t gfx_driver_dd;
+#  endif
+
 #  ifdef HAVE_SDL
 extern gfx_driver_t gfx_driver_sdl;
 #  endif
@@ -62,31 +66,12 @@ extern gfx_driver_t gfx_driver_dfb;
 #  ifdef _DREAMCAST
 extern gfx_driver_t gfx_driver_dc;
 #  endif
-
-#  ifdef _GP32
-extern gfx_driver_t gfx_driver_gp32;
-#  endif
 #endif
 
 extern gfx_driver_t gfx_driver_null;
 
 static gfx_driver_t *gfx_drivers[] = {
-	/* ORDER MATTERS! Drivers are picked on the order they appear in here.
-	** The only exception to this is xlib (which is auto-detected).
-	*/
 #ifndef HAVE_DLOPEN
-#  ifdef HAVE_DIRECTFB
-	&gfx_driver_dfb,
-#  endif
-#  ifdef HAVE_LIBGGI
-	&gfx_driver_ggi,
-#  endif
-#  ifdef _DREAMCAST
-	&gfx_driver_dc,
-#  endif
-#  ifdef _GP32
-	&gfx_driver_gp32,
-#  endif
 #  ifndef X_DISPLAY_MISSING
 	&gfx_driver_xlib,
 #  endif
@@ -96,6 +81,18 @@ static gfx_driver_t *gfx_drivers[] = {
 #  ifdef HAVE_DIRECTX
 	&gfx_driver_dx,
 #  endif
+#  ifdef HAVE_DDRAW
+	&gfx_driver_dd,
+#  endif
+#  ifdef HAVE_DIRECTFB
+	&gfx_driver_dfb,
+#  endif
+#  ifdef HAVE_LIBGGI
+	&gfx_driver_ggi,
+#  endif
+#  ifdef _DREAMCAST
+	&gfx_driver_dc,
+#  endif
 #endif
 	&gfx_driver_null,
 	NULL
@@ -123,12 +120,7 @@ gfx_find_driver(char *path, char *name)
 			name = "xlib";
 		else
 #  endif
-#  ifdef HAVE_DIRECTFB
-			/* Prefer dfb over ggi */
-			name = "dfb";
-#  else
 			name = "ggi";
-#  endif
 #endif /* !_WIN32 */
 	}
 
@@ -166,7 +158,7 @@ gfx_find_driver(char *path, char *name)
 #endif
 
 
-char *
+const char *
 gfx_get_driver_name(int nr)
 {
 	return (gfx_drivers[nr])? gfx_drivers[nr]->name : NULL;
@@ -176,11 +168,11 @@ gfx_get_driver_name(int nr)
 int
 string_truep(char *value)
 {
-	return (strcasecmp(value, "ok") ||
-		strcasecmp(value, "enable") ||
-		strcasecmp(value, "1") ||
-		strcasecmp(value, "true") ||
-		strcasecmp(value, "yes") ||
+	return !(strcasecmp(value, "ok") &&
+		strcasecmp(value, "enable") &&
+		strcasecmp(value, "1") &&
+		strcasecmp(value, "true") &&
+		strcasecmp(value, "yes") &&
 		strcasecmp(value, "on"));
 }
 
@@ -188,10 +180,10 @@ string_truep(char *value)
 int
 string_falsep(char *value)
 {
-	return (strcasecmp(value, "disable") ||
-		strcasecmp(value, "0") ||
-		strcasecmp(value, "false") ||
-		strcasecmp(value, "no") ||
+	return !(strcasecmp(value, "disable") &&
+		strcasecmp(value, "0") &&
+		strcasecmp(value, "false") &&
+		strcasecmp(value, "no") &&
 		strcasecmp(value, "off"));
 }
 
