@@ -100,7 +100,7 @@ typedef struct {
 
 } gfx_ggi_struct_t;
 
-int flags;
+static int flags;
 
 static int
 ggi_set_param(gfx_driver_t *drv, char *attribute, char *value)
@@ -163,7 +163,7 @@ ggi_init_specific(gfx_driver_t *drv, int xres, int yres, int bpp)
 	if (ggiInit() < 0)
 		return GFX_FATAL;
 
-	meta = sci_calloc(sizeof(gfx_ggi_struct_t), 1);
+	meta = (gfx_ggi_struct_t *) sci_calloc(sizeof(gfx_ggi_struct_t), 1);
 
 	if (!(meta->vis = ggiOpen(NULL))) {
 		DEBUG_BASIC("ggiOpen() failed!\n");
@@ -176,7 +176,7 @@ ggi_init_specific(gfx_driver_t *drv, int xres, int yres, int bpp)
 		--frames;
 
 	if (!frames) {
-		DEBUG_BASIC("Initializing %dx%d at %d bpp failed\n", xres*320, yres*200, 1 << bpp);
+		DEBUG_BASIC("Initializing %dx%d at %d bpp failed\n", xres*320, yres*200, bpp << 3);
 		free(meta);
 		ggiExit();
 		return GFX_ERROR;
@@ -213,7 +213,7 @@ ggi_init_specific(gfx_driver_t *drv, int xres, int yres, int bpp)
 	}
 
 	if (frames < 2) {
-		meta->alt_back_buffer = sci_malloc(bpp * 320 * 200 * xres * yres);
+		meta->alt_back_buffer = (byte *) sci_malloc(bpp * 320 * 200 * xres * yres);
 		meta->back_vis = ggiOpen("memory:pointer", meta->alt_back_buffer, NULL);
 		if (ggiSetSimpleMode(meta->back_vis, xres * 320, yres * 200, 1, GT_8BIT)) {
 			sciprintf("GFXGGI: Warning: Setting mode for memory visual failed\n");
@@ -221,7 +221,7 @@ ggi_init_specific(gfx_driver_t *drv, int xres, int yres, int bpp)
 	} else meta->alt_back_buffer = NULL;
 
 	if (frames < 3) {
-		meta->static_buffer = sci_malloc(bpp * 320 * 200 * xres * yres);
+		meta->static_buffer = (byte *) sci_malloc(bpp * 320 * 200 * xres * yres);
 		meta->static_vis = ggiOpen("memory:pointer", meta->static_buffer, NULL);
 		if (ggiSetSimpleMode(meta->static_vis, xres * 320, yres * 200, 1, GT_8BIT)) {
 			sciprintf("GFXGGI: Warning: Setting mode for memory visual #2 failed\n");
@@ -253,7 +253,7 @@ ggi_init(gfx_driver_t *drv)
 	if (ggiInit() < 0)
 		return GFX_FATAL;
 
-	meta = sci_calloc(sizeof(gfx_ggi_struct_t), 1);
+	meta = (gfx_ggi_struct_t *) sci_calloc(sizeof(gfx_ggi_struct_t), 1);
 
 	if (!(meta->vis = ggiOpen(NULL))) {
 		DEBUG_BASIC("ggiOpen() failed!\n");
@@ -289,14 +289,18 @@ ggi_init(gfx_driver_t *drv)
 	DEBUG_BASIC("Attempting to create mode with %dx%d, graphtype=%08x, %d frames\n",
 		    mode.visible.x, mode.visible.y, mode.graphtype, mode.frames);
 
-	mode.virt.x = mode.size.x = mode.visible.x;
-	mode.virt.y = mode.size.y = mode.visible.y;
+	mode.virt.x = mode.visible.x;
+	mode.virt.y = mode.visible.y;
+	mode.size.x = GGI_AUTO;
+	mode.size.y = GGI_AUTO;
 
 	if (ggiSetMode(meta->vis, &mode)) {
 		sciprintf("GFXGGI: Could not set proposed graphics mode!\n");
 
-		mode.virt.x = mode.size.x = mode.visible.x += x_blank;
-		mode.virt.y = mode.size.y = mode.visible.y += y_blank;
+		mode.virt.x = mode.visible.x += x_blank;
+		mode.virt.y = mode.visible.y += y_blank;
+		mode.size.x = GGI_AUTO;
+		mode.size.y = GGI_AUTO;
 
 		DEBUG_BASIC("Attempting to create augmented mode with %dx%d, graphtype=%08x, %d frames\n",
 			    mode.visible.x, mode.visible.y, mode.graphtype, mode.frames);
@@ -346,7 +350,7 @@ ggi_init(gfx_driver_t *drv)
 	}
 
 	if (meta->frames < 2) {
-		meta->alt_back_buffer = sci_malloc((pixelformat->size >> 3) * mode.visible.x * mode.visible.y);
+		meta->alt_back_buffer = (byte *) sci_malloc((pixelformat->size >> 3) * mode.visible.x * mode.visible.y);
 		meta->back_vis = ggiOpen("memory:pointer", meta->alt_back_buffer, NULL);
 		if (ggiSetSimpleMode(meta->back_vis, mode.visible.x, mode.visible.y, 1, GT_8BIT)) {
 			sciprintf("GFXGGI: Warning: Setting mode for memory visual failed\n");
@@ -354,7 +358,7 @@ ggi_init(gfx_driver_t *drv)
 	} else meta->alt_back_buffer = NULL;
 
 	if (meta->frames < 3) {
-		meta->static_buffer = sci_malloc((pixelformat->size >> 3) * mode.visible.x * mode.visible.y);
+		meta->static_buffer = (byte *) sci_malloc((pixelformat->size >> 3) * mode.visible.x * mode.visible.y);
 		meta->static_vis = ggiOpen("memory:pointer", meta->static_buffer, NULL);
 		if (ggiSetSimpleMode(meta->static_vis, mode.visible.x, mode.visible.y, 1, GT_8BIT)) {
 			sciprintf("GFXGGI: Warning: Setting mode for memory visual #2 failed\n");
@@ -444,7 +448,7 @@ ggi_draw_filled_rect(gfx_driver_t *drv, rect_t box, gfx_color_t color1, gfx_colo
 		     gfx_rectangle_fill_t shade_mode);
 
 static int
-ggi_draw_line(gfx_driver_t *drv, rect_t line, gfx_color_t color,
+ggi_draw_line(gfx_driver_t *drv, point_t start, point_t end, gfx_color_t color,
 	      gfx_line_mode_t line_mode, gfx_line_style_t line_style)
 {
 	ggi_pixel pixel;
@@ -459,12 +463,12 @@ ggi_draw_line(gfx_driver_t *drv, rect_t line, gfx_color_t color,
 		xw = MODE->xfact, yw = MODE->yfact;
 	}
 
-	rx = line.x;
-	ry = line.y;
-	endx = rx + line.xl;
-	endy = ry + line.yl;
+	rx = start.x;
+	ry = start.y;
+	endx = end.x;
+	endy = end.y;
 
-	if (!line.xl && !line.yl)
+	if ((rx == endx) && (ry == endy))
 		return ggi_draw_filled_rect(drv, gfx_rect(rx, ry, xw, yw), color, color, GFX_SHADE_FLAT);
 
 	if (color.mask & GFX_MASK_PRIORITY) {
@@ -621,7 +625,7 @@ ggi_draw_pixmap(gfx_driver_t *drv, gfx_pixmap_t *pxm, int priority,
 		return GFX_FATAL;
 	}
 
-	gfx_crossblit_pixmap(MODE, pxm, priority, src, dest, dbuf->write,
+	gfx_crossblit_pixmap(MODE, pxm, priority, src, dest, (byte *) dbuf->write,
 			     dbuf->buffer.plb.stride,
 			     pri_map, MODE->xfact * 320, 1, 0);
 
@@ -961,7 +965,6 @@ ggi_get_event(gfx_driver_t *drv)
 static void
 init_input_ggi()
 {
-	gettimeofday(&_sci_ggi_redraw_loopt, NULL);
 	_sci_ggi_loopt = _sci_ggi_redraw_loopt;
 	buckybits = SCI_EVM_INSERT; /* Start up in "insert" mode */
 	/* reset timers, leave them at current time to send redraw events ASAP */
@@ -1008,8 +1011,8 @@ gfx_driver_t gfx_driver_ggi = {
 	SCI_GFX_DRIVER_VERSION,
 	NULL,
 	0,0,
-	GFX_CAPABILITY_PIXMAP_GRABBING | GFX_CAPABILITY_FINE_LINES,
-	GFX_DEBUG_POINTER | GFX_DEBUG_UPDATES | GFX_DEBUG_PIXMAPS | GFX_DEBUG_BASIC,
+	GFX_CAPABILITY_FINE_LINES, GFX_DEBUG_POINTER
+	| GFX_DEBUG_UPDATES | GFX_DEBUG_PIXMAPS | GFX_DEBUG_BASIC,
 	ggi_set_param,
 	ggi_init_specific,
 	ggi_init,
