@@ -1,5 +1,5 @@
 /***************************************************************************
- xlib_driver.c Copyright (C) 2000-2005 Christoph Reichenbach
+ xlib_driver.h Copyright (C) 2000 Christoph Reichenbach
 
 
  This program may be modified and copied freely according to the terms of
@@ -21,7 +21,7 @@
 
  Current Maintainer:
 
-    Christoph Reichenbach (CR) <jameson@linuxgames.com>
+    Christoph Reichenbach (CR) <creichen@gmail.com>
 
 ***************************************************************************/
 
@@ -43,6 +43,9 @@
 #if defined(HAVE_X11_EXTENSIONS_XRENDER_H)
 #  define HAVE_RENDER
 #  include <X11/extensions/Xrender.h>
+#  if defined(HAVE_X11_XFT_XFT_H)
+#    include <X11/Xft/Xft.h>
+#  endif
 #endif
 #include <errno.h>
 #endif
@@ -60,7 +63,14 @@
 
 #define X_COLOR_EXT(c) ((c << 8) | c)
 
-int flags;
+/* In C++ mode, we re-name ``class'' to ``class_''.  However, this screws up
+** our interface to xlib, so we're not doing it in here.
+*/
+#ifdef class
+#  undef class
+#endif
+
+static int flags;
 
 struct _xlib_state {
 	Display *display;
@@ -79,11 +89,14 @@ struct _xlib_state {
 	Cursor mouse_cursor;
 	byte *pointer_data[2];
 	int used_bytespp; /* bytes actually used to display stuff, rather than bytes occupied in data space */
+	XVisualInfo visinfo;
 #ifdef HAVE_RENDER
 	Picture picture;
 #endif
 };
 
+#define VISUAL S->visinfo.visual
+
 #define S ((struct _xlib_state *)(drv->state))
 
 #define XASS(foo) { int val = foo; if (!val) xlderror(drv, __LINE__); }
@@ -138,7 +151,7 @@ static int x_have_render(Display *display)
 static int debugline = 0;
 
 static void
-xldprintf(char *fmt, ...)
+xldprintf(const char *fmt, ...)
 {
 	va_list argp;
 	fprintf(stderr,"GFX-XLIB %d:", debugline);
@@ -179,17 +192,6 @@ xlib_map_color(gfx_driver_t *drv, gfx_color_t color)
 	return retval;
 }
 
-static unsigned long
-xlib_map_pixmap_color(gfx_driver_t *drv, gfx_pixmap_color_t pc)
-{
-	gfx_color_t color;
-
-	color.mask = GFX_MASK_VISUAL;
-	memcpy(&(color.visual), &pc, sizeof(gfx_pixmap_color_t));
-
-	return xlib_map_color(drv, color);
-}
-
 
 static int
 xlib_error_handler(Display *display, XErrorEvent *error)
@@ -277,15 +279,13 @@ xlib_draw_pixmap(struct _gfx_driver *drv, gfx_pixmap_t *pxm, int priority,
 		 rect_t src, rect_t dest, gfx_buffer_t buffer);
 
 
-
 static int
 xlib_init_specific(struct _gfx_driver *drv, int xfact, int yfact, int bytespp)
 {
 	XVisualInfo xvisinfo;
 	XSetWindowAttributes win_attr;
-	int default_screen, num_aux_buffers;
+	int default_screen;
 	int vistype = (bytespp == 1)? 3 /* PseudoColor */ : 4 /* TrueColor */;
-	int found_vistype = 0;
 	int red_shift, green_shift, blue_shift, alpha_shift;
 	int bytespp_physical;
 	int depth_mod; /* Number of bits to subtract from the depth while checking */
@@ -351,16 +351,18 @@ xlib_init_specific(struct _gfx_driver *drv, int xfact, int yfact, int bytespp)
 
 	depth_mod = 0;
 
-	do {
-		while ((((bytespp > 1) && (vistype >= 4))
-			|| ((bytespp == 1) && (vistype == 3)))
-		       && (!(found_vistype = XMatchVisualInfo(S->display, default_screen,
-							     (bytespp << 3) - depth_mod, vistype, &xvisinfo))))
-			vistype--;
-		++depth_mod;
-	} while (!found_vistype
-		 && (bytespp == 1
-		     || (depth_mod <= 7)));
+	/* Try all bit size twiddling */
+	for (depth_mod = 0; depth_mod < 8; depth_mod++)
+		/* Try all viable depths */
+		for (; bytespp > 0; bytespp--)
+			/* Try all interesting visuals */
+			for (vistype = 4; vistype >= 3; vistype--)
+				if (XMatchVisualInfo(S->display, default_screen,
+						     (bytespp << 3) - depth_mod, vistype, &xvisinfo))
+					goto found_visual;
+ found_visual:
+
+	S->visinfo = xvisinfo;
 
 	if (vistype < 3 || ((vistype == 3) && (bytespp != 1))) {
 		if (bytespp == 1) {
@@ -373,7 +375,7 @@ xlib_init_specific(struct _gfx_driver *drv, int xfact, int yfact, int bytespp)
 
 	S->colormap = win_attr.colormap =
 		XCreateColormap(S->display, RootWindow(S->display, default_screen),
-				xvisinfo.visual, (bytespp == 1)? AllocAll : AllocNone);
+				VISUAL, (bytespp == 1)? AllocAll : AllocNone);
 
 	win_attr.event_mask = PointerMotionMask | StructureNotifyMask | ButtonPressMask
 		| ButtonReleaseMask | KeyPressMask | KeyReleaseMask | ExposureMask;
@@ -381,7 +383,7 @@ xlib_init_specific(struct _gfx_driver *drv, int xfact, int yfact, int bytespp)
 
 	S->window = XCreateWindow(S->display, RootWindow(S->display, default_screen),
 				  0, 0, xsize, ysize, 0, xvisinfo.depth, InputOutput,
-				  xvisinfo.visual, (CWBackPixel | CWBorderPixel | CWColormap | CWEventMask),
+				  VISUAL, (CWBackPixel | CWBorderPixel | CWColormap | CWEventMask),
 				  &win_attr);
 
 	if (!S->window) {
@@ -454,9 +456,8 @@ xlib_init_specific(struct _gfx_driver *drv, int xfact, int yfact, int bytespp)
 		}
 	}
 
-	foo_image = XCreateImage(S->display, DefaultVisual(S->display,
-							   DefaultScreen(S->display)),
-				 bytespp << 3, ZPixmap, 0, sci_malloc(23), 2, 2, 8, 0);
+	foo_image = XCreateImage(S->display, VISUAL,
+				 bytespp << 3, ZPixmap, 0, (char*)sci_malloc(23), 2, 2, 8, 0);
 
 	bytespp_physical = foo_image->bits_per_pixel >> 3;
 #ifdef WORDS_BIGENDIAN
@@ -469,13 +470,12 @@ xlib_init_specific(struct _gfx_driver *drv, int xfact, int yfact, int bytespp)
 #ifdef HAVE_MITSHM
 	/* set up and test the XSHM extension to make sure it's sane */
 	if (have_shmem) {
-	  void *old_handler;
+	  XErrorHandler old_handler;
 
 	  x11_error = 0;
 	  old_handler = XSetErrorHandler(xlib_error_handler);
 
-	  foo_image = XShmCreateImage(S->display, DefaultVisual(S->display,
-								DefaultScreen(S->display)),
+	  foo_image = XShmCreateImage(S->display, VISUAL,
 				      bytespp_physical << 3, ZPixmap, 0, &shminfo, 2, 2);
 	  if (foo_image)
 	    shminfo.shmid = shmget(IPC_PRIVATE,
@@ -536,7 +536,7 @@ xlib_init_specific(struct _gfx_driver *drv, int xfact, int yfact, int bytespp)
 	/* create the visual buffers */
 	for (i = 0; i < 3; i++) {
 #ifdef HAVE_MITSHM
-	  void *old_handler;
+	  XErrorHandler old_handler;
 
 	  if (have_shmem && have_shmem != 2) {
 	    ERROR("Shared memory pixmaps not supported.  Reverting\n");
@@ -547,7 +547,7 @@ xlib_init_specific(struct _gfx_driver *drv, int xfact, int yfact, int bytespp)
 	  if (have_shmem) {
 	    old_handler = XSetErrorHandler(xlib_error_handler);
 
-	    if ((S->shm[i] = sci_malloc(sizeof(XShmSegmentInfo))) == 0) {
+	    if ((S->shm[i] = (XShmSegmentInfo*)sci_malloc(sizeof(XShmSegmentInfo))) == 0) {
 	      ERROR("AIEEEE!  Malloc failed!\n");
 	      return GFX_FATAL;
 	    }
@@ -618,8 +618,7 @@ xlib_init_specific(struct _gfx_driver *drv, int xfact, int yfact, int bytespp)
 	if (S->use_render) {
 		XRenderPictFormat * format
 			= XRenderFindVisualFormat (S->display,
-						   DefaultVisual(S->display,
-								 DefaultScreen(S->display)));
+						   VISUAL);
 		S->picture =  XRenderCreatePicture (S->display,
 						    (Drawable) S->visual[1],
 						    format,
@@ -665,7 +664,7 @@ xlib_xdpy_info()
 	XVisualInfo *visuals;
 	int visuals_nr;
 	Display *display = XOpenDisplay(NULL);
-	char *vis_classes[6] = {"StaticGray", "GrayScale", "StaticColor",
+	const char *vis_classes[6] = {"StaticGray", "GrayScale", "StaticColor",
 			    "PseudoColor", "TrueColor", "DirectColor"};
 
 	printf("Visuals provided by X11 server:\n");
@@ -677,15 +676,20 @@ xlib_xdpy_info()
 
 	for (i = 0; i < visuals_nr; i++) {
 		XVisualInfo *visual = visuals + i;
+
+		/* This works around an incompatibility between C++ and xlib: access visual->class */
+		int visual_class = *((int *) (((byte *)(&(visual->depth))) + sizeof(unsigned int)));
+
 		printf("%d:\t%d bpp %s(%d)\n"
-		       "\tR:%08x G:%08x B:%08x\n"
+		       "\tR:%08lx G:%08lx B:%08lx\n"
 		       "\tbits_per_rgb=%d\n"
 		       "\tcolormap_size=%d\n\n",
 		       i,
 		       visual->depth,
-		       (visual->class < 0 || visual->class >5)?
-		       "INVALID" : vis_classes[visual->class],
-		       visual->class,
+		       (visual_class < 0 || visual_class >5)?
+		       "INVALID" :
+		       vis_classes[visual_class],
+		       visual_class,
 		       visual->red_mask, visual->green_mask, visual->blue_mask,
 		       visual->bits_per_rgb, visual->colormap_size);
 
@@ -700,10 +704,14 @@ xlib_init(struct _gfx_driver *drv)
 {
 	int i;
 
-	for (i = 4; i > 0; i--)
+	/* Try 32-bit mode last due to compiz issue with bit depth 32. */
+	for (i = 3; i > 0; i--)
 		if (!xlib_init_specific(drv, -1, -1, i))
 			return GFX_OK;
 
+	if (!xlib_init_specific(drv, -1, -1, 4))
+		return GFX_OK;
+
 	fprintf(stderr, "Could not find supported mode!\n");
 	xlib_xdpy_info();
 
@@ -755,7 +763,7 @@ xlib_exit(struct _gfx_driver *drv)
   /*** Drawing operations ***/
 
 static int
-xlib_draw_line(struct _gfx_driver *drv, rect_t line, gfx_color_t color,
+xlib_draw_line(struct _gfx_driver *drv, point_t start, point_t end, gfx_color_t color,
 	       gfx_line_mode_t line_mode, gfx_line_style_t line_style)
 {
 	int linewidth = (line_mode == GFX_LINE_MODE_FINE)? 1:
@@ -776,24 +784,26 @@ xlib_draw_line(struct _gfx_driver *drv, rect_t line, gfx_color_t color,
 
 		XChangeGC(S->display, S->gc, GCLineWidth | GCLineStyle | GCForeground | GCCapStyle, &(S->gc_values));
 
-		XASS(XDrawLine(S->display, S->visual[1], S->gc, line.x + xmod,
-			       line.y + ymod, line.x + line.xl + xmod,
-			       line.y + line.yl + ymod));
+		XASS(XDrawLine(S->display, S->visual[1], S->gc,
+			       start.x + xmod, start.y + ymod,
+			       end.x + xmod, end.y + ymod));
 	}
 
 	if (color.mask & GFX_MASK_PRIORITY) {
 		int xc, yc;
-		rect_t newline;
-
-		newline.xl = line.xl;
-		newline.yl = line.yl;
+		point_t nstart, nend;
 
 		linewidth--;
 		for (xc = 0; xc <= linewidth; xc++)
 			for (yc = -linewidth; yc <= linewidth; yc++) {
-				newline.x = line.x + xc;
-				newline.y = line.y + yc;
-				gfx_draw_line_pixmap_i(S->priority[0], newline, color.priority);
+				nstart.x = start.x + xc;
+				nstart.y = start.y + yc;
+
+				nend.x = end.x + xc;
+				nend.y = end.y + yc;
+
+				gfx_draw_line_pixmap_i(S->priority[0],
+						       nstart, nend, color.priority);
 			}
 	}
 
@@ -805,7 +815,6 @@ xlib_draw_filled_rect(struct _gfx_driver *drv, rect_t rect,
 		      gfx_color_t color1, gfx_color_t color2,
 		      gfx_rectangle_fill_t shade_mode)
 {
-
 	if (color1.mask & GFX_MASK_VISUAL) {
 		S->gc_values.foreground = xlib_map_color(drv, color1);
 		XChangeGC(S->display, S->gc, GCForeground, &(S->gc_values));
@@ -825,6 +834,9 @@ xlib_draw_filled_rect_RENDER(struct _gfx_driver *drv, rect_t rect,
 			     gfx_color_t color1, gfx_color_t color2,
 			     gfx_rectangle_fill_t shade_mode)
 {
+	if (S->used_bytespp == 1) /* No room for alpha! */
+		return xlib_draw_filled_rect(drv, rect, color1, color2, shade_mode);
+
 	if (color1.mask & GFX_MASK_VISUAL) {
 		XRenderColor fg;
 
@@ -833,6 +845,7 @@ xlib_draw_filled_rect_RENDER(struct _gfx_driver *drv, rect_t rect,
 		fg.blue = X_COLOR_EXT(color1.visual.b);
 		fg.alpha = 0xffff - X_COLOR_EXT(color1.alpha);
 
+
 		XRenderFillRectangle(S->display,
 				     PictOpOver,
 				     S->picture,
@@ -857,7 +870,7 @@ xlib_register_pixmap(struct _gfx_driver *drv, gfx_pixmap_t *pxm)
 		ERROR("Attempt to register pixmap twice!\n");
 		return GFX_ERROR;
 	}
-	pxm->internal.info = XCreateImage(S->display, DefaultVisual(S->display, DefaultScreen(S->display)),
+	pxm->internal.info = XCreateImage(S->display, VISUAL,
 					    S->used_bytespp << 3, ZPixmap, 0, (char *) pxm->data, pxm->xl,
 					    pxm->yl, 8, 0);
 
@@ -1027,7 +1040,7 @@ xlib_create_cursor_data(gfx_driver_t *drv, gfx_pixmap_t *pointer, int mode)
 	int lines = pointer->yl;
 	int xc, yc;
 	int xfact = drv->mode->xfact;
-	byte *data = sci_calloc(linewidth, lines);
+	byte *data = (byte*)sci_calloc(linewidth, lines);
 	byte *linebase = data, *pos;
 	byte *src = pointer->index_data;
 
@@ -1082,8 +1095,8 @@ xlib_set_pointer(struct _gfx_driver *drv, gfx_pixmap_t *pointer)
 		XColor cols[2];
 		Pixmap visual, mask;
 		byte *mask_data, *visual_data;
-		int real_xl = (pointer->xl + 7 >> 3) << 3;
-		int i, j;
+		int real_xl = ((pointer->xl + 7) >> 3) << 3;
+		int i;
 
 		for (i = 0; i < 2; i++) {
 			cols[i].red = pointer->colors[i].r;
@@ -1135,7 +1148,7 @@ xlib_set_palette(struct _gfx_driver *drv, int index, byte red, byte green, byte
 
 
   /*** Event management ***/
-
+/*
 int
 x_unmap_key(gfx_driver_t *drv, int keycode)
 {
@@ -1143,7 +1156,7 @@ x_unmap_key(gfx_driver_t *drv, int keycode)
 
 	return 0;
 }
-
+*/
 int
 x_map_key(gfx_driver_t *drv, XEvent *key_event, char *character)
 {
@@ -1269,7 +1282,6 @@ x_get_event(gfx_driver_t *drv, int eventmask, long wait_usec, sci_event_t *sci_e
 
 	do {
 		int hasnext_event = 1;
-		int redraw_pointer_request = 0;
 
 		while (hasnext_event) {
 			if (sci_event) { /* Capable of handling any event? */
@@ -1342,6 +1354,9 @@ x_get_event(gfx_driver_t *drv, int eventmask, long wait_usec, sci_event_t *sci_e
 
 					    drv->pointer_x = event.xmotion.x;
 					    drv->pointer_y = event.xmotion.y;
+					    if (!sci_event)
+					    	    /* Wake up from sleep */
+					    	    return;
 				    }
 					    break;
 
@@ -1409,15 +1424,15 @@ xlib_usec_sleep(struct _gfx_driver *drv, long usecs)
 gfx_driver_t
 gfx_driver_xlib = {
 	"xlib",
-	"0.6",
+	"0.6a",
 	SCI_GFX_DRIVER_MAGIC,
 	SCI_GFX_DRIVER_VERSION,
 	NULL,
 	0, 0,
 	GFX_CAPABILITY_STIPPLED_LINES | GFX_CAPABILITY_MOUSE_SUPPORT
 	| GFX_CAPABILITY_MOUSE_POINTER | GFX_CAPABILITY_PIXMAP_REGISTRY
-	| GFX_CAPABILITY_PIXMAP_GRABBING | GFX_CAPABILITY_FINE_LINES
-	| GFX_CAPABILITY_WINDOWED,
+	| GFX_CAPABILITY_FINE_LINES | GFX_CAPABILITY_WINDOWED
+	| GFX_CAPABILITY_KEYTRANSLATE,
 	0/*GFX_DEBUG_POINTER | GFX_DEBUG_UPDATES | GFX_DEBUG_PIXMAPS | GFX_DEBUG_BASIC*/,
 	xlib_set_parameter,
 	xlib_init_specific,
