@@ -92,7 +92,7 @@ scale_char(byte *dest, byte *src, int width, int height, int newwidth, int xfact
 gfx_bitmap_font_t *
 gfxr_scale_font_unfiltered(gfx_bitmap_font_t *orig_font, gfx_mode_t *mode)
 {
-	gfx_bitmap_font_t *font = sci_malloc(sizeof(gfx_bitmap_font_t));
+	gfx_bitmap_font_t *font = (gfx_bitmap_font_t*)sci_malloc(sizeof(gfx_bitmap_font_t));
 	int height = orig_font->height * mode->yfact;
 	int width = 0;
 	int byte_width;
@@ -114,9 +114,9 @@ gfxr_scale_font_unfiltered(gfx_bitmap_font_t *orig_font, gfx_mode_t *mode)
 	font->height = height;
 	font->line_height = orig_font->line_height * mode->yfact;
 
-	font->widths = sci_malloc(sizeof(int) * orig_font->chars_nr);
+	font->widths = (int*)sci_malloc(sizeof(int) * orig_font->chars_nr);
 	font->char_size = byte_width * height;
-	font->data = sci_malloc(font->chars_nr * font->char_size);
+	font->data = (byte*)sci_malloc(font->chars_nr * font->char_size);
 
 	for (i = 0; i < font->chars_nr; i++) {
 		font->widths[i] = orig_font->widths[i] * mode->xfact;
@@ -150,8 +150,10 @@ gfxr_scale_font(gfx_bitmap_font_t *orig_font, gfx_mode_t *mode, gfxr_font_scale_
 
 
 text_fragment_t *
-gfxr_font_calculate_size(gfx_bitmap_font_t *font, int max_width, char *text,
-			 int *width, int *height, int *lines, int flags)
+gfxr_font_calculate_size(gfx_bitmap_font_t *font, int max_width, const char *text,
+			 int *width, int *height,
+			 int *lines, int *line_height_p, int *last_offset_p,
+			 int flags)
 {
 	int est_char_width = font->widths[(font->chars_nr > 'M')? 'M' : font->chars_nr - 1];
 	/* 'M' is typically among the widest chars */
@@ -164,13 +166,15 @@ gfxr_font_calculate_size(gfx_bitmap_font_t *font, int max_width, char *text,
 	int max_allowed_width = max_width;
 	int maxwidth = 0, localmaxwidth = 0;
 	int current_fragment = 1;
-	char *breakpoint_ptr = NULL;
+	const char *breakpoint_ptr = NULL;
 	unsigned char foo;
 
+	if (line_height_p)
+		*line_height_p = lineheight;
 
 	if (max_width>1) fragments_nr = 3 + (strlen(text) * est_char_width)*3 / (max_width << 1); else fragments_nr = 1;
 
-	fragments = sci_calloc(sizeof(text_fragment_t), fragments_nr);
+	fragments = (text_fragment_t*)sci_calloc(sizeof(text_fragment_t), fragments_nr);
 
 
 	fragments[0].offset = text;
@@ -205,7 +209,7 @@ gfxr_font_calculate_size(gfx_bitmap_font_t *font, int max_width, char *text,
 				maxwidth = localmaxwidth;
 
 			if (current_fragment == fragments_nr)
-				fragments = sci_realloc(fragments, sizeof(text_fragment_t) * (fragments_nr <<= 1));
+				fragments = (text_fragment_t*)sci_realloc(fragments, sizeof(text_fragment_t) * (fragments_nr <<= 1));
 
 			localmaxwidth = 0;
 
@@ -239,7 +243,7 @@ gfxr_font_calculate_size(gfx_bitmap_font_t *font, int max_width, char *text,
 				fragments[current_fragment++].offset = text;
 
 				if (current_fragment == fragments_nr)
-					fragments = sci_realloc(fragments, sizeof(text_fragment_t *) * (fragments_nr <<= 1));
+					fragments = (text_fragment_t*)sci_realloc(fragments, sizeof(text_fragment_t *) * (fragments_nr <<= 1));
 
 				localmaxwidth = localmaxwidth - last_breakpoint;
 				if (!(flags & GFXR_FONT_FLAG_COUNT_WHITESPACE))
@@ -260,8 +264,13 @@ gfxr_font_calculate_size(gfx_bitmap_font_t *font, int max_width, char *text,
 	else
 		*width = maxwidth;
 
-	*height = maxheight;
-	*lines = current_fragment;
+	if (last_offset_p)
+		*last_offset_p = localmaxwidth;
+
+	if (height)
+		*height = maxheight;
+	if (lines)
+		*lines = current_fragment;
 
 	fragments[current_fragment-1].length = text - fragments[current_fragment-1].offset - 1;
 
@@ -300,16 +309,17 @@ render_char(byte *dest, byte *src, int width, int line_width, int lines, int byt
 }
 
 gfx_pixmap_t *
-gfxr_draw_font(gfx_bitmap_font_t *font, char *stext, int characters,
+gfxr_draw_font(gfx_bitmap_font_t *font, const char *stext, int characters,
 	       gfx_pixmap_color_t *fg0, gfx_pixmap_color_t *fg1, gfx_pixmap_color_t *bg)
 {
 	unsigned char *text = (unsigned char *) stext;
 	int height = font->height;
-	int line_height = font->line_height;
 	int width = 0;
 	gfx_pixmap_t *pxm;
 	int fore_0, fore_1, back;
 	int i;
+	int hack = 0;
+	gfx_pixmap_color_t dummy = {0};
 	byte *offset;
 
 	for (i = 0; i < characters; i++) {
@@ -326,7 +336,14 @@ gfxr_draw_font(gfx_bitmap_font_t *font, char *stext, int characters,
 	pxm = gfx_pixmap_alloc_index_data(gfx_new_pixmap(width, height, GFX_RESID_NONE, 0, 0));
 
 	pxm->colors_nr = !!fg0 + !!fg1 + !!bg;
-	pxm->colors = sci_malloc(sizeof(gfx_pixmap_color_t) * pxm->colors_nr);
+	if (pxm->colors_nr == 0)
+	  {
+	    GFXWARN("Pixmap would have zero colors, resetting!\n");
+	    pxm->colors_nr = 3;
+	    hack = 1;
+	    fg0 = fg1 = bg = &dummy;
+	  }
+	pxm->colors = (gfx_pixmap_color_t*)sci_malloc(sizeof(gfx_pixmap_color_t) * pxm->colors_nr);
 #ifdef SATISFY_PURIFY
 	memset(pxm->colors, 0, sizeof(gfx_pixmap_color_t) * pxm->colors_nr);
 #endif
@@ -334,27 +351,27 @@ gfxr_draw_font(gfx_bitmap_font_t *font, char *stext, int characters,
 
 	i = 0;
 
-	if (fg0) {
+	if (fg0 || hack) {
 		memcpy(pxm->colors + i, fg0, sizeof(gfx_pixmap_color_t));
 		fore_0 = i++;
-	} else fore_0 = GFX_COLOR_INDEX_TRANSPARENT;
+	} else fore_0 = pxm->color_key;
 
-	if (fg1) {
+	if (fg1 || hack) {
 		memcpy(pxm->colors + i, fg1, sizeof(gfx_pixmap_color_t));
 		fore_1 = i++;
-	} else fore_1 = GFX_COLOR_INDEX_TRANSPARENT;
+	} else fore_1 = pxm->color_key;
 
-	if (bg) {
+	if (bg || hack) {
 		memcpy(pxm->colors + i, bg, sizeof(gfx_pixmap_color_t));
 		back = i++;
-	} else back = GFX_COLOR_INDEX_TRANSPARENT;
+	} else back = pxm->color_key;
 
 	offset = pxm->index_data;
 
 	memset(pxm->index_data, back, pxm->index_xl * pxm->index_yl);
 	for (i = 0; i < characters; i++) {
 		unsigned char ch = text[i];
-		int width = font->widths[ch];
+		width = font->widths[ch];
 
 		render_char(offset, font->data + (ch * font->char_size), width,
 			    pxm->index_xl, pxm->index_yl, font->row_size,
