@@ -28,9 +28,11 @@
 
 
 #include <console.h>
+#include <ctype.h>
 
 #ifdef WANT_CONSOLE
 #  define CON_MAX_CLUSTERS 16
+
 #define CON_CLUSTER_SIZE 64
 
 /* Number of console entries stored = CON_MAX_CLUSTERS * CON_CLUSTER_SIZE */
@@ -187,7 +189,7 @@ con_gfx_show(gfx_state_t *state)
 	if (!con.input_prompt) {
 		con.input_prompt = _con_render_text(state, CON_GFX_PROMPT, -1,
 						    &con.color_input, NULL);
-		con.input_text = sci_malloc(con.input_bufsize = 64);
+		con.input_text = (char*)sci_malloc(con.input_bufsize = 64);
 		con.input_text[0] = 0;
 		con.input_history_pos = 0;
 		con.partial_write = 0;
@@ -246,7 +248,7 @@ con_gfx_hide(gfx_state_t *state)
 static inline con_buffer_t *
 _create_con_buffer(con_buffer_t *prev)
 {
-	con_buffer_t *buf = sci_malloc(sizeof (con_buffer_t));
+	con_buffer_t *buf = (con_buffer_t *)sci_malloc(sizeof (con_buffer_t));
 	int i;
 
 	for (i = 0; i < CON_CLUSTER_SIZE; i++)
@@ -296,7 +298,7 @@ _add_into_con_buffer(gfx_pixmap_t *pixmap, char *text)
 
 		needlen += strlen(target->text);
 		oldtext = target->text;
-		target->text = sci_malloc(needlen+1);
+		target->text = (char *)sci_malloc(needlen+1);
 		strcpy(target->text, oldtext);
 		strcat(target->text, text);
 		free(oldtext);
@@ -322,7 +324,7 @@ _add_into_con_buffer(gfx_pixmap_t *pixmap, char *text)
 		target->pixmaps = NULL;
 		target->pixmaps_nr = 0;
 	} else {
-		target->pixmaps = sci_malloc(sizeof(gfx_pixmap_t *));
+		target->pixmaps = (gfx_pixmap_t **)sci_malloc(sizeof(gfx_pixmap_t *));
 		target->pixmaps[0] = pixmap;
 		target->pixmaps_nr = 1;
 	}
@@ -447,7 +449,7 @@ con_gfx_init()
 	con_set_pixmap_callback(con_gfx_insert_pixmap);
 	con.input_prompt = NULL;
 	con.input_text = NULL;
-	con.input_window = con.input_prompt_pos = -strlen(CON_GFX_PROMPT);
+	con.input_window = con.input_prompt_pos = -(int)strlen(CON_GFX_PROMPT);
 	con.cursor_position = 0;
 	con.input_precursor = NULL;
 	con.input_postcursor = NULL;
@@ -462,7 +464,7 @@ _init_con_font()
 
 	con_font.ID = 0;
 	con_font.chars_nr = CON_BUILTIN_CHARS_NR;
-	con_font.widths = sci_malloc(sizeof(int) * CON_BUILTIN_CHARS_NR);
+	con_font.widths = (int *)sci_malloc(sizeof(int) * CON_BUILTIN_CHARS_NR);
 	for (i = 0; i < CON_BUILTIN_CHARS_NR; i++)
 		con_font.widths[i] = CON_BUILTIN_CHARS_WIDTH;
 	con_font.row_size = (CON_BUILTIN_CHARS_WIDTH + 7) >> 3;
@@ -477,8 +479,8 @@ static gfx_pixmap_t **
 _con_render_text_multiline(gfx_state_t *state, char *text, int maxchars, int *nr)
 {
 	int pixmaps_allocd = 1;
-	gfx_pixmap_t **retval = sci_malloc(sizeof(gfx_pixmap_t *) * pixmaps_allocd);
-	char *printbuf = sci_malloc(maxchars + 8);
+	gfx_pixmap_t **retval = (gfx_pixmap_t **)sci_malloc(sizeof(gfx_pixmap_t *) * pixmaps_allocd);
+	char *printbuf = (char *)sci_malloc(maxchars + 8);
 	int index = 0;
 	int overwrap = 0;
 
@@ -508,7 +510,7 @@ _con_render_text_multiline(gfx_state_t *state, char *text, int maxchars, int *nr
 			overwrap = 1;
 
 		if (index == pixmaps_allocd)
-			retval = sci_realloc(retval, sizeof(gfx_pixmap_t *) * (pixmaps_allocd+= 4));
+			retval = (gfx_pixmap_t **)sci_realloc(retval, sizeof(gfx_pixmap_t *) * (pixmaps_allocd+= 4));
 
 		retval[index++] = _con_render_text(state, printbuf, len,
 						   &con.color_text, NULL);
@@ -641,7 +643,7 @@ con_gfx_read(gfx_state_t *state)
 		int must_rewin = 0; /* Re-calculate window */
 		int must_redraw_text = 0; /* Redraw display field */
 		if (slen+1 >= con.input_bufsize)
-			con.input_text = sci_realloc(con.input_text, con.input_bufsize += 64); 
+			con.input_text = (char *)sci_realloc(con.input_text, con.input_bufsize += 64); 
 
 		evt.type = 0;
 		while (!evt.type)
@@ -868,9 +870,8 @@ con_gfx_read(gfx_state_t *state)
 		}
 	} while (!done);
 
-	sciprintf(CON_GFX_PROMPT "%s\n", con.input_text);
 	retval = con.input_text;
-	con.input_text = sci_malloc(64);
+	con.input_text = (char *)sci_malloc(64);
 	con.input_text[0] = 0;
 	con.input_window = con.input_prompt_pos;
 	con.cursor_position = 0;
@@ -923,11 +924,13 @@ _con_redraw(gfx_state_t *state, int update_display_field, int update_input_field
 
 	if (update_input_field) {
 
-		if (con_border_width >= 2)
-			gfxop_draw_line(state, gfx_rect(0, con_displayed_lines
-							+ con_border_width - 2,
-							320, 0), con.color_input,
-					GFX_LINE_MODE_FINE, GFX_LINE_STYLE_NORMAL);
+		if (con_border_width >= 2) {
+			int y = con_displayed_lines + con_border_width - 2;
+
+			gfxop_draw_line(state, gfx_point(0, y), gfx_point(319, y),
+					con.color_input, GFX_LINE_MODE_FINE,
+					GFX_LINE_STYLE_NORMAL);
+		}
 
 		if (con.input_prompt) {
 			int promptlen = strlen(CON_GFX_PROMPT);
