@@ -35,8 +35,18 @@
 ** BYTESPP: Bytes per pixel
 */
 
+/* set optimisations for Win32: */
+/* g on: enable global optimizations */
+/* t on: use fast code */
+/* y on: suppress creation of frame pointers on stack */
+/* s off: disable minimize size code */
 #ifdef _WIN32
 #	include <memory.h>
+#	ifndef SATISFY_PURIFY
+#		pragma optimize( "s", off )
+#		pragma optimize( "gty", on )
+#		pragma intrinsic( memcpy, memset )
+#	endif
 #endif
 
 static void FUNCTION_NAME(byte *dest, byte *src, int bytes_per_dest_line, int bytes_per_src_line,
@@ -50,7 +60,6 @@ static void FUNCTION_NAME(byte *dest, byte *src, int bytes_per_dest_line, int by
 	int x, y;
 	int alpha_end = xl * bytes_per_alpha_pixel;
 
-
 	for (y = 0; y < yl; y++) {
 		int pixel_offset = 0;
 		int alpha_offset = 0;
@@ -59,7 +68,13 @@ static void FUNCTION_NAME(byte *dest, byte *src, int bytes_per_dest_line, int by
 #endif /* USE_PRIORITY */
 
 		for (x = 0; x < alpha_end; x += bytes_per_alpha_pixel) {
-			if ((alpha_test_mask & alpha[x]) < alpha_min)
+			if ((alpha_test_mask & alpha[x])
+#ifdef REVERSE_ALPHA
+			    >=
+#else
+			    <
+#endif
+			    alpha_min)
 #ifdef USE_PRIORITY
 				if (priority_buffer[priority_offset] <= priority) {
 					priority_buffer[priority_offset] = priority;
