@@ -4,8 +4,18 @@
 ** EXTRA_BYTE_OFFSET: Extra source byte offset for copying (used on big-endian machines in 24 bit mode)
 */
 
+/* set optimisations for Win32: */
+/* g on: enable global optimizations */
+/* t on: use fast code */
+/* y on: suppress creation of frame pointers on stack */
+/* s off: disable minimize size code */
 #ifdef _WIN32
 #	include <memory.h>
+#	ifndef SATISFY_PURIFY
+#		pragma optimize( "s", off )
+#		pragma optimize( "gty", on )
+#		pragma intrinsic( memcpy, memset )
+#	endif
 #endif
 
 #include <sci_memory.h>
@@ -31,7 +41,7 @@ FUNCNAME(gfx_mode_t *mode, gfx_pixmap_t *pxm, int scale)
 	byte *src = pxm->index_data;
 	byte *dest = pxm->data;
 	byte *alpha_dest = pxm->alpha_map;
-	int using_alpha = pxm->colors_nr < GFX_PIC_COLORS;
+	int using_alpha = pxm->color_key != GFX_PIXMAP_COLOR_KEY_NONE;
 	int separate_alpha_map = (!mode->alpha_mask) && using_alpha;
 
 	if (mode->flags & GFX_MODE_FLAG_REVERSE_ALPHA) {
@@ -42,10 +52,7 @@ FUNCNAME(gfx_mode_t *mode, gfx_pixmap_t *pxm, int scale)
         assert(bytespp == COPY_BYTES);
 
 	if (separate_alpha_map && !alpha_dest)
-		alpha_dest = pxm->alpha_map = sci_malloc(pxm->index_xl * xfact * pxm->index_yl * yfact);
-
-	if (!separate_alpha_map)
-		result_colors[GFX_COLOR_INDEX_TRANSPARENT] = alpha_color;
+		alpha_dest = pxm->alpha_map = (byte*)sci_malloc(pxm->index_xl * xfact * pxm->index_yl * yfact);
 
 	/* Calculate all colors */
 	for (i = 0; i < pxm->colors_nr; i++) {
@@ -62,6 +69,10 @@ FUNCNAME(gfx_mode_t *mode, gfx_pixmap_t *pxm, int scale)
 		result_colors[i] = col;
 	}
 
+	if (!separate_alpha_map && pxm->color_key != GFX_PIXMAP_COLOR_KEY_NONE)
+		result_colors[pxm->color_key] = alpha_color;
+
+	src = pxm->index_data; /* Workaround for gcc 4.2.3 bug on EMT64 */
 	for (y = 0; y < pxm->index_yl; y++) {
 		byte *prev_dest = dest;
 		byte *prev_alpha_dest = alpha_dest;
@@ -69,7 +80,7 @@ FUNCNAME(gfx_mode_t *mode, gfx_pixmap_t *pxm, int scale)
 		for (x = 0; x < pxm->index_xl; x++) {
 			int isalpha;
 			SIZETYPE col = result_colors[isalpha = *src++] << (EXTRA_BYTE_OFFSET * 8);
-			isalpha = (isalpha == 255) && using_alpha;
+			isalpha = (isalpha == pxm->color_key) && using_alpha;
 
 			/* O(n) loops. There is an O(ln(n)) algorithm for this, but its slower for small n (which we're optimizing for here).
 			** And, anyway, most of the time is spent in memcpy() anyway. */
@@ -181,7 +192,7 @@ FUNCNAME_LINEAR(gfx_mode_t *mode, gfx_pixmap_t *pxm, int scale)
 	byte *src = pxm->index_data;
 	byte *dest = pxm->data;
 	byte *alpha_dest = pxm->alpha_map;
-	int using_alpha = pxm->colors_nr < GFX_PIC_COLORS;
+	int using_alpha = pxm->color_key != GFX_PIXMAP_COLOR_KEY_NONE;
 	int separate_alpha_map = (!mode->alpha_mask) && using_alpha;
 	unsigned int masks[4], shifts[4], zero[3];
 	int x,y;
@@ -208,7 +219,7 @@ FUNCNAME_LINEAR(gfx_mode_t *mode, gfx_pixmap_t *pxm, int scale)
 	shifts[3] = mode->alpha_shift;
 
 	if (separate_alpha_map && !alpha_dest)
-		alpha_dest = pxm->alpha_map = sci_malloc(pxm->index_xl * xfact * pxm->index_yl * yfact);
+		alpha_dest = pxm->alpha_map = (byte*)sci_malloc(pxm->index_xl * xfact * pxm->index_yl * yfact);
 
 	for (y = 0; y < pxm->index_yl; y++) {
 		byte *linepos = dest;
@@ -226,7 +237,7 @@ FUNCNAME_LINEAR(gfx_mode_t *mode, gfx_pixmap_t *pxm, int scale)
 			ctexel[0] = ctexel[1] = ctexel[2] = ctexel[3] = 0;
 
 #define MAKE_PIXEL(cond, rec, other, nr) \
-			if ((cond) || (using_alpha && nr == 255)) { \
+			if ((cond) || (using_alpha && nr == pxm->color_key)) { \
 				rec[0] = other[0] - ctexel[0]; \
 				rec[1] = other[1] - ctexel[1]; \
 				rec[2] = other[2] - ctexel[2]; \
@@ -309,7 +320,7 @@ gfx_apply_delta(unsigned int *color, int *delta, int factor)
 #endif
 
 #define MAKE_PIXEL_TRILINEAR(cond, rec, nr) \
-			if (!(cond) || (using_alpha && nr == 255)) { \
+			if (!(cond) || (using_alpha && nr == pxm->color_key)) { \
 				rec[0] = 0; \
 				rec[1] = 0; \
 				rec[2] = 0; \
@@ -334,7 +345,7 @@ FUNCNAME_TRILINEAR(gfx_mode_t *mode, gfx_pixmap_t *pxm, int scale)
 	byte *src = pxm->index_data;
 	byte *dest = pxm->data;
 	byte *alpha_dest = pxm->alpha_map;
-	int using_alpha = pxm->colors_nr < GFX_PIC_COLORS;
+	int using_alpha = pxm->color_key != GFX_PIXMAP_COLOR_KEY_NONE;
 	int separate_alpha_map = (!mode->alpha_mask) && using_alpha;
 	unsigned int masks[4], shifts[4];
 	unsigned int pixels[4][4];
@@ -364,7 +375,7 @@ FUNCNAME_TRILINEAR(gfx_mode_t *mode, gfx_pixmap_t *pxm, int scale)
 		return; /* Duh. */
 
 	if (separate_alpha_map && !alpha_dest)
-		alpha_dest = pxm->alpha_map = sci_malloc(pxm->index_xl * xfact * pxm->index_yl * yfact);
+		alpha_dest = pxm->alpha_map = (byte*)sci_malloc(pxm->index_xl * xfact * pxm->index_yl * yfact);
 
 	src -= pxm->index_xl + 1;
 
