@@ -44,8 +44,8 @@
 #include <gfx_tools.h>
 #include <gfx_operations.h>
 
-
-int sci0_palette;
+int
+sci0_palette;
 
 gfx_pixmap_color_t gfx_sci0_image_colors[1][16];
 
@@ -55,11 +55,6 @@ gfxr_interpreter_get_static_palette(gfx_resstate_t *state, int version, int *col
 	return NULL;
 }
 
-
-void sci_sched_yield()
-{
-}
-
 int
 sciprintf(const char *fmt, ...)
 {
@@ -120,7 +115,7 @@ gfx_options_t graphics_options;
 gfx_options_t *options = &graphics_options;
 
 
-#define CAPABILITY_IDS_NR 8
+#define CAPABILITY_IDS_NR 11
 
 static struct {
 	int mask;
@@ -131,9 +126,12 @@ static struct {
 	{GFX_CAPABILITY_COLOR_MOUSE_POINTER, "multi-color mouse pointers"},
 	{GFX_CAPABILITY_PIXMAP_REGISTRY, "pixmaps must be registered"},
 	{GFX_CAPABILITY_SCALEABLE_PIXMAPS, "pixmap scaling"},
-	{GFX_CAPABILITY_PIXMAP_GRABBING, "pixmap grabbing"},
 	{GFX_CAPABILITY_STIPPLED_LINES, "stippled lines"},
-	{GFX_CAPABILITY_POINTER_PIXMAP_REGISTRY, "pointer pixmaps must be registered"}
+	{GFX_CAPABILITY_MOUSE_SUPPORT, "pointing device input"},
+	{GFX_CAPABILITY_POINTER_PIXMAP_REGISTRY, "pointer pixmaps must be registered"},
+	{GFX_CAPABILITY_FINE_LINES, "fine lines"},
+	{GFX_CAPABILITY_WINDOWED, "windowed mode active"},
+	{GFX_CAPABILITY_KEYTRANSLATE, "built-in keyboard translation"}
 };
 
 int
@@ -184,7 +182,7 @@ int test_fonts[TEST_FONTS_NR] = {0};
 int test_cursors[TEST_CURSORS_NR] = {0, 1};
 
 int
-gfxr_interpreter_options_hash(gfx_resource_types_t type, int version, gfx_options_t *options, void *internal)
+gfxr_interpreter_options_hash(gfx_resource_type_t type, int version, gfx_options_t *options, void *internal, int palette)
 {
 	return 0;
 }
@@ -199,7 +197,7 @@ arrdup(int *src, int count)
 }
 
 int *
-gfxr_interpreter_get_resources(gfx_resstate_t *resstate, gfx_resource_types_t type,
+gfxr_interpreter_get_resources(gfx_resstate_t *resstate, gfx_resource_type_t type,
 			       int version, int *entries_nr, void *internal)
 {
 	switch (type) {
@@ -270,6 +268,7 @@ gfxr_interpreter_init_pic(int version, gfx_mode_t *mode, int ID, void *internal)
 
 	pic->mode = mode;
 	pic->undithered_buffer = NULL;
+	pic->internal = NULL;
 
 	pic->control_map = gfx_pixmap_alloc_index_data(gfx_new_pixmap(320, 200, ID, 2, 0));
 	pic->priority_map = gfx_pixmap_alloc_index_data(gfx_new_pixmap(320 * mode->xfact,
@@ -379,7 +378,7 @@ gfx_pixmap_color_t view_colors[VIEW_COLORS_NR] = {
 };
 
 gfxr_view_t *
-gfxr_interpreter_get_view(gfx_resstate_t *state, int nr, void *internal)
+gfxr_interpreter_get_view(gfx_resstate_t *state, int nr, void *internal, int palette)
 {
 	gfxr_view_t *view;
 	gfxr_loop_t *loop;
@@ -550,7 +549,7 @@ gfxr_interpreter_get_cursor(gfx_resstate_t *state, int nr, void *internal)
 }
 
 gfx_pixmap_color_t *
-gfxr_interpreter_get_palette(int version, int *colors_nr, void *internal)
+gfxr_interpreter_get_palette(gfx_resstate_t *state, int version, int *colors_nr, void *internal, int nr)
 {
 	return NULL;
 }
@@ -721,9 +720,9 @@ test_a(void)
 
 
 #define TEST_LINE(x, y, xl, yl) \
-	gfxop_draw_line(state, gfx_rect(x, y, xl, yl), blue, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL); \
+	gfxop_draw_line(state, gfx_point(x, y), gfx_point((x)+(xl), (y)+(yl)), blue, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL); \
 	gfxop_set_clip_zone(state, gfx_rect(140, 60, 40, 40)); \
-	gfxop_draw_line(state, gfx_rect(x, y, xl, yl), red, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL); \
+	gfxop_draw_line(state, gfx_point(x, y), gfx_point((x)+(xl), (y)+(yl)), red, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL); \
 	gfxop_set_clip_zone(state, gfx_rect(0, 0, 320, 200));
 
 #define LINES_NR 19
@@ -760,20 +759,20 @@ test_b(void)
 	MESSAGE("Some tests will include 'fine' lines.\nNote that support for those is\noptional.");
 	waitkey();
 
-	gfxop_draw_line(state, gfx_rect(30, 30, 260, 0), red, GFX_LINE_MODE_FINE, GFX_LINE_STYLE_NORMAL);
-	gfxop_draw_line(state, gfx_rect(30, 40, 260, 0), blue, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL);
-	gfxop_draw_line(state, gfx_rect(30, 50, 260, 0), green, GFX_LINE_MODE_FINE, GFX_LINE_STYLE_STIPPLED);
-	gfxop_draw_line(state, gfx_rect(30, 60, 260, 0), white, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_STIPPLED);
+	gfxop_draw_line(state, gfx_point(30, 30), gfx_point(290, 30), red, GFX_LINE_MODE_FINE, GFX_LINE_STYLE_NORMAL);
+	gfxop_draw_line(state, gfx_point(30, 40), gfx_point(290, 40), blue, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL);
+	gfxop_draw_line(state, gfx_point(30, 50), gfx_point(290, 50), green, GFX_LINE_MODE_FINE, GFX_LINE_STYLE_STIPPLED);
+	gfxop_draw_line(state, gfx_point(30, 60), gfx_point(290, 60), white, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_STIPPLED);
 	update();
 	MESSAGE("B.0: horizontal lines:\nYou should now be seeing (top-down):\nred-fine, blue-normal,\ngreen-fine-stippled, white-stippled");
 	waitkey();
 
 	clear();
-	gfxop_draw_line(state, gfx_rect(30, 30, 260, 100), blue, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL);
-	gfxop_draw_line(state, gfx_rect(30, 130, 260, -100), blue, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL);
+	gfxop_draw_line(state, gfx_point(30, 30), gfx_point(290, 130), blue, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL);
+	gfxop_draw_line(state, gfx_point(30, 130), gfx_point(290, 30), blue, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL);
 	gfxop_set_clip_zone(state, gfx_rect(140, 60, 40, 40));
-	gfxop_draw_line(state, gfx_rect(30, 30, 260, 100), red, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL);
-	gfxop_draw_line(state, gfx_rect(30, 130, 260, -100), red, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL);
+	gfxop_draw_line(state, gfx_point(30, 30), gfx_point(290, 130), red, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL);
+	gfxop_draw_line(state, gfx_point(30, 130), gfx_point(290, 30), red, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL);
 	gfxop_set_clip_zone(state, gfx_rect(0, 0, 320, 200));
 	update();
 	MESSAGE("B.1: line clipping:\nTwo identical pairs of lines.\nblue lines are unclipped,\nred ones are clipped.");
@@ -896,10 +895,11 @@ test_d(void)
 		} else if (event.type == SCI_EVT_MOUSE_RELEASE)
 
 			if (pressed) {
+				point_t line_pt = gfx_point(line.x, line.y);
 				pressed = 0;
 				line.xl = state->pointer_pos.x - line.x;
 				line.yl = state->pointer_pos.y - line.y;
-				gfxop_draw_line(state, line, red, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL);
+				gfxop_draw_line(state, state->pointer_pos, line_pt, red, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL);
 
 				if (line.xl < 0) {
 					line.x += line.xl;
@@ -955,7 +955,7 @@ test_e(void)
 
 	for (x = -20; x < 340; x++) {
 		clear_buffer();
-		gfxop_draw_cel(state, 0, 0, 0, gfx_point(x, 40), white);
+		gfxop_draw_cel(state, 0, 0, 0, gfx_point(x, 40), white, 0);
 		update();
 		gfxop_usleep(state, 10000);
 	}
@@ -963,10 +963,10 @@ test_e(void)
 	MESSAGE("E.2: Pic views\nFour pic views will now be added to\nthe static buffer");
 	waitkey();
 
-	gfxop_draw_cel_static(state, 0, 0, 1, gfx_point(50, 100), white24);
-	gfxop_draw_cel_static(state, 0, 0, 1, gfx_point(50, 20), white24);
-	gfxop_draw_cel_static(state, 0, 0, 1, gfx_point(220, 20), white16);
-	gfxop_draw_cel_static(state, 0, 0, 1, gfx_point(220, 100), white16);
+	gfxop_draw_cel_static(state, 0, 0, 1, gfx_point(50, 100), white24, 0);
+	gfxop_draw_cel_static(state, 0, 0, 1, gfx_point(50, 20), white24, 0);
+	gfxop_draw_cel_static(state, 0, 0, 1, gfx_point(220, 20), white16, 0);
+	gfxop_draw_cel_static(state, 0, 0, 1, gfx_point(220, 100), white16, 0);
 
 	update();
 	MESSAGE("E.2: Pic views\nThe pic views should NOT\nbe visible yet!\n");
@@ -982,8 +982,8 @@ test_e(void)
 
 	for (x = -20; x < 340; x++) {
 		clear_buffer();
-		gfxop_draw_cel(state, 0, 0, 2, gfx_point(x, 20), white8);
-		gfxop_draw_cel(state, 0, 0, 2, gfx_point(x, 100), white16);
+		gfxop_draw_cel(state, 0, 0, 2, gfx_point(x, 20), white8, 0);
+		gfxop_draw_cel(state, 0, 0, 2, gfx_point(x, 100), white16, 0);
 		update();
 		gfxop_usleep(state, 10000);
 	}
@@ -998,8 +998,8 @@ test_e(void)
 	for (x = -20; x < 340; x++) {
 		gfxop_clear_box(state, gfx_rect(x-9, 40-8, 17, 16));
 		gfxop_clear_box(state, gfx_rect(x-9, 70-8, 17, 16));
-		gfxop_draw_cel(state, 0, 0, 2, gfx_point(x, 40), white16);
-		gfxop_draw_cel(state, 0, 0, 2, gfx_point(x, 70), white16);
+		gfxop_draw_cel(state, 0, 0, 2, gfx_point(x, 40), white16, 0);
+		gfxop_draw_cel(state, 0, 0, 2, gfx_point(x, 70), white16, 0);
 		gfxop_update(state);
 		/*		gfxop_update_box(state, gfx_rect(x-1, 40, 17, 16)); */
 		/*		gfxop_update_box(state, gfx_rect(x-1, 70, 17, 16)); */
