@@ -31,8 +31,12 @@
 /* set optimisations for Win32: */
 #ifdef _WIN32
 #	include <memory.h>
+#	ifndef SATISFY_PURIFY
+#		pragma intrinsic( memcpy, memset )
+#	endif
 #endif
 
+rect_t gfx_rect_fullscreen = {0, 0, 320, 200};
 
 void
 gfx_clip_box_basic(rect_t *box, int maxx, int maxy)
@@ -56,7 +60,7 @@ gfx_new_mode(int xfact, int yfact, int bytespp, unsigned int red_mask, unsigned
 	     unsigned int blue_mask, unsigned int alpha_mask, int red_shift, int green_shift,
 	     int blue_shift, int alpha_shift, int palette, int flags)
 {
-	gfx_mode_t *mode = sci_malloc(sizeof(gfx_mode_t));
+	gfx_mode_t *mode = (gfx_mode_t*)sci_malloc(sizeof(gfx_mode_t));
 #ifdef SATISFY_PURIFY
 	memset(mode, 0, sizeof(gfx_mode_t));
 #endif
@@ -75,12 +79,12 @@ gfx_new_mode(int xfact, int yfact, int bytespp, unsigned int red_mask, unsigned
 	mode->flags = flags;
 
 	if (palette) {
-		mode->palette = sci_malloc(sizeof(gfx_palette_t));
+		mode->palette = (gfx_palette_t*)sci_malloc(sizeof(gfx_palette_t));
 #ifdef SATISFY_PURIFY
 		memset(mode->palette, 0, sizeof(gfx_palette_t));
 #endif
 		mode->palette->max_colors_nr = palette;
-		mode->palette->colors = sci_calloc(sizeof(gfx_palette_color_t), palette); /* Initialize with empty entries */
+		mode->palette->colors = (gfx_palette_color_t*)sci_calloc(sizeof(gfx_palette_color_t), palette); /* Initialize with empty entries */
 	} else mode->palette = NULL;
 
   return mode;
@@ -110,6 +114,9 @@ gfx_copy_pixmap_box_i(gfx_pixmap_t *dest, gfx_pixmap_t *src, rect_t box)
 
 	gfx_clip_box_basic(&box, dest->index_xl, dest->index_yl);
 
+	if (box.xl <= 0 || box.yl <= 0)
+		return;
+
 	height = box.yl;
 	width = box.xl;
 
@@ -125,7 +132,7 @@ gfx_copy_pixmap_box_i(gfx_pixmap_t *dest, gfx_pixmap_t *src, rect_t box)
 gfx_pixmap_t *
 gfx_clone_pixmap(gfx_pixmap_t *pxm, gfx_mode_t *mode)
 {
-	gfx_pixmap_t *clone = malloc(sizeof(gfx_pixmap_t));
+	gfx_pixmap_t *clone = (gfx_pixmap_t*)sci_malloc(sizeof(gfx_pixmap_t));
 	*clone = *pxm;
 	clone->index_data = NULL;
 	clone->colors = NULL;
@@ -134,7 +141,7 @@ gfx_clone_pixmap(gfx_pixmap_t *pxm, gfx_mode_t *mode)
 
 	memcpy(clone->data, pxm->data, clone->data_size);
 	if (clone->alpha_map) {
-		clone->alpha_map = malloc(clone->xl * clone->yl);
+		clone->alpha_map = (byte *) sci_malloc(clone->xl * clone->yl);
 		memcpy(clone->alpha_map, pxm->alpha_map, clone->xl * clone->yl);
 	}
 
@@ -144,12 +151,15 @@ gfx_clone_pixmap(gfx_pixmap_t *pxm, gfx_mode_t *mode)
 gfx_pixmap_t *
 gfx_new_pixmap(int xl, int yl, int resid, int loop, int cel)
 {
-	gfx_pixmap_t *pxm = sci_malloc(sizeof(gfx_pixmap_t));
+	gfx_pixmap_t *pxm = (gfx_pixmap_t*)sci_malloc(sizeof(gfx_pixmap_t));
 #ifdef SATISFY_PURIFY
 	memset(pxm, 0, sizeof(gfx_pixmap_t));
 #endif
 
-	pxm->alpha_map = pxm->data = pxm->internal.info = pxm->colors = NULL;
+	pxm->alpha_map = NULL;
+	pxm->data = NULL;
+	pxm->internal.info = NULL;
+	pxm->colors = NULL;
 	pxm->internal.handle = 0;
 
 	pxm->index_xl = xl;
@@ -163,6 +173,8 @@ gfx_new_pixmap(int xl, int yl, int resid, int loop, int cel)
 
 	pxm->flags = 0;
 
+	pxm->color_key = 0xff;
+
 	return pxm;
 }
 
@@ -203,7 +215,7 @@ gfx_free_pixmap(gfx_driver_t *driver, gfx_pixmap_t *pxm)
 	if (pxm->data)
 		free(pxm->data);
 
-	if ((pxm->colors) && !(pxm->flags & GFX_PIXMAP_FLAG_EXTERNAL_PALETTE))
+	if (pxm->colors && !(pxm->flags & GFX_PIXMAP_FLAG_EXTERNAL_PALETTE))
 		free(pxm->colors);
 
 	free(pxm);
@@ -224,10 +236,10 @@ gfx_pixmap_alloc_index_data(gfx_pixmap_t *pixmap)
 	if (!size)
 		size = 1;
 
-	pixmap->index_data = sci_malloc(size);
-#ifdef SATISFY_PURIFY
+	pixmap->index_data = (byte*)sci_malloc(size);
+
 	memset(pixmap->index_data, 0, size);
-#endif
+
 	return pixmap;
 }
 
@@ -268,7 +280,7 @@ gfx_pixmap_alloc_data(gfx_pixmap_t *pixmap, gfx_mode_t *mode)
 	if (!size)
 		size = 1;
 
-	pixmap->data = sci_malloc(pixmap->data_size = size);
+	pixmap->data = (byte*)sci_malloc(pixmap->data_size = size);
 	return pixmap;
 }
 
@@ -350,7 +362,7 @@ gfx_alloc_color(gfx_palette_t *pal, gfx_pixmap_color_t *color)
 
 	color->global_index = bestcolor;
 
-	GFXWARN("Out of palette colors- doing approximated mapping!\n");
+//	GFXWARN("Out of palette colors- doing approximated mapping!\n");
 	return GFX_OK;
 }
 
@@ -370,13 +382,13 @@ gfx_free_color(gfx_palette_t *pal, gfx_pixmap_color_t *color)
 	}
 
 	if (color->global_index >= pal->max_colors_nr) {
-		GFXERROR("Attempt to free invalid color index 0x%d (%02x/%02x/%02x)!\n",
+		GFXERROR("Attempt to free invalid color index %d (%02x/%02x/%02x)!\n",
 			 color->global_index, color->r, color->g, color->b);
 		return GFX_ERROR;
 	}
 
 	if (!palette_color->lockers) {
-		GFXERROR("Attempt to free unused color index 0x%d (%02x/%02x/%02x)!\n",
+		GFXERROR("Attempt to free unused color index %d (%02x/%02x/%02x)!\n",
 			 color->global_index, color->r, color->g, color->b);
 		return GFX_ERROR;
 	}
@@ -392,7 +404,7 @@ gfx_free_color(gfx_palette_t *pal, gfx_pixmap_color_t *color)
 gfx_pixmap_t *
 gfx_pixmap_scale_index_data(gfx_pixmap_t *pixmap, gfx_mode_t *mode)
 {
-	byte *old, *new, *initial_new;
+	byte *old_data, *new_data, *initial_new_data;
 	byte *linestart;
 	int linewidth;
 	int xl, yl;
@@ -409,9 +421,9 @@ gfx_pixmap_scale_index_data(gfx_pixmap_t *pixmap, gfx_mode_t *mode)
 	if (pixmap->flags & GFX_PIXMAP_FLAG_SCALED_INDEX)
 		return pixmap; /* Already done */
 
-	old = pixmap->index_data;
+	old_data = pixmap->index_data;
 
-	if (!old) {
+	if (!old_data) {
 		GFXERROR("Attempt to scale index data without index data!\n");
 		return pixmap;
 	}
@@ -419,30 +431,30 @@ gfx_pixmap_scale_index_data(gfx_pixmap_t *pixmap, gfx_mode_t *mode)
 	xl = pixmap->index_xl;
 	yl = pixmap->index_yl;
 	linewidth = xfact * xl;
-	initial_new = new = sci_malloc(linewidth * yfact * yl);
+	initial_new_data = new_data = (byte *) sci_malloc(linewidth * yfact * yl);
 
 	for (yc = 0; yc < yl; yc++) {
 
-		linestart = new;
+		linestart = new_data;
 
 		if (xfact == 1) {
-			memcpy(new, old, linewidth);
-			new += linewidth;
-			old += linewidth;
+			memcpy(new_data, old_data, linewidth);
+			new_data += linewidth;
+			old_data += linewidth;
 		} else for (i = 0; i < xl; i++) {
-			byte fillc = *old++;
-			memset(new, fillc, xfact);
-			new += xfact;
+			byte fillc = *old_data++;
+			memset(new_data, fillc, xfact);
+			new_data += xfact;
 		}
 
 		for (i = 1; i < yfact; i++) {
-			memcpy(new, linestart, linewidth);
-			new += linewidth;
+			memcpy(new_data, linestart, linewidth);
+			new_data += linewidth;
 		}
 	}
 
 	free(pixmap->index_data);
-	pixmap->index_data = initial_new;
+	pixmap->index_data = initial_new_data;
 
 	pixmap->flags |= GFX_PIXMAP_FLAG_SCALED_INDEX;
 
