@@ -1,4 +1,3 @@
-
 /***************************************************************************
  gfx_operations Copyright (C) 2000 Christoph Reichenbach
 
@@ -29,6 +28,7 @@
 #include <sci_memory.h>
 #include <gfx_operations.h>
 
+#include <ctype.h>
 
 #define PRECISE_PRIORITY_MAP /* Duplicate all operations on the local priority map as appropriate */
 
@@ -89,6 +89,16 @@ _gfxop_scale_rect(rect_t *rect, gfx_mode_t *mode)
 	rect->yl *= yfact;
 }
 
+static void
+_gfxop_scale_point(point_t *point, gfx_mode_t *mode)
+{
+	int xfact = mode->xfact;
+	int yfact = mode->yfact;
+
+	point->x *= xfact;
+	point->y *= yfact;
+}
+
 static void
 _gfxop_alloc_colors(gfx_state_t *state, gfx_pixmap_color_t *colors, int colors_nr)
 {
@@ -117,6 +127,11 @@ _gfxop_free_colors(gfx_state_t *state, gfx_pixmap_color_t *colors, int colors_nr
 int _gfxop_clip(rect_t *rect, rect_t clipzone)
 /* Returns 1 if nothing is left */
 {
+#if 0
+	printf ("Clipping (%d, %d) size (%d, %d)  by (%d,%d)(%d,%d)\n", rect->x, rect->y, rect->xl, rect->yl,
+		clipzone.x, clipzone.y, clipzone.xl, clipzone.yl);
+#endif
+
 	if (rect->x < clipzone.x) {
 		rect->xl -= (clipzone.x - rect->x);
 		rect->x = clipzone.x;
@@ -133,6 +148,14 @@ int _gfxop_clip(rect_t *rect, rect_t clipzone)
 	if (rect->y + rect->yl > clipzone.y + clipzone.yl)
 		rect->yl = (clipzone.y + clipzone.yl) - rect->y;
 
+	if (rect->xl < 0)
+		rect->xl = 0;
+	if (rect->yl < 0)
+		rect->yl = 0;
+
+#if 0
+	printf (" => (%d, %d) size (%d, %d)\n", rect->x, rect->y, rect->xl, rect->yl);
+#endif
 	return (rect->xl <= 0 || rect->yl <= 0);
 }
 
@@ -147,11 +170,6 @@ _gfxop_grab_pixmap(gfx_state_t *state, gfx_pixmap_t **pxmp, int x, int y,
 	int unscaled_yl = (yl + yfact - 1) / yfact;
 	*zone = gfx_rect(x, y, xl, yl);
 
-	if (!(state->driver->capabilities & GFX_CAPABILITY_PIXMAP_GRABBING)) {
-		GFXERROR("Attempt to grab pixmap even though driver does not support pixmap grabbing!");
-		return GFX_FATAL;
-	}
-
 	if (_gfxop_clip(zone, gfx_rect(0, 0,
 				       320 * state->driver->mode->xfact,
 				       200 * state->driver->mode->yfact)))
@@ -174,6 +192,7 @@ _gfxop_grab_pixmap(gfx_state_t *state, gfx_pixmap_t **pxmp, int x, int y,
 					  priority? GFX_MASK_PRIORITY : GFX_MASK_VISUAL);
 }
 
+
 #define DRAW_LOOP(condition)										\
 {													\
 	rect_t drawrect = gfx_rect(pos.x, pos.y, pxm->index_xl, pxm->index_yl);				\
@@ -195,7 +214,7 @@ _gfxop_grab_pixmap(gfx_state_t *state, gfx_pixmap_t **pxmp, int x, int y,
 													\
 	for (y = 0; y < drawrect.yl; y++) {								\
 		for (x = 0; x < drawrect.xl; x++)							\
-			if (pxm->index_data[read_offset++] != GFX_COLOR_INDEX_TRANSPARENT) {		\
+			if (pxm->index_data[read_offset++] != pxm->color_key) {				\
 				if (condition)								\
 					map->index_data[offset++] = color;				\
 				else									\
@@ -219,7 +238,6 @@ DRAW_LOOP(map->index_data[offset] < color) /* Draw only lower priority */
 
 #undef DRAW_LOOP
 
-
 static int
 _gfxop_install_pixmap(gfx_driver_t *driver, gfx_pixmap_t *pxm)
 {
@@ -284,6 +302,7 @@ _gfxop_draw_pixmap(gfx_driver_t *driver, gfx_pixmap_t *pxm, int priority, int co
 #endif
 	}
 
+					   
 	if (_gfxop_clip(&clipped_dest, clip))
 		return GFX_OK;
 
@@ -363,7 +382,7 @@ _gfxop_get_pointer_bounds(gfx_state_t *state, rect_t *rect)
 }
 
 static int
-_gfxop_buffer_propagate_box(gfx_state_t *state, rect_t box, int buffer);
+_gfxop_buffer_propagate_box(gfx_state_t *state, rect_t box, gfx_buffer_t buffer);
 
 static int
 _gfxop_draw_pointer(gfx_state_t *state)
@@ -383,12 +402,15 @@ _gfxop_draw_pointer(gfx_state_t *state)
 		state->old_pointer_draw_pos.x = x;
 		state->old_pointer_draw_pos.y = y;
 
+		/* FIXME: we are leaking the mouse_pointer_bg, but freeing it causes weirdness in jones
+		 * we should reuse the buffer instead of malloc/free for better performance */
+
 		retval = _gfxop_grab_pixmap(state, &(state->mouse_pointer_bg), x, y,
 					    ppxm->xl, ppxm->yl, 0,
 					    &(state->pointer_bg_zone));
 
 		if (retval == GFX_ERROR) {
-			state->pointer_bg_zone = gfx_rect(320, 200, 0, 0);
+			state->pointer_bg_zone = gfx_rect(0, 0, 320, 200);
 			state->mouse_pointer_visible = POINTER_VISIBLE_BUT_CLIPPED;
 			return GFX_OK;
 		}
@@ -411,9 +433,9 @@ _gfxop_draw_pointer(gfx_state_t *state)
 }
 
 gfx_pixmap_t *
-_gfxr_get_cel(gfx_state_t *state, int nr, int *loop, int *cel)
+_gfxr_get_cel(gfx_state_t *state, int nr, int *loop, int *cel, int palette)
 {
-	gfxr_view_t *view = gfxr_get_view(state->resstate, nr, loop, cel);
+	gfxr_view_t *view = gfxr_get_view(state->resstate, nr, loop, cel, palette);
 	gfxr_loop_t *indexed_loop;
 
 	if (!view)
@@ -459,7 +481,7 @@ _rect_create(rect_t box)
 {
 	struct _dirty_rect *rect;
 
-	rect = sci_malloc(sizeof(struct _dirty_rect));
+	rect = (struct _dirty_rect*)sci_malloc(sizeof(struct _dirty_rect));
 	rect->next = NULL;
 	rect->rect = box;
 
@@ -585,12 +607,6 @@ _gfxop_init_common(gfx_state_t *state, gfx_options_t *options, void *misc_payloa
 {
 	state->options = options;
 
-	if ((state->static_palette =
-	     gfxr_interpreter_get_palette(state->version,
-					  &(state->static_palette_entries),
-					  misc_payload)))
-		_gfxop_alloc_colors(state, state->static_palette, state->static_palette_entries);
-
 	if (!((state->resstate = gfxr_new_resource_manager(state->version,
 							   state->options,
 							   state->driver,
@@ -599,22 +615,17 @@ _gfxop_init_common(gfx_state_t *state, gfx_options_t *options, void *misc_payloa
 		return GFX_FATAL;
 	}
 
+	if ((state->static_palette =
+	     gfxr_interpreter_get_static_palette(state->resstate,
+						 state->version,
+					         &(state->static_palette_entries),
+					         misc_payload)))
+		_gfxop_alloc_colors(state, state->static_palette, state->static_palette_entries);
+
 	state->visible_map = GFX_MASK_VISUAL;
+	state->fullscreen_override = NULL; /* No magical override */
 	gfxop_set_clip_zone(state, gfx_rect(0, 0, 320, 200));
 
-	if (!state->driver->capabilities & GFX_CAPABILITY_PIXMAP_GRABBING) {
-		if (!state->driver->capabilities & GFX_CAPABILITY_MOUSE_POINTER) {
-			GFXWARN("Graphics driver does not support drawing mouse pointers; disabling mouse input support.\n");
-			state->driver->capabilities &= ~GFX_CAPABILITY_MOUSE_SUPPORT;
-		} else if (gfxr_interpreter_needs_multicolored_pointers
-			   (state->version, misc_payload)
-			   && !state->driver->capabilities & GFX_CAPABILITY_COLOR_MOUSE_POINTER) {
-			GFXWARN("Graphics driver only supports monochrome mouse pointers, but colored pointers are needed; disabling mouse input support.\n");
-			state->driver->capabilities &= ~GFX_CAPABILITY_MOUSE_SUPPORT;
-		}
-
-	}
-
 	state->mouse_pointer = state->mouse_pointer_bg = NULL;
 	state->mouse_pointer_visible = 0;
 
@@ -656,6 +667,7 @@ gfxop_init(gfx_state_t *state, int xfact, int yfact, gfx_color_mode_t bpp,
 {
 	int color_depth = bpp? bpp : 1;
 	int initialized = 0;
+
 	BASIC_CHECKS(GFX_FATAL);
 
 	do {
@@ -769,37 +781,6 @@ gfxop_scan_bitmask(gfx_state_t *state, rect_t area, gfx_map_mask_t map)
 }
 
 
-int
-gfxop_set_visible_map(gfx_state_t *state, gfx_map_mask_t visible_map)
-{
-	switch (visible_map) {
-
-	case GFX_MASK_VISUAL:
-		state->fullscreen_override = NULL;
-		if (visible_map != state->visible_map) {
-			rect_t rect = gfx_rect(0, 0, 320, 200);
-			gfxop_clear_box(state, rect);
-			gfxop_update_box(state, rect);
-		}
-		break;
-
-	case GFX_MASK_PRIORITY:
-		state->fullscreen_override = state->priority_map;
-		break;
-
-	case GFX_MASK_CONTROL:
-		state->fullscreen_override = state->control_map;
-		break;
-
-	default:
-		fprintf(stderr, "Invalid display map %d selected!\n", visible_map);
-		return GFX_ERROR;
-	}
-
-	state->visible_map = visible_map;
-	return GFX_OK;
-}
-
 #define MIN_X 0
 #define MIN_Y 0
 #define MAX_X 319
@@ -1025,64 +1006,103 @@ line_clip(rect_t *line, rect_t clip, int xfact, int yfact)
 	return 0;
 }
 
+static int
+point_clip(point_t *start, point_t *end, rect_t clip, int xfact, int yfact)
+{
+	rect_t line = gfx_rect(start->x, start->y, end->x - start->x, end->y - start->y);
+	int retval = line_clip(&line, clip, xfact, yfact);
+
+	start->x = line.x;
+	start->y = line.y;
+
+	end->x = line.x + line.xl;
+	end->y = line.y + line.yl;
+
+	return retval;
+}
+
+
 
 static void
-draw_line_to_control_map(gfx_state_t *state, rect_t line, gfx_color_t color)
+draw_line_to_control_map(gfx_state_t *state, point_t start, point_t end, gfx_color_t color)
 {
 	if (color.mask & GFX_MASK_CONTROL)
-		if (!line_clip(&line, state->clip_zone_unscaled, 0, 0))
-			gfx_draw_line_pixmap_i(state->control_map, line, color.control);
+		if (!point_clip(&start, &end, state->clip_zone_unscaled, 0, 0))
+			gfx_draw_line_pixmap_i(state->control_map, start, end, color.control);
 }
 
 static int
-simulate_stippled_line_draw(gfx_driver_t *driver, int skipone, rect_t line, gfx_color_t color, gfx_line_mode_t line_mode)
+simulate_stippled_line_draw(gfx_driver_t *driver, int skipone, point_t start, point_t end, gfx_color_t color, gfx_line_mode_t line_mode)
      /* Draws a stippled line if this isn't supported by the driver (skipone is ignored ATM) */
 {
-	int stepwidth = (line.xl)? driver->mode->xfact : driver->mode->yfact;
+	int xl = end.x - start.x;
+	int yl = end.y - start.y;
+	int stepwidth = (xl)? driver->mode->xfact : driver->mode->yfact;
 	int dbl_stepwidth = 2*stepwidth;
 	int linelength = (line_mode == GFX_LINE_MODE_FINE)? stepwidth - 1 : 0;
-	int *posvar = (line.xl)? &line.x : &line.y;
-	int length = (line.xl)? line.xl : line.yl;
-	int length_left = length;
+	int *posvar;
+	int length;
+	int delta;
+	int length_left;
+
+	if (!xl) { /* xl = 0, so we move along yl */
+		posvar = &start.y;
+		length = yl;
+		delta = (yl < 0)? -dbl_stepwidth : dbl_stepwidth;
+	} else {
+		assert (!yl); /* We don't do diagonals; that's not needed ATM. */
+		posvar = &start.x;
+		length = xl;
+		delta = (xl < 0)? -dbl_stepwidth : dbl_stepwidth;
+	}
+
+	length_left = length;
 
 	if (skipone) {
 		length_left -= stepwidth;
 		*posvar += stepwidth;
 	}
 
-	length /= dbl_stepwidth;
+	length /= delta;
 
 	length_left -= length * dbl_stepwidth;
 
-	if (line.xl)
-		line.xl = linelength;
+	if (xl)
+		xl = linelength;
 	else
-		line.yl = linelength;
+		yl = linelength;
 
 	while (length--) {
 		int retval;
+		point_t nextpos = gfx_point(start.x + xl, start.y + yl);
 
-		if ((retval = driver->draw_line(driver, line, color, line_mode, GFX_LINE_STYLE_NORMAL))) {
-			GFXERROR("Failed to draw partial stippled line (%d,%d)+(%d,%d)\n", line.x, line.y, line.xl, line.yl);
+		if ((retval = driver->draw_line(driver, start, nextpos,
+						color, line_mode, GFX_LINE_STYLE_NORMAL))) {
+			GFXERROR("Failed to draw partial stippled line (%d,%d) -- (%d,%d)\n",
+				 GFX_PRINT_POINT(start), GFX_PRINT_POINT(nextpos));
 			return retval;
 		}
-		*posvar += dbl_stepwidth;
+		*posvar += delta;
 	}
 
 	if (length_left) {
 		int retval;
+		point_t nextpos;
 
 		if (length_left > stepwidth)
 			length_left = stepwidth;
 
-		if (line.xl)
-			line.xl = length_left;
+		if (xl)
+			xl = length_left;
 		else
-			if (line.yl)
-				line.yl = length_left;
+			if (yl)
+				yl = length_left;
 
-		if ((retval = driver->draw_line(driver, line, color, line_mode, GFX_LINE_STYLE_NORMAL))) {
-			GFXERROR("Failed to draw partial stippled line (%d,%d)+(%d,%d)\n", line.x, line.y, line.xl, line.yl);
+		nextpos = gfx_point(start.x + xl, start.y + yl);
+
+		if ((retval = driver->draw_line(driver, start, nextpos, color, line_mode, GFX_LINE_STYLE_NORMAL))) {
+			GFXERROR("Failed to draw partial stippled line (%d,%d) -- (%d,%d)\n",
+				 GFX_PRINT_POINT(start), GFX_PRINT_POINT(nextpos));
 			return retval;
 		}
 	}
@@ -1092,75 +1112,79 @@ simulate_stippled_line_draw(gfx_driver_t *driver, int skipone, rect_t line, gfx_
 
 
 static int
-_gfxop_draw_line_clipped(gfx_state_t *state, rect_t line, gfx_color_t color, gfx_line_mode_t line_mode,
+_gfxop_draw_line_clipped(gfx_state_t *state, point_t start, point_t end, gfx_color_t color, gfx_line_mode_t line_mode,
 			 gfx_line_style_t line_style)
 {
 	int retval;
-	int skipone = (line.x ^ line.y) & 1; /* Used for simulated line stippling */
+	int skipone = (start.x ^ end.y) & 1; /* Used for simulated line stippling */
 
 	BASIC_CHECKS(GFX_FATAL);
 	REMOVE_POINTER;
 
 	/* First, make sure that the line is normalized */
-	if (line.yl < 0) {
-		line.y += line.yl;
-		line.x += line.xl;
-		line.yl = -line.yl;
-		line.xl = -line.xl;
+	if (start.y > end.y) {
+		point_t swap = start;
+		start = end;
+		end = swap;
 	}
 
-	if (line.x < state->clip_zone.x
-	    || line.x < state->clip_zone.x
-	    || line.y < state->clip_zone.y
-	    || (line.x + line.xl) >= (state->clip_zone.x + state->clip_zone.xl)
-	    || (line.y + line.yl) >= (state->clip_zone.y + state->clip_zone.yl))
-		if (line_clip(&line, state->clip_zone, state->driver->mode->xfact - 1,
+	if (start.x < state->clip_zone.x
+	    || start.y < state->clip_zone.y
+	    || end.x >= (state->clip_zone.x + state->clip_zone.xl)
+	    || end.y >= (state->clip_zone.y + state->clip_zone.yl))
+		if (point_clip(&start, &end, state->clip_zone, state->driver->mode->xfact - 1,
 			      state->driver->mode->yfact - 1))
 			return GFX_OK; /* Clipped off */
 
 	if (line_style == GFX_LINE_STYLE_STIPPLED) {
-		if (line.xl && line.yl) {
-			GFXWARN("Attempt to draw stippled line which is neither an hbar nor a vbar: (%d,%d)+(%d,%d)\n",
-				line.x, line.y, line.xl, line.yl);
+		if (start.x != end.x && start.y != end.y) {
+			GFXWARN("Attempt to draw stippled line which is neither an hbar nor a vbar: (%d,%d) -- (%d,%d)\n",
+				 GFX_PRINT_POINT(start), GFX_PRINT_POINT(end));
 			return GFX_ERROR;
 		}
 		if (!(state->driver->capabilities & GFX_CAPABILITY_STIPPLED_LINES))
-			return simulate_stippled_line_draw(state->driver, skipone, line, color, line_mode);
+			return simulate_stippled_line_draw(state->driver, skipone, start, end, color, line_mode);
 	}
 
 	if (line_mode == GFX_LINE_MODE_FINE
 	    && !(state->driver->capabilities & GFX_CAPABILITY_FINE_LINES))
 		line_mode = GFX_LINE_MODE_FAST;
 
-	if ((retval = state->driver->draw_line(state->driver, line, color, line_mode, line_style))) {
-		GFXERROR("Failed to draw line (%d,%d)+(%d,%d)\n", line.x, line.y, line.xl, line.yl);
+	if ((retval = state->driver->draw_line(state->driver, start, end, color, line_mode, line_style))) {
+		GFXERROR("Failed to draw line (%d,%d) -- (%d,%d)\n",
+			 GFX_PRINT_POINT(start), GFX_PRINT_POINT(end));
 		return retval;
 	}
 	return GFX_OK;
 }
 
 int
-gfxop_draw_line(gfx_state_t *state, rect_t line, gfx_color_t color, gfx_line_mode_t line_mode,
+gfxop_draw_line(gfx_state_t *state, point_t start, point_t end,
+		gfx_color_t color, gfx_line_mode_t line_mode,
 		gfx_line_style_t line_style)
 {
 	int xfact, yfact;
 
 	BASIC_CHECKS(GFX_FATAL);
-	_gfxop_add_dirty_x(state, line);
+	_gfxop_add_dirty_x(state, gfx_rect(start.x, start.y, end.x - start.x, end.y - start.y));
 
 	xfact = state->driver->mode->xfact;
 	yfact = state->driver->mode->yfact;
 
-	draw_line_to_control_map(state, line, color);
+	draw_line_to_control_map(state, start, end, color);
 
-	_gfxop_scale_rect(&line, state->driver->mode);
+	_gfxop_scale_point(&start, state->driver->mode);
+	_gfxop_scale_point(&end, state->driver->mode);
 
 	if (line_mode == GFX_LINE_MODE_FINE) {
-		line.x += xfact >> 1;
-		line.y += yfact >> 1;
+		start.x += xfact >> 1;
+		start.y += yfact >> 1;
+
+		end.x += xfact >> 1;
+		end.y += yfact >> 1;
 	}
 
-	return _gfxop_draw_line_clipped(state, line, color, line_mode, line_style);
+	return _gfxop_draw_line_clipped(state, start, end, color, line_mode, line_style);
 }
 
 int
@@ -1168,11 +1192,11 @@ gfxop_draw_rectangle(gfx_state_t *state, rect_t rect, gfx_color_t color, gfx_lin
 		     gfx_line_style_t line_style)
 {
 	int retval = 0;
-	rect_t line, unscaled_line = gfx_rect(rect.x, rect.y, rect.xl, rect.yl);
 	int xfact, yfact;
 	int xunit, yunit;
-	int ystart;
-	int xl, yl;
+	int x, y, xl, yl;
+	point_t upper_left_u, upper_right_u, lower_left_u, lower_right_u;
+	point_t upper_left, upper_right, lower_left, lower_right;
 
 	BASIC_CHECKS(GFX_FATAL);
 	REMOVE_POINTER;
@@ -1180,62 +1204,44 @@ gfxop_draw_rectangle(gfx_state_t *state, rect_t rect, gfx_color_t color, gfx_lin
 	xfact = state->driver->mode->xfact;
 	yfact = state->driver->mode->yfact;
 
-	if (rect.xl == 0 || rect.yl == 0) {
-		GFXWARN("Rectangle too small: (%d,%d)+(%d,%d)\n", rect.x, rect.y, rect.xl, rect.yl);
-		return GFX_ERROR;
-	}
-
 	if (line_mode == GFX_LINE_MODE_FINE
 	    && state->driver->capabilities & GFX_CAPABILITY_FINE_LINES) {
 		xunit = yunit = 1;
 		xl = 1 + (rect.xl - 1) * xfact;
 		yl = 1 + (rect.yl - 1) * yfact;
-		line.x = rect.x * xfact + (xfact - 1);
-		line.y = rect.y * yfact + (yfact - 1);
+		x = rect.x * xfact + (xfact - 1);
+		y = rect.y * yfact + (yfact - 1);
 	} else {
 		xunit = xfact;
 		yunit = yfact;
 		xl = rect.xl * xfact;
 		yl = rect.yl * yfact;
-		line.x = rect.x * xfact;
-		line.y = rect.y * yfact;
+		x = rect.x * xfact;
+		y = rect.y * yfact;
 	}
 
-	ystart = line.y;
-	line.xl = xl;
-	line.yl = 0;
-	retval |= _gfxop_draw_line_clipped(state, line, color, line_mode, line_style);
-	unscaled_line.yl = 0;
-	draw_line_to_control_map(state, unscaled_line, color);
-	_gfxop_add_dirty_x(state, unscaled_line);
+	upper_left_u = gfx_point(rect.x, rect.y);
+	upper_right_u = gfx_point(rect.x + rect.xl, rect.y);
+	lower_left_u = gfx_point(rect.x, rect.y + rect.yl);
+	lower_right_u = gfx_point(rect.x + rect.xl, rect.y + rect.yl);
 
+	upper_left = gfx_point(x, y);
+	upper_right = gfx_point(x + xl, y);
+	lower_left = gfx_point(x, y + yl);
+	lower_right = gfx_point(x + xl, y + yl);
 
-	line.y += yl;
-	line.x += xunit;
-	line.xl -= xunit;
-	retval |= _gfxop_draw_line_clipped(state, line, color, line_mode, line_style);
-	unscaled_line.y += rect.yl;
-	draw_line_to_control_map(state, unscaled_line, color);
-	_gfxop_add_dirty_x(state, unscaled_line);
+#define PARTIAL_LINE(pt1, pt2)	 								\
+	retval |= _gfxop_draw_line_clipped(state, pt1, pt2, color, line_mode, line_style);	\
+	draw_line_to_control_map(state, pt1##_u, pt2##_u, color);				\
+	_gfxop_add_dirty_x(state,								\
+			   gfx_rect(pt1##_u.x, pt1##_u.y, pt2##_u.x - pt1##_u.x, pt2##_u.y - pt1##_u.y))
 
-	line.xl = 0;
-	line.yl = yl - yunit;
-	line.y = ystart + yunit;
-	line.x -= xunit;
-	retval |= _gfxop_draw_line_clipped(state, line, color, line_mode, line_style);
-	unscaled_line.y = rect.y;
-	unscaled_line.yl = rect.yl;
-	unscaled_line.xl = rect.xl;
-	draw_line_to_control_map(state, unscaled_line, color);
-	_gfxop_add_dirty_x(state, unscaled_line);
-
-	line.x += xl;
-	line.y = ystart;
-	retval |= _gfxop_draw_line_clipped(state, line, color, line_mode, line_style);
-	unscaled_line.x += rect.xl;
-	draw_line_to_control_map(state, unscaled_line, color);
-	_gfxop_add_dirty_x(state, unscaled_line);
+	PARTIAL_LINE (upper_left, upper_right);
+	PARTIAL_LINE (upper_right, lower_right);
+	PARTIAL_LINE (lower_right, lower_left);
+	PARTIAL_LINE (lower_left, upper_left);
 
+#undef PARTIAL_LINE
 	if (retval) {
 		GFXERROR("Failed to draw rectangle (%d,%d)+(%d,%d)\n", rect.x, rect.y, rect.xl, rect.yl);
 		return retval;
@@ -1254,9 +1260,9 @@ gfxop_draw_box(gfx_state_t *state, rect_t box, gfx_color_t color1, gfx_color_t c
 	gfx_driver_t *drv = state->driver;
 	int reverse = 0; /* switch color1 and color2 */
 	float mod_offset = 0.0, mod_breadth = 1.0; /* 0.0 to 1.0: Color adjustment */
-	int driver_shade_type;
+	gfx_rectangle_fill_t driver_shade_type;
 	rect_t new_box;
-	gfx_color_t draw_color1, draw_color2	= {0};
+	gfx_color_t draw_color1, draw_color2	= {{0, 0, 0}, 0, 0, 0, 0};
 
 	BASIC_CHECKS(GFX_FATAL);
 	REMOVE_POINTER;
@@ -1286,7 +1292,6 @@ gfxop_draw_box(gfx_state_t *state, rect_t box, gfx_color_t color1, gfx_color_t c
 	}
 
 	memcpy(&new_box, &box, sizeof(rect_t));
-
 	if (_gfxop_clip(&new_box, state->clip_zone))
 		return GFX_OK;
 
@@ -1362,7 +1367,7 @@ gfxop_fill_box(gfx_state_t *state, rect_t box, gfx_color_t color)
 
 
 static int
-_gfxop_buffer_propagate_box(gfx_state_t *state, rect_t box, int buffer)
+_gfxop_buffer_propagate_box(gfx_state_t *state, rect_t box, gfx_buffer_t buffer)
 {
 	int error;
 
@@ -1402,6 +1407,36 @@ gfxop_clear_box(gfx_state_t *state, rect_t box)
 	return _gfxop_buffer_propagate_box(state, box, GFX_BUFFER_BACK);
 }
 
+int
+gfxop_set_visible_map(gfx_state_t *state, gfx_map_mask_t visible_map)
+{
+	switch (visible_map) {
+
+	case GFX_MASK_VISUAL:
+		state->fullscreen_override = NULL;
+		if (visible_map != state->visible_map) {
+			rect_t rect = gfx_rect(0, 0, 320, 200);
+			gfxop_clear_box(state, rect);
+			gfxop_update_box(state, rect);
+		}
+		break;
+
+	case GFX_MASK_PRIORITY:
+		state->fullscreen_override = state->priority_map;
+		break;
+
+	case GFX_MASK_CONTROL:
+		state->fullscreen_override = state->control_map;
+		break;
+
+	default:
+		fprintf(stderr, "Invalid display map %d selected!\n", visible_map);
+		return GFX_ERROR;
+	}
+
+	state->visible_map = visible_map;
+	return GFX_OK;
+}
 
 int
 gfxop_update(gfx_state_t *state)
@@ -1587,6 +1622,10 @@ _gfxop_set_pointer(gfx_state_t *state, gfx_pixmap_t *pxm)
 
 	} else draw_new = 1;
 
+	if (!state->mouse_pointer_in_hw)
+		draw_old = state->mouse_pointer != NULL;
+
+
 	if (draw_old) {
 		_gfxop_get_pointer_bounds(state, &old_pointer_bounds);
 		REMOVE_POINTER;
@@ -1637,7 +1676,7 @@ gfxop_set_pointer_cursor(gfx_state_t *state, int nr)
 
 
 int
-gfxop_set_pointer_view(gfx_state_t *state, int nr, int loop, int cel)
+gfxop_set_pointer_view(gfx_state_t *state, int nr, int loop, int cel, point_t  *hotspot)
 {
 	int real_loop = loop;
 	int real_cel = cel;
@@ -1645,17 +1684,26 @@ gfxop_set_pointer_view(gfx_state_t *state, int nr, int loop, int cel)
 
 	BASIC_CHECKS(GFX_FATAL);
 
-	new_pointer = _gfxr_get_cel(state, nr, &real_loop, &real_cel);
+	new_pointer = _gfxr_get_cel(state, nr, &real_loop, &real_cel,
+				    0); /* FIXME: For now, don't palettize pointers */
 
-
-	if (!state->mouse_pointer || (real_loop == -1)) {
-		GFXWARN("Attempt to set invalid pointer #%d\n", nr);
-	} else if (real_loop != loop || real_cel != cel) {
-		GFXDEBUG("Changed loop/cel from %d/%d to %d/%d in view %d\n",
-			 loop, cel, real_loop, real_cel, nr);
+	if (hotspot)
+	{
+		new_pointer->xoffset = hotspot->x;
+		new_pointer->yoffset = hotspot->y;
 	}
 
-	return _gfxop_set_pointer(state, new_pointer);
+	if (!new_pointer) {
+		GFXWARN("Attempt to set invalid pointer #%d\n", nr);
+		return GFX_ERROR;
+	} else 
+	{
+		if (real_loop != loop || real_cel != cel) {
+			GFXDEBUG("Changed loop/cel from %d/%d to %d/%d in view %d\n",
+				 loop, cel, real_loop, real_cel, nr);
+		}
+		return _gfxop_set_pointer(state, new_pointer);
+	}
 }
 
 int
@@ -1664,6 +1712,13 @@ gfxop_set_pointer_position(gfx_state_t *state, point_t pos)
 	BASIC_CHECKS(GFX_ERROR);
 
 	state->pointer_pos = pos;
+
+	if (pos.x > 320 || pos.y > 200)
+	{
+		GFXWARN("Attempt to place pointer at invalid coordinates (%d, %d)\n", pos.x, pos.y);
+		return 0; /* Not fatal */
+	}
+
 	state->driver->pointer_x = pos.x * state->driver->mode->xfact;
 	state->driver->pointer_y = pos.y * state->driver->mode->yfact;
 
@@ -1673,9 +1728,9 @@ gfxop_set_pointer_position(gfx_state_t *state, point_t pos)
 
 #define SCANCODE_ROWS_NR 3
 
-struct {
+struct scancode_row {
 	int offset;
-	char *keys;
+	const char *keys;
 } scancode_rows[SCANCODE_ROWS_NR] = {
 	{0x10, "QWERTYUIOP[]"},
 	{0x1e, "ASDFGHJKL;'\\"},
@@ -1690,7 +1745,7 @@ _gfxop_scancode(int ch)
 	int c = toupper((char)ch);
 
 	for (row = 0; row < SCANCODE_ROWS_NR; row++) {
-		char *keys = scancode_rows[row].keys;
+		const char *keys = scancode_rows[row].keys;
 		int offset = scancode_rows[row].offset;
 
 		while (*keys) {
@@ -1705,7 +1760,7 @@ _gfxop_scancode(int ch)
 	return ch; /* not found */
 }
 
-static int
+/* static */ int
 _gfxop_shiftify(int c)
 {
 	char shifted_numbers[] = ")!@#$%^&*(";
@@ -1750,6 +1805,8 @@ _gfxop_shiftify(int c)
 	case SCI_K_F9 : return SCI_K_SHIFT_F9;
 	case SCI_K_F10 : return SCI_K_SHIFT_F10;
 	}
+
+	return c;
 }
 
 static int
@@ -1795,20 +1852,23 @@ gfxop_get_event(gfx_state_t *state, unsigned int mask)
 	} else {
 		event.type = 0;
 
-		do {
-			if (event.type) {
-				*seekerp = sci_malloc(sizeof(gfx_input_event_t));
-				(*seekerp)->next = NULL;
-
-				event.data = (char) (event.data);
-				/* Clip illegal bits */
-
-				(*seekerp)->event = event;
-				seekerp = &((*seekerp)->next);
-			}
-			event = state->driver->get_event(state->driver);
-
-		} while (event.type && !(event.type & mask));
+		if (!(mask & SCI_EVT_NONBLOCK))
+		{
+			do {
+				if (event.type) {
+					*seekerp = (gfx_input_event_t*)sci_malloc(sizeof(gfx_input_event_t));
+					(*seekerp)->next = NULL;
+					
+					event.data = (char) (event.data);
+					/* Clip illegal bits */
+					
+					(*seekerp)->event = event;
+					seekerp = &((*seekerp)->next);
+				}
+				event = state->driver->get_event(state->driver);
+				
+			} while (event.type && !(event.type & mask));
+		}
 	}
 
 	if (_gfxop_full_pointer_refresh(state)) {
@@ -1855,7 +1915,7 @@ gfxop_lookup_view_get_loops(gfx_state_t *state, int nr)
 
 	BASIC_CHECKS(GFX_ERROR);
 
-	view = gfxr_get_view(state->resstate, nr, &loop, &cel);
+	view = gfxr_get_view(state->resstate, nr, &loop, &cel, 0);
 
 	if (!view) {
 		GFXWARN("Attempt to retreive number of loops from invalid view %d\n", nr);
@@ -1874,19 +1934,16 @@ gfxop_lookup_view_get_cels(gfx_state_t *state, int nr, int loop)
 
 	BASIC_CHECKS(GFX_ERROR);
 
-	view = gfxr_get_view(state->resstate, nr, &real_loop, &cel);
+	view = gfxr_get_view(state->resstate, nr, &real_loop, &cel, 0);
 
 	if (!view) {
-		GFXWARN("Attempt to retreive number of cels from invalid view %d\n", nr);
+		GFXWARN("Attempt to retreive number of cels from invalid/broken view %d\n", nr);
 		return 0;
 	} else if (real_loop != loop) {
 		GFXWARN("Loop number was corrected from %d to %d in view %d\n", loop, real_loop, nr);
 	}
 
-	if (loop == -1)
-		return 0;
-	else
-		return view->loops[real_loop].cels_nr;
+	return view->loops[real_loop].cels_nr;
 }
 
 
@@ -1895,15 +1952,12 @@ gfxop_check_cel(gfx_state_t *state, int nr, int *loop, int *cel)
 {
         BASIC_CHECKS(GFX_ERROR);
 
-	if (!gfxr_get_view(state->resstate, nr, loop, cel)) {
+	if (!gfxr_get_view(state->resstate, nr, loop, cel, 0)) {
 		GFXWARN("Attempt to verify loop/cel values for invalid view %d\n", nr);
 		return GFX_ERROR;
 	}
 
-	if (*loop == -1)
-		return GFX_ERROR;
-	else
-		return GFX_OK;
+	return GFX_OK;
 }
 
 int
@@ -1913,7 +1967,7 @@ gfxop_overflow_cel(gfx_state_t *state, int nr, int *loop, int *cel)
 	int cel_v = *cel;
         BASIC_CHECKS(GFX_ERROR);
 
-	if (!gfxr_get_view(state->resstate, nr, &loop_v, &cel_v)) {
+	if (!gfxr_get_view(state->resstate, nr, &loop_v, &cel_v, 0)) {
 		GFXWARN("Attempt to verify loop/cel values for invalid view %d\n", nr);
 		return GFX_ERROR;
 	}
@@ -1937,18 +1991,11 @@ gfxop_get_cel_parameters(gfx_state_t *state, int nr, int loop, int cel,
 	gfx_pixmap_t *pxm = NULL;
 	BASIC_CHECKS(GFX_ERROR);
 
-	if (!(view = gfxr_get_view(state->resstate, nr, &loop, &cel))) {
+	if (!(view = gfxr_get_view(state->resstate, nr, &loop, &cel, 0))) {
 		GFXWARN("Attempt to get cel parameters for invalid view %d\n", nr);
 		return GFX_ERROR;
 	}
 
-	if (loop == -1) {
-		*width = 0;
-		*height = 0;
-		offset-> x = offset->y = 0;
-		return GFX_OK;
-	}
-
 	pxm = view->loops[loop].cels[cel];
 	*width = pxm->index_xl;
 	*height = pxm->index_yl;
@@ -1961,7 +2008,8 @@ gfxop_get_cel_parameters(gfx_state_t *state, int nr, int loop, int cel,
 
 static int
 _gfxop_draw_cel_buffer(gfx_state_t *state, int nr, int loop, int cel,
-		       point_t pos, gfx_color_t color, int static_buf)
+		       point_t pos, gfx_color_t color, int static_buf,
+		       int palette)
 {
 	int priority = (color.mask & GFX_MASK_PRIORITY)? color.priority : -1;
 	int control = (color.mask & GFX_MASK_CONTROL)? color.control : -1;
@@ -1970,14 +2018,10 @@ _gfxop_draw_cel_buffer(gfx_state_t *state, int nr, int loop, int cel,
 	int old_x, old_y;
 	BASIC_CHECKS(GFX_FATAL);
 
-	if (!(view = gfxr_get_view(state->resstate, nr, &loop, &cel))) {
+	if (!(view = gfxr_get_view(state->resstate, nr, &loop, &cel, palette))) {
 		GFXWARN("Attempt to draw loop/cel %d/%d in invalid view %d\n", loop, cel, nr);
 		return GFX_ERROR;
 	}
-
-	if (loop == -1)
-		return GFX_OK;
-
 	pxm = view->loops[loop].cels[cel];
 
 	old_x = pos.x -= pxm->xoffset;
@@ -2002,22 +2046,23 @@ _gfxop_draw_cel_buffer(gfx_state_t *state, int nr, int loop, int cel,
 
 int
 gfxop_draw_cel(gfx_state_t *state, int nr, int loop, int cel, point_t pos,
-	       gfx_color_t color)
+	       gfx_color_t color, int palette)
 {
-	return _gfxop_draw_cel_buffer(state, nr, loop, cel, pos, color, 0);
+	return _gfxop_draw_cel_buffer(state, nr, loop, cel, pos, color, 0, palette);
 }
 
 
 int
 gfxop_draw_cel_static(gfx_state_t *state, int nr, int loop, int cel, point_t pos,
-		      gfx_color_t color)
+		      gfx_color_t color, int palette)
 {
 	int retval;
 	rect_t oldclip = state->clip_zone;
 
 	state->clip_zone = gfx_rect_fullscreen;
 	_gfxop_scale_rect(&(state->clip_zone), state->driver->mode);
-	retval = gfxop_draw_cel_static_clipped(state, nr, loop, cel, pos, color);
+	retval = gfxop_draw_cel_static_clipped(state, nr, loop, cel, pos, color,
+					       palette);
 	/* Except that the area it's clipped against is... unusual ;-) */
 	state->clip_zone = oldclip;
 
@@ -2027,9 +2072,9 @@ gfxop_draw_cel_static(gfx_state_t *state, int nr, int loop, int cel, point_t pos
 
 int
 gfxop_draw_cel_static_clipped(gfx_state_t *state, int nr, int loop, int cel,
-			      point_t pos, gfx_color_t color)
+			      point_t pos, gfx_color_t color, int palette)
 {
-	return _gfxop_draw_cel_buffer(state, nr, loop, cel, pos, color, 1);
+	return _gfxop_draw_cel_buffer(state, nr, loop, cel, pos, color, 1, palette);
 }
 
 
@@ -2040,12 +2085,14 @@ gfxop_draw_cel_static_clipped(gfx_state_t *state, int nr, int loop, int cel,
 static int
 _gfxop_set_pic(gfx_state_t *state)
 {
-	byte unscaled = (state->options->pic0_unscaled);
 	gfx_copy_pixmap_box_i(state->control_map, state->pic->control_map, gfx_rect(0, 0, 320, 200));
-	gfx_copy_pixmap_box_i(state->priority_map, state->pic_unscaled->priority_map, gfx_rect(0, 0, 320, 200));
+	gfx_copy_pixmap_box_i(state->priority_map, state->pic_unscaled->priority_map, gfx_rect(0, 0, 320, 200)); 
 	gfx_copy_pixmap_box_i(state->static_priority_map, state->pic_unscaled->priority_map, gfx_rect(0, 0, 320, 200));
 
 	_gfxop_install_pixmap(state->driver, state->pic->visual_map);
+
+	if (state->options->pic0_unscaled)
+		state->pic->priority_map = gfx_pixmap_scale_index_data(state->pic->priority_map, state->driver->mode);
 	return state->driver->set_static_buffer(state->driver, state->pic->visual_map, state->pic->priority_map);
 }
 
@@ -2136,12 +2183,12 @@ gfxop_get_font_height(gfx_state_t *state, int font_nr)
 }
 
 int
-gfxop_get_text_params(gfx_state_t *state, int font_nr, char *text,
-		      int maxwidth, int *width, int *height, int text_flags)
+gfxop_get_text_params(gfx_state_t *state, int font_nr, const char *text,
+		      int maxwidth, int *width, int *height, int text_flags,
+		      int *lines_nr, int *lineheight, int *lastline_width)
 {
 	text_fragment_t *textsplits;
 	gfx_bitmap_font_t *font;
-	int lines;
 
 	BASIC_CHECKS(GFX_FATAL);
 
@@ -2153,7 +2200,9 @@ gfxop_get_text_params(gfx_state_t *state, int font_nr, char *text,
 		return GFX_ERROR;
 	}
 
-	textsplits = gfxr_font_calculate_size(font, maxwidth, text, width, height, &lines,
+	textsplits = gfxr_font_calculate_size(font, maxwidth, text, width,
+					      height, lines_nr,
+					      lineheight, lastline_width,
 					      (state->options->workarounds & GFX_WORKAROUND_WHITESPACE_COUNT)
 					      | text_flags);
 
@@ -2170,8 +2219,19 @@ gfxop_get_text_params(gfx_state_t *state, int font_nr, char *text,
 
 
 #define COL_XLATE(des,src) \
-  des = src.visual; /* The new gfx_color_t structure makes things a lot easier :-) */
-
+  des = src.visual; /* The new gfx_color_t structure makes things a lot easier :-) */ /* \
+  if (gfxop_set_color(state, &src, \
+		      src.visual.r, \
+		      src.visual.g, \
+		      src.visual.b, \
+		      src.alpha, \
+		      src.priority, \
+		      src.control)) \
+  { \
+	  GFXERROR("Unable to set up colors"); \
+	  return NULL; \
+  }
+*/
 
 gfx_text_handle_t *
 gfxop_new_text(gfx_state_t *state, int font_nr, char *text, int maxwidth,
@@ -2196,9 +2256,9 @@ gfxop_new_text(gfx_state_t *state, int font_nr, char *text, int maxwidth,
 		return NULL;
 	}
 
-	handle = sci_malloc(sizeof(gfx_text_handle_t));
+	handle = (gfx_text_handle_t*)sci_malloc(sizeof(gfx_text_handle_t));
 
-	handle->text = sci_malloc(strlen(text) + 1);
+	handle->text = (char*)sci_malloc(strlen(text) + 1);
 	strcpy(handle->text, text);
 	handle->halign = halign;
 	handle->valign = valign;
@@ -2207,6 +2267,7 @@ gfxop_new_text(gfx_state_t *state, int font_nr, char *text, int maxwidth,
 	handle->lines =
 		gfxr_font_calculate_size(font, maxwidth, handle->text, &(handle->width), &(handle->height),
 					 &(handle->lines_nr),
+					 NULL, NULL,
 					 ((state->options->workarounds & GFX_WORKAROUND_WHITESPACE_COUNT)?
 					  GFXR_FONT_FLAG_COUNT_WHITESPACE : 0)
 					 | flags);
@@ -2224,7 +2285,7 @@ gfxop_new_text(gfx_state_t *state, int font_nr, char *text, int maxwidth,
 		handle->lines->length = strlen(text);
 	}
 
-	handle->text_pixmaps = sci_malloc(sizeof(gfx_pixmap_t *) * handle->lines_nr);
+	handle->text_pixmaps = (gfx_pixmap_t**)sci_malloc(sizeof(gfx_pixmap_t *) * handle->lines_nr);
 
 	for (i = 0; i < handle->lines_nr; i++) {
 		int chars_nr = handle->lines[i].length;
