@@ -33,6 +33,8 @@
 #include <gfx_resmgr.h>
 #include <gfx_state_internal.h>
 
+#undef TIME_PICDRAWING
+
 /* Invalid hash mode: Used to invalidate modified pics */
 #define MODE_INVALID -1
 
@@ -46,8 +48,8 @@ gfx_resstate_t *
 gfxr_new_resource_manager(int version, gfx_options_t *options,
 			  gfx_driver_t *driver, void *misc_payload)
 {
-	gfx_resstate_t *state = sci_malloc(sizeof(gfx_resstate_t));
-	int i;
+	gfx_resstate_t *state = (gfx_resstate_t*)sci_malloc(sizeof(gfx_resstate_t));
+	int ii;
 
 	state->version = version;
 	state->options = options;
@@ -55,7 +57,8 @@ gfxr_new_resource_manager(int version, gfx_options_t *options,
 	state->misc_payload = misc_payload;
 
 	state->tag_lock_counter = state->lock_counter = 0;
-	for (i = 0; i < GFX_RESOURCE_TYPES_NR; i++) {
+	for (ii = 0; ii < GFX_RESOURCE_TYPES_NR; ii++) {
+		gfx_resource_type_t i = (gfx_resource_type_t) ii;
 		sbtree_t *tree;
 		int entries_nr;
 		int *resources = gfxr_interpreter_get_resources(state, i, version,
@@ -255,7 +258,7 @@ gfxr_free_tagged_resources(gfx_driver_t *driver, gfx_resstate_t *state)
 
 static gfxr_pic_t *
 gfxr_pic_xlate_common(gfx_resource_t *res, int maps, int scaled,
-		      int force, gfx_mode_t *mode, int filter, int endianize,
+		      int force, gfx_mode_t *mode, gfx_xlate_filter_t filter, int endianize,
 		      gfx_options_t *options)
 {
 	XLATE_AS_APPROPRIATE(GFX_MASK_VISUAL, visual_map);
@@ -275,11 +278,11 @@ gfxr_pic_t *
 gfxr_get_pic(gfx_resstate_t *state, int nr, int maps, int flags, int default_palette, int scaled)
 {
 	gfxr_pic_t *npic = NULL;
-	int restype = GFX_RESOURCE_TYPE_PIC;
+	gfx_resource_type_t restype = GFX_RESOURCE_TYPE_PIC;
 	sbtree_t *tree = state->resource_trees[restype];
 	gfx_resource_t *res = NULL;
 	int hash = gfxr_interpreter_options_hash(restype, state->version,
-						 state->options, state->misc_payload);
+						 state->options, state->misc_payload, 0);
         int must_post_process_pic = 0;
 	int need_unscaled =
 		(state->driver->mode->xfact != 1 || state->driver->mode->yfact != 1);
@@ -325,6 +328,11 @@ gfxr_get_pic(gfx_resstate_t *state, int nr, int maps, int flags, int default_pal
 			gfxr_interpreter_clear_pic(state->version, unscaled_pic,
 						   state->misc_payload);
 		}
+#ifdef TIME_PICDRAWING
+		{long start_sec, start_usec;
+		long end_sec, end_usec;
+		sci_gettime(&start_sec, &start_usec);
+#endif
 		if (gfxr_interpreter_calculate_pic(state, pic, unscaled_pic, flags,
 						   default_palette, nr,
 						   state->misc_payload)) {
@@ -334,12 +342,15 @@ gfxr_get_pic(gfx_resstate_t *state, int nr, int maps, int flags, int default_pal
 
 			return NULL;
 		}
-
-		if (state->options->pic0_unscaled)
-			pic->priority_map = gfx_pixmap_scale_index_data(pic->priority_map, state->driver->mode);
+#ifdef TIME_PICDRAWING
+		sci_gettime(&end_sec, &end_usec);
+		printf("\nTIME:	%d	for drawing pic.%03d\n",
+			(end_sec - start_sec) * 1000000 + (end_usec - start_usec), nr);
+		}
+#endif
 
 		if (!res) {
-			res = sci_malloc(sizeof(gfx_resource_t));
+			res = (gfx_resource_t*)sci_malloc(sizeof(gfx_resource_t));
 			res->ID = GFXR_RES_ID(restype, nr);
 			res->lock_sequence_nr = state->options->buffer_pics_nr;
 			sbtree_set(tree, nr, (void *) res);
@@ -379,6 +390,33 @@ gfxr_get_pic(gfx_resstate_t *state, int nr, int maps, int flags, int default_pal
 }
 
 
+static void
+set_pic_id(gfx_resource_t *res, int id)
+{
+	if (res->scaled_data.pic) {
+		gfxr_pic_t *pic = res->scaled_data.pic;
+		pic->control_map->ID = id;
+		pic->priority_map->ID = id;
+		pic->visual_map->ID = id;
+	}
+
+	if (res->unscaled_data.pic) {
+		gfxr_pic_t *pic = res->unscaled_data.pic;
+		pic->control_map->ID = id;
+		pic->priority_map->ID = id;
+		pic->visual_map->ID = id;
+	}
+}
+
+static int
+get_pic_id(gfx_resource_t *res)
+{
+	if (res->scaled_data.pic)
+		return res->scaled_data.pic->visual_map->ID;
+	else
+		return res->unscaled_data.pic->visual_map->ID;
+}
+
 static void
 _gfxr_unscale_pixmap_index_data(gfx_pixmap_t *pxm, gfx_mode_t *mode)
 {
@@ -411,44 +449,17 @@ _gfxr_unscale_pixmap_index_data(gfx_pixmap_t *pxm, gfx_mode_t *mode)
 }
 
 
-static void
-set_pic_id(gfx_resource_t *res, int id)
-{
-	if (res->scaled_data.pic) {
-		gfxr_pic_t *pic = res->scaled_data.pic;
-		pic->control_map->ID = id;
-		pic->priority_map->ID = id;
-		pic->visual_map->ID = id;
-	}
-
-	if (res->unscaled_data.pic) {
-		gfxr_pic_t *pic = res->unscaled_data.pic;
-		pic->control_map->ID = id;
-		pic->priority_map->ID = id;
-		pic->visual_map->ID = id;
-	}
-}
-
-static int
-get_pic_id(gfx_resource_t *res)
-{
-	if (res->scaled_data.pic)
-		return res->scaled_data.pic->visual_map->ID;
-	else
-		return res->unscaled_data.pic->visual_map->ID;
-}
-
 gfxr_pic_t *
 gfxr_add_to_pic(gfx_resstate_t *state, int old_nr, int new_nr, int maps, int flags,
 		int old_default_palette, int default_palette, int scaled)
 {
-	int restype = GFX_RESOURCE_TYPE_PIC;
+	gfx_resource_type_t restype = GFX_RESOURCE_TYPE_PIC;
 	sbtree_t *tree = state->resource_trees[restype];
 	gfxr_pic_t *pic = NULL;
 	gfx_resource_t *res = NULL;
 	int hash = gfxr_interpreter_options_hash(restype, state->version,
 						 state->options,
-						 state->misc_payload);
+						 state->misc_payload, 0);
 	int need_unscaled = !(state->options->pic0_unscaled)
 		&& (state->driver->mode->xfact != 1 || state->driver->mode->yfact != 1);
 
@@ -479,24 +490,23 @@ gfxr_add_to_pic(gfx_resstate_t *state, int old_nr, int new_nr, int maps, int fla
 		res->lock_sequence_nr = state->options->buffer_pics_nr;
 
 		gfxr_interpreter_calculate_pic(state, res->scaled_data.pic, need_unscaled? res->unscaled_data.pic : NULL,
-					       flags, default_palette, new_nr,
-					       state->misc_payload);
+					       flags | DRAWPIC01_FLAG_OVERLAID_PIC, 
+					       default_palette, new_nr, state->misc_payload);
 	}
 
 	res->mode = MODE_INVALID; /* Invalidate */
 
+	if (state->options->pic0_unscaled) /* Scale priority map again, if needed */
+		res->scaled_data.pic->priority_map = gfx_pixmap_scale_index_data(res->scaled_data.pic->priority_map, state->driver->mode);
 	{
 		int old_ID = get_pic_id(res);
 		set_pic_id(res, GFXR_RES_ID(restype, new_nr)); /* To ensure that our graphical translation optoins work properly */
-		pic = gfxr_pic_xlate_common(res, maps, scaled, 1/*force*/, state->driver->mode,
+		pic = gfxr_pic_xlate_common(res, maps, scaled, 1, state->driver->mode,
 					    state->options->pic_xlate_filter, 1,
 					    state->options);
 		set_pic_id(res, old_ID);
 	}
 
-	if (state->options->pic0_unscaled) /* Scale priority map again, if needed */
-		res->scaled_data.pic->priority_map = gfx_pixmap_scale_index_data(res->scaled_data.pic->priority_map, state->driver->mode);
-
 	if (scaled || state->options->pic0_unscaled && maps & GFX_MASK_VISUAL)
 		gfxr_antialiase(pic->visual_map, state->driver->mode,
 				state->options->pic0_antialiasing);
@@ -505,15 +515,15 @@ gfxr_add_to_pic(gfx_resstate_t *state, int old_nr, int new_nr, int maps, int fla
 }
 
 
-
 gfxr_view_t *
-gfxr_get_view(gfx_resstate_t *state, int nr, int *loop, int *cel)
+gfxr_get_view(gfx_resstate_t *state, int nr, int *loop, int *cel, int palette)
 {
-	int restype = GFX_RESOURCE_TYPE_VIEW;
+	gfx_resource_type_t restype = GFX_RESOURCE_TYPE_VIEW;
 	sbtree_t *tree = state->resource_trees[restype];
 	gfx_resource_t *res = NULL;
 	int hash = gfxr_interpreter_options_hash(restype, state->version,
-						 state->options, state->misc_payload);
+						 state->options, state->misc_payload,
+						 palette);
 	gfxr_view_t *view = NULL;
 	gfxr_loop_t *loop_data = NULL;
 	gfx_pixmap_t *cel_data = NULL;
@@ -524,13 +534,13 @@ gfxr_get_view(gfx_resstate_t *state, int nr, int *loop, int *cel)
 	res = (gfx_resource_t *) sbtree_get(tree, nr);
 
 	if (!res || res->mode != hash) {
-		view = gfxr_interpreter_get_view(state, nr, state->misc_payload);
+		view = gfxr_interpreter_get_view(state, nr, state->misc_payload, palette);
 
 		if (!view)
 			return NULL;
 
 		if (!res) {
-			res = sci_malloc(sizeof(gfx_resource_t));
+			res = (gfx_resource_t*)sci_malloc(sizeof(gfx_resource_t));
 			res->scaled_data.view = NULL;
 			res->ID = GFXR_RES_ID(restype, nr);
 			res->lock_sequence_nr = state->tag_lock_counter;
@@ -540,6 +550,7 @@ gfxr_get_view(gfx_resstate_t *state, int nr, int *loop, int *cel)
 			gfxr_free_view(state->driver, res->unscaled_data.view);
 		}
 
+		res->mode = hash;
 		res->unscaled_data.view = view;
 
 	} else {
@@ -553,29 +564,45 @@ gfxr_get_view(gfx_resstate_t *state, int nr, int *loop, int *cel)
 		if (*loop >= view->loops_nr)
 			*loop = view->loops_nr - 1;
 
-	if (view->loops_nr <= 0) {
-		sciprintf("Warning: view.%x has no loops! This game is probably corrupt.\n", nr);
-		*loop = -1;
-		*cel = 0;
-	} else {
-
-		loop_data = view->loops + (*loop);
-
-		if (*cel < 0)
-			*cel = 0;
-		else
-			if (*cel >= loop_data->cels_nr)
-				*cel = loop_data->cels_nr - 1;
-
-		cel_data = loop_data->cels[*cel];
-
-		if (!cel_data->data) {
-			gfx_get_res_config(state->options, cel_data);
-			gfx_xlate_pixmap(cel_data, state->driver->mode, state->options->view_xlate_filter);
-			gfxr_endianness_adjust(cel_data, state->driver->mode);
-		}
+	if (*loop < 0) {
+		GFXWARN("View %d has no loops\n", nr);
+		return NULL;
 	}
 
+	loop_data = view->loops + (*loop);
+	if (loop_data == NULL)
+	{
+		GFXWARN("Trying to load invalid loop %d of view %d\n", *loop, nr);
+		return NULL;
+	}
+
+	if (*cel < 0)
+	{
+		sciprintf("Resetting cel! %d\n", *cel);
+		*cel = 0;
+	}
+	else
+		if (*cel >= loop_data->cels_nr)
+			*cel = loop_data->cels_nr - 1;
+
+	if (*cel < 0) {
+		GFXWARN("View %d loop %d has no cels\n", nr, *loop);
+		return NULL;
+	}
+
+	cel_data = loop_data->cels[*cel];
+	if (loop_data == NULL)
+	{
+		GFXWARN("Trying to load invalid view/loop/cel %d/%d/%d\n", nr, *loop, *cel);
+		return NULL;
+	}
+
+	if (!cel_data->data) {
+		gfx_get_res_config(state->options, cel_data);
+		gfx_xlate_pixmap(cel_data, state->driver->mode, state->options->view_xlate_filter);
+                gfxr_endianness_adjust(cel_data, state->driver->mode);
+        }
+
 	return view;
 }
 
@@ -585,7 +612,7 @@ extern gfx_bitmap_font_t gfxfont_6x10;
 gfx_bitmap_font_t *
 gfxr_get_font(gfx_resstate_t *state, int nr, int scaled)
 {
-	int restype = GFX_RESOURCE_TYPE_FONT;
+	gfx_resource_type_t restype = GFX_RESOURCE_TYPE_FONT;
 	sbtree_t *tree = NULL;
 	gfx_resource_t *res = NULL;
 	int hash;
@@ -598,7 +625,8 @@ gfxr_get_font(gfx_resstate_t *state, int nr, int scaled)
 	tree = state->resource_trees[restype];
 
 	hash = gfxr_interpreter_options_hash(restype, state->version,
-					     state->options, state->misc_payload);
+					     state->options, state->misc_payload, 0);
+
 	if (!tree)
 		return NULL;
 
@@ -612,7 +640,7 @@ gfxr_get_font(gfx_resstate_t *state, int nr, int scaled)
 			return NULL;
 
 		if (!res) {
-			res = sci_malloc(sizeof(gfx_resource_t));
+			res = (gfx_resource_t*)sci_malloc(sizeof(gfx_resource_t));
 			res->scaled_data.font = NULL;
 			res->ID = GFXR_RES_ID(restype, nr);
 			res->lock_sequence_nr = state->tag_lock_counter;
@@ -638,11 +666,11 @@ gfxr_get_font(gfx_resstate_t *state, int nr, int scaled)
 gfx_pixmap_t *
 gfxr_get_cursor(gfx_resstate_t *state, int nr)
 {
-	int restype = GFX_RESOURCE_TYPE_CURSOR;
+	gfx_resource_type_t restype = GFX_RESOURCE_TYPE_CURSOR;
 	sbtree_t *tree = state->resource_trees[restype];
 	gfx_resource_t *res = NULL;
 	int hash = gfxr_interpreter_options_hash(restype, state->version,
-						 state->options, state->misc_payload);
+						 state->options, state->misc_payload, 0);
 
 	if (!tree)
 		return NULL;
@@ -657,7 +685,7 @@ gfxr_get_cursor(gfx_resstate_t *state, int nr)
 			return NULL;
 
 		if (!res) {
-			res = sci_malloc(sizeof(gfx_resource_t));
+			res = (gfx_resource_t*)sci_malloc(sizeof(gfx_resource_t));
 			res->scaled_data.pointer = NULL;
 			res->ID = GFXR_RES_ID(restype, nr);
 			res->lock_sequence_nr = state->tag_lock_counter;
