@@ -37,6 +37,8 @@
 #define GFXR_PIC0_PALETTE_SIZE 40
 #define GFXR_PIC0_NUM_PALETTES 4
 
+#define INTERCOL(a, b) ((int) sqrt((((3.3 * (a))*(a)) + ((1.7 * (b))*(b))) / 5.0))
+/* Macro for color interpolation */
 
 #define SCI_PIC0_MAX_FILL 30 /* Number of times to fill before yielding to scheduler */
 
@@ -44,9 +46,10 @@
 
 int sci0_palette = 0;
 
+/* Copied from include/kernel.h */
+#define SCI0_PRIORITY_BAND_FIRST_14_ZONES(nr) ((((nr) == 0)? 0 :  \
+        ((first) + (((nr)-1) * (last - first)) / 14)))
 
-#define INTERCOL(a, b) ((int) sqrt((((3.3 * (a))*(a)) + ((1.7 * (b))*(b))) / 5.0))
-/* Macro for color interpolation */
 /* Default color maps */
 gfx_pixmap_color_t gfx_sci0_image_colors[SCI0_MAX_PALETTE+1][GFX_SCI0_IMAGE_COLORS_NR] = {
 	{{GFX_COLOR_SYSTEM, 0x00, 0x00, 0x00}, {GFX_COLOR_SYSTEM, 0x00, 0x00, 0xaa},
@@ -102,6 +105,8 @@ gfx_pixmap_color_t gfx_sci0_pic_colors[GFX_SCI0_PIC_COLORS_NR]; /* Initialized d
 
 static int _gfxr_pic0_colors_initialized = 0;
 
+#define SCI1_PALETTE_SIZE 1284
+
 #ifdef FILL_RECURSIVE_DEBUG
 /************************************/
 int fillc = 100000000;
@@ -109,6 +114,14 @@ int fillmagc = 30000000;
 /************************************/
 #endif
 
+/* Color mapping used while scaling embedded views. */
+gfx_pixmap_color_t embedded_view_colors[16] = {
+	{0x00, 0, 0, 0}, {0x11, 0, 0, 0}, {0x22, 0, 0, 0}, {0x33, 0, 0, 0},
+	 {0x44, 0, 0, 0}, {0x55, 0, 0, 0}, {0x66, 0, 0, 0}, {0x77, 0, 0, 0},
+	 {0x88, 0, 0, 0}, {0x99, 0, 0, 0}, {0xaa, 0, 0, 0}, {0xbb, 0, 0, 0},
+	 {0xcc, 0, 0, 0}, {0xdd, 0, 0, 0}, {0xee, 0, 0, 0}, {0xff, 0, 0, 0}
+};
+
 void
 gfxr_init_static_palette()
 {
@@ -124,16 +137,16 @@ gfxr_init_static_palette()
 			gfx_sci0_pic_colors[i].b = INTERCOL(gfx_sci0_image_colors[sci0_palette][i & 0xf].b,
 							    gfx_sci0_image_colors[sci0_palette][i >> 4].b);
 		}
-WARNING("Uncomment me after fixing sci0_palette changes to reset me");
+		WARNING("Uncomment me after fixing sci0_palette changes to reset me");
                 /*  _gfxr_pic0_colors_initialized = 1; */
 	}
 }
 
 
 gfxr_pic_t *
-gfxr_init_pic(gfx_mode_t *mode, int ID)
+gfxr_init_pic(gfx_mode_t *mode, int ID, int sci1)
 {
-	gfxr_pic_t *pic = sci_malloc(sizeof(gfxr_pic_t));
+	gfxr_pic_t *pic = (gfxr_pic_t*)sci_malloc(sizeof(gfxr_pic_t));
 
 	pic->mode = mode;
 
@@ -147,6 +160,7 @@ gfxr_init_pic(gfx_mode_t *mode, int ID)
 								     200 * mode->yfact, ID, 0, 0));
 	pic->visual_map->colors = gfx_sci0_pic_colors;
 	pic->visual_map->colors_nr = GFX_SCI0_PIC_COLORS_NR;
+	pic->visual_map->color_key = GFX_PIXMAP_COLOR_KEY_NONE;
 
 	pic->visual_map->flags = GFX_PIXMAP_FLAG_EXTERNAL_PALETTE;
 	pic->priority_map->flags = GFX_PIXMAP_FLAG_EXTERNAL_PALETTE;
@@ -162,7 +176,10 @@ gfxr_init_pic(gfx_mode_t *mode, int ID)
 	pic->control_map->colors_nr = GFX_SCI0_IMAGE_COLORS_NR;
 
 	/* Initialize colors */
-	gfxr_init_static_palette();
+	if (!sci1) {
+		pic->ID = ID;
+		gfxr_init_static_palette();
+	}
 
 	pic->undithered_buffer_size = pic->visual_map->index_xl * pic->visual_map->index_yl;
 	pic->undithered_buffer = NULL;
@@ -177,14 +194,14 @@ gfxr_init_pic(gfx_mode_t *mode, int ID)
 /****************************/
 
 void
-gfxr_clear_pic0(gfxr_pic_t *pic)
+gfxr_clear_pic0(gfxr_pic_t *pic, int sci_titlebar_size)
 {
-	memset(pic->visual_map->index_data, 0x00, (320 * pic->mode->xfact * SCI_TITLEBAR_SIZE * pic->mode->yfact));
-	memset(pic->visual_map->index_data + (320 * pic->mode->xfact * SCI_TITLEBAR_SIZE * pic->mode->yfact),
-	       0xff, pic->mode->xfact * 320 * pic->mode->yfact * (200 - SCI_TITLEBAR_SIZE)); /* white */
-	memset(pic->priority_map->index_data + (320 * pic->mode->xfact * SCI_TITLEBAR_SIZE * pic->mode->yfact),
-	       0x0, pic->mode->xfact * 320 * pic->mode->yfact * (200 - SCI_TITLEBAR_SIZE));
-	memset(pic->priority_map->index_data, 0x0a, SCI_TITLEBAR_SIZE * (pic->mode->yfact * 320 * pic->mode->xfact));
+	memset(pic->visual_map->index_data, 0x00, (320 * pic->mode->xfact * sci_titlebar_size * pic->mode->yfact));
+	memset(pic->visual_map->index_data + (320 * pic->mode->xfact * sci_titlebar_size * pic->mode->yfact),
+	       0xff, pic->mode->xfact * 320 * pic->mode->yfact * (200 - sci_titlebar_size)); /* white */
+	memset(pic->priority_map->index_data + (320 * pic->mode->xfact * sci_titlebar_size * pic->mode->yfact),
+	       0x0, pic->mode->xfact * 320 * pic->mode->yfact * (200 - sci_titlebar_size));
+	memset(pic->priority_map->index_data, 0x0a, sci_titlebar_size * (pic->mode->yfact * 320 * pic->mode->xfact));
 	memset(pic->control_map->index_data, 0, GFXR_AUX_MAP_SIZE);
 	memset(pic->aux_map, 0, GFXR_AUX_MAP_SIZE);
 }
@@ -205,7 +222,7 @@ gfxr_clear_pic0(gfxr_pic_t *pic)
    d = nonlinearstart-1;  \
    while (linearvar != (linearend)) { \
      buffer[linewidth * y + x] operation color; \
-     color ^= color2; color2 ^= color; color ^= color2; /* Swap colors */ \
+/* color ^= color2; color2 ^= color; color ^= color2; */ /* Swap colors */ \
      linearvar += linearmod; \
      if ((d+=incrE) < 0) { \
        d += incrNE; \
@@ -215,11 +232,11 @@ gfxr_clear_pic0(gfxr_pic_t *pic)
   buffer[linewidth * y + x] operation color;
 
 static void
-_gfxr_auxbuf_line_draw(gfxr_pic_t *pic, rect_t line, int color, int color2)
+_gfxr_auxbuf_line_draw(gfxr_pic_t *pic, rect_t line, int color, int color2, int sci_titlebar_size)
 {
 	int dx, dy, incrE, incrNE, d, finalx, finaly;
 	int x = line.x;
-	int y = line.y + SCI_TITLEBAR_SIZE;
+	int y = line.y + sci_titlebar_size;
 	unsigned char *buffer = pic->aux_map;
 	int linewidth = 320;
 
@@ -263,11 +280,11 @@ _gfxr_auxbuf_line_draw(gfxr_pic_t *pic, rect_t line, int color, int color2)
 }
 
 static void
-_gfxr_auxbuf_line_clear(gfxr_pic_t *pic, rect_t line, int color)
+_gfxr_auxbuf_line_clear(gfxr_pic_t *pic, rect_t line, int color, int sci_titlebar_size)
 {
 	int dx, dy, incrE, incrNE, d, finalx, finaly;
 	int x = line.x;
-	int y = line.y + SCI_TITLEBAR_SIZE;
+	int y = line.y + sci_titlebar_size;
 	unsigned char *buffer = pic->aux_map;
 	int linewidth = 320;
 	int color2 = color;
@@ -383,7 +400,6 @@ _gfxr_auxbuf_spread(gfxr_pic_t *pic, int *min_x, int *min_y, int *max_x, int *ma
 		int ivi = y & 1; /* InterVal Index: Current intervals; !ivi is the list of old ones */
 		int old_intervals_start_offset = 0;
 		int width = 0;
-		int j;
 
 		old_intervals_nr = intervals_nr;
 		intervals_nr = 0;
@@ -479,7 +495,7 @@ _gfxr_auxbuf_spread(gfxr_pic_t *pic, int *min_x, int *min_y, int *max_x, int *ma
 #ifdef FILL_RECURSIVE_DEBUG
 		if (!fillmagc && intervals_nr) {
 			fprintf(stderr,"AI L#%03d:", y);
-			for (j = 0; j < intervals_nr; j++)
+			for (int j = 0; j < intervals_nr; j++)
 				fprintf(stderr, "%c[%03d,%03d]", intervals[ivi][j].tag? ' ':'-', intervals[ivi][j].xl, intervals[ivi][j].xr);
 			fprintf(stderr,"\n");
 		}
@@ -532,17 +548,31 @@ enum {
 };
 
 enum {
-	PIC_OPX_SET_PALETTE_ENTRIES = 0,
-	PIC_OPX_SET_PALETTE = 1,
-	PIC_OPX_MONO0 = 2,
-	PIC_OPX_MONO1 = 3,
-	PIC_OPX_MONO2 = 4,
-	PIC_OPX_MONO3 = 5,
-	PIC_OPX_MONO4 = 6,
-	PIC_OPX_EMBEDDED_VIEW,
-	PIC_OPX_SET_PRIORITY_TABLE
+	PIC_SCI0_OPX_SET_PALETTE_ENTRIES = 0,
+	PIC_SCI0_OPX_SET_PALETTE = 1,
+	PIC_SCI0_OPX_MONO0 = 2,
+	PIC_SCI0_OPX_MONO1 = 3,
+	PIC_SCI0_OPX_MONO2 = 4,
+	PIC_SCI0_OPX_MONO3 = 5,
+	PIC_SCI0_OPX_MONO4 = 6,
+	PIC_SCI0_OPX_EMBEDDED_VIEW,
+	PIC_SCI0_OPX_SET_PRIORITY_TABLE
 };
 
+/* We use this so we can keep OPX handling in one switch.
+   We simply add this constant to the op number if we're running an SCI1 game,
+   and offset the OPX constants below correspondingly. */
+#define SCI1_OP_OFFSET 42
+
+enum {
+	PIC_SCI1_OPX_SET_PALETTE_ENTRIES = 0+SCI1_OP_OFFSET,
+	PIC_SCI1_OPX_EMBEDDED_VIEW = 1+SCI1_OP_OFFSET,
+	PIC_SCI1_OPX_SET_PALETTE = 2+SCI1_OP_OFFSET,
+	PIC_SCI1_OPX_PRIORITY_TABLE_EQDIST = 3+SCI1_OP_OFFSET,
+	PIC_SCI1_OPX_PRIORITY_TABLE_EXPLICIT = 4+SCI1_OP_OFFSET
+};
+	
+
 #ifdef GFXR_DEBUG_PIC0
 #define p0printf sciprintf
 #else
@@ -820,7 +850,8 @@ _gfxr_plot_aux_pattern(gfxr_pic_t *pic, int x, int y, int size, int circle, int
 
 static void
 _gfxr_draw_pattern(gfxr_pic_t *pic, int x, int y, int color, int priority, int control, int drawenable,
-		   int pattern_code, int pattern_size, int pattern_nr, int brush_mode)
+		   int pattern_code, int pattern_size, int pattern_nr, gfx_brush_mode_t brush_mode,
+		   int sci_titlebar_size)
 {
 	int xsize = (pattern_size + 1) * pic->mode->xfact - 1;
 	int ysize = (pattern_size + 1) * pic->mode->yfact - 1;
@@ -830,13 +861,13 @@ _gfxr_draw_pattern(gfxr_pic_t *pic, int x, int y, int color, int priority, int c
 
 	p0printf(stderr, "Pattern at (%d,%d) size %d, rand=%d, code=%02x\n", x, y, pattern_size, pattern_nr, pattern_code);
 
-	y += SCI_TITLEBAR_SIZE;
+	y += sci_titlebar_size;
 
 	if (x - pattern_size < 0)
 		x = pattern_size;
 
-	if (y - pattern_size < SCI_TITLEBAR_SIZE)
-		y = SCI_TITLEBAR_SIZE + pattern_size;
+	if (y - pattern_size < sci_titlebar_size)
+		y = sci_titlebar_size + pattern_size;
 
 	if (x + pattern_size > max_x)
 		x = max_x - pattern_size;
@@ -850,8 +881,8 @@ _gfxr_draw_pattern(gfxr_pic_t *pic, int x, int y, int color, int priority, int c
 	if (scaled_x < xsize)
 		scaled_x = xsize;
 
-	if (scaled_y < ysize + SCI_TITLEBAR_SIZE * pic->mode->yfact)
-		scaled_y = ysize + SCI_TITLEBAR_SIZE * pic->mode->yfact;
+	if (scaled_y < ysize + sci_titlebar_size * pic->mode->yfact)
+		scaled_y = ysize + sci_titlebar_size * pic->mode->yfact;
 
 	if (scaled_x > (320 * pic->mode->xfact) - 1 - xsize)
 		scaled_x = (320 * pic->mode->xfact) - 1 - xsize;
@@ -875,7 +906,9 @@ _gfxr_draw_pattern(gfxr_pic_t *pic, int x, int y, int color, int priority, int c
 
 			_gfxr_plot_aux_pattern(pic, x, y, pattern_size, 0,
 					       PLOT_AUX_PATTERN_NO_RANDOM,
-					       drawenable, 0, 0, control, 0, GFX_MASK_CONTROL);
+					       drawenable, 0, 0, control,
+					       GFX_BRUSH_MODE_SCALED,
+					       GFX_MASK_CONTROL);
 
 			if (drawenable & GFX_MASK_VISUAL)
 				gfx_draw_box_pixmap_i(pic->visual_map, boundaries, color);
@@ -896,19 +929,25 @@ _gfxr_draw_pattern(gfxr_pic_t *pic, int x, int y, int color, int priority, int c
 
 			_gfxr_plot_aux_pattern(pic, x, y, pattern_size, 1,
 					       PLOT_AUX_PATTERN_NO_RANDOM,
-					       drawenable, 0, 0, control, 0, GFX_MASK_CONTROL);
+					       drawenable, 0, 0, control,
+					       GFX_BRUSH_MODE_SCALED,
+					       GFX_MASK_CONTROL);
 
 			if (pic->mode->xfact == 1 && pic->mode->yfact == 1) {
 
 				if (drawenable & GFX_MASK_VISUAL)
 					_gfxr_plot_aux_pattern(pic, x, y, pattern_size, 1,
 							       PLOT_AUX_PATTERN_NO_RANDOM,
-							       drawenable, 0, 0, color, 0, GFX_MASK_VISUAL);
+							       drawenable, 0, 0, color,
+							       GFX_BRUSH_MODE_SCALED,
+							       GFX_MASK_VISUAL);
 
 				if (drawenable & GFX_MASK_PRIORITY)
 					_gfxr_plot_aux_pattern(pic, x, y, pattern_size, 1,
 							       PLOT_AUX_PATTERN_NO_RANDOM,
-							       drawenable, 0, 0, priority, 0, GFX_MASK_PRIORITY);
+							       drawenable, 0, 0, priority,
+							       GFX_BRUSH_MODE_SCALED,
+							       GFX_MASK_PRIORITY);
 			} else {
 
 				if (drawenable & GFX_MASK_VISUAL)
@@ -929,30 +968,32 @@ _gfxr_draw_pattern(gfxr_pic_t *pic, int x, int y, int color, int priority, int c
 static inline void
 _gfxr_draw_subline(gfxr_pic_t *pic, int x, int y, int ex, int ey, int color, int priority, int drawenable)
 {
-	rect_t line;
+	point_t start;
+	point_t end;
+	
+	start.x = x;
+	start.y = y;
+	end.x = ex;
+	end.y = ey;
 
-	line.x = x;
-	line.y = y;
-	line.xl = ex - x;
-	line.yl = ey - y;
         if (ex >= pic->visual_map->index_xl || ey >= pic->visual_map->index_yl || x < 0 || y < 0) {
                 fprintf(stderr,"While drawing pic0: INVALID LINE %d,%d,%d,%d\n",
-                        GFX_PRINT_RECT(line));
+                        GFX_PRINT_POINT(start), GFX_PRINT_POINT(end));
                 return;
         }
 
 	if (drawenable & GFX_MASK_VISUAL)
-		gfx_draw_line_pixmap_i(pic->visual_map, line, color);
+		gfx_draw_line_pixmap_i(pic->visual_map, start, end, color);
 
 	if (drawenable & GFX_MASK_PRIORITY)
-		gfx_draw_line_pixmap_i(pic->priority_map, line, priority);
+		gfx_draw_line_pixmap_i(pic->priority_map, start, end, priority);
 
 }
 
 static void
 _gfxr_draw_line(gfxr_pic_t *pic, int x, int y, int ex, int ey, int color,
 		int priority, int control, int drawenable, int line_mode,
-		int cmd)
+		int cmd, int sci_titlebar_size)
 {
 	int scale_x = pic->mode->xfact;
 	int scale_y = pic->mode->yfact;
@@ -973,13 +1014,13 @@ _gfxr_draw_line(gfxr_pic_t *pic, int x, int y, int ex, int ey, int color,
 		return;
 	}
 
-	y += SCI_TITLEBAR_SIZE;
-	ey += SCI_TITLEBAR_SIZE;
+	y += sci_titlebar_size;
+	ey += sci_titlebar_size;
 
 	if (drawenable & GFX_MASK_CONTROL) {
 
 		p0printf(" ctl:%x", control);
-		gfx_draw_line_pixmap_i(pic->control_map, gfx_rect(x, y, line.xl, line.yl), control);
+		gfx_draw_line_pixmap_i(pic->control_map, gfx_point(x, y), gfx_point(x + line.xl, y + line.yl), control);
 	}
 
 
@@ -995,7 +1036,7 @@ _gfxr_draw_line(gfxr_pic_t *pic, int x, int y, int ex, int ey, int color,
 		int mask2 = mask;
 		if (partially_white)
 			mask2 = mask &= ~GFX_MASK_VISUAL;
-		_gfxr_auxbuf_line_draw(pic, line, mask, mask2);
+		_gfxr_auxbuf_line_draw(pic, line, mask, mask2, sci_titlebar_size);
 	}
 
 	/* Calculate everything that is changed to TRANSPARENT */
@@ -1007,7 +1048,7 @@ _gfxr_draw_line(gfxr_pic_t *pic, int x, int y, int ex, int ey, int color,
 			);
 
 	if (mask)
-		_gfxr_auxbuf_line_clear(pic, line, ~mask);
+		_gfxr_auxbuf_line_clear(pic, line, ~mask, sci_titlebar_size);
 
 	x *= scale_x;
 	y *= scale_y;
@@ -1180,8 +1221,8 @@ _gfxr_find_fill_point(gfxr_pic_t *pic, int min_x, int min_y, int max_x, int max_
 #define AUXBUF_FILL_HELPER _gfxr_auxbuf_fill_any_recursive
 #define AUXBUF_FILL _gfxr_auxbuf_fill_any
 #define DRAW_SCALED
-#  include "sci_picfill_aux.c"
-#  include "sci_picfill.c"
+# include "sci_picfill_aux.c"
+# include "sci_picfill.c"
 #undef DRAW_SCALED
 #undef AUXBUF_FILL
 #undef AUXBUF_FILL_HELPER
@@ -1195,8 +1236,8 @@ _gfxr_find_fill_point(gfxr_pic_t *pic, int min_x, int min_y, int max_x, int max_
 #define FILL_FUNCTION_RECURSIVE _gfxr_fill_1_recursive
 #define AUXBUF_FILL_HELPER _gfxr_auxbuf_fill_1_recursive
 #define AUXBUF_FILL _gfxr_auxbuf_fill_1
-#  include "sci_picfill_aux.c"
-#  include "sci_picfill.c"
+# include "sci_picfill_aux.c"
+# include "sci_picfill.c"
 #undef AUXBUF_FILL
 #undef AUXBUF_FILL_HELPER
 #undef FILL_FUNCTION_RECURSIVE
@@ -1231,82 +1272,6 @@ _gfxr_find_fill_point(gfxr_pic_t *pic, int min_x, int min_y, int max_x, int max_
   x = oldx + *((signed char *) resource + pos++);
 
 
-static void
-_gfxr_vismap_remove_artifacts_old(gfxr_pic_t *pic)
-{
-	/* Check the visual map for things that look like artifacs and remove them,
-	** if that appears to be appropriate.
-	*/
-	int x,y;
-	int linewidth = 320 * pic->mode->xfact;
-	int maxx = 320 * pic->mode->xfact - 1;
-	int maxy = 200 * pic->mode->yfact - 1;
-	int miny = SCI_TITLEBAR_SIZE * pic->mode->yfact;
-	int offset = linewidth * miny;
-	byte *vismap = pic->visual_map->index_data;
-	byte *primap = pic->priority_map->index_data;
-
-	if (pic->mode->xfact == 1 || pic->mode->yfact == 1)
-		return;
-
-	for (y = miny; y <= maxy; y++)
-		for (x = 0; x <= maxx; x++) {
-			if (vismap[offset] == 0xff) { /* Potential offender */
-				int horiz = 0, vertic = 0;
-
-				if ((x > 0 && vismap[offset-1] == 0xff)
-				    || (x < maxx && vismap[offset+1] == 0xff))
-					horiz = 1;
-
-				if ((y > miny && vismap[offset-linewidth] == 0xff)
-				    || (y < maxy && vismap[offset+linewidth] == 0xff))
-					vertic = 1;
-
-				if (!(horiz && vertic)) { /* otherwise, we're inside a white block */
-					if (horiz) {
-						if (y == miny) {
-							vismap[offset] = vismap[offset + linewidth];
-							primap[offset] = vismap[offset + linewidth];
-						} else if (y == maxy) {
-							vismap[offset] = vismap[offset - linewidth];
-							primap[offset] = vismap[offset - linewidth];
-						} else {
-							if ((primap[offset + linewidth] == primap[offset - linewidth])
-							    &&(vismap[offset - linewidth] == vismap[offset + linewidth])) {
-								vismap[offset] = vismap[offset + linewidth];
-								primap[offset] = primap[offset + linewidth];
-							} else {
-								vismap[offset] = (vismap[offset + linewidth] & 0x0f) | (vismap[offset - linewidth] & 0xf0);
-								primap[offset] = (primap[offset + linewidth] + primap[offset - linewidth]) >> 1;
-							}
-						}
-					} else { /* vertical or neither */
-						if (x == 0) {
-							vismap[offset] = vismap[offset + 1];
-							primap[offset] = vismap[offset + 1];
-						} else if (x == maxx) {
-							vismap[offset] = vismap[offset - 1];
-							primap[offset] = vismap[offset - 1];
-						} else {
-							if ((primap[offset + 1] == primap[offset - 1])
-							    &&(vismap[offset - 1] == vismap[offset + 1])) {
-								vismap[offset] = vismap[offset + 1];
-								primap[offset] = primap[offset + 1];
-							} else {
-								vismap[offset] = (vismap[offset + 1] & 0x0f) | (vismap[offset - 1] & 0xf0);
-								primap[offset] = (primap[offset + 1] + primap[offset - 1]) >> 1;
-							}
-						}
-					}
-				}
-			}
-
-			++offset;
-		}
-
-}
-
-
 inline static void
 check_and_remove_artifact(byte *dest, byte* srcp, int legalcolor, byte l, byte r, byte u, byte d)
 {
@@ -1384,10 +1349,35 @@ gfxr_remove_artifacts_pic0(gfxr_pic_t *dest, gfxr_pic_t *src)
 
 }
 
+static void
+view_transparentize(gfx_pixmap_t *view, byte *pic_index_data,
+		    int posx, int posy,
+		    int width, int height)
+{
+	int i,j;
+
+	for (i=0;i<width;i++)
+		for (j=0;j<height;j++)
+		{
+			if (view->index_data[j*width+i] == view->color_key)
+			{
+				view->index_data[j*width+i] = 
+					pic_index_data[(j+posy)*width+i+posx];
+			}
+		}
+}
+
+extern gfx_pixmap_t *
+gfxr_draw_cel0(int id, int loop, int cel, byte *resource, int size, gfxr_view_t *view, int mirrored);
+extern gfx_pixmap_t *
+gfxr_draw_cel1(int id, int loop, int cel, int mirrored, byte *resource, int size, gfxr_view_t *view, int amiga_game);
+extern void
+_gfx_crossblit_simple(byte *dest, byte *src, int dest_line_width, int src_line_width, int xl, int yl, int bpp);
 
 void
-gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size,
-	       byte *resource, gfxr_pic0_params_t *style, int resid)
+gfxr_draw_pic01(gfxr_pic_t *pic, int flags, int default_palette, int size,
+		byte *resource, gfxr_pic0_params_t *style, int resid, int sci1,
+		gfx_pixmap_color_t *static_pal, int static_pal_nr)
 {
 	const int default_palette_table[GFXR_PIC0_PALETTE_SIZE] = {
 		0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
@@ -1420,13 +1410,14 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 	int pal, index;
 	int temp;
 	int line_mode = style->line_mode;
+	int sci_titlebar_size = style->pic_port_bounds.y;
 	int fill_count = 0;
 	byte op, opx;
 
 #ifdef FILL_RECURSIVE_DEBUG
 	fillmagc = atoi(getenv("FOO"));
 	fillc = atoi(getenv("FOO2"));
-#endif /* GFXR_DEBUG_PIC0 */
+#endif /* FILL_RECURSIVE_DEBUG */
 
 	/* Initialize palette */
 	for (i = 0; i < GFXR_PIC0_NUM_PALETTES; i++)
@@ -1443,47 +1434,51 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 		case PIC_OP_SET_COLOR:
 			p0printf("Set color @%d\n", pos);
 
-			pal = *(resource + pos++);
-			index = pal % GFXR_PIC0_PALETTE_SIZE;
-			pal /= GFXR_PIC0_PALETTE_SIZE;
+			if (!sci1) {
+				pal = *(resource + pos++);
+				index = pal % GFXR_PIC0_PALETTE_SIZE;
+				pal /= GFXR_PIC0_PALETTE_SIZE;
 
-			pal += default_palette;
+				pal += default_palette;
 
-			if (pal >= GFXR_PIC0_NUM_PALETTES) {
-				GFXERROR("Attempt to access invalid palette %d\n", pal);
-				return;
-			}
+				if (pal >= GFXR_PIC0_NUM_PALETTES) {
+					GFXERROR("Attempt to access invalid palette %d\n", pal);
+					return;
+				}
 
-			color = palette[pal][index];
+				color = palette[pal][index];
+			} else color = *(resource + pos++);
 			p0printf("  color <- %02x [%d/%d]\n", color, pal, index);
 			drawenable |= GFX_MASK_VISUAL;
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_DISABLE_VISUAL:
 			p0printf("Disable visual @%d\n", pos);
 			drawenable &= ~GFX_MASK_VISUAL;
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_SET_PRIORITY:
 			p0printf("Set priority @%d\n", pos);
 
-			pal = *(resource + pos++);
-			index = pal % GFXR_PIC0_PALETTE_SIZE;
-			pal /= GFXR_PIC0_PALETTE_SIZE; /* Ignore pal */
-
-			priority = priority_table[index];
+			if (!sci1) {
+				pal = *(resource + pos++);
+				index = pal % GFXR_PIC0_PALETTE_SIZE;
+				pal /= GFXR_PIC0_PALETTE_SIZE; /* Ignore pal */
+				
+				priority = priority_table[index];
+			} else priority = *(resource + pos++);
 
 			p0printf("  priority <- %d [%d/%d]\n", priority, pal, index);
 			drawenable |= GFX_MASK_PRIORITY;
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_DISABLE_PRIORITY:
 			p0printf("Disable priority @%d\n", pos);
 			drawenable &= ~GFX_MASK_PRIORITY;
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_SHORT_PATTERNS:
@@ -1496,7 +1491,7 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 			GET_ABS_COORDS(x, y);
 
 			_gfxr_draw_pattern(pic, x, y, color, priority, control, drawenable, pattern_code,
-					   pattern_size, pattern_nr, style->brush_mode);
+					   pattern_size, pattern_nr, style->brush_mode, sci_titlebar_size);
 
 			while (*(resource + pos) < PIC_OP_FIRST) {
 				if (pattern_code & PATTERN_FLAG_USE_PATTERN) {
@@ -1507,9 +1502,9 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 				GET_REL_COORDS(x, y);
 
 				_gfxr_draw_pattern(pic, x, y, color, priority, control, drawenable, pattern_code,
-						   pattern_size, pattern_nr, style->brush_mode);
+						   pattern_size, pattern_nr, style->brush_mode, sci_titlebar_size);
 			}
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_MEDIUM_LINES:
@@ -1524,10 +1519,11 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 #if 0
 				fprintf(stderr, " to %d,%d\n", x, y);
 #endif
-				_gfxr_draw_line(pic, oldx, oldy, x, y, color, priority, control, drawenable, line_mode, PIC_OP_MEDIUM_LINES);
+				_gfxr_draw_line(pic, oldx, oldy, x, y, color, priority, control, drawenable, line_mode, 
+						PIC_OP_MEDIUM_LINES, sci_titlebar_size);
 				oldx = x; oldy = y;
 			}
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_LONG_LINES:
@@ -1535,10 +1531,11 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 			GET_ABS_COORDS(oldx, oldy);
 			while (*(resource + pos) < PIC_OP_FIRST) {
 				GET_ABS_COORDS(x,y);
-				_gfxr_draw_line(pic, oldx, oldy, x, y, color, priority, control, drawenable, line_mode, PIC_OP_LONG_LINES);
+				_gfxr_draw_line(pic, oldx, oldy, x, y, color, priority, control, drawenable, line_mode, 
+						PIC_OP_LONG_LINES, sci_titlebar_size);
 				oldx = x; oldy = y;
 			}
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_SHORT_LINES:
@@ -1547,10 +1544,11 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 			x = oldx; y = oldy;
 			while (*(resource + pos) < PIC_OP_FIRST) {
 				GET_REL_COORDS(x,y);
-				_gfxr_draw_line(pic, oldx, oldy, x, y, color, priority, control, drawenable, line_mode, PIC_OP_SHORT_LINES);
+				_gfxr_draw_line(pic, oldx, oldy, x, y, color, priority, control, drawenable, line_mode, 
+						PIC_OP_SHORT_LINES, sci_titlebar_size);
 				oldx = x; oldy = y;
 			}
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_FILL:
@@ -1559,21 +1557,24 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 				/*fprintf(stderr,"####################\n"); */
 				GET_ABS_COORDS(x, y);
 				p0printf("Abs coords %d,%d\n", x, y);
-				/*fprintf(stderr,"C=(%d,%d)\n", x, y + SCI_TITLEBAR_SIZE);*/
+				/*fprintf(stderr,"C=(%d,%d)\n", x, y + sci_titlebar_size);*/
 #ifdef WITH_PIC_SCALING
 				if (pic->mode->xfact > 1
 				    || pic->mode->yfact > 1)
-					_gfxr_fill_any(pic, x, y + SCI_TITLEBAR_SIZE, (fill_normally)? color : 0, priority, control, drawenable);
+				_gfxr_fill_any(pic, x, y + sci_titlebar_size, (flags & DRAWPIC01_FLAG_FILL_NORMALLY)? 
+					       color : 0, priority, control, drawenable, sci_titlebar_size);
+
 				else
 #endif
-				_gfxr_fill_1(pic, x, y + SCI_TITLEBAR_SIZE, (fill_normally)? color : 0, priority, control, drawenable);
+				_gfxr_fill_1(pic, x, y + sci_titlebar_size, (flags & DRAWPIC01_FLAG_FILL_NORMALLY)? 
+					     color : 0, priority, control, drawenable, sci_titlebar_size);
 
 				if (fill_count++ > SCI_PIC0_MAX_FILL) {
 					sci_sched_yield();
 					fill_count = 0;
 				}
 
-#ifdef  FILL_RECURSIVE_DEBUG
+#ifdef FILL_RECURSIVE_DEBUG
 				if (!fillmagc) {
 					int x,y;
 					if (getenv("FOO1"))
@@ -1610,14 +1611,14 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 				} --fillmagc;
 #endif /* GFXR_DEBUG_PIC0 */
 			}
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_SET_PATTERN:
 			p0printf("Set pattern @%d\n", pos);
 			pattern_code = (*(resource + pos++));
 			pattern_size = pattern_code & 0x07;
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_ABSOLUTE_PATTERN:
@@ -1631,22 +1632,22 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 				GET_ABS_COORDS(x, y);
 
 				_gfxr_draw_pattern(pic, x, y, color, priority, control, drawenable, pattern_code,
-						   pattern_size, pattern_nr, style->brush_mode);
+						   pattern_size, pattern_nr, style->brush_mode, sci_titlebar_size);
 			}
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_SET_CONTROL:
 			p0printf("Set control @%d\n", pos);
 			control = (*(resource + pos++)) & 0xf;
 			drawenable |= GFX_MASK_CONTROL;
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_DISABLE_CONTROL:
 			p0printf("Disable control @%d\n", pos);
 			drawenable &= ~GFX_MASK_CONTROL;
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_MEDIUM_PATTERNS:
@@ -1659,7 +1660,7 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 			GET_ABS_COORDS(oldx, oldy);
 
 			_gfxr_draw_pattern(pic, oldx, oldy, color, priority, control, drawenable, pattern_code,
-					   pattern_size, pattern_nr, style->brush_mode);
+					   pattern_size, pattern_nr, style->brush_mode, sci_titlebar_size);
 
 			x = oldx; y = oldy;
 			while (*(resource + pos) < PIC_OP_FIRST) {
@@ -1671,19 +1672,24 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 				GET_MEDREL_COORDS(x, y);
 
 				_gfxr_draw_pattern(pic, x, y, color, priority, control, drawenable, pattern_code,
-						   pattern_size, pattern_nr, style->brush_mode);
+						   pattern_size, pattern_nr, style->brush_mode, sci_titlebar_size);
 			}
-			break;
+			goto end_op_loop;
 
 
 		case PIC_OP_OPX:
 			opx = *(resource + pos++);
 			p0printf("OPX: ");
 
+			if (sci1) opx += SCI1_OP_OFFSET; /* See comment at the definition of SCI1_OP_OFFSET. */
+
 			switch (opx) {
 
+			case PIC_SCI1_OPX_SET_PALETTE_ENTRIES:
+				GFXWARN("SCI1 Set palette entried not implemented\n");
+				goto end_op_loop;
 
-			case PIC_OPX_SET_PALETTE_ENTRIES:
+			case PIC_SCI0_OPX_SET_PALETTE_ENTRIES:
 				p0printf("Set palette entry @%d\n", pos);
 				while (*(resource + pos) < PIC_OP_FIRST) {
 					index = *(resource + pos++);
@@ -1696,10 +1702,10 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 					}
 					palette[pal][index] = *(resource + pos++);
 				}
-				break;
+				goto end_op_loop;
 
 
-			case PIC_OPX_SET_PALETTE:
+			case PIC_SCI0_OPX_SET_PALETTE:
 				p0printf("Set palette @%d\n", pos);
 				pal = *(resource + pos++);
 				if (pal >= GFXR_PIC0_NUM_PALETTES) {
@@ -1717,44 +1723,135 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 					p0printf("%02x", palette[pal][index]);
 				}
 				p0printf(")\n");
-				break;
+				goto end_op_loop;
 
+			case PIC_SCI1_OPX_SET_PALETTE:
+				p0printf("Set palette @%d\n", pos);
+				pic->visual_map->flags &= ~GFX_PIXMAP_FLAG_EXTERNAL_PALETTE;
+				pic->visual_map->colors = gfxr_read_pal1(resid, &pic->visual_map->colors_nr,
+									 resource+pos, SCI1_PALETTE_SIZE);
+				pos += SCI1_PALETTE_SIZE;
+				goto end_op_loop;
 
-			case PIC_OPX_MONO0:
+			case PIC_SCI0_OPX_MONO0:
 				p0printf("Monochrome opx 0 @%d\n", pos);
 				pos += 41;
-				break;
+				goto end_op_loop;
 
 
-			case PIC_OPX_MONO1:
-			case PIC_OPX_MONO3:
+			case PIC_SCI0_OPX_MONO1:
+			case PIC_SCI0_OPX_MONO3:
 				++pos;
 				p0printf("Monochrome opx %d @%d\n", opx, pos);
-				break;
+				goto end_op_loop;
 
 
-			case PIC_OPX_MONO2:
-			case PIC_OPX_MONO4: /* Monochrome ops: Ignored by us */
+			case PIC_SCI0_OPX_MONO2:
+			case PIC_SCI0_OPX_MONO4: /* Monochrome ops: Ignored by us */
 				p0printf("Monochrome opx %d @%d\n", opx, pos);
-				break;
+				goto end_op_loop;
 
 
-			case PIC_OPX_EMBEDDED_VIEW:
-				GFXWARN("Embedded view @%d\n", pos);
-				GFXWARN("-- not implemented- aborting --\n");
-				return;
-				break;
+			case PIC_SCI0_OPX_EMBEDDED_VIEW:
+			case PIC_SCI1_OPX_EMBEDDED_VIEW:
+			{
+				int posx, posy;
+				int bytesize;
+/*				byte *vismap = pic->visual_map->index_data; */
+				int nodraw = 0;
 
+				gfx_pixmap_t *view;
+				gfx_mode_t *mode;
 
-			case PIC_OPX_SET_PRIORITY_TABLE: {
+				p0printf("Embedded view @%d\n", pos);
+
+				/* Set up mode structure for resizing the view */
+				mode = gfx_new_mode(
+					pic->visual_map->index_xl/320,
+					pic->visual_map->index_yl/200, 
+					1, /* 1bpp, which handles masks and the rest for us */
+					0, 0, 0, 0, 0, 0, 0, 0, 16, 0);
+
+				GET_ABS_COORDS(posx, posy);
+				bytesize = (*(resource + pos))+(*(resource + pos + 1) << 8);
+				p0printf("(%d, %d)\n", posx, posy);
+				pos += 2;
+				if (!sci1 && !nodraw) 
+					view = gfxr_draw_cel0(-1,-1,-1, resource + pos, bytesize, NULL, 0); 
+				else
+					view = gfxr_draw_cel1(-1,-1,-1, 0, resource + pos, bytesize, NULL,
+							      static_pal_nr == GFX_SCI1_AMIGA_COLORS_NR);
+				pos+=bytesize;
+				if (nodraw) continue;
+				p0printf("(%d, %d)-(%d, %d)\n", 
+					 posx, 
+					 posy, 
+					 posx + view->index_xl,
+					 posy + view->index_yl);
+
+				/* we can only safely replace the palette if it's static
+				 *if it's not for some reason, we should die
+				 */
+				if (!(view->flags & GFX_PIXMAP_FLAG_EXTERNAL_PALETTE) && !sci1) {
+					sciprintf("gfx_draw_pic0(): can't set a non-static palette for an embedded view!\n");
+				}
+
+				/* For SCI0, use special color mapping to copy the low
+				** nibble of the color index to the high
+				** nibble. 
+				*/
+				if (sci1) {
+					if (static_pal_nr == GFX_SCI1_AMIGA_COLORS_NR) {
+						/* Assume Amiga game */
+						pic->visual_map->colors = static_pal;
+						pic->visual_map->colors_nr = static_pal_nr;
+						pic->visual_map->flags |= GFX_PIXMAP_FLAG_EXTERNAL_PALETTE;
+					}
+					view->colors = pic->visual_map->colors;
+					view->colors_nr = pic->visual_map->colors_nr;
+				} else
+					view->colors = embedded_view_colors;
+
+				/* Hack to prevent overflowing the visual map buffer.
+				   Yes, this does happen otherwise. */
+				if (view->index_yl + sci_titlebar_size > 200)
+					sci_titlebar_size = 0;
+
+				gfx_xlate_pixmap(view, mode, GFX_XLATE_FILTER_NONE);
+
+				if (flags & DRAWPIC01_FLAG_OVERLAID_PIC)
+					view_transparentize(view, pic->visual_map->index_data, 
+							    posx, sci_titlebar_size+posy, 
+							    view->index_xl, view->index_yl);
+
+				_gfx_crossblit_simple(pic->visual_map->index_data+(sci_titlebar_size*320)+
+						      posy*320+posx,
+						      view->index_data,
+						      pic->visual_map->index_xl, view->index_xl,
+						      view->index_xl,
+						      view->index_yl,
+						      1);
+
+				gfx_free_mode(mode);
+				gfx_free_pixmap(NULL, view);
+			}
+			goto end_op_loop;
+
+			case PIC_SCI0_OPX_SET_PRIORITY_TABLE: 
+			case PIC_SCI1_OPX_PRIORITY_TABLE_EXPLICIT: {
 				int i;
 				int *pri_table;
 
-				p0printf("Set priority table @%d\n", pos);
+				p0printf("Explicit priority table @%d\n", pos);
 				if (!pic->internal)
-					pic->internal = sci_malloc(16 * sizeof(int));
+				{
+					pic->internal = sci_malloc(16 * sizeof(int)); 
+				} else
+				{
+					GFXERROR("pic->internal is not NULL (%08x); this only occurs with overlaid pics, otherwise it's a bug!\n", pic->internal); 
+				}	
 
-				pri_table = pic->internal;
+				pri_table = (int*)pic->internal;
 
 				pri_table[0] = 0;
 				pri_table[15] = 190;
@@ -1762,13 +1859,35 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 				for (i = 1; i < 15; i++)
 					pri_table[i] = resource[pos++];
 			}
-			break;
+				goto end_op_loop;
 
+			case PIC_SCI1_OPX_PRIORITY_TABLE_EQDIST:
+			{
+				int first = getInt16(resource + pos);
+				int last = getInt16(resource + pos + 2);
+				int nr;
+				int *pri_table;
 
-			default: sciprintf("%s L%d: Warning: Unknown opx %02x\n", __FILE__, __LINE__, op);
+				if (!pic->internal)
+				{
+					pic->internal = sci_malloc(16 * sizeof(int)); 
+				} else
+				{
+					GFXERROR("pic->internal is not NULL (%08x); possible memory corruption!\n", pic->internal); 
+				}	
+
+				pri_table = (int*)pic->internal;
+				
+				for (nr = 0; nr < 16; nr ++)
+					pri_table[nr] = SCI0_PRIORITY_BAND_FIRST_14_ZONES(nr);					
+				pos += 4;
+				goto end_op_loop;
+			}
+
+			default: sciprintf("%s L%d: Warning: Unknown opx %02x\n", __FILE__, __LINE__, opx);
 				return;
 			}
-			break;
+			goto end_op_loop;
 
 		case PIC_OP_TERMINATE:
 			p0printf("Terminator\n");
@@ -1779,11 +1898,71 @@ gfxr_draw_pic0(gfxr_pic_t *pic, int fill_normally, int default_palette, int size
 		default: GFXWARN("Unknown op %02x\n", op);
 			return;
 		}
+end_op_loop: 
+		{}
 	}
 
 	GFXWARN("Reached end of pic resource %04x\n", resid);
 }
 
+void
+gfxr_draw_pic11(gfxr_pic_t *pic, int flags, int default_palette, int size,
+	       byte *resource, gfxr_pic0_params_t *style, int resid,
+	       gfx_pixmap_color_t *static_pal, int static_pal_nr)
+{
+	int has_bitmap = getUInt16(resource + 4);
+	int vector_data_ptr = getUInt16(resource + 16);
+	int palette_data_ptr = getUInt16(resource + 28);
+	int bitmap_data_ptr = getUInt16(resource + 32);
+	int sci_titlebar_size = style->pic_port_bounds.y;
+	gfx_mode_t *mode;
+	gfx_pixmap_t *view = NULL;
+	/* Set up mode structure for resizing the view */
+	mode = gfx_new_mode(
+			    pic->visual_map->index_xl/320,
+			    pic->visual_map->index_yl/200, 
+			    1, /* 1bpp, which handles masks and the rest for us */
+			    0, 0, 0, 0, 0, 0, 0, 0, 16, 0);
+
+	pic->visual_map->colors = gfxr_read_pal11(-1, &(pic->visual_map->colors_nr), resource + palette_data_ptr, 1284);
+
+	if (has_bitmap)
+		view = gfxr_draw_cel11(-1, 0, 0, 0, resource, resource + bitmap_data_ptr, size - bitmap_data_ptr, NULL);
+
+	if (view)
+	{
+		view->colors = pic->visual_map->colors;
+		view->colors_nr = pic->visual_map->colors_nr;
+		
+		gfx_xlate_pixmap(view, mode, GFX_XLATE_FILTER_NONE);
+		
+		if (flags & DRAWPIC01_FLAG_OVERLAID_PIC)
+			view_transparentize(view, pic->visual_map->index_data, 
+					    0, 0, 
+					    view->index_xl, view->index_yl);
+		
+		/* Hack to prevent overflowing the visual map buffer.
+		   Yes, this does happen otherwise. */
+		if (view->index_yl + sci_titlebar_size > 200)
+			sci_titlebar_size = 0;
+
+		_gfx_crossblit_simple(pic->visual_map->index_data+sci_titlebar_size*view->index_xl,
+				      view->index_data,
+				      pic->visual_map->index_xl, view->index_xl,
+				      view->index_xl,
+				      view->index_yl,
+				      1);
+	} else 
+	{
+		GFXWARN("No view was contained in SCI1.1 pic resource");
+	}
+		
+
+
+	gfxr_draw_pic01(pic, flags, default_palette, size - vector_data_ptr,
+			resource + vector_data_ptr, style, resid, 1,
+			static_pal, static_pal_nr);
+}
 
 void
 gfxr_dither_pic0(gfxr_pic_t *pic, int dmode, int pattern)
