@@ -1,5 +1,5 @@
 /***************************************************************************
- sci_widgets.c Copyright (C) 2000,01,02 Christoph Reichenbach
+ sci_widgets.c Copyright (C) 2000 Christoph Reichenbach
 
 
  This program may be modified and copied freely according to the terms of
@@ -48,14 +48,14 @@ clear_titlebar(gfxw_port_t *titlebar)
 static gfxw_list_t *
 make_titlebar_list(state_t *s, rect_t bounds, gfxw_port_t *status_bar)
 {
-	gfx_color_t white = s->ega_colors[0xf];
+	gfx_color_t color = status_bar->bgcolor;
 	gfxw_list_t *list;
 	gfxw_box_t *bgbox;
 
 
 	list = gfxw_new_list(status_bar->bounds, 0);
 	bgbox = gfxw_new_box(s->gfx_state, gfx_rect(0, 0, status_bar->bounds.xl, status_bar->bounds.yl - 1),
-			     white, white, GFX_BOX_SHADE_FLAT);
+			     color, color, GFX_BOX_SHADE_FLAT);
 
 	list->add((gfxw_container_t *) list, (gfxw_widget_t *) bgbox);
 
@@ -68,7 +68,8 @@ finish_titlebar_list(state_t *s, gfxw_list_t *list, gfxw_port_t *status_bar)
 	gfx_color_t black = s->ega_colors[0];
 	gfxw_primitive_t *line;
 
-	line = gfxw_new_line(gfx_rect(0, status_bar->bounds.yl - 1, status_bar->bounds.xl, 0),
+	line = gfxw_new_line(gfx_point(0, status_bar->bounds.yl - 1),
+			     gfx_point(status_bar->bounds.xl, status_bar->bounds.yl - 1),
 			     black, GFX_LINE_MODE_CORRECT, GFX_LINE_STYLE_NORMAL);
 	list->add((gfxw_container_t *) list, (gfxw_widget_t *) line);
 
@@ -76,11 +77,12 @@ finish_titlebar_list(state_t *s, gfxw_list_t *list, gfxw_port_t *status_bar)
 }
 
 void
-sciw_set_status_bar(state_t *s, gfxw_port_t *status_bar, char *text)
+sciw_set_status_bar(state_t *s, gfxw_port_t *status_bar, char *text, int fgcolor, int bgcolor)
 {
 	gfx_state_t *state;
 	gfxw_list_t *list;
-	gfx_color_t white = s->ega_colors[0xf];
+	gfx_color_t bg = status_bar->bgcolor;
+	gfx_color_t fg = status_bar->color;
 	gfx_color_t black = s->ega_colors[0];
 
 	if (!status_bar->visual) {
@@ -100,7 +102,7 @@ sciw_set_status_bar(state_t *s, gfxw_port_t *status_bar, char *text)
 	if (text) {
 		gfxw_text_t *textw = gfxw_new_text(state, gfx_rect(0, 0, status_bar->bounds.xl, status_bar->bounds.yl),
 						   status_bar->font_nr, text, ALIGN_LEFT, ALIGN_CENTER,
-						   black, black, white, GFXR_FONT_FLAG_NO_NEWLINES);
+						   fg, fg, bg, GFXR_FONT_FLAG_NO_NEWLINES);
 
 		list = make_titlebar_list(s, status_bar->bounds, status_bar);
 
@@ -122,19 +124,63 @@ sciw_set_status_bar(state_t *s, gfxw_port_t *status_bar, char *text)
 	gfxop_update(state);
 }
 
+static void
+sciw_make_window_fit(rect_t *rect, gfxw_port_t *parent)
+{
+  /* This window is meant to cover the whole screen, so we allow it to go through. */
+  if (rect->xl == 319 && rect->yl == 189) return;
+
+  if (rect->x + rect->xl > parent->bounds.x + parent->bounds.xl)
+	  rect->x -= (rect->x + rect->xl) - (parent->bounds.x + parent->bounds.xl) + 2;
+
+  if (rect->y + rect->yl > parent->bounds.y + parent->bounds.yl)
+	  rect->y -= (rect->y + rect->yl) - (parent->bounds.y + parent->bounds.yl) + 2;
+}
 
 gfxw_port_t *
 sciw_new_window(state_t *s, rect_t area, int font, gfx_color_t color, gfx_color_t bgcolor,
 		int title_font, gfx_color_t title_color, gfx_color_t title_bgcolor,
-		char *title, int flags)
+		const char *title, int flags)
 {
 	gfxw_visual_t *visual = s->visual;
 	gfx_state_t *state = s->gfx_state;
 	int shadow_offset = 2;
 	rect_t frame;
 	gfx_color_t black	= {0};
-	gfxw_port_t *win = gfxw_new_port(visual, s->port, area, color, bgcolor);
+	gfxw_port_t *win;
 	gfxw_list_t *decorations;
+	int xextra = !(flags & WINDOW_FLAG_NOFRAME) ? 1 : 0;
+	int yextra = !(flags & WINDOW_FLAG_NOFRAME) ? 2 : 0;
+
+	if (area.xl == 319 && area.yl == 189)
+	{
+		flags |= WINDOW_FLAG_NOFRAME;
+		/* The below line makes the points bar in QfG2 work, but breaks
+		   the one in QfG1. Hm. */
+		if (bgcolor.priority == 255) /* Yep, QfG2 */
+			area.y += 3;
+	}
+
+	/*
+	if (area.y + area.yl > visual->bounds.y + visual->bounds.yl)
+	{
+		area.y -= (area.y + area.yl) - (visual->bounds.y + visual->bounds.yl) + yextra;
+	}
+
+	if (area.x + area.xl > visual->bounds.x + visual->bounds.xl)
+	{
+		area.x -= (area.x + area.xl) - (visual->bounds.x + visual->bounds.xl) + xextra;
+	}
+	*/
+
+	if (flags & WINDOW_FLAG_TITLE)
+		area. y += 10;
+
+	if (!(flags & (WINDOW_FLAG_TITLE | WINDOW_FLAG_NOFRAME)))
+		area.yl -= 1; /* Normal windows are drawn one pixel too small. */
+
+	sciw_make_window_fit(&area, s->wm_port);
+	win = gfxw_new_port(visual, s->wm_port, area, color, bgcolor);
 
 	win->font_nr = font;
 	win->title_text = title;
@@ -148,11 +194,10 @@ sciw_new_window(state_t *s, rect_t area, int font, gfx_color_t color, gfx_color_
 	if (flags == (WINDOW_FLAG_TRANSPARENT | WINDOW_FLAG_NOFRAME))
 		return win; /* Fully transparent window */
 
-
 	if (flags & WINDOW_FLAG_TITLE)
-		frame = gfx_rect(area.x -1, area.y -11, area.xl + 2, area.yl + 12);
+		frame = gfx_rect(area.x-1, area.y-10, area.xl + 2, area.yl + 11);
 	else
-		frame = gfx_rect(area.x -1, area.y -1, area.xl + 2, area.yl + 2);
+		frame = gfx_rect(area.x-1, area.y-1, area.xl + 2, area.yl + 2);
 
 	/* Set visible window boundaries */
 	win->bounds = gfx_rect(frame.x, frame.y, frame.xl + shadow_offset, frame.yl + shadow_offset);
@@ -163,7 +208,7 @@ sciw_new_window(state_t *s, rect_t area, int font, gfx_color_t color, gfx_color_
 	if (!(flags & WINDOW_FLAG_TRANSPARENT)) {
 		/* Draw window background */
 		win->port_bg = (gfxw_widget_t *) gfxw_new_box (state,
-							       gfx_rect(1, (flags & WINDOW_FLAG_TITLE)? 11 : 1,
+							       gfx_rect(1, (flags & WINDOW_FLAG_TITLE)? 10 : 1,
 									area.xl, area.yl),
 							       bgcolor, bgcolor, GFX_BOX_SHADE_FLAT);
 		decorations->add((gfxw_container_t *) decorations, win->port_bg);
@@ -172,7 +217,7 @@ sciw_new_window(state_t *s, rect_t area, int font, gfx_color_t color, gfx_color_
 
 	if (flags & WINDOW_FLAG_TITLE) {
 		/* Add window title */
-		rect_t title_rect = gfx_rect(1, 1, area.xl, 10);
+		rect_t title_rect = gfx_rect(1, 1, area.xl, 8);
 
 		decorations->add((gfxw_container_t *) decorations, (gfxw_widget_t *)
 				 gfxw_new_box(state, title_rect, title_bgcolor, title_bgcolor, GFX_BOX_SHADE_FLAT));
@@ -218,7 +263,8 @@ sciw_new_window(state_t *s, rect_t area, int font, gfx_color_t color, gfx_color_
 
 			if (flags & WINDOW_FLAG_TITLE)
 				decorations->add((gfxw_container_t *) decorations, (gfxw_widget_t *)
-						 gfxw_new_line(gfx_rect(1, 10, frame.xl - 3, 0),
+						 gfxw_new_line(gfx_point(1, 9),
+							       gfx_point(frame.xl - 2, 9),
 							       black, GFX_LINE_MODE_CORRECT, GFX_LINE_STYLE_NORMAL));
 		} else {
 			decorations->add((gfxw_container_t *) decorations, (gfxw_widget_t *)
@@ -282,52 +328,55 @@ _sciw_add_text_to_list(gfxw_list_t *list, gfxw_port_t *port, rect_t zone, char *
 }
 
 gfxw_list_t *
-sciw_new_button_control(gfxw_port_t *port, int ID, rect_t zone, char *text, int font, char selected, char inverse, char grayed_out)
+sciw_new_button_control(gfxw_port_t *port, reg_t ID, rect_t zone, char *text, int font, char selected, char inverse, char grayed_out)
 {
         gfx_color_t *frame_col = (inverse)? &(port->bgcolor) : &(port->color);
-	gfxw_list_t *list = gfxw_new_list(_move_and_extend_rect(zone, gfx_point(port->zone.x, port->zone.y), 1), 0);
+	gfxw_list_t *list;
 
-	gfxw_set_id(GFXW(list), ID);
+	zone.x--;
+	zone.y--;
+	zone.xl++;
+	zone.yl++;
+
+	list = gfxw_new_list(_move_and_extend_rect(zone, gfx_point(port->zone.x, port->zone.y), 1), 0);
+
+	gfxw_set_id(GFXW(list), ID.segment, ID.offset);
 
 	zone.x = 0;
 	zone.y = 0;
 
 	if (inverse)
-                list->add(GFXWC(list), GFXW(gfxw_new_box(NULL, gfx_rect(zone.x + 1, zone.y + 1, zone.xl - 2, zone.yl),
+                list->add(GFXWC(list), GFXW(gfxw_new_box(NULL, gfx_rect(zone.x, zone.y, zone.xl + 1, zone.yl + 1),
 							 port->color, port->color, GFX_BOX_SHADE_FLAT)));
 
 	if (!inverse)
-		list = _sciw_add_text_to_list(list, port, gfx_rect(zone.x, zone.y + 2, zone.xl, zone.yl),
+		list = _sciw_add_text_to_list(list, port, gfx_rect(zone.x + 1, zone.y + 2, zone.xl - 1, zone.yl),
 					      text, font, ALIGN_CENTER, 0, inverse, GFXR_FONT_FLAG_EAT_TRAILING_LF, grayed_out);
 
 	if (!inverse)
 		list->add(GFXWC(list),
 			  GFXW(gfxw_new_rect(zone, *frame_col, GFX_LINE_MODE_CORRECT, GFX_LINE_STYLE_NORMAL)));
 
+	if (inverse)
+		list = _sciw_add_text_to_list(list, port, gfx_rect(zone.x + 1, zone.y + 2, zone.xl - 1, zone.yl),
+					      text, font, ALIGN_CENTER, 0, inverse, GFXR_FONT_FLAG_EAT_TRAILING_LF, grayed_out);
+
         if (selected)
                 list->add(GFXWC(list),
                           GFXW(gfxw_new_rect(gfx_rect(zone.x + 1, zone.y + 1, zone.xl - 2, zone.yl - 2),
                                              *frame_col, GFX_LINE_MODE_CORRECT, GFX_LINE_STYLE_NORMAL)));
 
-	if (inverse)
-		list = _sciw_add_text_to_list(list, port, gfx_rect(zone.x, zone.y + 2, zone.xl, zone.yl),
-					      text, font, ALIGN_CENTER, 0, inverse, GFXR_FONT_FLAG_EAT_TRAILING_LF, grayed_out);
-
-	if (inverse)
-		list->add(GFXWC(list),
-			  GFXW(gfxw_new_rect(zone, *frame_col, GFX_LINE_MODE_CORRECT, GFX_LINE_STYLE_NORMAL)));
-
 	return list;
 }
 
 
 gfxw_list_t *
-sciw_new_text_control(gfxw_port_t *port, int ID, rect_t zone, char *text, int font,
+sciw_new_text_control(gfxw_port_t *port, reg_t ID, rect_t zone, char *text, int font,
 		      gfx_alignment_t align, char framed, char inverse)
 {
 	gfxw_list_t *list = gfxw_new_list(_move_and_extend_rect(zone, gfx_point(port->zone.x, port->zone.y), 2), 0);
 
-	gfxw_set_id(GFXW(list), ID);
+	gfxw_set_id(GFXW(list), ID.segment, ID.offset);
 
 	zone.x = 0;
 	zone.y = 0;
@@ -337,16 +386,24 @@ sciw_new_text_control(gfxw_port_t *port, int ID, rect_t zone, char *text, int fo
 
 
 gfxw_list_t *
-sciw_new_edit_control(gfxw_port_t *port, int ID, rect_t zone, char *text, int font, unsigned int cursor,
+sciw_new_edit_control(gfxw_port_t *port, reg_t ID, rect_t zone, char *text, int font, unsigned int cursor,
 		      char inverse)
 {
-	gfxw_list_t *list = gfxw_new_list(_move_and_extend_rect(zone, gfx_point(port->zone.x, port->zone.y), 1), 0);
 	gfxw_text_t *text_handle;
 	long draw_cursor;
 	long foo;
 
-	gfxw_set_id(GFXW(list), ID);
-	zone.x = 0;
+	gfxw_list_t *list;
+	int cursor_height = gfxop_get_font_height(port->visual->gfx_state, font);
+
+	zone.x--;
+	zone.y--;
+	zone.xl++;
+	zone.yl++;
+
+	list = gfxw_new_list(_move_and_extend_rect(zone, gfx_point(port->zone.x, port->zone.y), 1), 0);
+	gfxw_set_id(GFXW(list), ID.segment, ID.offset);
+	zone.x = 1;
 	zone.y = 1;
 
 	sci_gettime(&foo, &draw_cursor);
@@ -359,7 +416,7 @@ sciw_new_edit_control(gfxw_port_t *port, int ID, rect_t zone, char *text, int fo
 
 		list->add(GFXWC(list), GFXW(text_handle));
 	} else {
-		char *textdup = sci_malloc(strlen(text) + 1);
+		char *textdup = (char*)sci_malloc(strlen(text) + 1);
 
 		strncpy(textdup, text, cursor);
 
@@ -394,8 +451,9 @@ sciw_new_edit_control(gfxw_port_t *port, int ID, rect_t zone, char *text, int fo
 		};
 
 		if (cursor == strlen(text))
-			list->add(GFXWC(list), GFXW(gfxw_new_line(gfx_rect(zone.x + 1, zone.y, 0, zone.yl - 1),
-							  port->color, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL)));
+			list->add(GFXWC(list), GFXW(gfxw_new_line(gfx_point(zone.x, zone.y),
+								  gfx_point(zone.x, zone.y + cursor_height - 1),
+								  port->color, GFX_LINE_MODE_FAST, GFX_LINE_STYLE_NORMAL)));
 		free(textdup);
 	}
 
@@ -410,12 +468,12 @@ sciw_new_edit_control(gfxw_port_t *port, int ID, rect_t zone, char *text, int fo
 
 
 gfxw_list_t *
-sciw_new_icon_control(gfxw_port_t *port, int ID, rect_t zone, int view, int loop, int cel,
+sciw_new_icon_control(gfxw_port_t *port, reg_t ID, rect_t zone, int view, int loop, int cel,
 		      char frame, char inverse)
 {
 	gfxw_list_t *list = gfxw_new_list(_move_and_extend_rect(zone, gfx_point(port->zone.x, port->zone.y), 1), 0);
 	gfxw_widget_t *icon;
-	gfxw_set_id(GFXW(list), ID);
+	gfxw_set_id(GFXW(list), ID.segment, ID.offset);
 
 	if (!port->visual) {
 		GFXERROR("Attempting to create icon control for virtual port!\n");
@@ -425,7 +483,7 @@ sciw_new_icon_control(gfxw_port_t *port, int ID, rect_t zone, int view, int loop
 	zone.x = 0;
 	zone.y = 0;
 
-	icon = GFXW(gfxw_new_view(port->visual->gfx_state, gfx_point(zone.x, zone.y), view, loop, cel, -1, -1,
+	icon = GFXW(gfxw_new_view(port->visual->gfx_state, gfx_point(zone.x, zone.y), view, loop, cel, 0, -1, -1,
 				  ALIGN_LEFT, ALIGN_TOP, GFXW_VIEW_FLAG_DONT_MODIFY_OFFSET));
 
 	if (!icon) {
@@ -442,14 +500,26 @@ sciw_new_icon_control(gfxw_port_t *port, int ID, rect_t zone, int view, int loop
 
 
 gfxw_list_t *
-sciw_new_list_control(gfxw_port_t *port, int ID, rect_t zone, int font_nr, char **entries_list,
+sciw_new_list_control(gfxw_port_t *port, reg_t ID, rect_t zone, int font_nr, char **entries_list,
 		      int entries_nr, int list_top, int selection, char inverse)
 {
-	gfxw_list_t *list = gfxw_new_list(_move_and_extend_rect(zone, gfx_point(port->zone.x, port->zone.y), 1), 0);
+	gfxw_list_t *list;
+
 	char arr_up[2], arr_down[2];
 	int i;
-	int font_height = gfxop_get_font_height(port->visual->gfx_state, font_nr);
-	int columns = (zone.yl - 20);
+
+	int font_height;
+	int columns;
+
+	zone.x--;
+	zone.y--;
+	zone.xl++;
+	zone.yl++;
+
+	list = gfxw_new_list(_move_and_extend_rect(zone, gfx_point(port->zone.x, port->zone.y), 1), 0);
+
+	font_height = gfxop_get_font_height(port->visual->gfx_state, font_nr);
+	columns = (zone.yl - 20);
 
 	if (font_height <= 0) {
 		GFXERROR("Attempt to create list control with invalid font %d\n", font_nr);
@@ -459,7 +529,7 @@ sciw_new_list_control(gfxw_port_t *port, int ID, rect_t zone, int font_nr, char
 
 	columns /= font_height;
 
-	gfxw_set_id(GFXW(list), ID);
+	gfxw_set_id(GFXW(list), ID.segment, ID.offset);
 
 	arr_up[0] = SCI_SPECIAL_CHAR_ARROW_UP;
 	arr_down[0] = SCI_SPECIAL_CHAR_ARROW_DOWN;
@@ -494,6 +564,18 @@ sciw_new_list_control(gfxw_port_t *port, int ID, rect_t zone, int font_nr, char
 	zone.x = 0;
 	zone.y = 0;
 
+	/* Add up arrow */
+	list->add(GFXWC(list),
+		  GFXW(gfxw_new_text(port->visual->gfx_state, gfx_rect(1, 0, zone.xl-2, 8),
+				     port->font_nr, arr_up, ALIGN_CENTER, ALIGN_CENTER,
+				     port->color, port->color, port->bgcolor, 0)));
+
+	/* Add down arrow */
+	list->add(GFXWC(list),
+		  GFXW(gfxw_new_text(port->visual->gfx_state, gfx_rect(1, zone.yl-9, zone.xl-2, 8),
+				     port->font_nr, arr_down, ALIGN_CENTER, ALIGN_CENTER,
+				     port->color, port->color, port->bgcolor, 0)));
+
 	if (list_top & 1) { /* Hack to work around aggressive caching */
 
 		list->add(GFXWC(list),
@@ -513,18 +595,6 @@ sciw_new_list_control(gfxw_port_t *port, int ID, rect_t zone, int font_nr, char
 					     port->color, GFX_LINE_MODE_CORRECT, GFX_LINE_STYLE_NORMAL)));
 	}
 
-	/* Add up arrow */
-	list->add(GFXWC(list),
-		  GFXW(gfxw_new_text(port->visual->gfx_state, gfx_rect(1, 1, zone.xl-2, 8),
-				     port->font_nr, arr_up, ALIGN_CENTER, ALIGN_CENTER,
-				     port->color, port->color, port->bgcolor, 0)));
-
-	/* Add down arrow */
-	list->add(GFXWC(list),
-		  GFXW(gfxw_new_text(port->visual->gfx_state, gfx_rect(1, zone.yl-9, zone.xl-2, 8),
-				     port->font_nr, arr_down, ALIGN_CENTER, ALIGN_CENTER,
-				     port->color, port->color, port->bgcolor, 0)));
-
 	return list;
 }
 
@@ -582,7 +652,7 @@ sciw_new_menu(state_t *s, gfxw_port_t *status_bar, menubar_t *menubar, int selec
 		area.x += menubar->menus[i].title_width;
 
 	area.xl = menu->width - 1;
-	area.yl = menu->items_nr * 10 - 1;
+	area.yl = menu->items_nr * 10;
 
 	retval = sciw_new_window(s, area, status_bar->font_nr, status_bar->color, status_bar->bgcolor,
 				 0, status_bar->color, status_bar->bgcolor,
@@ -612,7 +682,7 @@ _make_menu_entry(menu_item_t *item, int offset, int width, gfxw_port_t *port, gf
 {
 	rect_t area		= gfx_rect(MENU_BOX_LEFT_PADDING, 0, width - MENU_BOX_LEFT_PADDING, 10);
 	rect_t list_area	= gfx_rect(port->zone.x, area.y + offset + port->zone.y, width, area.yl);
-	gfxw_list_t *list	= (gfxw_list_t *) gfxw_set_id(GFXW(gfxw_new_list(list_area, 0)), ID);
+	gfxw_list_t *list	= (gfxw_list_t *) gfxw_set_id(GFXW(gfxw_new_list(list_area, 0)), ID, GFXW_NO_ID);
 	gfx_color_t xcolor	= {0};
 
 	color = un_prioritize(color);
@@ -638,13 +708,15 @@ _make_menu_hbar(int offset, int width, gfxw_port_t *port, gfx_color_t color, gfx
 {
 	rect_t area = gfx_rect(0, 0, width, 10);
 	rect_t list_area = gfx_rect(area.x + port->zone.x, area.y + offset + port->zone.y, area.xl, area.yl);
-	gfxw_list_t *list = (gfxw_list_t *) gfxw_set_id(GFXW(gfxw_new_list(list_area, 0)), ID);
+	gfxw_list_t *list = (gfxw_list_t *) gfxw_set_id(GFXW(gfxw_new_list(list_area, 0)), ID, GFXW_NO_ID);
 
 	color = un_prioritize(color);
 	bgcolor = un_prioritize(bgcolor);
 
 	list->add(GFXWC(list), GFXW(gfxw_new_box(port->visual->gfx_state, area, bgcolor, bgcolor, GFX_BOX_SHADE_FLAT)));
-	list->add(GFXWC(list), GFXW(gfxw_new_line(gfx_rect(0, 5, width, 0), color,
+	list->add(GFXWC(list), GFXW(gfxw_new_line(gfx_point(0, 5),
+						  gfx_point(width, 5),
+						  color,
 						  GFX_LINE_MODE_FAST, GFX_LINE_STYLE_STIPPLED)));
 
 	return GFXW(list);
