@@ -1,5 +1,5 @@
 /***************************************************************************
- widgets.c Copyright (C) 2000,01,02 Christoph Reichenbach
+ widgets.c Copyright (C) 2000,01 Christoph Reichenbach
 
 
  This program may be modified and copied freely according to the terms of
@@ -36,6 +36,8 @@
 #  define DDIRTY if (0) fprintf
 #endif
 
+point_t gfxw_point_zero = {0, 0};
+
 #define MAX_SERIAL_NUMBER 0x7fffffff
 static int widget_serial_number_counter = 0x10000; /* Avoid confusion with IDs */
 
@@ -109,8 +111,14 @@ _gfxw_print_widget(gfxw_widget_t *widget, int indentation)
 
 	sciprintf("S%08x", widget->serial);
 
-	if (widget->ID != GFXW_NO_ID)
-		sciprintf("#%x ", widget->ID);
+	if (widget->ID != GFXW_NO_ID) {
+		sciprintf("#%x", widget->ID);
+
+		if (widget->subID != GFXW_NO_ID)
+			sciprintf(":%x ", widget->subID);
+		else
+			sciprintf(" ");
+	}
 
 	sciprintf("[(%d,%d)(%dx%d)]", widget->bounds.x, widget->bounds.y, widget->bounds.xl, widget->bounds.yl);
 
@@ -132,9 +140,9 @@ _gfxwop_print_empty(gfxw_widget_t *widget, int indentation)
 
 
 gfxw_widget_t *
-_gfxw_new_widget(int size, int type)
+_gfxw_new_widget(int size, gfxw_widget_type_t type)
 {
-	gfxw_widget_t *widget = sci_malloc(size);
+	gfxw_widget_t *widget = (gfxw_widget_t*)sci_malloc(size);
 #ifdef SATISFY_PURIFY
 	memset(widget, 0, size);
 #endif
@@ -147,6 +155,7 @@ _gfxw_new_widget(int size, int type)
 	widget->bounds = gfx_rect(0, 0, 0, 0);
 	widget->flags = GFXW_FLAG_DIRTY;
 	widget->ID = GFXW_NO_ID;
+	widget->subID = GFXW_NO_ID;
 	widget->serial = widget_serial_number_counter++;
 	widget->widget_priority = -1;
 
@@ -427,6 +436,15 @@ _move_rect(rect_t rect, point_t point)
 	return gfx_rect(rect.x + point.x, rect.y + point.y, rect.xl, rect.yl);
 }
 
+static inline void
+_split_rect(rect_t rect, point_t *p1, point_t *p2)
+{
+	p1->x = rect.x;
+	p1->y = rect.y;
+	p2->x = rect.x + rect.xl;
+	p2->y = rect.y + rect.yl;
+}
+
 static inline point_t
 _move_point(rect_t rect, point_t point)
 {
@@ -537,7 +555,8 @@ gfxw_new_box(gfx_state_t *state, rect_t area, gfx_color_t color1, gfx_color_t co
 
 
 static inline gfxw_primitive_t *
-_gfxw_new_primitive(rect_t area, gfx_color_t color, gfx_line_mode_t mode, gfx_line_style_t style, int type)
+_gfxw_new_primitive(rect_t area, gfx_color_t color, gfx_line_mode_t mode,
+		    gfx_line_style_t style, gfxw_widget_type_t type)
 {
 	gfxw_primitive_t *widget = (gfxw_primitive_t *) _gfxw_new_widget(sizeof(gfxw_primitive_t), type);
 
@@ -636,6 +655,7 @@ _gfxwop_line_draw(gfxw_widget_t *widget, point_t pos)
 {
 	gfxw_primitive_t *line = (gfxw_primitive_t *) widget;
 	rect_t linepos = widget->bounds;
+	point_t p1, p2;
 
 	linepos.xl--;
 	linepos.yl--;
@@ -648,7 +668,8 @@ _gfxwop_line_draw(gfxw_widget_t *widget, point_t pos)
 	}
 
 
-	GFX_ASSERT(gfxop_draw_line(line->visual->gfx_state, _move_rect(linepos, pos),
+	_split_rect(_move_rect(linepos, pos), &p1, &p2);
+	GFX_ASSERT(gfxop_draw_line(line->visual->gfx_state, p1, p2,
 				   line->color, line->line_mode, line->line_style));
 
 	return 0;
@@ -679,9 +700,12 @@ _gfxw_set_ops_LINE(gfxw_widget_t *prim)
 }
 
 gfxw_primitive_t *
-gfxw_new_line(rect_t line, gfx_color_t color, gfx_line_mode_t line_mode, gfx_line_style_t line_style)
+gfxw_new_line(point_t start, point_t end, gfx_color_t color, gfx_line_mode_t line_mode, gfx_line_style_t line_style)
 {
 	gfxw_primitive_t *prim;
+	/* Encode into internal representation */
+	rect_t line = gfx_rect (start.x, start.y, end.x - start.x, end.y - start.y);
+
 	byte inverse = 0;
 
 	if (line.xl < 0) {
@@ -713,8 +737,8 @@ gfxw_new_line(rect_t line, gfx_color_t color, gfx_line_mode_t line_mode, gfx_lin
 
 
 gfxw_view_t *
-_gfxw_new_simple_view(gfx_state_t *state, point_t pos, int view, int loop, int cel, int priority, int control,
-		      gfx_alignment_t halign, gfx_alignment_t valign, int size, int type)
+_gfxw_new_simple_view(gfx_state_t *state, point_t pos, int view, int loop, int cel, int palette, int priority, int control,
+		      gfx_alignment_t halign, gfx_alignment_t valign, int size, gfxw_widget_type_t type)
 {
 	gfxw_view_t *widget;
 	int width, height;
@@ -736,14 +760,15 @@ _gfxw_new_simple_view(gfx_state_t *state, point_t pos, int view, int loop, int c
 	widget->widget_priority = priority;
 	widget->pos = pos;
 	widget->color.mask =
-		(priority < 0)? 0 : GFX_MASK_PRIORITY
-		| (control < 0)? 0 : GFX_MASK_CONTROL;
+		((priority < 0)? 0 : GFX_MASK_PRIORITY)
+		| ((control < 0)? 0 : GFX_MASK_CONTROL);
 	widget->color.priority = priority;
 	widget->color.control = control;
 	widget->view = view;
 	widget->loop = loop;
 	widget->cel = cel;
-
+	widget->palette = palette;
+	
 	if (halign == ALIGN_CENTER)
 		widget->pos.x -= width >> 1;
 	else if (halign == ALIGN_RIGHT)
@@ -761,7 +786,7 @@ _gfxw_new_simple_view(gfx_state_t *state, point_t pos, int view, int loop, int c
 	return widget;
 }
 
-static int
+int
 _gfxwop_view_draw(gfxw_widget_t *widget, point_t pos)
 {
 	gfxw_view_t *view = (gfxw_view_t *) widget;
@@ -769,7 +794,7 @@ _gfxwop_view_draw(gfxw_widget_t *widget, point_t pos)
 
 	GFX_ASSERT(gfxop_draw_cel(view->visual->gfx_state, view->view, view->loop,
 				  view->cel, gfx_point(view->pos.x + pos.x, view->pos.y + pos.y),
-				  view->color));
+				  view->color, view->palette));
 
 	return 0;
 }
@@ -782,13 +807,13 @@ _gfxwop_static_view_draw(gfxw_widget_t *widget, point_t pos)
 
 	GFX_ASSERT(gfxop_draw_cel_static(view->visual->gfx_state, view->view, view->loop,
 					 view->cel, _move_point(view->bounds, pos),
-					 view->color));
+					 view->color, view->palette));
 
 	return 0;
 }
 
 static int
-_w_gfxwop_view_print(gfxw_widget_t *widget, char *name, int indentation)
+_w_gfxwop_view_print(gfxw_widget_t *widget, const char *name, int indentation)
 {
 	gfxw_view_t *view = (gfxw_view_t *) widget;
 	_gfxw_print_widget(widget, indentation);
@@ -826,7 +851,7 @@ _gfxw_set_ops_VIEW(gfxw_widget_t *view, char stat)
 }
 
 gfxw_view_t *
-gfxw_new_view(gfx_state_t *state, point_t pos, int view_nr, int loop, int cel, int priority, int control,
+gfxw_new_view(gfx_state_t *state, point_t pos, int view_nr, int loop, int cel, int palette, int priority, int control,
 	      gfx_alignment_t halign, gfx_alignment_t valign, int flags)
 {
 	gfxw_view_t *view;
@@ -839,10 +864,10 @@ gfxw_new_view(gfx_state_t *state, point_t pos, int view_nr, int loop, int cel, i
 		pos.y += offset.y;
 	}
 
-	view = _gfxw_new_simple_view(state, pos, view_nr, loop, cel, priority, control, halign, valign,
+	view = _gfxw_new_simple_view(state, pos, view_nr, loop, cel, palette, priority, control, halign, valign,
 				     sizeof(gfxw_view_t), (flags & GFXW_VIEW_FLAG_STATIC) ? GFXW_STATIC_VIEW : GFXW_VIEW);
 
-	_gfxw_set_ops_VIEW(GFXW(view), flags & GFXW_VIEW_FLAG_STATIC);
+	_gfxw_set_ops_VIEW(GFXW(view), (char)(flags & GFXW_VIEW_FLAG_STATIC));
 
 	return view;
 }
@@ -859,7 +884,7 @@ _gfxwop_dyn_view_draw(gfxw_widget_t *widget, point_t pos)
 
 	GFX_ASSERT(gfxop_draw_cel(view->visual->gfx_state, view->view, view->loop,
 				  view->cel, _move_point(view->draw_bounds, pos),
-				  view->color));
+				  view->color, view->palette));
 
 	/*
 	  gfx_color_t red; red.visual.r = 0xff; red.visual.g = red.visual.b = 0; red.mask = GFX_MASK_VISUAL;
@@ -891,14 +916,14 @@ _gfxwop_pic_view_draw(gfxw_widget_t *widget, point_t pos)
 						 view->view, view->loop,
 						 view->cel,
 						 _move_point(view->draw_bounds, pos),
-						 view->color));
+						 view->color, view->palette));
 
 	/* Draw again on the back buffer */
 	GFX_ASSERT(gfxop_draw_cel(view->visual->gfx_state,
 				  view->view, view->loop,
 				  view->cel,
 				  _move_point(view->draw_bounds, pos),
-				  view->color));
+				  view->color, view->palette));
 
 
 	widget->draw = _gfxwop_draw_nop; /* No more drawing needs to be done */
@@ -908,7 +933,7 @@ _gfxwop_pic_view_draw(gfxw_widget_t *widget, point_t pos)
 }
 
 static int
-_gfxwop_some_view_print(gfxw_widget_t *widget, int indentation, char *type_string)
+_gfxwop_some_view_print(gfxw_widget_t *widget, int indentation, const char *type_string)
 {
 	gfxw_dyn_view_t *view = (gfxw_dyn_view_t *) widget;
 
@@ -1012,7 +1037,7 @@ _gfxw_set_ops_PICVIEW(gfxw_widget_t *widget)
 }
 
 gfxw_dyn_view_t *
-gfxw_new_dyn_view(gfx_state_t *state, point_t pos, int z, int view, int loop, int cel, int priority, int control,
+gfxw_new_dyn_view(gfx_state_t *state, point_t pos, int z, int view, int loop, int cel, int palette, int priority, int control,
 		  gfx_alignment_t halign, gfx_alignment_t valign, int sequence)
 {
 	gfxw_dyn_view_t *widget;
@@ -1050,7 +1075,7 @@ gfxw_new_dyn_view(gfx_state_t *state, point_t pos, int z, int view, int loop, in
 	widget->cel = cel;
 	widget->sequence = sequence;
 	widget->force_precedence = 0;
-
+	widget->palette = palette;
 
 	if (halign == ALIGN_CENTER)
 		xalignmod = width >> 1;
@@ -1077,7 +1102,8 @@ gfxw_new_dyn_view(gfx_state_t *state, point_t pos, int z, int view, int loop, in
 
 	_gfxw_set_ops_DYNVIEW(GFXW(widget));
 
-	widget->signalp = widget->signal = 0;
+	widget->signalp = NULL;
+	widget->signal = 0;
 
 	return widget;
 }
@@ -1197,7 +1223,7 @@ _gfxw_set_ops_TEXT(gfxw_widget_t *widget)
 }
 
 gfxw_text_t *
-gfxw_new_text(gfx_state_t *state, rect_t area, int font, char *text, gfx_alignment_t halign,
+gfxw_new_text(gfx_state_t *state, rect_t area, int font, const char *text, gfx_alignment_t halign,
 	      gfx_alignment_t valign, gfx_color_t color1, gfx_color_t color2,
 	      gfx_color_t bgcolor, int text_flags)
 {
@@ -1206,7 +1232,7 @@ gfxw_new_text(gfx_state_t *state, rect_t area, int font, char *text, gfx_alignme
 
 	widget->widget_priority = _gfxw_color_get_priority(color1);
 	widget->font_nr = font;
-	widget->text = sci_malloc(strlen(text) + 1);
+	widget->text = (char*)sci_malloc(strlen(text) + 1);
 	widget->halign = halign;
 	widget->valign = valign;
 	widget->color1 = color1;
@@ -1217,7 +1243,10 @@ gfxw_new_text(gfx_state_t *state, rect_t area, int font, char *text, gfx_alignme
 
 	strcpy(widget->text, text);
 
-	gfxop_get_text_params(state, font, text, area.xl, &(widget->width), &(widget->height), text_flags);
+	gfxop_get_text_params(state, font, text, area.xl, &(widget->width),
+			      &(widget->height), text_flags,
+			      &(widget->lines_nr), &(widget->lineheight),
+			      &(widget->lastline_width));
 
 	/* FIXME: Window is too big
 	area.x += _calc_needmove(halign, area.xl, widget->width);
@@ -1239,6 +1268,17 @@ gfxw_new_text(gfx_state_t *state, rect_t area, int font, char *text, gfx_alignme
 }
 
 
+void
+gfxw_text_info(gfx_state_t *state, gfxw_text_t *text, int *lines,
+	       int *lineheight, int *offset)
+{
+	if (lines)
+		*lines = text->lines_nr;
+	if (lineheight)
+		*lineheight = text->lineheight;
+	if (offset)
+		*offset = text->lastline_width;
+}
 
 
 /***********************/
@@ -1277,7 +1317,7 @@ _gfxw_set_container_ops(gfxw_container_t *container, gfxw_point_op *draw, gfxw_o
 }
 
 static int
-_w_gfxwop_container_print_contents(char *name, gfxw_widget_t *widget, int indentation)
+_w_gfxwop_container_print_contents(const char *name, gfxw_widget_t *widget, int indentation)
 {
 	gfxw_widget_t *seeker = widget;
 
@@ -1326,10 +1366,10 @@ _w_gfxwop_container_print(gfxw_widget_t *widget, int indentation)
 
 
 gfxw_container_t *
-_gfxw_new_container_widget(rect_t area, int size, int type)
+_gfxw_new_container_widget(rect_t area, int size, gfxw_widget_type_t type)
 {
 	gfxw_container_t *widget = (gfxw_container_t *)
-		_gfxw_new_widget(size, type);
+	  _gfxw_new_widget(size, type);
 
 	widget->bounds = widget->zone = area;
 	widget->contents = NULL;
@@ -1497,18 +1537,18 @@ _gfxwop_container_set_visual(gfxw_widget_t *widget, gfxw_visual_t *visual)
 static int
 _gfxwop_container_free_tagged(gfxw_container_t *container)
 {
-        gfxw_widget_t *seekerp = (container->contents);
+	gfxw_widget_t *seekerp = (container->contents);
 
-        while (seekerp) {
-                gfxw_widget_t *redshirt = seekerp;
+	while (seekerp) {
+		gfxw_widget_t *redshirt = seekerp;
 
-                if (redshirt->flags & GFXW_FLAG_TAGGED) {
-                        seekerp = (redshirt->next);
-                        redshirt->widfree(redshirt); /* He's dead, Jim. */
-                } else
-                        seekerp = (seekerp)->next;
-        }
-        return 0;
+		if (redshirt->flags & GFXW_FLAG_TAGGED) {
+			seekerp = (redshirt->next);
+			redshirt->widfree(redshirt); /* He's dead, Jim. */
+		} else
+			seekerp = (seekerp)->next;
+	}
+	return 0;
 }
 
 static int
@@ -1566,7 +1606,9 @@ _gfxw_container_id_equals(gfxw_container_t *container, gfxw_widget_t *widget)
 	if (widget->ID == GFXW_NO_ID)
 		return 0;
 
-	while (*seekerp && (*seekerp)->ID != widget->ID)
+	while (*seekerp
+	       && ((*seekerp)->ID != widget->ID
+		   || (*seekerp)->subID != widget->subID))
 		seekerp = &((*seekerp)->next);
 
 	if (!*seekerp)
@@ -1649,7 +1691,7 @@ _gfxwop_sorted_list_draw(gfxw_widget_t *list, point_t pos)
 }
 
 static inline int
-_w_gfxwop_list_print(gfxw_widget_t *list, char *name, int indentation)
+_w_gfxwop_list_print(gfxw_widget_t *list, const char *name, int indentation)
 {
 	_gfxw_print_widget(list, indentation);
 	sciprintf(name);
@@ -1753,7 +1795,7 @@ _gfxwop_list_add_dirty(gfxw_container_t *container, rect_t dirty, int propagate)
 	return _gfxwop_container_add_dirty(container, dirty, propagate);
 }
 
-static inline int
+/* static inline */ int
 _gfxwop_ordered_add(gfxw_container_t *container, gfxw_widget_t *widget, int compare_all)
      /* O(n) */
 {
@@ -1831,7 +1873,7 @@ gfxw_new_list(rect_t area, int sorted)
 	gfxw_list_t *list = (gfxw_list_t *) _gfxw_new_container_widget(area, sizeof(gfxw_list_t),
 								       sorted? GFXW_SORTED_LIST : GFXW_LIST);
 
-	_gfxw_set_ops_LIST(GFXWC(list), sorted);
+	_gfxw_set_ops_LIST(GFXWC(list), (char)sorted);
 
 	return list;
 }
@@ -1846,7 +1888,6 @@ _gfxwop_visual_draw(gfxw_widget_t *widget, point_t pos)
 {
 	gfxw_visual_t *visual = (gfxw_visual_t *) widget;
 	gfx_dirty_rect_t *dirty = visual->dirty;
-	gfxw_widget_t *c_widget;
 	DRAW_ASSERT(widget, GFXW_VISUAL);
 
 	while (dirty) {
@@ -1958,7 +1999,7 @@ gfxw_new_visual(gfx_state_t *state, int font)
 	visual->font_nr = font;
 	visual->gfx_state = state;
 
-	visual->port_refs = sci_calloc(sizeof(gfxw_port_t), visual->port_refs_nr = 16);
+	visual->port_refs = (struct _gfxw_port**)sci_calloc(sizeof(gfxw_port_t), visual->port_refs_nr = 16);
 
 	_gfxw_set_ops_VISUAL(GFXWC(visual));
 
@@ -1976,7 +2017,7 @@ _visual_find_free_ID(gfxw_visual_t *visual)
 		id++;
 
 	if (id == visual->port_refs_nr) {/* Out of ports? */
-		visual->port_refs = sci_realloc(visual->port_refs, visual->port_refs_nr += newports);
+		visual->port_refs = (struct _gfxw_port**)sci_realloc(visual->port_refs, visual->port_refs_nr += newports);
 		memset(visual->port_refs + id, 0, newports * sizeof(gfxw_port_t *)); /* Clear new port refs */
 	}
 
@@ -2175,6 +2216,7 @@ gfxw_new_port(gfxw_visual_t *visual, gfxw_port_t *predecessor, rect_t area, gfx_
 	widget->bgcolor = bgcolor;
 	widget->font_nr = visual->font_nr;
 	widget->ID = _visual_find_free_ID(visual);
+	widget->chrono_port = 0;
 	visual->port_refs[widget->ID] = widget;
 
 	_gfxw_set_ops_PORT(GFXWC(widget));
@@ -2182,6 +2224,11 @@ gfxw_new_port(gfxw_visual_t *visual, gfxw_port_t *predecessor, rect_t area, gfx_
 	return widget;
 }
 
+void gfxw_port_auto_restore_background(gfxw_visual_t *visual, gfxw_port_t *window, rect_t auto_rect)
+{
+	window->port_flags |= WINDOW_FLAG_AUTO_RESTORE;
+	window->restore_snap = gfxw_make_snapshot(visual, auto_rect);
+}
 
 gfxw_port_t *
 gfxw_remove_port(gfxw_visual_t *visual, gfxw_port_t *port)
@@ -2196,6 +2243,9 @@ gfxw_remove_port(gfxw_visual_t *visual, gfxw_port_t *port)
 	}
 
 	parent = (gfxw_port_t *) port->parent;
+	if (port->port_flags & WINDOW_FLAG_AUTO_RESTORE)
+		gfxw_restore_snapshot(visual, port->restore_snap);
+
 	if (port->widfree(GFXW(port)))
 		return parent;
 
@@ -2235,16 +2285,19 @@ gfxw_find_default_port(gfxw_visual_t *visual)
 /*** - other functions - ***/
 
 gfxw_widget_t *
-gfxw_set_id(gfxw_widget_t *widget, int ID)
+gfxw_set_id(gfxw_widget_t *widget, int ID, int subID)
 {
-	if (widget)
+	if (widget) {
 		widget->ID = ID;
+		widget->subID = subID;
+	}
 
 	return widget;
 }
 
 gfxw_dyn_view_t *
-gfxw_dyn_view_set_params(gfxw_dyn_view_t *widget, int under_bits, int under_bitsp, int signal, int signalp)
+gfxw_dyn_view_set_params(gfxw_dyn_view_t *widget, int under_bits, void *under_bitsp,
+			 int signal, void *signalp)
 {
 	if (!widget)
 		return NULL;
@@ -2258,12 +2311,14 @@ gfxw_dyn_view_set_params(gfxw_dyn_view_t *widget, int under_bits, int under_bits
 }
 
 gfxw_widget_t *
-gfxw_remove_id(gfxw_container_t *container, int ID)
+gfxw_remove_id(gfxw_container_t *container, int ID, int subID)
 {
 	gfxw_widget_t **wp = &(container->contents);
 
 	while (*wp) {
-		if ((*wp)->ID == ID) {
+		if ((*wp)->ID == ID
+		    && (subID == GFXW_NO_ID
+			|| (*wp)->subID == subID)) {
 			gfxw_widget_t *widget = *wp;
 
 			*wp = (*wp)->next;
@@ -2311,12 +2366,16 @@ gfxw_show_widget(gfxw_widget_t *widget)
 gfxw_snapshot_t *
 gfxw_make_snapshot(gfxw_visual_t *visual, rect_t area)
 {
-	gfxw_snapshot_t *retval = sci_malloc(sizeof(gfxw_snapshot_t));
+	gfxw_snapshot_t *retval = (gfxw_snapshot_t*)sci_malloc(sizeof(gfxw_snapshot_t));
 
 	retval->serial = widget_serial_number_counter++;
 
 	retval->area = area;
 
+	/* Work around subset semantics in gfx_rect_subset.
+	   This fixes the help icon in LSL5. */
+	if (retval->area.xl == 320) retval->area.xl = 321;
+
 	return retval;
 }
 
@@ -2406,3 +2465,136 @@ gfxw_picviewize_dynview(gfxw_dyn_view_t *dynview)
 
 	return dynview;
 }
+
+/* Chrono-Ports (tm) */
+
+gfxw_port_t *
+gfxw_get_chrono_port(gfxw_visual_t *visual, gfxw_list_t **temp_widgets_list, int flags)
+{
+	gfxw_port_t *result = NULL;
+	gfx_color_t transparent = {0};
+	int id = 0;
+
+	if (!(flags & GFXW_CHRONO_NON_TOPMOST))
+	{
+		result = gfxw_find_default_port(visual);
+	} else
+	{
+		id = visual->port_refs_nr;
+		while (id >= 0 && (!visual->port_refs[id] ||
+				   !visual->port_refs[id]->chrono_port))
+			id--;
+
+		if (id >= 0)
+			result = visual->port_refs[id];
+	}
+		
+	if (!result || !result->chrono_port)
+	{
+		if (flags & GFXW_CHRONO_NO_CREATE) return NULL;
+		result = gfxw_new_port(visual, NULL, gfx_rect(0, 0, 320, 200), transparent, transparent);
+		*temp_widgets_list = gfxw_new_list(gfx_rect(0, 0, 320, 200), 1);
+		result->add(GFXWC(result), GFXW(*temp_widgets_list));
+		result->chrono_port = 1;
+		if (temp_widgets_list)
+			*temp_widgets_list = GFXWC(result->contents);
+		return result;
+	};
+
+	if (temp_widgets_list)
+		*temp_widgets_list = GFXWC(result->contents);
+	return result;
+}
+
+static int
+gfxw_check_chrono_overlaps(gfxw_port_t *chrono, gfxw_widget_t *widget)
+{
+	gfxw_widget_t *seeker = GFXWC(chrono->contents)->contents;
+
+	while (seeker) {
+		if (gfx_rect_equals(seeker->bounds, widget->bounds))
+		{
+			gfxw_annihilate(GFXW(seeker));
+			return 1;
+		}
+
+		seeker = seeker->next;
+	}
+
+	return 0;
+}
+
+void
+gfxw_add_to_chrono(gfxw_visual_t *visual, gfxw_widget_t *widget)
+{
+	gfxw_list_t *tw;
+	gfxw_port_t *chrono = 
+		gfxw_get_chrono_port(visual, &tw, 0);
+
+	gfxw_check_chrono_overlaps(chrono, widget);
+	chrono->add(GFXWC(chrono), widget);
+}
+
+static gfxw_widget_t *
+gfxw_widget_intersects_chrono(gfxw_list_t *tw, gfxw_widget_t *widget)
+{
+	gfxw_widget_t *seeker;
+
+	assert(tw->type == GFXW_SORTED_LIST);
+
+	seeker = tw->contents;
+	while (seeker)
+	{
+		point_t origin;
+		rect_t bounds = widget->bounds;
+
+		bounds = widget->bounds;
+		origin.x = seeker->parent->zone.x;
+		origin.y = seeker->parent->zone.y;
+		gfx_rect_translate(bounds, origin);
+
+		if (gfx_rects_overlap(bounds, seeker->bounds))
+			return seeker;
+
+		seeker = seeker->next;
+	}
+
+	return 0;
+}
+
+void
+gfxw_widget_reparent_chrono(gfxw_visual_t *visual, gfxw_widget_t *view, gfxw_list_t *target)
+{
+	gfxw_list_t *tw;
+	gfxw_port_t *chrono;
+	gfxw_widget_t *intersector;
+
+	chrono = gfxw_get_chrono_port(visual, &tw, GFXW_CHRONO_NO_CREATE);
+	if (chrono == NULL) return;
+
+	intersector = gfxw_widget_intersects_chrono(tw, view);
+	if (intersector)
+	{
+		point_t origin = gfx_point(intersector->parent->zone.x, 
+					   intersector->parent->zone.y);
+
+		gfxw_remove_widget_from_container(GFXWC(chrono), GFXW(tw));
+		gfxw_remove_widget_from_container(GFXWC(chrono->parent), GFXW(chrono));
+		gfxw_annihilate(GFXW(chrono));
+
+		gfx_rect_translate(tw->zone, origin);
+		target->add(GFXWC(target), GFXW(tw));
+	}
+}
+
+void
+gfxw_widget_kill_chrono(gfxw_visual_t *visual, int window)
+{
+	int i;
+
+	for (i=window; i < visual->port_refs_nr ; i++)
+	{
+		if (visual->port_refs[i] && visual->port_refs[i]->chrono_port)
+			gfxw_annihilate(GFXW(visual->port_refs[i]));
+	}
+}
