@@ -32,11 +32,8 @@
 
 #include <resource.h>
 #include <sciresource.h>
-#include <heap.h>
 #include <script.h>
 #include <vocabulary.h>
-#include <sound.h>
-#include <soundserver.h>
 #include <uinput.h>
 #include <console.h>
 #include <vm.h>
@@ -45,54 +42,35 @@
 #include <versions.h>
 #include <kernel.h>
 #include <gfx_state_internal.h>
+#include <seg_manager.h>
+#include <sfx_engine.h>
 
-#ifdef _MSC_VER
-#	ifndef VERSION
-#		ifdef NDEBUG
-#			define VERSION "0.3.5"
-#		else
-#			define VERSION __TIMESTAMP__
-#		endif
-#	endif
+#define FREESCI_CURRENT_SAVEGAME_VERSION 7
+#define FREESCI_MINIMUM_SAVEGAME_VERSION 7
+
+#ifdef WIN32
+#  define FREESCI_GAMEDIR "FreeSCI"
+#  define STRLEN_FREESCI_GAMEDIR 7
+#else
+#  define FREESCI_GAMEDIR ".freesci"
+#  define STRLEN_FREESCI_GAMEDIR 8
 #endif
 
-#define FREESCI_SAVEGAME_VERSION 5
-
-#ifndef FREESCI_GAMEDIR
-#define FREESCI_GAMEDIR ".freesci"
-#define STRLEN_FREESCI_GAMEDIR 8
-#endif
 #define FREESCI_CONFFILE "config"
 #define FREESCI_SAVEDIR_PREFIX "save_"
 #define FREESCI_CONFFILE_DOS "freesci.cfg"
-#define FREESCI_FILE_VISUAL_MAP "visual_map.png"
-#define FREESCI_FILE_PRIORITY_MAP "priority_map.png"
-#define FREESCI_FILE_CONTROL_MAP "control_map.png"
-#define FREESCI_FILE_AUXILIARY_MAP "auxiliary_map.png"
+#define FREESCI_GAMES_DIR "games"
 
 #define FREESCI_FILE_STATE "state"
-#define FREESCI_FILE_HEAP "heap"
 #define FREESCI_ID_SUFFIX ".id"
 /* Used for <gamename>.id files ("real" save games) */
 
-#ifndef MAX_GAMEDIR_SIZE
 #define MAX_GAMEDIR_SIZE 32 /* Used for subdirectory inside of "~/.freesci/" */
-#endif
 #define MAX_SAVEGAME_NR 20 /* Maximum number of savegames */
 
-#define MAX_HUNK_BLOCKS 256 /* Used for SCI "far memory"; only used for sci_memory in FreeSCI */
-
 #define MAX_SAVE_DIR_SIZE MAX_HOMEDIR_SIZE + STRLEN_FREESCI_GAMEDIR + MAX_GAMEDIR_SIZE + 4
 /* +4 for the three slashes and trailing \0 */
 
-typedef struct
-{
-	int size;
-	int type; /* See above */
-	char *data;
-} hunk_block_t; /* Used to store dynamically allocated "far" memory */
-
-
 /* values for state_t.restarting_flag */
 #define SCI_GAME_IS_NOT_RESTARTING 0
 #define SCI_GAME_WAS_RESTARTED 1
@@ -104,7 +82,6 @@ typedef struct {
 	int palette;
 } drawn_pic_t;
 
-
 typedef struct _state
 {
 	int savegame_version;
@@ -115,17 +92,18 @@ typedef struct _state
 	char *work_dir; /* Directory the game metadata should be written to */
 	resource_mgr_t *resmgr; /* The resource manager */
 
-	unsigned char *game_name; /* Designation of the primary object (which inherits from Game) */
+	char *game_name; /* Designation of the primary object (which inherits from Game) */
+	char *game_version;
 
 	/* Non-VM information */
 
 	gfx_state_t *gfx_state; /* Graphics state and driver */
 	gfx_pixmap_t *old_screen; /* Old screen content: Stored during kDrawPic() for kAnimate() */
 
-	sound_server_t *sound_server; /* Sound server */
-
+	sfx_state_t sound; /* sound subsystem */
+	int sfx_init_flags; /* flags the sfx subsystem was initialised with */
 	unsigned int sound_volume; /* 0x0 -> 0xf Current volume of sound system */
- 	unsigned int sound_mute; /* 0 = not, else == saved value */
+	unsigned int sound_mute; /* 0 = not, else == saved value */
 
 	byte restarting_flags; /* Flags used for restarting */
 	byte have_mouse_flag;  /* Do we have a hardware pointing device? */
@@ -139,18 +117,20 @@ typedef struct _state
 
 	char *status_bar_text; /* Text on the status bar, or NULL if the title bar is blank */
 
-        int status_bar_foreground, status_bar_background;
+	int status_bar_foreground, status_bar_background;
 
 	long game_time; /* Counted at 60 ticks per second, reset during start time */
 
-	heap_ptr save_dir; /* Pointer to the allocated space for the save directory */
-	heap_ptr save_dir_copy; /* Last copy of the save dir */
+	reg_t save_dir_copy; /* Last copy of the save dir */
 	int save_dir_edit_offset; /* For kEdit(): Display offset for editing the savedir */
 	char *save_dir_copy_buf; /* Temp savedir buffer for kEdit() */
 
-	heap_ptr sound_object; /* Some sort of object for sound management */
-
-	int mouse_pointer_nr; /* Mouse pointer resource, or -1 if disabled */
+	int mouse_pointer_view; /* Mouse pointer resource, or -1 if disabled */
+	int mouse_pointer_loop; /* Mouse pointer resource, or -1 if disabled */
+	int mouse_pointer_cel; /* Mouse pointer resource, or -1 if disabled */
+	int save_mouse_pointer_view; /* Temporary storage for mouse pointer resource, when the pointer is hidden */
+	int save_mouse_pointer_loop; /* Temporary storage for mouse pointer resource, when the pointer is hidden */
+	int save_mouse_pointer_cel; /* Temporary storage for mouse pointer resource, when the pointer is hidden */
 
 	int port_serial; /* Port serial number, for save/restore */
 	gfxw_port_t *port; /* The currently active port */
@@ -162,8 +142,9 @@ typedef struct _state
 	gfxw_port_t *titlebar_port; /* Title bar viewport (0,0,9,319) */
 	gfxw_port_t *wm_port; /* window manager viewport and designated &heap[0] view (10,0,199,319) */
 	gfxw_port_t *picture_port; /* The background picture viewport (10,0,199,319) */
+	gfxw_port_t *iconbar_port; /* Full-screen port used for non-clipped icon bar draw in SCI1 */
 
-	int pic_visible_map; /* The number of the map to display in update commands */
+	gfx_map_mask_t pic_visible_map; /* The number of the map to display in update commands */
 	int pic_animate; /* The animation used by Animate() to display the picture */
 
 	int dyn_views_list_serial; /* Used for save/restore */
@@ -175,8 +156,6 @@ typedef struct _state
 	long animation_delay; /* A delay factor for pic opening animations. Defaults to 500. */
 	int animation_granularity; /* Number of animation steps to perform betwen updates for transition animations */
 
-	hunk_block_t hunk[MAX_HUNK_BLOCKS]; /* Hunk memory */
-
 	menubar_t *menubar; /* The menu bar */
 
 	int priority_first; /* The line where priority zone 0 ends */
@@ -200,7 +179,7 @@ typedef struct _state
 	int file_handles_nr; /* maximum numer of allowed file handles */
 	FILE **file_handles; /* Array of file handles. Dynamically increased if required. */
 
-	heap_ptr dirseeker_outbuffer;
+	reg_t dirseeker_outbuffer;
 	sci_dir_t dirseeker;
 
 	/* VM Information */
@@ -217,24 +196,30 @@ typedef struct _state
 					   ** should be re-evaluated by the vm
 					   */
 
-	heap_t *_heap; /* The heap structure */
-	byte *heap; /* The actual heap data (equal to _heap->start) */
-	gint16 acc; /* Accumulator */
-	gint16 amp_rest; /* &rest register (only used for save games) */
-	gint16 prev; /* previous comparison result */
+	reg_t r_acc; /* Accumulator */
+	unsigned int r_amp_rest; /* &rest register (only used for save games) */
+	reg_t r_prev; /* previous comparison result */
 
-	heap_ptr stack_base;   /* The base position of the stack; used for debugging */
-	heap_ptr stack_handle; /* The stack's heap handle */
-	heap_ptr parser_base;  /* A heap area used by the parser for error reporting */
-	heap_ptr parser_event; /* The event passed to Parse() and later used by Said() */
-	heap_ptr global_vars;  /* script 000 selectors */
+	seg_id_t stack_segment; /* Heap area for the stack to use */
+	stack_ptr_t stack_base; /* Pointer to the least stack element */
+	stack_ptr_t stack_top; /* First invalid stack element */
+
+	seg_id_t parser_segment;  /* A heap area used by the parser for error reporting */
+	reg_t parser_base; /* Base address for the parser error reporting mechanism */
+	reg_t parser_event; /* The event passed to Parse() and later used by Said() */
+	seg_id_t script_000_segment;
+	script_t *script_000;  /* script 000, e.g. for globals */
 
 	int parser_lastmatch_word; /* Position of the input word the parser last matched on, or SAID_NO_MATCH */
 
 	/* Debugger data: */
 	breakpoint_t *bp_list;   /* List of breakpoints */
 	int have_bp;  /* Bit mask specifying which types of breakpoints are used in bp_list */
-	int debug_mode; /* Contains flags for the various debug modes */
+	unsigned int debug_mode; /* Contains flags for the various debug modes */
+
+	/* System strings */
+	seg_id_t sys_strings_segment;
+	sys_strings_t *sys_strings;
 
 	/* Parser data: */
 	word_t **parser_words;
@@ -251,19 +236,22 @@ typedef struct _state
 	synonym_t *synonyms; /* The list of synonyms */
 	int synonyms_nr;
 
-	heap_ptr game_obj; /* Pointer to the game object */
+	reg_t game_obj; /* Pointer to the game object */
 
 	int classtable_size; /* Number of classes in the table- for debugging */
 	class_t *classtable; /* Table of all classes */
-	script_t scripttable[1000]; /* Table of all scripts */
 
-	heap_ptr clone_list[SCRIPT_MAX_CLONES];
+	seg_manager_t seg_manager;
+	int gc_countdown; /* Number of kernel calls until next gc */
 
 	int selector_names_nr; /* Number of selector names */
 	char **selector_names; /* Zero-terminated selector name list */
 	int kernel_names_nr; /* Number of kernel function names */
 	char **kernel_names; /* List of kernel names */
-	kfunct **kfunct_table; /* Table of kernel functions */
+
+	kfunct_sig_pair_t *kfunct_table; /* Table of kernel functions */
+	int kfunct_nr; /* Number of mapped kernel functions; may be more than
+		       ** kernel_names_nr  */
 
 	opcode *opcodes;
 
@@ -276,6 +264,7 @@ typedef struct _state
 
 } state_t;
 
+
 #define STATE_T_DEFINED
 
 int
@@ -294,9 +283,26 @@ gamestate_restore(state_t *s, char *dirname);
 ** Returns   : (state_t *) NULL on failure, a pointer to a valid state_t otherwise
 */
 
+gfx_pixmap_color_t *
+get_pic_color(state_t *s, int color);
+/* Retrieves the gfx_pixmap_color_t associated with a game color index
+** Parameters: (state_t *) s: The game state
+**             (int) color: The color to look up
+** Returns   : (gfx_pixmap_color_t *) The requested color.
+*/
+
 void
-other_libs_exit();
+other_libs_exit(void);
 /* Called directly before FreeSCI ends to allow libraries to clean up
 */
 
+static inline
+reg_t not_register(state_t *s, reg_t r)
+{
+	if (s->version >= SCI_VERSION_FTU_INVERSE_CANBEHERE)
+		return make_reg(0, !r.offset); else
+		return r;
+
+}
+
 #endif /* !_SCI_ENGINE_H */
