@@ -44,7 +44,6 @@ typedef enum {
 #define GFX_CAPABILITY_COLOR_MOUSE_POINTER (1<<2)
 #define GFX_CAPABILITY_PIXMAP_REGISTRY (1<<3)
 #define GFX_CAPABILITY_SCALEABLE_PIXMAPS (1<<4)
-#define GFX_CAPABILITY_PIXMAP_GRABBING (1<<5)
 #define GFX_CAPABILITY_STIPPLED_LINES (1<<6)
 #define GFX_CAPABILITY_MOUSE_SUPPORT (1<<7)
 #define GFX_CAPABILITY_POINTER_PIXMAP_REGISTRY (1<<8)
@@ -86,11 +85,11 @@ typedef enum {
 
 typedef struct _gfx_driver { /* Graphics driver */
 
-	char *name; /* Graphics driver name. Unique identifier, should consist of
-	            ** lower-case (where applicable) alphanumerics
-	            */
+	const char *name; /* Graphics driver name. Unique identifier, should consist of
+	                   ** lower-case (where applicable) alphanumerics
+	                   */
 
-	char *version; /* Free-form version description (for informative purposes
+	const char *version; /* Free-form version description (for informative purposes
 		           ** only)
 		           */
 
@@ -112,11 +111,9 @@ typedef struct _gfx_driver { /* Graphics driver */
 	** GFX_CAPABILITY_SHADING: draw_filled_rect() supports drawing shaded
 	**    rectangles.
 	** GFX_CAPABILITY_MOUSE_POINTER: The driver has built-in support for mouse
-	**    pointers (monochrome or colored). If not set, GFX_CAPABILITY_PIXMAP_
-	**    GRABBING must be supported.
+	**    pointers (monochrome or colored).
 	** GFX_CAPABILITY_COLOR_MOUSE_POINTER: The driver has built-in support for
-	**    colored mouse pointers. If not available, GFX_CAPABILITY_PIXMAP_
-	**    GRABBING has to be supported.
+	**    colored mouse pointers.
 	** GFX_CAPABILITY_PIXMAP_REGISTRY: System provides a pixmap registry. The
 	**    invoking functions will assume that all pixmaps MUST be registered;
 	**    if this flag is not set, it assumes that pixmaps MUST NOT be
@@ -129,10 +126,6 @@ typedef struct _gfx_driver { /* Graphics driver */
 	** GFX_CAPABILITY_SCALEABLE_PIXMAPS: Pixmap scaling is fully supported.
 	**    If this capability is flag is set, all pixmaps passed to the driver
 	**    will be unscaled.
-	** GFX_CAPABILITY_PIXMAP_GRABBING: The driver supports grabbing images from
-	**    the back buffer. This option MUST be available if no colored
-	**    mouse pointer support (or no mouse pointer support at all) are
-	**    available.
 	** GFX_CAPABILITY_STIPPLED_LINES: The driver is able to draw stippled lines
 	**    horizontally and vertically (xl = 0 or yl = 0).
 	** GFX_CAPABILITY_MOUSE_SUPPORT: There is some support for mouse (or similar)
@@ -143,9 +136,14 @@ typedef struct _gfx_driver { /* Graphics driver */
 	**    fine (width 1) lines
 	** GFX_CAPABILITY_WINDOWED: Driver runs in a window and supports a debug
 	**    console running on stdin/stdout
+	** GFX_CAPABILITY_KEYTRANSLATE: The driver's input layer automatically
+	**    handles 'shifted' keys (i.e. turning shift-'a' to 'A' etc.).
+	**    Drivers only need to handle this if they desire to support
+	**    non-US keyboard layouts, usually by localisation methods
+	**    provided by the underlying windowing or operating system.
 	*/
 
-	int debug_flags; /* Driver debug flags */
+	unsigned int debug_flags; /* Driver debug flags */
 
 
 	/*** Initialization ***/
@@ -208,11 +206,14 @@ typedef struct _gfx_driver { /* Graphics driver */
 
 	/*** Drawing operations ***/
 
-	int (*draw_line) (struct _gfx_driver *drv, rect_t line, gfx_color_t color,
-		    gfx_line_mode_t line_mode, gfx_line_style_t line_style);
+	int (*draw_line) (struct _gfx_driver *drv,
+			  point_t start, point_t end,
+			  gfx_color_t color,
+			  gfx_line_mode_t line_mode, gfx_line_style_t line_style);
 	/* Draws a single line to the back buffer.
 	** Parameters: (gfx_driver_t *) drv: The driver affected
-	**             (rect_t *) line: The line to draw
+	**             (point_t) start: Starting point of the line to draw
+	**             (point_t) end: End point of the line to draw
 	**             (gfx_color_t *) color: The color to draw with
 	**             (int) line_mode: Any of the line modes
 	**             (int) line_style: Any of the line styles
@@ -291,7 +292,7 @@ typedef struct _gfx_driver { /* Graphics driver */
 	** Returns   : (int) GFX_OK, GFX_FATAL, or GFX_ERROR for invalid map values
 	** pxm may be assumed to be empty and pre-allocated with an appropriate
 	** memory size.
-	** This function is optional if GFX_CAPABILITY_PIXMAP_GRABBING is not set.
+	** This function is now mandatory.
 	*/
 
 
@@ -314,7 +315,7 @@ typedef struct _gfx_driver { /* Graphics driver */
 	*/
 
 	int (*set_static_buffer) (struct _gfx_driver *drv, gfx_pixmap_t *pic,
-			    gfx_pixmap_t *priority);
+				  gfx_pixmap_t *priority);
 	/* Sets the contents of the static visual and priority buffers
 	** Parameters: (gfx_driver_t *) drv: The affected driver
 	**             (gfx_pixmap_t *) pic: The image defining the new content
@@ -404,7 +405,7 @@ gfx_find_driver(char *, char *name);
 **                              was found
 */
 
-char *
+const char *
 gfx_get_driver_name(int nr);
 /* Retreives the name of the driver with the specified number
 ** Parameters: (int) nr: Number of the driver
