@@ -110,7 +110,7 @@ typedef struct {
 
 	gfx_pixmap_t *priority_map; /* back buffer priority map (unscaled) */
 	gfx_pixmap_t *static_priority_map; /* static buffer priority map (unscaled) */
-	gfx_pixmap_t *control_map; /* back buffer control map */
+	gfx_pixmap_t *control_map; /* back buffer control map (only exists unscaled in the first place) */
 
 
 	int mouse_pointer_visible; /* Whether the pointer is drawn right now */
@@ -215,6 +215,10 @@ gfxop_set_visible_map(gfx_state_t *state, gfx_map_mask_t map);
 ** Parameters: (gfx_state_t *) state: The state to modify
 **             (gfx_map_mask_t) map: The GFX_MASK to set
 ** Returns   : (int) GFX_OK, or GFX_ERROR if map was invalid
+** 'visible_map' can be any of GFX_MASK_VISUAL, GFX_MASK_PRIORITY and GFX_MASK_CONTROL; the appropriate
+** map (as far as its contents are known to the graphics subsystem) is then subsequently drawn to the
+** screen at each update. If this is set to anything other than GFX_MASK_VISUAL, slow full-screen updates
+** are performed. Mostly useful for debugging.
 ** The screen needs to be updated for the changes to take effect.
 */
 
@@ -239,11 +243,14 @@ gfxop_have_mouse(gfx_state_t *state);
 /******************************/
 
 int
-gfxop_draw_line(gfx_state_t *state, rect_t line, gfx_color_t color, gfx_line_mode_t line_mode,
+gfxop_draw_line(gfx_state_t *state,
+		point_t start, point_t end,
+		gfx_color_t color, gfx_line_mode_t line_mode,
 		gfx_line_style_t line_style);
 /* Renders a clipped line to the back buffer
 ** Parameters: (gfx_state_t *) state: The state affected
-**             (rect_t) line: The line position and parameters
+**             (point_t) start: Starting point of the line
+**	       (point_t) end: End point of the line
 **             (gfx_color_t) color: The color to use for drawing
 **             (gfx_line_mode_t) line_mode: Any valid line mode to use
 **             (gfx_line_style_t) line_style: The line style to use
@@ -411,12 +418,13 @@ gfxop_set_pointer_cursor(gfx_state_t *state, int nr);
 */
 
 int
-gfxop_set_pointer_view(gfx_state_t *state, int nr, int loop, int cel);
+gfxop_set_pointer_view(gfx_state_t *state, int nr, int loop, int cel, point_t *hotspot);
 /* Sets the mouse pointer to a view resource
 ** Parameters: (gfx_state_t *) state: The affected state
 **             (int) nr: Number of the view resource to use
 **             (int) loop: View loop to use
 **             (int) cel: View cel to use
+**             (point_t *) hotspot: Manually set hotspot to use, or NULL for default.
 ** Returns   : (int) GFX_OK or GFX_FATAL
 ** Use gfxop_set_pointer_cursor(state, GFXOP_NO_POINTER) to disable the
 ** pointer.
@@ -510,7 +518,7 @@ gfxop_get_cel_parameters(gfx_state_t *state, int nr, int loop, int cel,
 
 int
 gfxop_draw_cel(gfx_state_t *state, int nr, int loop, int cel, point_t pos,
-	       gfx_color_t color);
+	       gfx_color_t color, int palette);
 /* Draws (part of) a cel to the back buffer
 ** Parameters: (gfx_state_t *) state: The state encapsulating the driver to draw with
 **             (int) nr: Number of the view to draw
@@ -518,13 +526,14 @@ gfxop_draw_cel(gfx_state_t *state, int nr, int loop, int cel, point_t pos,
 **             (int) cel: The cel number of the cel to draw
 **             (point_t) pos: The positino the cel is to be drawn to
 **             (gfx_color_t color): The priority and control values to use for drawing
+**	       (int) palette: The palette to use
 ** Returns   : (int) GFX_OK or GFX_FATAL
 */
 
 
 int
 gfxop_draw_cel_static(gfx_state_t *state, int nr, int loop, int cel, point_t pos,
-		      gfx_color_t color);
+		      gfx_color_t color, int palette);
 /* Draws a cel to the static buffer; no clipping is performed
 ** Parameters: (gfx_state_t *) state: The state encapsulating the driver to draw with
 **             (int) nr: Number of the view to draw
@@ -532,6 +541,7 @@ gfxop_draw_cel_static(gfx_state_t *state, int nr, int loop, int cel, point_t pos
 **             (int) cel: The cel number of the cel to draw
 **             (point_t) pos: The positino the cel is to be drawn to
 **             (gfx_color_t color): The priority and control values to use for drawing
+**	       (int) palette: The palette to use
 ** Returns   : (int) GFX_OK or GFX_FATAL
 ** Let me repeat, no clipping (except for the display borders) is performed.
 */
@@ -539,7 +549,7 @@ gfxop_draw_cel_static(gfx_state_t *state, int nr, int loop, int cel, point_t pos
 
 int
 gfxop_draw_cel_static_clipped(gfx_state_t *state, int nr, int loop, int cel, point_t pos,
-			      gfx_color_t color);
+			      gfx_color_t color, int palette);
 /* Draws (part of) a clipped cel to the static buffer
 ** Parameters: (gfx_state_t *) state: The state encapsulating the driver to draw with
 **             (int) nr: Number of the view to draw
@@ -547,6 +557,7 @@ gfxop_draw_cel_static_clipped(gfx_state_t *state, int nr, int loop, int cel, poi
 **             (int) cel: The cel number of the cel to draw
 **             (point_t) pos: The positino the cel is to be drawn to
 **             (gfx_color_t color): The priority and control values to use for drawing
+**	       (int) palette: The palette to use
 ** Returns   : (int) GFX_OK or GFX_FATAL
 ** This function does clip.
 */
@@ -605,19 +616,22 @@ gfxop_get_font_height(gfx_state_t *state, int font_nr);
 */
 
 int
-gfxop_get_text_params(gfx_state_t *state, int font_nr, char *text,
-		      int maxwidth, int *width, int *height, int flags);
+gfxop_get_text_params(gfx_state_t *state, int font_nr, const char *text,
+		      int maxwidth, int *width, int *height, int flags,
+		      int *lines_nr, int *lineheight, int *lastline_width);
 /* Calculates the width and height of a specified text in a specified font
 ** Parameters: (gfx_state_t *) state: The state to use
 **             (int) font_nr: Font number to use for the calculation
-**             (char *) text: The text to examine
-**             (int) maxwidth: The maximum pixel width to allow for the text
-**             (int *) width: The variable the actual width will be stored in
-**                            afterwards
-**             (int *) height: The variable the resulting height will be stored
-**                             in
+**             (const char *) text: The text to examine
 **             (int) flags: ORred GFXR_FONT_FLAGs
+**             (int) maxwidth: The maximum pixel width to allow for the text
 ** Returns   : (int) GFX_OK or GFX_ERROR if the font didn't exist
+**             (int) *width: The resulting width
+**             (int) *height: The resulting height
+**             (int) *lines_nr: Number of lines used in the text
+**             (int) *lineheight: Pixel height (SCI scale) of each text line
+**             (int) *lastline_wdith: Pixel offset (SCI scale) of the space
+**                   after the last character in the last line
 */
 
 gfx_text_handle_t *
