@@ -140,7 +140,7 @@ typedef struct { /* gfx_color_t: Full color */
 
 typedef struct { /* rect_t: Rectangle description */
 	int x, y;
-	int xl, yl; /* width-1, height-1: (x,y,xl,yl)=(5,5,1,1) occupies 4 pixels */
+	int xl, yl; /* width, height: (x,y,xl,yl)=(5,5,1,1) occupies 1 pixel */
 } rect_t;
 
 
@@ -192,9 +192,8 @@ else SUBMERGE_PARTIAL(b, a, z, zl)
 }
 
 
-/* This is defined in Win32 and should be renamed if it still needs to be used
+
 #define RECT(a) a.x, a.y, a.xl, a.yl
-*/
 
 /* Merges two rects
 ** Parameters: (rect_t x rect_t) a,b: The two rects to merge
@@ -239,7 +238,8 @@ gfx_rect_equals(rect_t a, rect_t b)
 }
 
 
-static rect_t gfx_rect_fullscreen = {0, 0, 320, 200};
+/* gfx_rect_fullscreen is declared in gfx/gfx_tools.c */
+extern rect_t gfx_rect_fullscreen;
 
 /*** points ***/
 
@@ -247,6 +247,7 @@ typedef struct {
 	int x, y;
 } point_t;
 
+#define GFX_PRINT_POINT(p) (p).x, (p).y
 
 /* Generates a point_t from index data
 ** Parameters: (int x int) x,y: Indicated point
@@ -278,9 +279,8 @@ gfx_rect_translate(rect_t rect, point_t offset)
 }
 
 #define GFX_RESID_NONE -1
-#define GFX_COLOR_INDEX_TRANSPARENT 0xff
 
-#define GFX_PIC_COLORS 0x100
+#define GFX_PIC_COLORS 256
 
 #define GFX_PIXMAP_FLAG_SCALED_INDEX      (1<<0) /* Index data is scaled already */
 #define GFX_PIXMAP_FLAG_EXTERNAL_PALETTE  (1<<1) /* The colors pointer points to an external palette */
@@ -288,6 +288,9 @@ gfx_rect_translate(rect_t rect, point_t offset)
 #define GFX_PIXMAP_FLAG_PALETTE_ALLOCATED (1<<3) /* Palette has been allocated */
 #define GFX_PIXMAP_FLAG_PALETTE_SET       (1<<4) /* Palette has been propagated to the driver */
 #define GFX_PIXMAP_FLAG_DONT_UNALLOCATE_PALETTE (1<<5) /* Used by text, which uses preallocated colors */
+#define GFX_PIXMAP_FLAG_PALETTIZED	  (1<<6) /* Indicates a palettized view */
+
+#define GFX_PIXMAP_COLOR_KEY_NONE -1 /* No transpacency colour key */
 
 typedef struct { /* gfx_pixmap_t: Pixel map */
 
@@ -326,6 +329,8 @@ typedef struct { /* gfx_pixmap_t: Pixel map */
 			 ** graphics mode's alpha_mask is zero.
 			 */
 
+	int color_key;
+
 	/*** Data reserved for gfx_driver use ***/
 	struct pixmap_internal { /* Internal state management data for use by drivers */
 		int handle; /* initialized to 0 */
@@ -334,6 +339,7 @@ typedef struct { /* gfx_pixmap_t: Pixel map */
 
 } gfx_pixmap_t;
 
+
 #define GFX_FONT_BUILTIN_5x8  -1
 #define GFX_FONT_BUILTIN_6x10 -2
 
@@ -382,6 +388,7 @@ enum gfx_return_value_t {
 
 
 typedef enum {/* Map masks */
+	GFX_MASK_NONE = 0,
 	GFX_MASK_VISUAL = 1,
 	GFX_MASK_PRIORITY = 2,
 	GFX_MASK_CONTROL = 4
