@@ -38,6 +38,15 @@
 /* Our strategy for dirty rectangle management */
 #define GFXW_DIRTY_STRATEGY GFXOP_DIRTY_FRAMES_CLUSTERS
 
+/* Properly belongs in sci_widgets.h, but we need it here */
+#define WINDOW_FLAG_AUTO_RESTORE 0x2000000
+
+/* Indicates that a Chrono-Port should not be created even if it doesn't exist. */
+#define GFXW_CHRONO_NO_CREATE 1
+
+/* Indicates that non-topmost ports should be scanned for a Chrono-Port. */
+#define GFXW_CHRONO_NON_TOPMOST 2
+
 /* Terminology
 **
 ** Two special terms are used in here: /equivalent/ and /clear/. Their meanings
@@ -70,7 +79,8 @@
 #define GFXWC(foo) ((gfxw_container_t *) foo)
 /* Typecasts a container widget to gfxw_container_widget_t *. */
 
-static point_t gfxw_point_zero = {0, 0};
+/* gfxw_point_zero is declared in gfx/widgets.c */
+extern point_t gfxw_point_zero;
 
 /*********************/
 /* Widget operations */
@@ -252,9 +262,9 @@ gfxw_new_rect(rect_t rect, gfx_color_t color, gfx_line_mode_t line_mode, gfx_lin
 */
 
 gfxw_primitive_t *
-gfxw_new_line(rect_t line, gfx_color_t color, gfx_line_mode_t line_mode, gfx_line_style_t line_style);
+gfxw_new_line(point_t start, point_t end, gfx_color_t color, gfx_line_mode_t line_mode, gfx_line_style_t line_style);
 /* Creates a new line
-** Parameters: (rect_t) line: The line origin and relative destination coordinates
+** Parameters: (point_t * point_t) (start, line): The line origin and end point
 **             (gfx_color_t) color: The line's color
 **             (gfx_line_mode_t) line_mode: The line mode to use for drawing
 **             (gfx_line_style_t) line_style: The line style
@@ -269,7 +279,7 @@ gfxw_new_line(rect_t line, gfx_color_t color, gfx_line_mode_t line_mode, gfx_lin
 #define GFXW_VIEW_FLAG_DONT_MODIFY_OFFSET (1 << 1)
 
 gfxw_view_t *
-gfxw_new_view(gfx_state_t *state, point_t pos, int view, int loop, int cel, int priority, int control,
+gfxw_new_view(gfx_state_t *state, point_t pos, int view, int loop, int cel, int palette, int priority, int control,
 	      gfx_alignment_t halign, gfx_alignment_t valign, int flags);
 /* Creates a new view (a cel, actually)
 ** Parameters: (gfx_state_t *) state: The graphics state
@@ -284,7 +294,7 @@ gfxw_new_view(gfx_state_t *state, point_t pos, int view, int loop, int cel, int
 */
 
 gfxw_dyn_view_t *
-gfxw_new_dyn_view(gfx_state_t *state, point_t pos, int z, int view, int loop, int cel,
+gfxw_new_dyn_view(gfx_state_t *state, point_t pos, int z, int view, int loop, int cel, int palette,
 		  int priority, int control, gfx_alignment_t halign, gfx_alignment_t valign,
 		  int sequence);
 /* Creates a new dyn view
@@ -304,7 +314,7 @@ gfxw_new_dyn_view(gfx_state_t *state, point_t pos, int z, int view, int loop, in
 */
 
 gfxw_text_t *
-gfxw_new_text(gfx_state_t *state, rect_t area, int font, char *text, gfx_alignment_t halign,
+gfxw_new_text(gfx_state_t *state, rect_t area, int font, const char *text, gfx_alignment_t halign,
 	      gfx_alignment_t valign, gfx_color_t color1, gfx_color_t color2,
 	      gfx_color_t bgcolor, int flags);
 /* Creates a new text widget
@@ -321,11 +331,23 @@ gfxw_new_text(gfx_state_t *state, rect_t area, int font, char *text, gfx_alignme
 ** Returns   : (gfxw_text_t *) The resulting text widget
 */
 
+void
+gfxw_text_info(gfx_state_t *state, gfxw_text_t *text, int *lines_nr,
+	       int *lineheight, int *offset);
+/* Determines text widget meta-information
+** Parameters: (gfx_state_t *) state: The state to operate on
+**             (gfx_text_t *) text: The widget to query
+** Returns   : (int) lines_nr: Number of lines used in the text
+**             (int) lineheight: Pixel height (SCI scale) of each text line
+**             (int) offset: Pixel offset (SCI scale) of the space after the
+**                           last character in the last line
+*/
+
 gfxw_widget_t *
-gfxw_set_id(gfxw_widget_t *widget, int ID);
+gfxw_set_id(gfxw_widget_t *widget, int ID, int subID);
 /* Sets a widget's ID
 ** Parmaeters: (gfxw_widget_t *) widget: The widget whose ID should be set
-**             (int) ID: The ID to set
+**             (int x int) ID, subID: The ID to set
 ** Returns   : (gfxw_widget_t *) widget
 ** A widget ID is unique within the container it is stored in, if and only if it was
 ** added to that container with gfxw_add().
@@ -333,20 +355,21 @@ gfxw_set_id(gfxw_widget_t *widget, int ID);
 */
 
 gfxw_widget_t *
-gfxw_remove_id(gfxw_container_t *container, int ID);
+gfxw_remove_id(gfxw_container_t *container, int ID, int subID);
 /* Finds a widget with a specific ID in a container and removes it from there
 ** Parameters: (gfxw_container_t *) container: The container to search in
 **             (int) ID: The ID to look for
+**             (int) subID: The subID to look for, or GFXW_NO_ID for any
 ** Returns   : (gfxw_widget_t *) The resulting widget or NULL if no match was found
 ** Search is non-recursive; widgets with IDs hidden in subcontainers will not be found.
 */
 
 
 gfxw_dyn_view_t *
-gfxw_dyn_view_set_params(gfxw_dyn_view_t *widget, int under_bits, int under_bitsp, int signal, int signalp);
+gfxw_dyn_view_set_params(gfxw_dyn_view_t *widget, int under_bits, void *under_bitsp, int signal, void *signalp);
 /* Initializes a dyn view's interpreter attributes
 ** Parameters: (gfxw_dyn_view_t *) widget: The widget affected
-**             (int x int x int x int) under_bits, inder_bitsp, signal, signalp: Interpreter-dependant data
+**             (int x void * x int x void *) under_bits, inder_bitsp, signal, signalp: Interpreter-dependant data
 ** Returns   : (gfxw_dyn_view_t *) widget
 */
 
@@ -429,6 +452,15 @@ gfxw_find_default_port(gfxw_visual_t *visual);
 ** or highest-ranking port.
 */
 
+void
+gfxw_port_set_auto_restore(gfxw_visual_t *visual, gfxw_port_t *window, rect_t auto_rect);
+/* Sets rectangle to be restored upon port removal
+** Parameters: (state_t *) s: The state to operate on
+               (gfxw_port_t *) window: The affected window
+**             (rect_t) auto_rect: The area to restore
+** Returns   : (void)
+*/
+
 gfxw_port_t *
 gfxw_remove_port(gfxw_visual_t *visual, gfxw_port_t *port);
 /* Removes a port from a visual
@@ -491,4 +523,26 @@ gfxw_picviewize_dynview(gfxw_dyn_view_t *dynview);
 ** The only changes are in function and type variables, actually.
 */
 
+void
+gfxw_port_auto_restore_background(gfxw_visual_t *visual, gfxw_port_t *window, rect_t auto_rect);
+/* Tags a window widget as automatically restoring the visual background upon removal
+** Parameters: (gfx_visual_t *) visual: The base visual
+**             (gfxw_port_t *) window: The window to tag
+**             (rect_t) auto_rect: The background to remember
+** Also records the specified background rectangle, for later recovery
+*/
+
+
+gfxw_port_t *
+gfxw_get_chrono_port(gfxw_visual_t *visual, gfxw_list_t **temp_widgets_list, int flags);
+
+void
+gfxw_add_to_chrono(gfxw_visual_t *visual, gfxw_widget_t *widget);
+
+void
+gfxw_widget_reparent_chrono(gfxw_visual_t *visual, gfxw_widget_t *view, gfxw_list_t *target);
+
+void
+gfxw_widget_kill_chrono(gfxw_visual_t *visual, int window);
+
 #endif /* !_GFX_WIDGETS_H_ */
