@@ -30,10 +30,7 @@
 #define _SCI_KDEBUG_H_
 
 struct _state;
-#define SCIk_DEBUG_MODES 17
-
-extern const char *SCIk_Debug_Names[SCIk_DEBUG_MODES];
-/* The various debug areas */
+#define SCIk_DEBUG_MODES 18
 
 #define SCIkERROR_NR -2
 #define SCIkWARNING_NR -1
@@ -43,6 +40,7 @@ extern const char *SCIk_Debug_Names[SCIk_DEBUG_MODES];
 #define SCIkGFXDRIVER_NR 8
 #define SCIkBASESETTER_NR 9
 #define SCIkPARSER_NR 10
+#define SCIkAVOIDPATH_NR 17
 
 #define SCIkERROR      s, __FILE__, __LINE__, SCIkERROR_NR
 #define SCIkWARNING    s, __FILE__, __LINE__, SCIkWARNING_NR
@@ -62,6 +60,8 @@ extern const char *SCIk_Debug_Names[SCIk_DEBUG_MODES];
 #define SCIkFILE       s, __FILE__, __LINE__, 13
 #define SCIkTIME       s, __FILE__, __LINE__, 14
 #define SCIkROOM       s, __FILE__, __LINE__, 15
+#define SCIkEMU	       s, __FILE__, __LINE__, 16
+#define SCIkAVOIDPATH  s, __FILE__, __LINE__, SCIkAVOIDPATH_NR
 
 #define SCI_KERNEL_DEBUG
 
@@ -109,7 +109,7 @@ _SCIGNUkdebug(const char *funcname, struct _state *s, const char *file, int line
 */
 
 void
-set_debug_mode (struct _state *s, int mode, char *areas);
+set_debug_mode (struct _state *s, int mode, const char *areas);
 
 extern int sci_debug_flags;
 
