@@ -41,12 +41,7 @@
  ** -- Christoph Reichenbach
  **/
 
-#ifdef _DREAMCAST
-#  define SCI_INVALID_FD 0
-#else
-#  define SCI_INVALID_FD -1
-#endif
-
+#define SCI_INVALID_FD -1
 #define IS_VALID_FD(a) ((a) != SCI_INVALID_FD) /* Tests validity of a file descriptor */
 
 #ifdef _WIN32
@@ -98,36 +93,11 @@
 
 #ifdef _WIN32
 #  include <io.h>
-#  undef inline /* just to be sure it is not defined */
-#  define inline __inline
-#  define strcasecmp stricmp
-#  define strncasecmp strnicmp
-#  define vsnprintf _vsnprintf
+#  include <sci_win32.h>
 #else /* !_WIN32 */
 #  define DLLEXTERN
 #endif /* !_WIN32 */
 
-#ifdef _DREAMCAST
-#  define strcasecmp stricmp
-#  define open fs_open
-#  define read fs_read
-#  define close fs_close
-#  define chdir fs_chdir
-#  define lseek fs_seek
-#  define perror(S) fprintf(stderr, S)
-#  define unlink fs_unlink
-#  define write fs_write
-#  define mkdir(D,M) fs_mkdir(D)
-#  define creat(D,M) fs_open(D, O_WRONLY)
-#  define putchar(C) fputc(C, stdout)
-#  define getc(S) fgetc(S)
-#  define sleep(S) thd_sleep((S)*1000)
-#endif
-
-#ifdef _GP32
-#  include "wrap.h"
-#endif
-
 #ifdef __BEOS__
 #  include <kernel/OS.h>
 #  define usleep snooze
@@ -135,14 +105,15 @@
 
 #ifdef _MSC_VER
 #	include <sys/timeb.h>
+#   include <fcntl.h>
 #	include <windows.h>
 #endif
 
 
-#include <errno.h>
 #ifdef HAVE_FCNTL_H
 #  include <fcntl.h>
 #endif
+#include <errno.h>
 #include <sys/types.h>
 #ifdef HAVE_SYS_STAT_H
 #  include <sys/stat.h>
@@ -166,31 +137,14 @@
 #  define DLLEXTERN
 #endif
 
-#if _MSC_VER || _DOS || defined(_GP32)
+#if _MSC_VER || _DOS
 #  define G_DIR_SEPARATOR_S "\\"
 #else
 #  define G_DIR_SEPARATOR_S "/"
 #endif
 
-#if defined(__MORPHOS__)
-#  define G_DIR_CURRENT_S ""
-#  define G_DIR_PARENT_S "/"
-#elif defined(_DREAMCAST)
-#  define G_DIR_CURRENT_S fs_getwd()
-#  define G_DIR_PARENT_S fs_getwd() /* We should never change to parent dir. */
-#else
-#  define G_DIR_CURRENT_S "."
-#  define G_DIR_PARENT_S ".."
-#endif
-
-#if defined (ARM_WINCE)
-#  undef HAVE_FORK
-#endif
-
-#if defined(__MORPHOS__) || defined(_DREAMCAST) || defined(_MSC_VER) || defined(ARM_WINCE) || defined(_WIN32) || defined(_GP32)
-#  ifndef PATH_MAX
-#    define PATH_MAX 255
-#  endif
+#if defined(__MORPHOS__) || defined(_MSC_VER) || defined(ARM_WINCE) || defined(__amigaos4__)
+#  define PATH_MAX 255
 #endif
 
 #ifndef MIN
@@ -200,6 +154,7 @@
 #  define MAX(a, b)  (((a) > (b)) ? (a) : (b))
 #endif /* HP-UX defines both */
 
+
 #define GUINT16_SWAP_LE_BE_CONSTANT(val) ((((val) & 0x00ff) << 8) | (((val) & 0xff00) >> 8))
 
 #define GUINT32_SWAP_LE_BE_CONSTANT(val)  ( \
@@ -212,15 +167,6 @@
 /* The maximum allowed size for a compressed or decompressed resource */
 
 
-typedef gint8 sbyte;
-typedef guint8 byte;
-typedef guint16 word;
-
-
-#include <sys/types.h>
-#ifdef HAVE_SYS_STAT_H
-#  include <sys/stat.h>
-#endif
 
 #define MAX_HOMEDIR_SIZE 255
 
@@ -242,7 +188,7 @@ typedef guint16 word;
 
 /**** FUNCTION DECLARATIONS ****/
 
-#ifdef _MSC_VER
+#ifdef _WIN32
 #    define scimkdir(arg1,arg2) mkdir(arg1)
 #else
 #    define scimkdir(arg1,arg2) mkdir(arg1,arg2)
@@ -265,6 +211,16 @@ getInt16(byte *d)
 ** getUInt16 returns the int unsigned.
 */
 
+static inline void
+putInt16(byte* dest, int src)
+{
+	dest[0]=(byte)src&0xff;
+	dest[1]=(byte)(src>>8)&0xff;
+}
+/* Converse of getInt16()
+** Parameters: (byte *) dest: The position to write to
+**             (int) src: value to write
+*/
 
 #ifndef HAVE_ISBLANK
 static inline int
@@ -274,6 +230,11 @@ isblank(int foo)
 }
 #endif
 
+#ifdef _cplusplus
+#  define delete _freesci_cplusplus_workaround_delete
+#  define new _freesci_cplusplus_workaround_new
+#endif
+
 #define SCI_MEMTEST memtest(__FILE__, __LINE__)
 
 /*-- queues --*/
@@ -318,9 +279,9 @@ sci_get_from_queue(sci_queue_t *queue, int *type);
 /* --- */
 
 int
-memtest(char *file, int line);
+memtest(const char *file, int line);
 /* Allocates, manipulates, and frees some memory
-** Parameters: (char *) file: The file name to print when starting the
+** Parameters: (const char *) file: The file name to print when starting the
 **				tests
 **             (int) line: The line number to claim it was executed on
 ** Returns   : (int) 0
@@ -360,10 +321,10 @@ sci_init_dir(sci_dir_t *dirent);
 */
 
 char *
-sci_find_first(sci_dir_t *dirent, char *mask);
+sci_find_first(sci_dir_t *dirent, const char *mask);
 /* Finds the first file matching the specified file mask
 ** Parameters: (sci_dir_t *) dirent: Pointer to an unused dirent structure
-**             (char *) mask: File mask to apply
+**             (const char *) mask: File mask to apply
 ** Returns   : (char *) Name of the first matching file found, or NULL
 */
 
@@ -387,18 +348,18 @@ sci_finish_find(sci_dir_t *dirent);
 */
 
 FILE *
-sci_fopen(char *fname, char *mode);
+sci_fopen(const char *fname, const char *mode);
 /* Opens a FILE* case-insensitively
-** Parameters: (char *) fname: Name of the file to open
-**             (char *) mode: Mode to open it with
+** Parameters: (const char *) fname: Name of the file to open
+**             (const char *) mode: Mode to open it with
 ** Returns   : (FILE *) A valid file handle, or NULL on failure
 ** Always refers to the cwd, cannot address subdirectories
 */
 
 int
-sci_open(char *fname, int flags);
+sci_open(const char *fname, int flags);
 /* Opens a file descriptor case-insensitively
-** Parameters: (char *) fname: Name of the file to open
+** Parameters: (const char *) fname: Name of the file to open
 **             (int) flags: open(2) flags for the file
 ** Returns   : (int) a file descriptor of the open file,
 **             or SCI_INVALID_FD on failure
@@ -417,7 +378,7 @@ sciprintf(const char *fmt, ...);
 */
 
 char *
-sci_getcwd();
+sci_getcwd(void);
 /* Returns the current working directory, malloc'd.
 ** Parameters: (void)
 ** Returns   : (char *) a malloc'd cwd, or NULL if it couldn't be determined.
@@ -425,7 +386,7 @@ sci_getcwd();
 
 
 char *
-sci_get_homedir();
+sci_get_homedir(void);
 /* Returns the user's home directory
 ** Parameters: (void)
 ** Returns   : (char *) Pointer to a static buffer containing the user's home,
@@ -433,9 +394,9 @@ sci_get_homedir();
 */
 
 int
-sci_mkpath(char *path);
+sci_mkpath(const char *path);
 /* Asserts that the specified path is available
-** Parameters: (char *) path: Path to verify/create
+** Parameters: (const char *) path: Path to verify/create
 ** Returns   : (int) 0 on success, <0 on error
 ** This function will create any directories that couldn't be found
 */
@@ -445,21 +406,35 @@ sci_fd_size(int fd);
 /* Returns the filesize of an open file
 ** Paramaters: (int) fd: File descriptor of open file
 ** Returns   : (int) filesize of file pointed to by fd, -1 on error
-*/
+*/ 
 
 int
-sci_file_size(char *fname);
+sci_file_size(const char *fname);
 /* Returns the filesize of a file
-** Parameters: (char *) fname: Name of file to get filesize of
+** Parameters: (const char *) fname: Name of file to get filesize of
 ** Returns   : (int) filesize of the file, -1 on error
 */
 
-int
-sci_create_file(char *fname);
-/* Creates a writeable file
-** Parameters: (char *) fname: The name of the file to create
-** Returns   : (int) A file handle on success, SCI_INVALID_FD on failure
-*/
+/* Simple heuristic to work around array handling peculiarity in SQ4:
+It uses StrAt() to read the individual elements, so we must determine
+whether a string is really a string or an array. */
+int is_print_str(char *str);
+
+#ifdef HAVE_UNLINK
+#  define sci_unlink unlink
+#else  /* !HAVE_UNLINK */
+#  error "Please provide an int sci_unlink(const char *) for removing filesystem entries"
+#endif /* !HAVE_UNLINK */
+
+#ifdef HAVE_RMDIR
+#  define sci_rmdir rmdir
+#else  /* !HAVE_RMDIR */
+#  ifdef WIN32
+#    define sci_rmdir _rmdir
+#  else
+#    error "Please provide an int sci_rmdir(const char *) for removing directories"
+#  endif
+#endif /* !HAVE_RMDIR */
 
 #ifndef HAVE_FFS
 int sci_ffs(int _mask);
@@ -469,7 +444,7 @@ int sci_ffs(int _mask);
 
 
 void
-sci_sched_yield();
+sci_sched_yield(void);
 /* Yields the running process/thread to the scheduler
 ** Parameters: (void)
 ** Returns   : after a while.
@@ -496,10 +471,15 @@ sci_sched_yield();
 #  endif /* !__i386__ && !__alpha__ */
 #endif
 #ifndef BREAKPOINT
-#  define BREAKPOINT() { fprintf(stderr, "Missed breakpoint in %s, line %d\n", __FILE__, __LINE__); *((int *)NULL) = 42; }
+#  define BREAKPOINT() { fprintf(stderr, "Missed breakpoint in %s, line %d\n", __FILE__, __LINE__); *((int *) NULL) = 42; }
 #endif  /* !BREAKPOINT() */
 
 #define WARNING(foo) {char i; i = 500;}
 
 #endif
 
+
+
+
+
+
