@@ -31,12 +31,8 @@
 
 #include <versions.h>
 #include <gfx_options.h>
-#include <soundserver.h>
-#include <midiout.h>
-#include <midi_device.h>
-#include <pcmout.h>
 
-#define FREESCI_DRIVER_SUBSYSTEMS_NR 3
+#define FREESCI_DRIVER_SUBSYSTEMS_NR 1
 
 #define FREESCI_DRIVER_SUBSYSTEM_GFX 0
 #define FREESCI_DRIVER_SUBSYSTEM_PCM 1
@@ -44,8 +40,6 @@
 
 #ifdef _WIN32
 #  define SCI_DEFAULT_MODULE_PATH "."
-#elif defined(__MORPHOS__)
-#  define SCI_DEFAULT_MODULE_PATH "PROGDIR:Modules"
 #else
 #  define SCI_DEFAULT_MODULE_PATH "/usr/local/lib/freesci/:/usr/lib/freesci/"
 #endif
@@ -85,42 +79,29 @@ typedef struct {
 	char *resource_dir; /* Resource directory */
 	char *gfx_driver_name; /* The graphics driver to use */
 	char *console_log; /* The file to which console output should be echoed */
+	char *menu_dir; /* Directory where the game menu searches for games */
 	char debug_mode [80]; /* Characters specifying areas for which debug output should be enabled */
 	int mouse; /* Whether the mouse should be active */
 	int reverse_stereo;
-
+	int res_version;
+	
+#ifdef __GNUC__
+#  warning "Re-enable sound stuff"
+#endif
+#if 0
 	midiout_driver_t *midiout_driver; /* the midiout method to use */
 	midi_device_t *midi_device; /* the midi device to use */
-
+	
 	pcmout_driver_t *pcmout_driver; /* the pcm driver to use */
 	sound_server_t *sound_server; /* The sound server */
-	int pcmout_rate;  /* Sample rate */
-	int pcmout_stereo;  /* Stereo? */
-	int pcmout_buffer_size; /* Buffer size (in frames) */
+#endif
+        guint16 pcmout_rate;  /* Sample rate */
+        guint8 pcmout_stereo;  /* Stereo? */
 
 	char *module_path; /* path to directories modules are loaded from */
-
+	void *dummy; /* This is sad... */
 } config_entry_t;
 
-
-/* Command line options: Used to store options passed through the
-** command line, overriding  other configuration  */
-typedef struct {
-	int script_debug_flag;
-	int scale_x, scale_y, color_depth;
-	int mouse;
-	int show_rooms;
-	sci_version_t version;
-	char *gfx_driver_name;
-	char *gamedir;
-	char *midiout_driver_name;
-	char *midi_device_name;
-	char *sound_server_name;
-	char *pcmout_driver_name;
-} cl_options_t;
-
-
-
 int
 config_init(config_entry_t **conf, char *conffil);
 /* Initializes the config entry structurre based on information found in the config file.
@@ -178,11 +159,11 @@ parse_sound_server(char *driver_name);
 */
 
 driver_option_t *
-get_driver_options(config_entry_t *config, int subsystem, char *name);
+get_driver_options(config_entry_t *config, int subsystem, const char *name);
 /* Retreives the driver options for one specific driver in a subsystem
 ** Parameters: (config_entry_t *) config: The config entry to search in
 **             (int) subsystem: Any of the FREESCI_DRIVER_SUBSYSTEMs
-**             (char *) name: Name of the driver to look for
+**             (const char *) name: Name of the driver to look for
 ** Returns   : (driver_option_t *) A pointer to the first option in
 **             a singly-linked list of options, or NULL if none was
 **             found
