@@ -35,13 +35,12 @@
 /*#define _SCI_RESOURCE_DEBUG */
 /*#define _SCI_DECOMPRESS_DEBUG*/
 
-#define findResource "Error: Using findResource!"
-
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
 #include <resource.h>
+#include <versions.h>
 
 #define SCI_MAX_RESOURCE_SIZE 0x0400000
 /* The maximum allowed size for a compressed or decompressed resource */
@@ -89,10 +88,12 @@
 #define SCI_VERSION_AUTODETECT 0
 #define SCI_VERSION_0 1
 #define SCI_VERSION_01 2
-#define SCI_VERSION_1_EARLY 3
-#define SCI_VERSION_1_LATE 4
-#define SCI_VERSION_1_1 5
-#define SCI_VERSION_32 6
+#define SCI_VERSION_01_VGA 3
+#define SCI_VERSION_01_VGA_ODD 4
+#define SCI_VERSION_1_EARLY 5
+#define SCI_VERSION_1_LATE 6
+#define SCI_VERSION_1_1 7
+#define SCI_VERSION_32 8
 #define SCI_VERSION_LAST SCI_VERSION_1_LATE /* The last supported SCI version */
 
 #define SCI_VERSION_1 SCI_VERSION_1_EARLY
@@ -113,6 +114,7 @@ enum ResourceTypes {
 };
 
 #define sci0_last_resource sci_patch
+#define sci1_last_resource sci_heap
 /* Used for autodetection */
 
 
@@ -238,8 +240,8 @@ scir_free_resource_manager(resource_mgr_t *mgr);
 /**--- Resource map decoding functions ---*/
 
 int
-sci0_read_resource_map(char *path, resource_t **resources, int *resource_nr_p);
-/* Reads the resource.map file from a local directory
+sci0_read_resource_map(char *path, resource_t **resources, int *resource_nr_p, int *sci_version);
+/* Reads the SCI0 resource.map file from a local directory
 ** Parameters: (char *) path: (unused)
 **             (resource_t **) resources: Pointer to a pointer
 **                                        that will be set to the
@@ -247,6 +249,21 @@ sci0_read_resource_map(char *path, resource_t **resources, int *resource_nr_p);
 **                                        (in one large chunk)
 **             (int *) resource_nr_p: Pointer to an int the number of resources
 **                                    read is stored in
+**             (int) sci_version: SCI resource version
+** Returns   : (int) 0 on success, an SCI_ERROR_* code otherwise
+*/
+
+int
+sci1_read_resource_map(char *path, resource_t **resources, int *resource_nr_p, int *sci_version);
+/* Reads the SCI1 resource.map file from a local directory
+** Parameters: (char *) path: (unused)
+**             (resource_t **) resources: Pointer to a pointer
+**                                        that will be set to the
+**                                        location of the resources
+**                                        (in one large chunk)
+**             (int *) resource_nr_p: Pointer to an int the number of resources
+**                                    read is stored in
+**             (int) sci_version: SCI resource version
 ** Returns   : (int) 0 on success, an SCI_ERROR_* code otherwise
 */
 
@@ -261,6 +278,15 @@ sci0_sprintf_patch_file_name(char *string, resource_t *res);
 ** Returns   : (void)
 */
 
+void
+sci1_sprintf_patch_file_name(char *string, resource_t *res);
+/* Prints the name of a matching patch to a string buffer
+** Parameters: (char *) string: The buffer to print to
+**             (resource_t *) res: Resource containing the number and type of the
+**                                 resource whose name is to be print
+** Returns   : (void)
+*/
+
 int
 sci0_read_resource_patches(char *path, resource_t **resources, int *resource_nr_p);
 /* Reads SCI0 patch files from a local directory
@@ -274,45 +300,55 @@ sci0_read_resource_patches(char *path, resource_t **resources, int *resource_nr_
 ** Returns   : (int) 0 on success, an SCI_ERROR_* code otherwise
 */
 
+int
+sci1_read_resource_patches(char *path, resource_t **resources, int *resource_nr_p);
+/* Reads SCI1 patch files from a local directory
+** Parameters: (char *) path: (unused)
+**             (resource_t **) resources: Pointer to a pointer
+**                                        that will be set to the
+**                                        location of the resources
+**                                        (in one large chunk)
+**             (int *) resource_nr_p: Pointer to an int the number of resources
+**                                    read is stored in
+** Returns   : (int) 0 on success, an SCI_ERROR_* code otherwise
+*/
+
 
 /**--- Decompression functions ---**/
 
 
-int decompress0(resource_t *result, int resh);
+int decompress0(resource_t *result, int resh, int sci_version);
 /* Decrypts resource data and stores the result for SCI0-style compression.
 ** Parameters : result: The resource_t the decompressed data is stored in.
 **              resh  : File handle of the resource file
+**              sci_version : Actual SCI resource version
 ** Returns    : (int) 0 on success, one of SCI_ERROR_* if a problem was
 **               encountered.
 */
 
-int decompress01(resource_t *result, int resh);
+int decompress01(resource_t *result, int resh, int sci_version);
 /* Decrypts resource data and stores the result for SCI01-style compression.
 ** Parameters : result: The resource_t the decompressed data is stored in.
 **              resh  : File handle of the resource file
+**              sci_version : Actual SCI resource version
 ** Returns    : (int) 0 on success, one of SCI_ERROR_* if a problem was
 **               encountered.
 */
 
-int decompress1e(resource_t *result, int resh);
-/* Decrypts resource data and stores the result for early SCI1-style compression.
-** Parameters : result: The resource_t the decompressed data is stored in.
-**              resh  : File handle of the resource file
-** Returns    : (int) 0 on success, one of SCI_ERROR_* if a problem was
-**               encountered.
-*/
-
-int decompress1l(resource_t *result, int resh);
-/* Decrypts resource data and stores the result for late SCI1-style compression.
-** Parameters : result: The resource_t the decompressed data is stored in.
-**              resh  : File handle of the resource file
-** Returns    : (int) 0 on success, one of SCI_ERROR_* if a problem was
-**               encountered.
-*/
-
-int decompress11(resource_t *result, int resh);
+int decompress1(resource_t *result, int resh, int sci_version);
 /* Decrypts resource data and stores the result for SCI1.1-style compression.
 ** Parameters : result: The resource_t the decompressed data is stored in.
+**              sci_version : Actual SCI resource version
+**              resh  : File handle of the resource file
+** Returns    : (int) 0 on success, one of SCI_ERROR_* if a problem was
+**               encountered.
+*/
+
+
+int decompress11(resource_t *result, int resh, int sci_version);
+/* Decrypts resource data and stores the result for SCI1.1-style compression.
+** Parameters : result: The resource_t the decompressed data is stored in.
+**              sci_version : Actual SCI resource version
 **              resh  : File handle of the resource file
 ** Returns    : (int) 0 on success, one of SCI_ERROR_* if a problem was
 **               encountered.
@@ -327,6 +363,11 @@ int decrypt4(guint8* dest, guint8* src, int length, int complength);
 /* DCL inflate- implemented in decompress1.c
 */
 
+byte *view_reorder(byte *inbuffer, int dsize);
+/* SCI1 style view compression */
+
+byte *pic_reorder(byte *inbuffer, int dsize);
+/* SCI1 style pic compression */
 
 /*--- Internal helper functions ---*/
 
@@ -368,13 +409,16 @@ _scir_add_altsource(resource_t *res, int file, unsigned int file_offset);
 /* Resource type encoding */
 #define SCI0_B1_RESTYPE_MASK  0xf8
 #define SCI0_B1_RESTYPE_SHIFT 3
-#define SCI0_B3_RESFILE_MASK  0xfd
+#define SCI0_B3_RESFILE_MASK  0xfc
 #define SCI0_B3_RESFILE_SHIFT 2
+#define SCI01V_B3_RESFILE_MASK  0xf0
+#define SCI01V_B3_RESFILE_SHIFT 4
 
 #define SCI0_RESID_GET_TYPE(bytes) \
     (((bytes)[1] & SCI0_B1_RESTYPE_MASK) >> SCI0_B1_RESTYPE_SHIFT)
 #define SCI0_RESID_GET_NUMBER(bytes) \
     ((((bytes)[1] & ~SCI0_B1_RESTYPE_MASK) << 8) | ((bytes)[0]))
+
 #define SCI0_RESFILE_GET_FILE(bytes) \
     (((bytes)[3] & SCI0_B3_RESFILE_MASK) >> SCI0_B3_RESFILE_SHIFT)
 #define SCI0_RESFILE_GET_OFFSET(bytes) \
@@ -383,6 +427,35 @@ _scir_add_altsource(resource_t *res, int file, unsigned int file_offset);
       | (((bytes)[1]) << 8) \
       | (((bytes)[0]) << 0))
 
+#define SCI01V_RESFILE_GET_FILE(bytes) \
+    (((bytes)[3] & SCI01V_B3_RESFILE_MASK) >> SCI01V_B3_RESFILE_SHIFT)
+#define SCI01V_RESFILE_GET_OFFSET(bytes) \
+    ((((bytes)[3] & ~SCI01V_B3_RESFILE_MASK) << 24) \
+      | (((bytes)[2]) << 16) \
+      | (((bytes)[1]) << 8) \
+      | (((bytes)[0]) << 0))
+
+#define SCI1_B5_RESFILE_MASK 0xf0
+#define SCI1_B5_RESFILE_SHIFT 4
+
+#define SCI1_RESFILE_GET_FILE(bytes) \
+  (((bytes)[5] & SCI1_B5_RESFILE_MASK) >> SCI1_B5_RESFILE_SHIFT)
+
+#define SCI1_RESFILE_GET_OFFSET(bytes) \
+    ((((bytes)[5] & ~SCI1_B5_RESFILE_MASK) << 24) \
+      | (((bytes)[4]) << 16) \
+      | (((bytes)[3]) << 8) \
+      | (((bytes)[2]) << 0))
+
+#define SCI1_RESFILE_GET_NUMBER(bytes) \
+      ((((bytes)[1]) << 8) \
+      | (((bytes)[0]) << 0))
+
+#define SCI11_RESFILE_GET_OFFSET(bytes) \
+    ((((bytes)[4]) << 17) \
+      | (((bytes)[3]) << 9) \
+      | (((bytes)[2]) << 1))
+
 #endif
 
 
