@@ -37,12 +37,10 @@
 #endif
 #ifdef _WIN32
 #  include <io.h>
-#endif
-#ifdef _GP32
-#  include "wrap.h"
+#  include <sci_win32.h>
 #endif
 #ifdef _DREAMCAST
-#  include <kos/fs.h>
+#  include <stdio.h>
 #endif
 
 #if defined(_WIN32) && defined(_MSC_VER)
@@ -87,6 +85,10 @@ typedef unsigned TYPE_32 guint32;
 #undef TYPE_16
 #undef TYPE_32
 
+typedef gint8 sbyte;
+typedef guint8 byte;
+typedef guint16 word;
+
 typedef struct {
         long tv_sec;
         long tv_usec;
@@ -96,18 +98,35 @@ typedef struct {
 #ifdef _MSC_VER
 	long search;
 	struct _finddata_t fileinfo;
-#elif defined(_DREAMCAST)
-	file_t dir;
-	char *mask_copy;
-#elif defined(_GP32)
-	DIR *dir;
-	int total, cur;
-	char *mask_copy;
 #else
 	DIR *dir;
 	char *mask_copy;
 #endif
 } sci_dir_t; /* used by sci_find_first and friends */
 
-#endif /* !SCI_TYPES */
+/*
+ * Fixed point type, borrowed from ScummVM.
+ * The precision of the fractional (fixed point) type we define below.
+ * Normally you should never have to modify this value.
+ */
+enum {
+	FRAC_BITS = 16,
+	FRAC_LO_MASK = ((1L << FRAC_BITS) - 1),
+	FRAC_HI_MASK = (((1L << (32 - FRAC_BITS)) - 1) << FRAC_BITS),
+ 
+	FRAC_ONE = (1L << FRAC_BITS),		// 1.0
+	FRAC_HALF = (1L << (FRAC_BITS-1)),	// 0.5
+};
 
+/*
+ * Fixed-point fractions, used by the sound rate converter and other code.
+ */
+typedef gint32 frac_t;
+
+static inline frac_t double_to_frac(double value) { return (frac_t)(value * FRAC_ONE); }
+static inline double frac_to_double(frac_t value) { return ((double)value) / FRAC_ONE; }
+
+static inline frac_t int_to_frac(gint32 value) { return value << FRAC_BITS; }
+static inline gint32 frac_to_int(frac_t value) { return value >> FRAC_BITS; }
+
+#endif /* !SCI_TYPES */
