@@ -35,7 +35,7 @@ typedef struct script_opcode_
 } script_opcode;
 
 
-enum {
+typedef enum {
   sci_obj_terminator,
   sci_obj_object,
   sci_obj_code,
@@ -67,6 +67,7 @@ typedef enum {
   Script_Local,
   Script_Temp,
   Script_Param,
+  Script_Offset,
   Script_End
 } opcode_format;
 
@@ -130,7 +131,7 @@ typedef enum { /* FIXME */
   op_push1,
   op_push2,
   op_pushSelf,
-  op_lag,
+  op_lag = 0x40,
   op_lal,
   op_lat,
   op_lap,
@@ -198,4 +199,17 @@ typedef enum { /* FIXME */
 
 extern DLLEXTERN opcode_format formats[128][4];
 
+void script_adjust_opcode_formats(int res_version);
+
+int
+script_find_selector(struct _state *s, const char *selector_name);
+/* Determines the selector ID of a selector by its name
+** Parameters: (state_t *) s: VM state
+**             (char *) selector_name: Name of the selector to look up
+** Returns   : (int) The appropriate selector ID, or -1 on error
+*/
+
+struct _state;
+void script_free_breakpoints(struct _state *s);
+
 #endif
