@@ -1,5 +1,5 @@
 /***************************************************************************
- uinput.h (C) 1999,2000,01 Christoph Reichenbach, TU Darmstadt
+ uinput.h (C) 1999,2000,01,03 Christoph Reichenbach
 
 
  This program may be modified and copied freely according to the terms of
@@ -66,18 +66,19 @@ typedef struct {
 /*Fake values for other events*/
 #define SCI_EVT_ERROR           (1<<10)
 #define SCI_EVT_QUIT            (1<<11)
+#define SCI_EVT_NONBLOCK	(1<<15)
 /* The QUIT event may be used to signal an external 'quit' command being
 ** issued to the gfx driver.  */
-#define SCI_EVT_ANY             0xffff
+#define SCI_EVT_ANY             0x7fff
 
 
 
 /* Keycodes of special keys: */
-#define SCI_K_ESC		27
-#define SCI_K_BACKSPACE	8
-#define SCI_K_ENTER		13
-#define SCI_K_TAB		'\t'
-#define SCI_K_SHIFT_TAB	(0xf << 8)
+#define SCI_K_ESC 27
+#define SCI_K_BACKSPACE 8
+#define SCI_K_ENTER 13
+#define SCI_K_TAB '\t'
+#define SCI_K_SHIFT_TAB (0xf << 8)
 
 #define SCI_K_END (79 << 8)
 #define SCI_K_DOWN (80 << 8)
@@ -113,12 +114,6 @@ typedef struct {
 #define SCI_K_SHIFT_F9 (92<<8)
 #define SCI_K_SHIFT_F10 (93<<8)
 
-#define SCI_K_PLUS		'+'
-#define SCI_K_MINUS		'-'
-#define SCI_K_MULTIPLY	'*'
-#define SCI_K_DIVIDE	'/'
-#define SCI_K_EQUALS	'='
-
 /*Values for buckybits */
 #define SCI_EVM_RSHIFT          (1<<0)
 #define SCI_EVM_LSHIFT          (1<<1)
@@ -129,20 +124,7 @@ typedef struct {
 #define SCI_EVM_CAPSLOCK        (1<<6)
 #define SCI_EVM_INSERT          (1<<7)
 
-
-extern int
-scancode(int ch);
-/* Determines the PC 'scancode' for the specified character
-** Parameters: (int) ch: The character to translate
-** Returns   : (int) a matching scancode number
-*/
-
-int
-sci_toupper(int ch);
-/* Uppercasifies the character
-** Parameters: (int) ch: The character to capitalize
-** Returns   : (int) The capitalized version of the character
-** Also selects special symbols underlying the key
-*/
+#define SCI_EVM_NO_FOOLOCK      (~(SCI_EVM_SCRLOCK | SCI_EVM_NUMLOCK | SCI_EVM_CAPSLOCK | SCI_EVM_INSERT))
+#define SCI_EVM_ALL             0xFF
 
 #endif /* _SCI_UINPUT_H */
