@@ -1,3 +1,4 @@
+
 /***************************************************************************
  versions.h Copyright (C) 1999,2000,01 Christoph Reichenbach
 
@@ -40,6 +41,7 @@ struct _state;
 #define SCI_VERSION_MAJOR(_version_) ((_version_) >> 20)
 #define SCI_VERSION_MINOR(_version_) (((_version_) >> 10) & 0x3ff)
 #define SCI_VERSION_PATCHLEVEL(_version_) ((_version_) & 0x3ff)
+#define SCI_VERSION_IGNORE_PATCHLEVEL(_version_) ((_version) & ~0x3ff)
 
 /* Version number guide:
 ** - Always use the version number of the first known version to have a special feature.
@@ -75,7 +77,7 @@ struct _state;
 */
 
 
-#define SCI_VERSION_FTU_NEW_SCRIPT_HEADER SCI_VERSION(0,000,397)
+#define SCI_VERSION_FTU_NEW_SCRIPT_HEADER SCI_VERSION(0,000,395)
 /* Last version known not to do this: 0.000.343
 ** Old SCI versions used two word header for script blocks (first word equal
 ** to 0x82, meaning of the second one unknown). New SCI versions used one
@@ -87,7 +89,7 @@ struct _state;
 ** When doing CanBeHere(), augment y offset by 1
 */
 
-#define SCI_VERSION_FTU_2ND_ANGLES SCI_VERSION(0,000,572)
+#define SCI_VERSION_FTU_2ND_ANGLES SCI_VERSION(0,000,395)
 /* Last version known not to use this: ?
 ** Earlier versions assign 120 degrees to left & right , and 60 to up and down.
 ** Later versions use an even 90 degree distribution.
@@ -100,9 +102,27 @@ struct _state;
    clobbered the old songs.
 */
 
+#define SCI_VERSION_FTU_INVERSE_CANBEHERE SCI_VERSION(1,000,510)
+/* FIXME: This shouldn't be a version number.
+ * But it'll do for now.
+ */
+
+#define SCI_VERSION_FTU_LOFS_ABSOLUTE SCI_VERSION(1,000,200)
+/* First version known to do this: ?
+   In later versions (SCI1 and beyond), the argument of lofs[as]
+   instructions is absolute rather than relative.
+*/
+
+#define SCI_VERSION_FTU_DISPLAY_COORDS_FUZZY SCI_VERSION(1,000,510)
+/* First version known to do this: ?
+   In later versions of SCI1 kDisplay(), if the text would not fit on
+   the screen, the text is moved to the left and upwards until it
+   fits.
+*/
+
 #define SCI_VERSION_FTU_DOSOUND_VARIANT_1 SCI_VERSION(1,000,000)
 #define SCI_VERSION_FTU_DOSOUND_VARIANT_2 SCI_VERSION(1,000,510)
-#define SCI_VERSION_FTU_NORMAL_PORTS SCI_VERSION(0,000,502)
+
 
 typedef int sci_version_t;
 
@@ -122,25 +142,27 @@ version_require_later_than(struct _state *s, sci_version_t version);
 **             (sci_version_t) version: The version that we're later than
 */
 
-sci_version_t
-version_parse(char *vn);
+int
+version_parse(char *vn, sci_version_t *result);
 /* Parse a string containing an SCI version number
 ** Parameters: (char *) vn: The string to parse
-** Returns   : (sci_version_t): The resulting version number
+** Returns   : (int) 0 on success, 1 on failure
+**             (sci_version_t) *result: The resulting version number on success
 */
 
 int
 version_detect_from_executable(sci_version_t *result);
 /* Try to detect version from Sierra executable in cwd
-** Returns   : (int) 0 on failure, 1 on success
+** Returns   : (int) 0 on success, 1 on failure
 **             (sci_version_t) *result: The version number detected on success
 */
 
-char *
-version_guess_from_hashcode(sci_version_t *result, guint32 *code);
+const char *
+version_guess_from_hashcode(sci_version_t *result, int *res_version, guint32 *code);
 /* Try to detect version from Sierra resource file(s) in cwd
-** Returns   : (char *) NULL on failure, the name of the associated game otherwise
+** Returns   : (const char *) NULL on failure, the name of the associated game otherwise
 **             (sci_version_t) *result: The version number detected on success
+**             (int) *res_version: The resource version number detected on success
 **             (guint32) *code: The resource hash  code
 */
 
