@@ -183,8 +183,6 @@ decrypt4_hdyn(byte *dest, int length, struct bit_read_struct *reader)
 		CALLC(value = getbits(reader, 1));
 
 		if (value) { /* (length,distance) pair */
-			int val_length_backup;
-
 			CALLC(value = huffman_lookup(reader, length_tree));
 
 			if (value < 8)
@@ -228,8 +226,6 @@ decrypt4_hdyn(byte *dest, int length, struct bit_read_struct *reader)
 				return -SCI_ERROR_DECOMPRESSION_INSANE;
 			}
 
-			val_length_backup = val_length;
-
 			while (val_length) {
 				int copy_length = (val_length > val_distance)? val_distance : val_length;
 
@@ -243,10 +239,10 @@ decrypt4_hdyn(byte *dest, int length, struct bit_read_struct *reader)
 				}
 
 				val_length -= copy_length;
+				val_distance += copy_length;
+				write_pos += copy_length;
 			}
 
-			write_pos += val_length_backup;
-
 		} else { /* Copy byte verbatim */
 			if (mode == DCL_ASCII_MODE) {
 				CALLC(value = huffman_lookup(reader, ascii_tree));
@@ -280,29 +276,18 @@ decrypt4(guint8* dest, guint8* src, int length, int complength)
 
 
 
-void decryptinit3();
+void decryptinit3(void);
 int decrypt3(guint8* dest, guint8* src, int length, int complength);
 int decompress1(resource_t *result, int resh, int early);
 
-int decompress1e(resource_t *result, int resh)
-{
-  return decompress1(result, resh, 1);
-}
-
-int decompress1l(resource_t *result, int resh)
-{
-  return decompress1(result, resh, 0);
-}
-
-
-int decompress1(resource_t *result, int resh, int early)
+int decompress1(resource_t *result, int resh, int sci_version)
 {
 	guint16 compressedLength;
 	guint16 compressionMethod, result_size;
 	guint8 *buffer;
 	guint8 tempid;
 
-	if (early) {
+	if (sci_version == SCI_VERSION_1_EARLY) {
 		if (read(resh, &(result->id),2) != 2)
 			return SCI_ERROR_IO_ERROR;
 
@@ -313,7 +298,7 @@ int decompress1(resource_t *result, int resh, int early)
 		result->number = result->id & 0x07ff;
 		result->type = result->id >> 11;
 
-		if ((result->number >= sci_max_resource_nr[SCI_VERSION_1_EARLY]) || (result->type > sci_invalid_resource))
+		if ((result->number >= sci_max_resource_nr[SCI_VERSION_1_LATE]) || (result->type > sci_invalid_resource))
 			return SCI_ERROR_DECOMPRESSION_INSANE;
 	} else {
 		if (read(resh, &tempid, 1) != 1)
@@ -356,8 +341,8 @@ int decompress1(resource_t *result, int resh, int early)
 		return SCI_ERROR_EMPTY_OBJECT;
 	}
 
-	buffer = sci_malloc(compressedLength);
-	result->data = sci_malloc(result->size);
+	buffer = (guint8*)sci_malloc(compressedLength);
+	result->data = (unsigned char*)sci_malloc(result->size);
 
 	if (read(resh, buffer, compressedLength) != compressedLength) {
 		free(result->data);
@@ -415,28 +400,35 @@ int decompress1(resource_t *result, int resh, int early)
 		result->status = SCI_STATUS_ALLOCATED;
 		break;
 
-	case 3: /* Some sort of Huffman encoding */
-		if (decrypt4(result->data, buffer, result->size, compressedLength)) {
+	case 3: 
+		decryptinit3();
+		if (decrypt3(result->data, buffer, result->size, compressedLength)) {
 			free(result->data);
 			result->data = 0; /* So that we know that it didn't work */
 			result->status = SCI_STATUS_NOMALLOC;
 			free(buffer);
 			return SCI_ERROR_DECOMPRESSION_OVERFLOW;
 		}
+		result->data = view_reorder(result->data, result->size);
 		result->status = SCI_STATUS_ALLOCATED;
 		break;
 
-	case 4: /* NYI */
-		fprintf(stderr,"Resource %d.%s: Warning: compression type #%d not yet implemented\n",
-			result->number, sci_resource_type_suffixes[result->type], compressionMethod);
-		free(result->data);
-		result->data = NULL;
-		result->status = SCI_STATUS_NOMALLOC;
+	case 4:
+		decryptinit3();
+		if (decrypt3(result->data, buffer, result->size, compressedLength)) {
+			free(result->data);
+			result->data = 0; /* So that we know that it didn't work */
+			result->status = SCI_STATUS_NOMALLOC;
+			free(buffer);
+			return SCI_ERROR_DECOMPRESSION_OVERFLOW;
+		}
+		result->data = pic_reorder(result->data, result->size);
+		result->status = SCI_STATUS_ALLOCATED;
 		break;
-
+		
 	default:
-		fprintf(stderr,"Resource %d.%s: Compression method SCI1/%hi not "
-			"supported!\n", result->number, sci_resource_type_suffixes[result->type],
+		fprintf(stderr,"Resource %s.%03hi: Compression method SCI1/%hi not "
+			"supported!\n", sci_resource_types[result->type], result->number,
 			compressionMethod);
 		free(result->data);
 		result->data = 0; /* So that we know that it didn't work */
