@@ -31,11 +31,11 @@
 
 #define DDEBUG if (0) printf
 
-void decryptinit3();
+void decryptinit3(void);
 int decrypt3(guint8* dest, guint8* src, int length, int complength);
 int decrypt4(guint8* dest, guint8* src, int length, int complength);
 
-int decompress11(resource_t *result, int resh)
+int decompress11(resource_t *result, int resh, int sci_version)
 {
 	guint16 compressedLength;
 	guint16 compressionMethod, result_size;
@@ -79,16 +79,16 @@ int decompress11(resource_t *result, int resh)
 	    (compressedLength > SCI_MAX_RESOURCE_SIZE))
 		return SCI_ERROR_RESOURCE_TOO_BIG;
 
-	if (compressedLength > 0/*4*/)
-		compressedLength -= 0/*4*/;
+	if (compressedLength > 0)
+		compressedLength -= 0;
 	else { /* Object has size zero (e.g. view.000 in sq3) (does this really exist?) */
 		result->data = 0;
 		result->status = SCI_STATUS_NOMALLOC;
 		return SCI_ERROR_EMPTY_OBJECT;
 	}
 
-	buffer = sci_malloc(compressedLength);
-	result->data = sci_malloc(result->size);
+	buffer = (guint8*)sci_malloc(compressedLength);
+	result->data = (unsigned char*)sci_malloc(result->size);
 
 	if (read(resh, buffer, compressedLength) != compressedLength) {
 		free(result->data);
@@ -126,12 +126,11 @@ int decompress11(resource_t *result, int resh)
 		}
 		memcpy(result->data, buffer, compressedLength);
 		result->status = SCI_STATUS_ALLOCATED;
-		printf("#################### %d.%s: compression type #%d\n",
-			result->number, sci_resource_type_suffixes[result->type], compressionMethod);
 		break;
 
 	case 18:
-		fprintf(stderr,"Preparing decompression...\n");
+	case 19:
+	case 20:
 		if (decrypt4(result->data, buffer, result->size, compressedLength)) {
 			free(result->data);
 			result->data = 0; /* So that we know that it didn't work */
@@ -142,8 +141,6 @@ int decompress11(resource_t *result, int resh)
 		result->status = SCI_STATUS_ALLOCATED;
 		break;
 
-	case 19:
-	case 20:
 	case 3:
 	case 4: /* NYI */
 		fprintf(stderr,"Resource %d.%s: Warning: compression type #%d not yet implemented\n",
