@@ -32,7 +32,6 @@
 
 #include <sci_memory.h>
 #include <sciresource.h>
-#include <assert.h>
 #include <vocabulary.h> /* For SCI version auto-detection */
 
 #include <ctype.h>
@@ -49,13 +48,15 @@ const char* sci_version_types[] = {
 	"SCI version undetermined (Autodetect failed / not run)",
 	"SCI version 0.xxx",
 	"SCI version 0.xxx w/ 1.000 compression",
+	"SCI version 1.000 w/ 0.xxx resource.map",
+	"SCI version 1.000 w/ special resource.map",
 	"SCI version 1.000 (early)",
 	"SCI version 1.000 (late)",
 	"SCI version 1.001",
 	"SCI WIN/32"
 };
 
-const int sci_max_resource_nr[] = {65536, 1000, 1000, 1000, 8192};
+const int sci_max_resource_nr[] = {65536, 1000, 2048, 2048, 2048, 8192, 8192, 65536};
 
 const char* sci_error_types[] = {
 	"No error",
@@ -85,15 +86,17 @@ const char *sci_resource_type_suffixes[] = {"v56","p56","scr","tex","snd",
 int resourcecmp(const void *first, const void *second);
 
 
-typedef int decomp_funct(resource_t *result, int resh);
+typedef int decomp_funct(resource_t *result, int resh, int sci_version);
 typedef void patch_sprintf_funct(char *string, resource_t *res);
 
 static decomp_funct *decompressors[] = {
 	NULL,
 	&decompress0,
 	&decompress01,
-	&decompress1e,
-	&decompress1l,
+	&decompress01,
+	&decompress01,
+	&decompress1,
+	&decompress1,
 	&decompress11,
 	NULL
 };
@@ -102,10 +105,12 @@ static patch_sprintf_funct *patch_sprintfers[] = {
 	NULL,
 	&sci0_sprintf_patch_file_name,
 	&sci0_sprintf_patch_file_name,
-	NULL,
-	NULL,
-	NULL,
-	NULL
+	&sci1_sprintf_patch_file_name,
+	&sci1_sprintf_patch_file_name,
+	&sci1_sprintf_patch_file_name,
+	&sci1_sprintf_patch_file_name,
+	&sci1_sprintf_patch_file_name,
+	&sci1_sprintf_patch_file_name
 };
 
 
@@ -133,7 +138,7 @@ int resourcecmp (const void *first, const void *second)
 void
 _scir_add_altsource(resource_t *res, int file, unsigned int file_offset)
 {
-	resource_source_t *rsrc = sci_malloc(sizeof(resource_source_t));
+	resource_source_t *rsrc = (resource_source_t*)sci_malloc(sizeof(resource_source_t));
 
 	rsrc->next = res->alt_sources;
 	rsrc->file = file;
@@ -157,179 +162,20 @@ _scir_find_resource_unsorted(resource_t *res, int res_nr, int type, int number)
 /** Resource manager constructors and operations **/
 /*------------------------------------------------*/
 
-void
+static void
 _scir_init_trivial(resource_mgr_t *mgr)
 {
 	mgr->resources_nr = 0;
-	mgr->resources = sci_malloc(1);
+	mgr->resources = (resource_t*)sci_malloc(1);
 }
 
 
-resource_mgr_t *
-scir_new_resource_manager(char *dir, int version,
-			  char allow_patches, int max_memory)
-{
-	int resource_error = 0;
-	resource_mgr_t *mgr = sci_malloc(sizeof(resource_mgr_t));
-	char *caller_cwd = sci_getcwd();
-	int resmap_version = version;
-
-	if (chdir(dir)) {
-		sciprintf("Resmgr: Directory '%s' is invalid!\n", dir);
-		free(caller_cwd);
-		return NULL;
-	}
-
-	mgr->max_memory = max_memory;
-
-	mgr->memory_locked = 0;
-	mgr->memory_lru = 0;
-
-	mgr->resource_path = dir;
-
-	mgr->resources = NULL;
-
-	if (version <= SCI_VERSION_01) {
-		resource_error =
-			sci0_read_resource_map(dir,
-					       &mgr->resources,
-					       &mgr->resources_nr);
-
-		if (resource_error >= SCI_ERROR_CRITICAL) {
-			sciprintf("Resmgr: Error while loading resource map: %s\n",
-				  sci_error_types[resource_error]);
-			if (resource_error == SCI_ERROR_RESMAP_NOT_FOUND)
-				sciprintf("Running SCI games without a resource map is not supported ATM\n");
-			sci_free(mgr);
-			chdir(caller_cwd);
-			free(caller_cwd);
-			return NULL;
-		}
-
-		if (resource_error == SCI_ERROR_RESMAP_NOT_FOUND) {
-			/* fixme: Try reading w/o resource.map */
-			resource_error = SCI_ERROR_NO_RESOURCE_FILES_FOUND;
-		}
-
-		if (resource_error == SCI_ERROR_NO_RESOURCE_FILES_FOUND) {
-			/* Initialize empty resource manager */
-			_scir_init_trivial(mgr);
-			resource_error = 0;
-		}
-
-		if (!resource_error)
-			sci0_read_resource_patches(dir,
-						   &mgr->resources,
-						   &mgr->resources_nr);
-
-		resmap_version = SCI_VERSION_0;
-	}
-
-
-	/* ADDME: Try again with sci1_read_resource_map() */
-	if (!mgr->resources || !mgr->resources_nr) {
-		if (mgr->resources) {
-			free(mgr->resources);
-			mgr->resources = NULL;
-		}
-		sciprintf("Resmgr: Could not retreive a resource list!\n");
-		sci_free(mgr);
-		chdir(caller_cwd);
-		free(caller_cwd);
-		return NULL;
-	}
-
-	mgr->lru_first = NULL;
-	mgr->lru_last = NULL;
-
-	qsort(mgr->resources, mgr->resources_nr, sizeof(resource_t),
-	      resourcecmp); /* Sort resources */
-
-	mgr->allow_patches = allow_patches;
-
-	if (version == SCI_VERSION_AUTODETECT)
-		switch (resmap_version) {
-		case SCI_VERSION_0:
-			if (scir_test_resource(mgr, sci_vocab,
-					       VOCAB_RESOURCE_SCI0_MAIN_VOCAB)) {
-				sciprintf("Resmgr: Detected SCI0\n");
-				version = SCI_VERSION_0;
-			} else if (scir_test_resource(mgr, sci_vocab,
-						      VOCAB_RESOURCE_SCI1_MAIN_VOCAB)) {
-				sciprintf("Resmgr: Detected SCI01\n");
-				if (scir_test_resource(mgr, sci_vocab, 912)) {
-					sciprintf("Resmgr: Running KQ1 or similar, using SCI0 resource encoding\n");
-					version = SCI_VERSION_0;
-				} else version = SCI_VERSION_01;
-			} else {
-				sciprintf("Resmgr: Warning: Could not find vocabulary; assuming SCI0 w/o parser\n");
-				version = SCI_VERSION_0;
-			} break;
-
-		default:
-			sciprintf("Resmgr: Warning: While autodetecting: Couldn't"
-				  " determine SCI version!\n");
-		}
-
-	mgr->sci_version = version;
-
-	chdir(caller_cwd);
-	free(caller_cwd);
-
-	return mgr;
-}
-
-static void
-_scir_free_resource_sources(resource_source_t *rss)
-{
-	if (rss) {
-		_scir_free_resource_sources(rss->next);
-		free(rss);
-	}
-}
-
-void
-_scir_free_altsources(resource_source_t *dynressrc)
-{
-	if (dynressrc) {
-		_scir_free_altsources(dynressrc->next);
-		free(dynressrc);
-	}
-}
-
-void
-_scir_free_resources(resource_t *resources, int resources_nr)
-{
-	int i;
-
-	for (i = 0; i < resources_nr; i++) {
-		resource_t *res = resources + i;
-
-		_scir_free_altsources(res->alt_sources);
-
-		if (res->status != SCI_STATUS_NOMALLOC)
-			sci_free(res->data);
-	}
-
-	sci_free(resources);
-}
-
-void
-scir_free_resource_manager(resource_mgr_t *mgr)
-{
-	_scir_free_resources(mgr->resources, mgr->resources_nr);
-
-	mgr->resources = NULL;
-
-	sci_free(mgr);
-}
-
 static void
 _scir_load_from_patch_file(int fh, resource_t *res, char *filename)
 {
 	int really_read;
 
-	res->data = sci_malloc(res->size);
+	res->data = (unsigned char*)sci_malloc(res->size);
 	really_read = read(fh, res->data, res->size);
 
 	if (really_read < res->size) {
@@ -341,14 +187,15 @@ _scir_load_from_patch_file(int fh, resource_t *res, char *filename)
 	res->status = SCI_STATUS_ALLOCATED;
 }
 
-
 static void
-_scir_load_resource(resource_mgr_t *mgr, resource_t *res)
+_scir_load_resource(resource_mgr_t *mgr, resource_t *res, int protect)
 {
 	char *cwd = sci_getcwd();
 	char filename[14];
 	int fh;
+	resource_t backup;
 
+	memcpy(&backup, res, sizeof(resource_t));
 	/* Enter resource directory */
 	chdir(mgr->resource_path);
 
@@ -384,6 +231,8 @@ _scir_load_resource(resource_mgr_t *mgr, resource_t *res)
 		res->data = NULL;
 		res->status = SCI_STATUS_NOMALLOC;
 		res->size = 0;
+		chdir(cwd);
+		free(cwd);
 		return;
 	}
 
@@ -399,16 +248,22 @@ _scir_load_resource(resource_mgr_t *mgr, resource_t *res)
 		exit(1);
 	} else {
 		int error = /* Decompress from regular resource file */
-			decompressors[mgr->sci_version](res, fh);
+			decompressors[mgr->sci_version](res, fh, mgr->sci_version);
 
 		if (error) {
 			sciprintf("Error %d occured while reading %s.%03d"
 				  " from resource file: %s\n",
 				  error, sci_resource_types[res->type], res->number,
 				  sci_error_types[error]);
+
+			if (protect)
+				memcpy(res, &backup, sizeof(resource_t));
+
 			res->data = NULL;
 			res->status = SCI_STATUS_NOMALLOC;
 			res->size = 0;
+			chdir(cwd);
+			free(cwd);
 			return;
 		}
 	}
@@ -418,7 +273,342 @@ _scir_load_resource(resource_mgr_t *mgr, resource_t *res)
 	free(cwd);
 }
 
+resource_t *
+scir_test_resource(resource_mgr_t *mgr, int type, int number)
+{
+	resource_t binseeker;
+	binseeker.type = type;
+	binseeker.number = number;
+	return (resource_t *)
+		bsearch(&binseeker, mgr->resources, mgr->resources_nr,
+			sizeof(resource_t), resourcecmp);
+}
+
+int sci0_get_compression_method(int resh);
+
+int
+sci_test_view_type(resource_mgr_t *mgr)
+{
+	int fh;
+	char filename[14];
+	int compression;
+	resource_t *res;
+	int i;
+
+	mgr->sci_version = SCI_VERSION_AUTODETECT;
+
+	for (i=0;i<1000;i++)
+	{
+		res = scir_test_resource(mgr, sci_view, i);
+
+		if (!res) continue;
+
+		if (res->file == SCI_RESOURCE_FILE_PATCH)
+			continue;
+
+		sprintf(filename, "resource.%03i", res->file);
+		fh = open(filename, O_RDONLY | O_BINARY);
+
+
+		if (!IS_VALID_FD(fh)) {
+			char *raiser = filename;
+			while (*raiser) {
+				*raiser = toupper(*raiser); /* Uppercasify */
+				++raiser;
+			}
+			fh = sci_open(filename, O_RDONLY|O_BINARY);
+		}    /* Try case-insensitively name */
+		
+		if (!IS_VALID_FD(fh)) continue;
+		lseek(fh, res->file_offset, SEEK_SET);
+
+		compression = sci0_get_compression_method(fh);
+		close(fh);
+
+		if (compression == 3)
+			return (mgr->sci_version = SCI_VERSION_01_VGA);
+	}
+
+	/* Try the same thing with pics */
+	for (i=0;i<1000;i++)
+	{
+		res = scir_test_resource(mgr, sci_pic, i);
+
+		if (!res) continue;
+
+		if (res->file == SCI_RESOURCE_FILE_PATCH)
+			continue;
+
+		sprintf(filename, "resource.%03i", res->file);
+		fh = open(filename, O_RDONLY | O_BINARY);
+
+
+		if (!IS_VALID_FD(fh)) {
+			char *raiser = filename;
+			while (*raiser) {
+				*raiser = toupper(*raiser); /* Uppercasify */
+				++raiser;
+			}
+			fh = sci_open(filename, O_RDONLY|O_BINARY);
+		}    /* Try case-insensitively name */
+		
+		if (!IS_VALID_FD(fh)) continue;
+		lseek(fh, res->file_offset, SEEK_SET);
+
+		compression = sci0_get_compression_method(fh);
+		close(fh);
+
+		if (compression == 3)
+			return (mgr->sci_version = SCI_VERSION_01_VGA);
+	}
+
+	return mgr->sci_version;
+}
+			
+		
+
+resource_mgr_t *
+scir_new_resource_manager(char *dir, int version,
+			  char allow_patches, int max_memory)
+{
+	int resource_error = 0;
+	resource_mgr_t *mgr = (resource_mgr_t*)sci_malloc(sizeof(resource_mgr_t));
+	char *caller_cwd = sci_getcwd();
+	int resmap_version = version;
+
+	if (chdir(dir)) {
+		sciprintf("Resmgr: Directory '%s' is invalid!\n", dir);
+		free(caller_cwd);
+		return NULL;
+	}
+
+	mgr->max_memory = max_memory;
+
+	mgr->memory_locked = 0;
+	mgr->memory_lru = 0;
+
+	mgr->resource_path = dir;
+
+	mgr->resources = NULL;
+
+	if (version <= SCI_VERSION_01_VGA_ODD
+	    /* || version == SCI_VERSION_AUTODETECT -- subsumed by the above line */) {
+		resource_error =
+			sci0_read_resource_map(dir,
+					       &mgr->resources,
+					       &mgr->resources_nr,
+					       &resmap_version);
+
+		if (resource_error >= SCI_ERROR_CRITICAL) {
+			sciprintf("Resmgr: Error while loading resource map: %s\n",
+				  sci_error_types[resource_error]);
+			if (resource_error == SCI_ERROR_RESMAP_NOT_FOUND)
+				sciprintf("Running SCI games without a resource map is not supported ATM\n");
+			sci_free(mgr);
+			chdir(caller_cwd);
+			free(caller_cwd);
+			return NULL;
+		}
+
+		if (resource_error == SCI_ERROR_RESMAP_NOT_FOUND) {
+			/* fixme: Try reading w/o resource.map */
+			resource_error = SCI_ERROR_NO_RESOURCE_FILES_FOUND;
+		}
+
+		if (resource_error == SCI_ERROR_NO_RESOURCE_FILES_FOUND) {
+			/* Initialize empty resource manager */
+			_scir_init_trivial(mgr);
+			resource_error = 0;
+		}
+
+	}
+
+	if ((version == SCI_VERSION_1_EARLY)||
+	    (version == SCI_VERSION_1_LATE)||
+	    (version == SCI_VERSION_1_1)||
+	    ((resmap_version == SCI_VERSION_AUTODETECT)&&(version == SCI_VERSION_AUTODETECT)))
+	{
+		resource_error =
+			sci1_read_resource_map(dir,
+					       &mgr->resources,
+					       &mgr->resources_nr,
+					       &resmap_version);
+
+		if (resource_error >= SCI_ERROR_CRITICAL) {
+			sciprintf("Resmgr: Error while loading resource map: %s\n",
+				  sci_error_types[resource_error]);
+			if (resource_error == SCI_ERROR_RESMAP_NOT_FOUND)
+				sciprintf("Running SCI games without a resource map is not supported ATM\n");
+			sci_free(mgr);
+			chdir(caller_cwd);
+			free(caller_cwd);
+			return NULL;
+		}
+
+		if (resource_error == SCI_ERROR_RESMAP_NOT_FOUND) {
+			/* fixme: Try reading w/o resource.map */
+			resource_error = SCI_ERROR_NO_RESOURCE_FILES_FOUND;
+		}
+
+		if (resource_error == SCI_ERROR_NO_RESOURCE_FILES_FOUND) {
+			/* Initialize empty resource manager */
+			_scir_init_trivial(mgr);
+			resource_error = 0;
+		}
+
+		resmap_version = SCI_VERSION_1;
+	}
+		
+	if (!mgr->resources || !mgr->resources_nr) {
+		if (mgr->resources) {
+			free(mgr->resources);
+			mgr->resources = NULL;
+		}
+		sciprintf("Resmgr: Could not retreive a resource list!\n");
+		sci_free(mgr);
+		chdir(caller_cwd);
+		free(caller_cwd);
+		return NULL;
+	}
+
+	mgr->lru_first = NULL;
+	mgr->lru_last = NULL;
+
+	mgr->allow_patches = allow_patches;
+
+	qsort(mgr->resources, mgr->resources_nr, sizeof(resource_t),
+	      resourcecmp); /* Sort resources */
+
+	if (version == SCI_VERSION_AUTODETECT)
+		switch (resmap_version) {
+		case SCI_VERSION_0:
+			if (scir_test_resource(mgr, sci_vocab,
+					       VOCAB_RESOURCE_SCI0_MAIN_VOCAB)) {
+				version = sci_test_view_type(mgr);
+				if (version == SCI_VERSION_01_VGA)
+				{
+					sciprintf("Resmgr: Detected KQ5 or similar\n");
+				} else {
+					sciprintf("Resmgr: Detected SCI0\n");
+					version = SCI_VERSION_0;
+				}
+			} else if (scir_test_resource(mgr, sci_vocab,
+						      VOCAB_RESOURCE_SCI1_MAIN_VOCAB)) {
+				version = sci_test_view_type(mgr);
+				if (version == SCI_VERSION_01_VGA)
+				{
+					sciprintf("Resmgr: Detected KQ5 or similar\n");
+				} else {
+					if (scir_test_resource(mgr, sci_vocab, 912)) {
+						sciprintf("Resmgr: Running KQ1 or similar, using SCI0 resource encoding\n");
+						version = SCI_VERSION_0;
+					} else {
+						version = SCI_VERSION_01;
+						sciprintf("Resmgr: Detected SCI01\n");
+					}
+				}
+			} else {
+				version = sci_test_view_type(mgr);
+				if (version == SCI_VERSION_01_VGA)
+				{
+					sciprintf("Resmgr: Detected KQ5 or similar\n");
+				} else {
+					sciprintf("Resmgr: Warning: Could not find vocabulary; assuming SCI0 w/o parser\n");
+					version = SCI_VERSION_0;
+				}
+			} break;
+		case SCI_VERSION_01_VGA_ODD:
+			version = resmap_version;
+			sciprintf("Resmgr: Detected Jones/CD or similar\n");
+			break;
+		case SCI_VERSION_1:
+		{
+			resource_t *res = scir_test_resource(mgr, sci_script, 0);
+			
+			mgr->sci_version = version = SCI_VERSION_1_EARLY;
+			_scir_load_resource(mgr, res, 1);
+			
+			if (res->status == SCI_STATUS_NOMALLOC)
+			    mgr->sci_version = version = SCI_VERSION_1_LATE;
+
+			/* No need to handle SCI 1.1 here - it was done in resource_map.c */
+			break;
+		}
+		default:
+			sciprintf("Resmgr: Warning: While autodetecting: Couldn't"
+				  " determine SCI version!\n");
+		}
+
+	if (!resource_error)
+	{
+		if (version <= SCI_VERSION_01)
+			sci0_read_resource_patches(dir,
+						   &mgr->resources,
+						   &mgr->resources_nr);
+		else
+			sci1_read_resource_patches(dir,
+						   &mgr->resources,
+						   &mgr->resources_nr);
+
+		qsort(mgr->resources, mgr->resources_nr, sizeof(resource_t),
+		      resourcecmp); /* Sort resources */
+	}
+
+	mgr->sci_version = version;
+
+	chdir(caller_cwd);
+	free(caller_cwd);
+
+	return mgr;
+}
+
+static void
+_scir_free_resource_sources(resource_source_t *rss)
+{
+	if (rss) {
+		_scir_free_resource_sources(rss->next);
+		free(rss);
+	}
+}
+
+static void
+_scir_free_altsources(resource_source_t *dynressrc)
+{
+	if (dynressrc) {
+		_scir_free_altsources(dynressrc->next);
+		free(dynressrc);
+	}
+}
+
 void
+_scir_free_resources(resource_t *resources, int resources_nr)
+{
+	int i;
+
+	for (i = 0; i < resources_nr; i++) {
+		resource_t *res = resources + i;
+
+		_scir_free_altsources(res->alt_sources);
+
+		if (res->status != SCI_STATUS_NOMALLOC)
+			sci_free(res->data);
+	}
+
+	sci_free(resources);
+}
+
+void
+scir_free_resource_manager(resource_mgr_t *mgr)
+{
+	_scir_free_resources(mgr->resources, mgr->resources_nr);
+
+	mgr->resources = NULL;
+
+	sci_free(mgr);
+}
+
+
+static void
 _scir_unalloc(resource_t *res)
 {
 	sci_free(res->data);
@@ -427,7 +617,7 @@ _scir_unalloc(resource_t *res)
 }
 
 
-void
+static void
 _scir_remove_from_lru(resource_mgr_t *mgr, resource_t *res)
 {
 	if (res->status != SCI_STATUS_ENQUEUED) {
@@ -450,7 +640,7 @@ _scir_remove_from_lru(resource_mgr_t *mgr, resource_t *res)
 	res->status = SCI_STATUS_ALLOCATED;
 }
 
-void
+static void
 _scir_add_to_lru(resource_mgr_t *mgr, resource_t *res)
 {
 	if (res->status != SCI_STATUS_ALLOCATED) {
@@ -507,7 +697,7 @@ _scir_free_old_resources(resource_mgr_t *mgr, int last_invulnerable)
 		if (!goner) {
 			fprintf(stderr,"Internal error: mgr->lru_last is NULL!\n");
 			fprintf(stderr,"LRU-mem= %d\n", mgr->memory_lru);
-			fprintf(stderr,"lru_first = %p\n", mgr->lru_first);
+			fprintf(stderr,"lru_first = %p\n", (void *)mgr->lru_first);
 			_scir_print_lru_list(mgr);
 		}
 
@@ -521,29 +711,29 @@ _scir_free_old_resources(resource_mgr_t *mgr, int last_invulnerable)
 	}
 }
 
-resource_t *
-scir_test_resource(resource_mgr_t *mgr, int type, int number)
-{
-	resource_t binseeker;
-	binseeker.type = type;
-	binseeker.number = number;
-	return (resource_t *)
-		bsearch(&binseeker, mgr->resources, mgr->resources_nr,
-			sizeof(resource_t), resourcecmp);
-}
-
 resource_t *
 scir_find_resource(resource_mgr_t *mgr, int type, int number, int lock)
 {
-	resource_t *retval = scir_test_resource(mgr, type, number);
+	resource_t *retval;
+
+	if (number >= sci_max_resource_nr[mgr->sci_version]) {
+		int modded_number = number % sci_max_resource_nr[mgr->sci_version];
+		sciprintf("[resmgr] Requested invalid resource %s.%d, mapped to %s.%d\n",
+			  sci_resource_types[type], number,
+			  sci_resource_types[type], modded_number);
+		number = modded_number;
+	}
+
+	retval = scir_test_resource(mgr, type, number);
 
 	if (!retval)
 		return NULL;
 
 	if (!retval->status)
-		_scir_load_resource(mgr, retval);
+		_scir_load_resource(mgr, retval, 0);
+
 	else if (retval->status == SCI_STATUS_ENQUEUED)
-			_scir_remove_from_lru(mgr, retval);
+		_scir_remove_from_lru(mgr, retval);
 	/* Unless an error occured, the resource is now either
 	** locked or allocated, but never queued or freed.  */
 
@@ -573,7 +763,7 @@ scir_find_resource(resource_mgr_t *mgr, int type, int number, int lock)
 }
 
 void
-scir_unlock_resource(resource_mgr_t *mgr, resource_t *res, int restype, int resnum)
+scir_unlock_resource(resource_mgr_t *mgr, resource_t *res, int resnum, int restype)
 {
 	if (!res) {
 		sciprintf("Resmgr: Warning: Attempt to unlock non-existant"
