@@ -1,5 +1,6 @@
 /***************************************************************************
  sci_memory.c Copyright (C) 2001 Alexander R Angas
+			Refcounted memory by Christoph Reichenbach
 
 
  This program may be modified and copied freely according to the terms of
@@ -34,6 +35,8 @@
 
 #include <sci_memory.h>
 
+/*#define POISON_MEMORY*/
+
 /* set optimisations for Win32: */
 /* g on: enable global optimizations */
 /* t on: use fast code */
@@ -42,10 +45,15 @@
 
 #ifdef _MSC_VER
 #	include <crtdbg.h>
+#	ifndef SATISFY_PURIFY
+#		pragma optimize( "s", off )
+#		pragma optimize( "gty", on )
+#		pragma intrinsic( memcpy, strlen )
+#	endif
 #endif
 
 
-inline void *
+void *
 _SCI_MALLOC(size_t size, const char *file, int line, const char *funct)
 {
 	void *res;
@@ -53,11 +61,16 @@ _SCI_MALLOC(size_t size, const char *file, int line, const char *funct)
 	INFO_MEMORY("_SCI_MALLOC()", size, file, line, funct);
 #endif
 	ALLOC_MEM((res = malloc(size)), size, file, line, funct)
+#ifdef POISON_MEMORY
+	{
+		memset(res, 0xa5, size);
+	}
+#endif
 	return res;
 }
 
 
-inline void *
+void *
 _SCI_CALLOC(size_t num, size_t size, const char *file, int line, const char *funct)
 {
 	void *res;
@@ -69,7 +82,7 @@ _SCI_CALLOC(size_t num, size_t size, const char *file, int line, const char *fun
 }
 
 
-inline void *
+void *
 _SCI_REALLOC(void *ptr, size_t size, const char *file, int line, const char *funct)
 {
 	void *res;
@@ -81,11 +94,11 @@ _SCI_REALLOC(void *ptr, size_t size, const char *file, int line, const char *fun
 }
 
 
-inline void
+void
 _SCI_FREE(void *ptr, const char *file, int line, const char *funct)
 {
 #ifdef MALLOC_DEBUG
-	INFO_MEMORY("_SCI_FREE()", size, file, line, funct);
+	INFO_MEMORY("_SCI_FREE()", 0, file, line, funct);
 #endif
 	if (!ptr)
 	{
@@ -98,7 +111,7 @@ _SCI_FREE(void *ptr, const char *file, int line, const char *funct)
 }
 
 
-inline void *
+void *
 _SCI_MEMDUP(const void *ptr, size_t size, const char *file, int line, const char *funct)
 {
 	void *res;
@@ -118,12 +131,12 @@ _SCI_MEMDUP(const void *ptr, size_t size, const char *file, int line, const char
 }
 
 
-inline char *
+char *
 _SCI_STRDUP(const char *src, const char *file, int line, const char *funct)
 {
 	void *res;
 #ifdef MALLOC_DEBUG
-	INFO_MEMORY("_SCI_STRDUP()", size, file, line, funct);
+	INFO_MEMORY("_SCI_STRDUP()", 0, file, line, funct);
 #endif
 	if (!src)
 	{
@@ -133,18 +146,18 @@ _SCI_STRDUP(const char *src, const char *file, int line, const char *funct)
 		BREAKPOINT();
 	}
 	ALLOC_MEM((res = strdup(src)), strlen(src), file, line, funct)
-	return res;
+	return (char*)res;
 }
 
 
-inline char *
+char *
 _SCI_STRNDUP(const char *src, size_t length, const char *file, int line, const char *funct)
 {
 	void *res;
 	char *strres;
-	int rlen = MIN(strlen(src), length) + 1;
+	size_t rlen = (int)MIN(strlen(src), length) + 1;
 #ifdef MALLOC_DEBUG
-	INFO_MEMORY("_SCI_STRNDUP()", size, file, line, funct);
+	INFO_MEMORY("_SCI_STRNDUP()", 0, file, line, funct);
 #endif
 	if (!src)
 	{
@@ -155,7 +168,7 @@ _SCI_STRNDUP(const char *src, size_t length, const char *file, int line, const c
 	}
 	ALLOC_MEM((res = malloc(rlen)), rlen, file, line, funct)
 
-	strres = res;
+	strres = (char*)res;
 	strncpy(strres, src, rlen);
 	strres[rlen - 1] = 0;
 
@@ -212,3 +225,87 @@ debug_win32_memory(int dbg_setting)
 #endif
 }
 #endif
+
+
+
+/*-------- Refcounting ----------*/
+
+#define REFCOUNT_OVERHEAD (sizeof(guint32)*3)
+#define REFCOUNT_MAGIC_LIVE_1 0xebdc1741
+#define REFCOUNT_MAGIC_LIVE_2 0x17015ac9
+#define REFCOUNT_MAGIC_DEAD_1 0x11dead11
+#define REFCOUNT_MAGIC_DEAD_2 0x22dead22
+
+#define REFCOUNT_CHECK(p) ((((guint32 *)(p))[-3] == REFCOUNT_MAGIC_LIVE_2)    \
+			   && (((guint32 *)(p))[-1] == REFCOUNT_MAGIC_LIVE_1))
+
+#define REFCOUNT(p) (((guint32 *)p)[-2])
+
+#undef TRACE_REFCOUNT
+
+
+extern void *
+sci_refcount_alloc(size_t length)
+{
+	guint32 *data = (guint32*)sci_malloc(REFCOUNT_OVERHEAD + length);
+#ifdef TRACE_REFCOUNT
+fprintf(stderr, "[] REF: Real-alloc at %p\n", data);
+#endif
+	data += 3;
+
+	data[-1] = REFCOUNT_MAGIC_LIVE_1;
+	data[-3] = REFCOUNT_MAGIC_LIVE_2;
+	REFCOUNT(data) = 1;
+#ifdef TRACE_REFCOUNT
+fprintf(stderr, "[] REF: Alloc'd %p (ref=%d) OK=%d\n", data, REFCOUNT(data),
+	REFCOUNT_CHECK(data));
+#endif
+	return data;
+}
+
+extern void *
+sci_refcount_incref(void *data)
+{
+	if (!REFCOUNT_CHECK(data)) {
+		BREAKPOINT();
+	} else
+		REFCOUNT(data)++;
+
+#ifdef TRACE_REFCOUNT
+fprintf(stderr, "[] REF: Inc'ing %p (now ref=%d)\n", data, REFCOUNT(data));
+#endif
+	return data;
+}
+
+extern void
+sci_refcount_decref(void *data)
+{
+#ifdef TRACE_REFCOUNT
+fprintf(stderr, "[] REF: Dec'ing %p (prev ref=%d) OK=%d\n", data, REFCOUNT(data),
+	REFCOUNT_CHECK(data));
+#endif
+	if (!REFCOUNT_CHECK(data)) {
+		BREAKPOINT();
+	} else if (--REFCOUNT(data) == 0) {
+		guint32 *fdata = (guint32*)data;
+
+		fdata[-1] = REFCOUNT_MAGIC_DEAD_1;
+		fdata[-3] = REFCOUNT_MAGIC_DEAD_2;
+
+#ifdef TRACE_REFCOUNT
+fprintf(stderr, "[] REF: Freeing (%p)...\n", fdata - 3);
+#endif
+		sci_free(fdata - 3);
+#ifdef TRACE_REFCOUNT
+fprintf(stderr, "[] REF: Done.\n");
+#endif
+	}
+}
+
+extern void *
+sci_refcount_memdup(void *data, size_t len)
+{
+	void *dest = sci_refcount_alloc(len);
+	memcpy(dest, data, len);
+	return dest;
+}
