@@ -1,5 +1,5 @@
 /***************************************************************************
- tools.c Copyright (C) 1999,2000,2001 Christoph Reichenbach
+ tools.c Copyright (C) 1999,2000,2001,2002 Christoph Reichenbach
 
  This program may be modified and copied freely according to the terms of
  the GNU general public license (GPL), as long as the above copyright
@@ -24,9 +24,8 @@
 
 ***************************************************************************/
 
-
+#include <stdlib.h>
 #include <engine.h>
-#include <kdebug.h>
 #ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 #endif
@@ -34,17 +33,20 @@
 #  include <sys/timeb.h>
 #  include <windows.h>
 #  include <sci_win32.h>
-#endif
-#if !defined(HAVE_FNMATCH) && !defined(_MSC_VER) && !defined(__MORPHOS__)
-#  include <beos/fnmatch.h>
-#endif
-#if defined(__MORPHOS__)
+#  include <sys/types.h>
 #  include <sys/stat.h>
+#else
+#ifdef _WIN32
+#  include <windows.h>
+#  include <win32/usleep.h>
+#  include <win32/sci_win32.h>
+#endif
+#endif
+#if !defined(HAVE_FNMATCH) && !defined(_MSC_VER)
+#  include <beos/fnmatch.h>
 #endif
 
 #ifdef _DREAMCAST
-#  include <arch/timer.h>
-#  include <dc.h>
 #  include <kos/thread.h>
 #endif
 
@@ -59,11 +61,14 @@ void *memfrob(void *s, size_t n);
 int script_debug_flag = 0; /* Defaulting to running mode */
 int sci_debug_flags = 0; /* Special flags */
 
+#ifndef con_file
+#	define con_file 0
+#endif
 
 #define MEMTEST_HARDNESS 31
 
 int
-memtest(char *file, int line)
+memtest(const char *file, int line)
 {
 	/* va_list argp; -- unused */
 	int i;
@@ -218,10 +223,8 @@ _SCIGNUkdebug(const char *funcname, state_t *s, const char *file, int line, int
 	}
 }
 
-#if defined (HAVE_SYS_TIME_H) && !(ARM_WINCE)
-/* Do not use this function (even if it's there) when compiling for StrongARM & Windows CE.
-** See below for more information.
-*/
+
+#if defined(HAVE_GETTIMEOFDAY)
 void
 sci_gettime(long *seconds, long *useconds)
 {
@@ -231,7 +234,7 @@ sci_gettime(long *seconds, long *useconds)
 	*seconds = tv.tv_sec;
 	*useconds = tv.tv_usec;
 }
-#elif defined (_MSC_VER)
+#elif defined (_WIN32)
 
 /*WARNING(Incorrect)*/
 /* Warning: This function only retrieves the amount of mseconds since the start of
@@ -254,36 +257,7 @@ void sci_gettime(long *seconds, long *useconds)
 	}
 
 	*seconds = tm/1000;
-	*useconds = (tm % 1000) * 1000;
-}
-#elif defined (_DREAMCAST)
-/* Warning: This function returns the amount of time that has passed since the
-** start of the program. A gettimeofday() function is available, but it
-** contains a bug. So this will have to do for now.
-*/
-void
-sci_gettime(long *seconds, long *useconds)
-{
-	uint32 s, m;
-	timer_ms_gettime(&s, &m);
-	*seconds = s;
-	*useconds = m*1000;
-}
-#elif defined (ARM_WINCE) && (HAVE_SDL)
-/* Warning: This function returns the amount of time that has passed since the
-** initialization of the SDL_Driver. A gettimeofday() function is available, but it is
-** somehow buggy.
-** NOTE!!
-** Due to this, the game_start_time contains junk (SDL wasn't init'd yet)
-*/
-void
-sci_gettime(long *seconds, long *useconds)
-{
-	long tm;
-	tm = SDL_GetTicks();
-	*seconds = tm/1000;
-	*useconds = (tm % 1000) * 1000;
-	
+	*useconds = (tm%1000)*1000;
 }
 #else
 #  error "You need to provide a microsecond resolution sci_gettime implementation for your platform!"
@@ -311,7 +285,7 @@ sci_init_dir(sci_dir_t *dir)
 }
 
 char *
-sci_find_first(sci_dir_t *dir, char *mask)
+sci_find_first(sci_dir_t *dir, const char *mask)
 {
 	dir->search = _findfirst(mask, &(dir->fileinfo));
 
@@ -395,131 +369,7 @@ sci_finish_find(sci_dir_t *dir)
 	}
 }
 
-#elif defined(_DREAMCAST)
-/******** Dir: DREAMCAST CODE ********/
-
-void
-sci_init_dir(sci_dir_t *dir)   
-{
-	dir->dir = 0;
-	dir->mask_copy = NULL;
-}
-
-char *
-sci_find_first(sci_dir_t *dir, char *mask)
-{
-	if (dir->dir)
-		fs_close(dir->dir);
-
-	/* Opening "." does not work */
-	if (!(dir->dir = fs_open(fs_getwd(), O_RDONLY | O_DIR))) {
-		sciprintf("%s, L%d: fs_open(fs_getwd(), O_RDONLY | O_DIR) failed!\n",__FILE__, __LINE__);
-		return NULL;
-	}
-
-	dir->mask_copy = sci_strdup(mask);
-
-	return sci_find_next(dir);
-}
-
-char *
-sci_find_next(sci_dir_t *dir)
-{
-	dirent_t *match;
-
-	while ((match = fs_readdir(dir->dir))) {
-		if (match->name[0] == '.')
-			continue;
-
-		if (!fnmatch(dir->mask_copy, match->name, 0))
-			return match->name;
-	}
-
-	sci_finish_find(dir);
-	return NULL;
-}
-
-void
-sci_finish_find(sci_dir_t *dir)
-{
-	if (dir->dir) {
-		fs_close(dir->dir);
-		dir->dir = 0;
-		free(dir->mask_copy);
-		dir->mask_copy = NULL;
-	}
-}
-
-#elif defined(_GP32)
-/******** Dir: GP32 CODE ********/
-
-void
-sci_init_dir(sci_dir_t *dir)   
-{
-	dir->dir = NULL;
-	dir->total = 0;
-	dir->cur = 0;
-	dir->mask_copy = NULL;
-}
-
-char *
-sci_find_first(sci_dir_t *dir, char *mask)
-{
-	char cur_dir[PATH_MAX + 1];
-	unsigned int total, read;
-
-	getcwd(cur_dir, PATH_MAX + 1);
-
-	if (smGetListNumDir(cur_dir, &total)) {
-		sciprintf("%s, L%d: smGetListNumDir() failed!\n", __FILE__, __LINE__);
-		return NULL;
-	}
-
-	if (total > 128)
-		total = 128;
-
-	dir->dir = sci_malloc(sizeof(DIR) * total);
-
-	if (smReadDir(cur_dir, 0, total, dir->dir, &read) || (read < total)) {
-		sciprintf("%s, L%d: smReadDir() failed!\n", __FILE__, __LINE__);
-		sci_free(dir->dir);
-		dir->dir = NULL;
-		return NULL;
-	}
-
-	dir->mask_copy = sci_strdup(mask);
-	dir->total = total;
-
-	return sci_find_next(dir);
-}
-
-char *
-sci_find_next(sci_dir_t *dir)
-{
-	while (dir->cur++ < dir->total)
-		if (!fnmatch(dir->mask_copy, dir->dir[dir->cur - 1].name, 0))
-			return dir->dir[dir->cur - 1].name;
-
-	sci_finish_find(dir);
-	return NULL;
-}
-
-void
-sci_finish_find(sci_dir_t *dir)
-{
-	if (dir->dir) {
-		sci_free(dir->dir);
-		dir->dir = NULL;
-	}
-	if (dir->mask_copy) {
-		sci_free(dir->mask_copy);
-		dir->mask_copy = NULL;
-	}
-	dir->total = 0;
-	dir->cur = 0;
-}
-
-#else /* !_WIN32 && !_DREAMCAST && !_GP32*/
+#else /* !_WIN32 */
 /******** Dir: UNIX CODE ********/
 
 void
@@ -530,12 +380,12 @@ sci_init_dir(sci_dir_t *dir)
 }
 
 char *
-sci_find_first(sci_dir_t *dir, char *mask)
+sci_find_first(sci_dir_t *dir, const char *mask)
 {
 	if (dir->dir)
 		closedir(dir->dir);
 
-	if (!(dir->dir = opendir(G_DIR_CURRENT_S))) {
+	if (!(dir->dir = opendir("."))) {
 		sciprintf("%s, L%d: opendir(\".\") failed!\n", __FILE__, __LINE__);
 		return NULL;
 	}
@@ -579,11 +429,10 @@ sci_finish_find(sci_dir_t *dir)
 
 
 int
-sci_mkpath(char *path)
+sci_mkpath(const char *path)
 {
-        char *nextsep = NULL, *path_pos = path;
-
-	fprintf(stderr, "sci_mkpath: PATH='%s'\n", path);
+	const char *path_pos = path;
+        char *nextsep = NULL;
 
         if (chdir(G_DIR_SEPARATOR_S)) { /* Go to root */
                 sciprintf("Error: Could not change to root directory '%s'!\n",
@@ -619,35 +468,28 @@ sci_mkpath(char *path)
 }
 
 
-#ifdef _WIN32
-static _path_buf[MAX_PATH];
-#endif
 
 char *
-sci_get_homedir()
+sci_get_homedir(void)
 {
 #ifdef _WIN32
+	char *_path_buf = (char*)malloc(MAX_PATH);
 	char *dr = getenv("HOMEDRIVE");
 	char *path = getenv("HOMEPATH");
 
 	if (!dr || !path)
 		return getenv("WINDIR");
 
-	strncpy((char *)_path_buf, dr, 4);
-	strncat((char *)_path_buf, path, MAX_PATH - 4);
-
-	return (char *)_path_buf;
+	strncpy(_path_buf, dr, 4);
+	strncat(_path_buf, path, MAX_PATH - 4);
 
+	return _path_buf;
 #elif defined(__unix__) || !defined(X_DISPLAY_MISSING) || defined (__BEOS__) || defined(MACOSX)
 	return getenv("HOME");
-#elif defined (_DREAMCAST)
-	return "/ram";
-#elif defined (_GP32)
-	return "dev0:\\GPMM";
-#elif defined(__MORPHOS__)
-	return "PROGDIR:";
-#elif defined(ARM_WINCE)
-	return "/";
+#elif defined(_DREAMCAST)
+	return NULL;
+#elif defined(__amigaos4__)
+	return "/PROGDIR/";
 #else
 #  error Please add a $HOME policy for your platform!
 #endif
@@ -664,7 +506,7 @@ sci_init_queue(sci_queue_t *queue)
 sci_queue_t *
 sci_add_to_queue(sci_queue_t *queue, void *data, int type)
 {
-	sci_queue_node_t *node = sci_malloc(sizeof(sci_queue_node_t));
+	sci_queue_node_t *node = (sci_queue_node_t*)sci_malloc(sizeof(sci_queue_node_t));
 
 	node->next = NULL;
 	node->data = data;
@@ -705,12 +547,10 @@ sci_get_from_queue(sci_queue_t *queue, int *type)
 /*-- Yielding to the scheduler --*/
 
 #ifdef HAVE_SCHED_YIELD
-# ifndef ARM_WINCE
 #  include <sched.h>
-# endif
 
 void
-sci_sched_yield()
+sci_sched_yield(void)
 {
 	sched_yield();
 }
@@ -731,7 +571,7 @@ sci_sched_yield()
 	snooze(0);
 }
 
-#else
+#elif defined (_WIN32)
 
 void
 sci_sched_yield()
@@ -739,11 +579,18 @@ sci_sched_yield()
 	sleep(1);
 }
 
+#else
+
+void
+sci_sched_yield()
+{
+}
+
 #endif /* !HAVE_SCHED_YIELD */
 
 
 static char *
-_fcaseseek(char *fname, sci_dir_t *dir)
+_fcaseseek(const char *fname, sci_dir_t *dir)
 		 /* Expects *dir to be uninitialized and the caller to
 		 ** free it afterwards  */
 {
@@ -757,7 +604,7 @@ _fcaseseek(char *fname, sci_dir_t *dir)
 	}
 
 	if (strlen(fname) > 12) /* not a DOS file? */
-		buf = sci_malloc(strlen(fname) + 1);
+		buf = (char*)sci_malloc(strlen(fname) + 1);
 	else
 		buf = _buf;
 
@@ -789,7 +636,7 @@ _fcaseseek(char *fname, sci_dir_t *dir)
 
 
 FILE *
-sci_fopen(char *fname, char *mode)
+sci_fopen(const char *fname, const char *mode)
 {
 	sci_dir_t dir;
 	char *name = _fcaseseek(fname, &dir);
@@ -806,7 +653,7 @@ sci_fopen(char *fname, char *mode)
 }
 
 int
-sci_open(char *fname, int flags)
+sci_open(const char *fname, int flags)
 {
 	sci_dir_t dir;
 	char *name = _fcaseseek(fname, &dir);
@@ -821,14 +668,14 @@ sci_open(char *fname, int flags)
 }
 
 char *
-sci_getcwd()
+sci_getcwd(void)
 {
 	int size = 0;
 	char *cwd = NULL;
 
 	while (size < 8192) {
 		size += 256;
-		cwd = sci_malloc(size);
+		cwd = (char*)sci_malloc(size);
 		if (getcwd(cwd, size-1))
 			return cwd;
 
@@ -839,7 +686,6 @@ sci_getcwd()
 	return NULL;
 }
 
-
 #ifdef _DREAMCAST
 
 int
@@ -849,16 +695,16 @@ sci_fd_size(int fd)
 }
 
 int
-sci_file_size(char *fname)
+sci_file_size(const char *fname)
 {
 	int fd = fs_open(fname, O_RDONLY);
 	int retval = -1;
-	
+
 	if (fd != 0) {
 		retval = sci_fd_size(fd);
 		fs_close(fd);
 	}
-	
+
 	return retval;
 }
 
@@ -873,7 +719,7 @@ sci_fd_size(int fd)
 }
 
 int
-sci_file_size(char *fname)
+sci_file_size(const char *fname)
 {
 	struct stat fn_stat;
 	if (stat(fname, &fn_stat)) return -1;
@@ -882,18 +728,21 @@ sci_file_size(char *fname)
 
 #endif
 
-
-int
-sci_create_file(char *fname)
+/* Simple heuristic to work around array handling peculiarity in SQ4:
+It uses StrAt() to read the individual elements, so we must determine
+whether a string is really a string or an array. */
+int is_print_str(char *str)
 {
-       int fd;
-#ifdef _MSC_VER
-       /* Visual C++ doesn't allow to specify O_BINARY with creat() */
-       fd = _open(fname, _O_CREAT | _O_BINARY | _O_RDWR, _S_IREAD | _S_IWRITE);
-#elif defined (ARM_WINCE)
-       fd = open(fname, O_CREAT | O_BINARY | O_RDWR, S_IREAD | S_IWRITE);
-#else
-       fd = creat(fname, 0600);
-#endif
-       return fd;
-};
+	int printable = 0;
+	int len = strlen(str);
+	
+	if (len == 0) return 1;
+
+	while (*str)
+	{
+		if (isprint(*str)) printable++;
+		str++;
+	}
+
+	return ((float) printable / (float) len >= 0.5);
+}
