@@ -43,19 +43,19 @@ int vocab_version;
 				    VOCAB_RESOURCE_SCI1_SUFFIX_VOCAB : \
 				    VOCAB_RESOURCE_SCI0_SUFFIX_VOCAB
 
-char *class_names[] =
+const char *class_names[] =
   {"",               /* These strange names were taken from an SCI01 interpreter */
    "",
-   "conj",
-   "ass",
-   "pos",
-   "art",
-   "adj",
-   "pron",
-   "noun",
-   "auxv",
-   "adv",
-   "verb",
+   "conj",		/* conjunction */
+   "ass",		/* ? */
+   "pos",		/* preposition ? */
+   "art",		/* article */
+   "adj",		/* adjective */
+   "pron",		/* pronoun */
+   "noun",		/* noun */
+   "auxv",		/* auxillary verb */
+   "adv",		/* adverb */
+   "verb",		/* verb */
    "",
    "",
    "",
@@ -68,74 +68,6 @@ _vocab_cmp_words(const void *word1, const void *word2)
 	return strcasecmp((*((word_t **) word1))->word, (*((word_t **) word2))->word);
 }
 
-/* FIXME: Unify this with the SCI0 routine */
-
-word_t **
-vocab_get_words_sci1(resource_mgr_t *resmgr, int *word_counter)
-{
-  int counter = 0;
-  unsigned int seeker;
-  word_t **words;
-
-  char currentword[256] = ""; /* They're not going to use words longer than 255 ;-) */
-  int currentwordpos = 0;
-
-  resource_t *resource = scir_find_resource(resmgr, sci_vocab,
-					    VOCAB_RESOURCE_SCI1_MAIN_VOCAB, 0);
-  vocab_version = 1;
-
-  if (!resource) {
-    fprintf(stderr,"SCI1: Could not find a main vocabulary!\n");
-    return NULL; /* NOT critical: SCI1 games and some demos don't have one! */
-  }
-
-  seeker = 0x1fe; /* vocab.900 starts with 255 16-bit pointers which we don't use */
-
-  if (resource->size < 510) {
-    fprintf(stderr, "Invalid main vocabulary encountered: Too small\n");
-    return NULL;
-    /* Now this ought to be critical, but it'll just cause parse() and said() not to work */
-  }
-
-  words = sci_malloc(sizeof(word_t *));
-
-  while (seeker < resource->size) {
-    byte c;
-
-    words = sci_realloc(words, (counter + 1) * sizeof(word_t *));
-
-    currentwordpos = resource->data[seeker++]; /* Parts of previous words may be re-used */
-
-    do {
-      c = resource->data[seeker++];
-      currentword[currentwordpos++] = c;
-    } while (c);
-
-    currentword[currentwordpos] = 0;
-
-    words[counter] = sci_malloc(sizeof(word_t) + currentwordpos);
-    /* Allocate more memory, so that the word fits into the structure */
-
-    strcpy(&(words[counter]->word[0]), &(currentword[0])); /* Copy the word */
-
-    /* Now decode class and group: */
-    c = resource->data[seeker + 1];
-    words[counter]->w_class = ((resource->data[seeker]) << 4) | ((c & 0xf0) >> 4);
-    words[counter]->group = (resource->data[seeker + 2]) | ((c & 0x0f) << 8);
-
-    ++counter;
-
-    seeker += 3;
-
-  }
-
-  *word_counter = counter;
-
-  qsort(words, counter, sizeof(word_t *), _vocab_cmp_words); /* Sort entries */
-
-  return words;
-}
-
 
 word_t **
 vocab_get_words(resource_mgr_t *resmgr, int *word_counter)
@@ -147,38 +79,68 @@ vocab_get_words(resource_mgr_t *resmgr, int *word_counter)
 	char currentword[256] = ""; /* They're not going to use words longer than 255 ;-) */
 	int currentwordpos = 0;
 
-	resource_t *resource = scir_find_resource(resmgr, sci_vocab,
+	resource_t *resource;
+
+	/* First try to load the SCI0 vocab resource. */
+	resource = scir_find_resource(resmgr, sci_vocab,
 						  VOCAB_RESOURCE_SCI0_MAIN_VOCAB, 0);
+	vocab_version = 0;
 
 	if (!resource) {
 		fprintf(stderr,"SCI0: Could not find a main vocabulary, trying SCI01.\n");
-		return vocab_get_words_sci1(resmgr, word_counter); /* NOT critical: SCI1 games and some demos don't have one! */
+		resource = scir_find_resource(resmgr, sci_vocab,
+							  VOCAB_RESOURCE_SCI1_MAIN_VOCAB, 0);
+		vocab_version = 1;
 	}
 
-	seeker = 52; /* vocab.000 starts with 26 16-bit pointers which we don't use */
+	if (!resource) {
+		fprintf(stderr,"SCI1: Could not find a main vocabulary!\n");
+		return NULL; /* NOT critical: SCI1 games and some demos don't have one! */
+	}
 
-	if (resource->size < 52) {
+	if (vocab_version == 1)
+		seeker = 255 * 2; /* vocab.900 starts with 255 16-bit pointers which we don't use */
+	else
+		seeker = 26 * 2; /* vocab.000 starts with 26 16-bit pointers which we don't use */
+
+	if (resource->size < seeker) {
 		fprintf(stderr, "Invalid main vocabulary encountered: Too small\n");
 		return NULL;
 		/* Now this ought to be critical, but it'll just cause parse() and said() not to work */
 	}
 
-	words = sci_malloc(sizeof(word_t *));
+	words = (word_t**)sci_malloc(sizeof(word_t *));
 
 	while (seeker < resource->size) {
 		byte c;
 
-		words = sci_realloc(words, (counter + 1) * sizeof(word_t *));
+		words = (word_t**)sci_realloc(words, (counter + 1) * sizeof(word_t *));
 
 		currentwordpos = resource->data[seeker++]; /* Parts of previous words may be re-used */
 
-		do {
-			c = resource->data[seeker++];
-			currentword[currentwordpos++] = c & 0x7f; /* 0x80 is used to terminate the string */
-		} while (c < 0x80);
+		if (vocab_version == 1) {
+			c = 1;
+			while (seeker < resource->size
+				   && currentwordpos < 255
+				   && c) {
+				c = resource->data[seeker++];
+				currentword[currentwordpos++] = c;
+			}
+			if (seeker == resource->size) {
+				fprintf(stderr, "SCI1: Vocabulary not usable, disabling.\n");
+				vocab_free_words(words, counter);
+				return NULL;
+			}
+		} else {
+			do {
+				c = resource->data[seeker++];
+				currentword[currentwordpos++] = c & 0x7f; /* 0x80 is used to terminate the string */
+			} while (c < 0x80);
+		}
+
 		currentword[currentwordpos] = 0;
 
-		words[counter] = sci_malloc(sizeof(word_t) + currentwordpos);
+		words[counter] = (word_t*)sci_malloc(sizeof(word_t) + currentwordpos);
 		/* Allocate more memory, so that the word fits into the structure */
 
 		strcpy(&(words[counter]->word[0]), &(currentword[0])); /* Copy the word */
@@ -186,19 +148,10 @@ vocab_get_words(resource_mgr_t *resmgr, int *word_counter)
 		/* Now decode class and group: */
 		c = resource->data[seeker + 1];
 		words[counter]->w_class = ((resource->data[seeker]) << 4) | ((c & 0xf0) >> 4);
-
-#if 0
-		if (c & 0xf0) {
-			fprintf(stderr,"Unexpected class mask-");
-			fprintf(stderr,"%s ", words[counter]->word);
-			fprintf(stderr, " class mask = %03x\n", (words[counter]->w_class));
-		}
-#endif
 		words[counter]->group = (resource->data[seeker + 2]) | ((c & 0x0f) << 8);
 
-		++counter;
-
 		seeker += 3;
+		++counter;
 
 	}
 
@@ -222,7 +175,7 @@ vocab_free_words(word_t **words, int words_nr)
 }
 
 
-char *
+const char *
 vocab_get_any_group_word(int group, word_t **words, int words_nr)
 {
   int i;
@@ -257,7 +210,7 @@ vocab_get_suffices(resource_mgr_t *resmgr, int *suffices_nr)
     return NULL; /* Not critical */
   }
 
-  suffices = sci_malloc(sizeof(suffix_t *));
+  suffices = (suffix_t**)sci_malloc(sizeof(suffix_t *));
 
   while ((seeker < resource->size-1) && (resource->data[seeker + 1] != 0xff)) {
 
@@ -266,13 +219,13 @@ vocab_get_suffices(resource_mgr_t *resmgr, int *suffices_nr)
     char *word_suffix;
     int word_len;
 
-    suffices = sci_realloc(suffices, sizeof(suffix_t *) * (counter + 1));
+    suffices = (suffix_t**)sci_realloc(suffices, sizeof(suffix_t *) * (counter + 1));
 
     seeker += alt_len + 1; /* Hit end of string */
     word_suffix = (char *) resource->data + seeker + 3; /* Beginning of next string +1 (ignore '*') */
     word_len = strlen(word_suffix);
 
-    suffices[counter] = sci_malloc(sizeof(suffix_t));
+    suffices[counter] = (suffix_t*)sci_malloc(sizeof(suffix_t));
     /* allocate enough memory to store the strings */
 
     suffices[counter]->word_suffix = word_suffix;
@@ -339,7 +292,7 @@ vocab_get_branches(resource_mgr_t * resmgr, int *branches_nr)
     return NULL;
   }
 
-  retval = sci_malloc(sizeof(parse_tree_branch_t) * *branches_nr);
+  retval = (parse_tree_branch_t*)sci_malloc(sizeof(parse_tree_branch_t) * *branches_nr);
 
   for (i = 0; i < *branches_nr; i++) {
     int k;
@@ -366,7 +319,7 @@ vocab_lookup_word(char *word, int word_len,
 		  word_t **words, int words_nr,
 		  suffix_t **suffices, int suffices_nr)
 {
-	word_t *tempword = sci_malloc(sizeof(word_t) + word_len + 256);
+	word_t *tempword = (word_t*)sci_malloc(sizeof(word_t) + word_len + 256);
 	/* 256: For suffices. Should suffice. */
 	word_t **dict_word;
 	result_word_t *retval;
@@ -377,12 +330,12 @@ vocab_lookup_word(char *word, int word_len,
 	tempword->word[word_len] = 0;
 
 	word_len_tmp = word_len;
-	while (tester = strchr(tempword->word, '-'))
+	while ((tester = strchr(tempword->word, '-')))
 		memmove(tester, tester + 1, (tempword->word + word_len_tmp--) - tester);
 
-	retval = sci_malloc(sizeof(result_word_t));
+	retval = (result_word_t*)sci_malloc(sizeof(result_word_t));
 
-	dict_word = bsearch(&tempword, words, words_nr, sizeof(word_t *), _vocab_cmp_words);
+	dict_word = (word_t**)bsearch(&tempword, words, words_nr, sizeof(word_t *), _vocab_cmp_words);
 
 	if (dict_word) {
 		free(tempword);
@@ -407,7 +360,7 @@ vocab_lookup_word(char *word, int word_len,
 				tempword->word[suff_index] = 0; /* Terminate word at suffix start position... */
 				strncat(&(tempword->word[0]), suffices[i]->word_suffix, suffices[i]->word_suffix_length); /* ...and append "correct" suffix */
 
-				dict_word = bsearch(&tempword, words, words_nr, sizeof(word_t *), _vocab_cmp_words);
+				dict_word = (word_t**)bsearch(&tempword, words, words_nr, sizeof(word_t *), _vocab_cmp_words);
 
 				if ((dict_word) && ((*dict_word)->w_class & suffices[i]->class_mask)) { /* Found it? */
 					free(tempword);
@@ -426,7 +379,7 @@ vocab_lookup_word(char *word, int word_len,
 	tempword->word[word_len] = 0;
 
 	word_len_tmp = word_len;
-	while (tester = strchr(tempword->word, '-'))
+	while ((tester = strchr(tempword->word, '-')))
 		memmove(tester, tester + 1, (tempword->word + word_len--) - tester);
 
 	if ((strtol(&(tempword->word[0]), &tester, 10) >= 0)
@@ -446,35 +399,35 @@ vocab_lookup_word(char *word, int word_len,
 
 
 void
-vocab_decypher_said_block(state_t *s, heap_ptr addr)
+vocab_decypher_said_block(state_t *s, byte *addr)
 {
-  int nextitem;
+	int nextitem;
 
-  do {
-    nextitem = s->heap[addr++];
+	do {
+		nextitem = *addr++;
 
-    if (nextitem < 0xf0) {
-      nextitem = nextitem << 8 | s->heap[addr++];
-      sciprintf(" %s[%03x]", vocab_get_any_group_word(nextitem, s->parser_words, s->parser_words_nr),
-		nextitem);
+		if (nextitem < 0xf0) {
+			nextitem = nextitem << 8 | *addr++;
+			sciprintf(" %s[%03x]", vocab_get_any_group_word(nextitem, s->parser_words, s->parser_words_nr),
+				  nextitem);
 
-      nextitem = 42; /* Make sure that group 0xff doesn't abort */
-    } else switch(nextitem) {
-    case 0xf0: sciprintf(" ,"); break;
-    case 0xf1: sciprintf(" &"); break;
-    case 0xf2: sciprintf(" /"); break;
-    case 0xf3: sciprintf(" ("); break;
-    case 0xf4: sciprintf(" )"); break;
-    case 0xf5: sciprintf(" ["); break;
-    case 0xf6: sciprintf(" ]"); break;
-    case 0xf7: sciprintf(" #"); break;
-    case 0xf8: sciprintf(" <"); break;
-    case 0xf9: sciprintf(" >"); break;
-    case 0xff: break;
-    }
-  } while (nextitem != 0xff);
+			nextitem = 42; /* Make sure that group 0xff doesn't abort */
+		} else switch(nextitem) {
+		case 0xf0: sciprintf(" ,"); break;
+		case 0xf1: sciprintf(" &"); break;
+		case 0xf2: sciprintf(" /"); break;
+		case 0xf3: sciprintf(" ("); break;
+		case 0xf4: sciprintf(" )"); break;
+		case 0xf5: sciprintf(" ["); break;
+		case 0xf6: sciprintf(" ]"); break;
+		case 0xf7: sciprintf(" #"); break;
+		case 0xf8: sciprintf(" <"); break;
+		case 0xf9: sciprintf(" >"); break;
+		case 0xff: break;
+		}
+	} while (nextitem != 0xff);
 
-  sciprintf("\n");
+	sciprintf("\n");
 }
 
 
@@ -586,7 +539,7 @@ vocab_tokenize_string(char *sentence, int *result_nr,
 	int pos_in_sentence = 0;
 	char c;
 	int wordlen = 0;
-	result_word_t *retval = sci_malloc(sizeof(result_word_t));
+	result_word_t *retval = (result_word_t*)sci_malloc(sizeof(result_word_t));
 	/* malloc'd size is always one result_word_t too big */
 
 	result_word_t *lookup_result;
@@ -615,7 +568,7 @@ vocab_tokenize_string(char *sentence, int *result_nr,
 				/* Look it up */
 
 				if (!lookup_result) { /* Not found? */
-					*error = sci_calloc(wordlen + 1, 1);
+					*error = (char*)sci_calloc(wordlen + 1, 1);
 					strncpy(*error, lastword, wordlen); /* Set the offending word */
 					free(retval);
 					return NULL; /* And return with error */
@@ -626,7 +579,7 @@ vocab_tokenize_string(char *sentence, int *result_nr,
 				++(*result_nr); /* Increase number of resulting words */
 				free(lookup_result);
 
-				retval = sci_realloc(retval, sizeof(result_word_t) * (*result_nr + 1));
+				retval = (result_word_t*)sci_realloc(retval, sizeof(result_word_t) * (*result_nr + 1));
 
 			}
 
@@ -717,7 +670,7 @@ _vocab_recursive_ptree_dump(parse_tree_node_t *nodes, int nr, int prevnr, int bl
 }
 
 void
-vocab_dump_parse_tree(char *tree_name, parse_tree_node_t *nodes)
+vocab_dump_parse_tree(const char *tree_name, parse_tree_node_t *nodes)
 {
   /*  _vocab_recursive_ptree_dump_treelike(nodes, 0, 0); */
   sciprintf("(setq %s \n'(", tree_name);
