@@ -37,11 +37,12 @@
 
 #include <sciresource.h>
 #include <engine.h>
-#include <sound.h>
 #include <console.h>
 
 /* #define DRAW_GRAPHICS */
 
+#undef HAVE_OBSTACK_H
+
 #ifdef _MSC_VER
 #  include <direct.h>
 #  define extern __declspec(dllimport) extern
@@ -61,7 +62,7 @@
 #  endif /* HAVE_LIBPNG */
 #endif /* DRAW_GRAPHICS */
 
-#if defined (_MSC_VER) || defined (__BEOS__) || defined(_DOS)
+#if defined (_MSC_VER) || defined (__BEOS__) || defined(_DOS) || defined(__amigaos4__)
 /* [DJ] fchmod is not in Visual C++ RTL - and probably not needed,anyway */
 /* [RS] (see comment above, but read MS-DOS instead of Visual C++ RTL) */
 #  define fchmod(file,mode)
@@ -78,7 +79,6 @@
 #define ACT_LIST 2
 #define ACT_SCRIPTDUMP 3
 #define ACT_VOCABDUMP 4
-#define ACT_SAIDSPECDUMP 5
 
 #define ACT_DEFAULT ACT_UNPACK
 
@@ -92,6 +92,10 @@ static guint8 midimask = 0x01;  /* MT-32 */
 
 resource_mgr_t *resmgr;
 
+#ifdef _WIN32
+#define fchmod(arg1, arg2)
+#endif
+
 void
 print_resource_filename(FILE* file, int type, int number)
 {
@@ -122,7 +126,6 @@ static struct option options[] = {
 	{"words", no_argument, &action, ACT_WORDS},
 	{"vocab", no_argument, &action, ACT_VOCABDUMP},
 	{"objects", no_argument, &action, ACT_SCRIPTDUMP},
-	{"said-specs", no_argument, &action, ACT_SAIDSPECDUMP},
 	{"with-header", no_argument, &with_header, 1},
 	{"without-header", no_argument, &with_header, 0},
 	{"sort-alpha", no_argument, &vocab_sort, SORT_METHOD_ALPHA},
@@ -142,74 +145,6 @@ static struct option options[] = {
 void unpack_resource(int stype, int snr, char *outfilename);
 
 
-int
-saidspec_dump()
-{
-	int i;
-	word_t **parser_words;
-	int parser_words_nr;
-
-	if (!conversion)
-		parser_words = vocab_get_words(resmgr, &parser_words_nr);
-
-	for (i = 0; i < 999; i++) {
-		resource_t *r = scir_find_resource(resmgr, sci_script, i, 0);
-
-		if (r) {
-			int offset = 0;
-			int type, size;
-
-			do {
-				type = getUInt16(r->data + offset);
-				size = getUInt16(r->data + offset + 2);
-
-				if (offset + size > r->size)
-					break;
-
-				if (type == sci_obj_said) {
-					byte *pos = r->data + offset + 4;
-					int nextitem;
-
-					while (pos - r->data +1 < size + offset) { do {
-						nextitem = *pos++;
-
-						if (nextitem < 0xf0) {
-							nextitem = nextitem << 8 | *pos++;
-
-							if (conversion)
-								printf(" W");
-							else
-								printf(" %s", vocab_get_any_group_word(nextitem, parser_words, parser_words_nr),
-									  nextitem);
-
-							nextitem = 42; /* Make sure that group 0xff doesn't abort */
-						} else switch(nextitem) {
-						case 0xf0: printf(" ,"); break;
-						case 0xf1: printf(" &"); break;
-						case 0xf2: printf(" /"); break;
-						case 0xf3: printf(" ("); break;
-						case 0xf4: printf(" )"); break;
-						case 0xf5: printf(" ["); break;
-						case 0xf6: printf(" ]"); break;
-						case 0xf7: printf(" #"); break;
-						case 0xf8: printf(" <"); break;
-						case 0xf9: printf(" >"); break;
-						case 0xff: printf("\n"); break;
-						}
-					} while (nextitem != 0xff); }
-				}
-
-				offset += size;
-			} while (type && (offset < r->size));
-		}
-	}
-
-	if (!conversion)
-		vocab_free_words(parser_words, parser_words_nr);
-
-	return 0;
-}
-
 int main(int argc, char** argv)
 {
 	int retval = 0;
@@ -221,16 +156,17 @@ int main(int argc, char** argv)
 	int optindex = 0;
 	int c;
 	char *gamedir = sci_getcwd();
-
+	int res_version = SCI_VERSION_AUTODETECT;
+	
 #ifdef HAVE_GETOPT_LONG
-	while ((c = getopt_long(argc, argv, "SWOVUvhLco:d:M:", options, &optindex)) > -1) {
+	while ((c = getopt_long(argc, argv, "WOVUvhLcr:o:d:M:", options, &optindex)) > -1) {
 #else /* !HAVE_GETOPT_LONG */
-	while ((c = getopt(argc, argv, "SWOVUvhLco:d:M:")) > -1) {
+	while ((c = getopt(argc, argv, "WOVUvhLcr:o:d:M:")) > -1) {
 #endif /* !HAVE_GETOPT_LONG */
 
 		switch (c) {
 		case 256:
-			printf("sciunpack");
+			printf("sciunpack ("PACKAGE") "VERSION"\n");
 			printf("This program is copyright (C) 1999, 2000, 2001 Christoph Reichenbach,\n"
 			       " Lars Skovlund, Magnus Reftel\n"
 			       "It comes WITHOUT WARRANTY of any kind.\n"
@@ -238,7 +174,7 @@ int main(int argc, char** argv)
 			exit(0);
 
 		case 'h': {
-			char *gcc_3_0_hates_me =
+			char *gcc_3_0_can_kiss_my_ass =
 			       "Usage: sciunpack [options] [-U] <resource.number>\n"
 			       "       sciunpack [options] [-U] <resource> <number>\n"
 			       "Unpacks resource data\n"
@@ -255,7 +191,6 @@ int main(int argc, char** argv)
 			       " --list        -L       List all resources\n"
 			       " --words       -W       List all vocabulary words\n"
 			       " --objects     -O       Print all objects\n"
-			       " --said-specs  -S       Prints all said specs\n"
 			       " --vocab       -V       Lists the complete vocabulary\n"
 			       "\nAvailable options:\n"
 			       "General:\n"
@@ -267,8 +202,6 @@ int main(int argc, char** argv)
 			       "Listing words:\n"
 			       " --sort-alpha		sort in alphabetical order\n"
 			       " --sort-group		sort in group order\n"
-			       "Dumping Said specs:\n"
-			       " --convert     -c       Print 'W' instead of actual words\n"
 			       "Unpacking:\n"
 			       " --convert     -c       Converts selected resources\n"
 			       " --output-file -o       Selects output file\n"
@@ -290,7 +223,7 @@ int main(int argc, char** argv)
 #endif /* DRAW_GRAPHICS */
 				;
 
-			printf(gcc_3_0_hates_me);
+			printf(gcc_3_0_can_kiss_my_ass);
 			exit(0);
 		}
 
@@ -314,10 +247,6 @@ int main(int argc, char** argv)
 			action = ACT_SCRIPTDUMP;
 			break;
 
-		case 'S':
-			action = ACT_SAIDSPECDUMP;
-			break;
-
 		case 'o':
 			outfilename = optarg;
 			break;
@@ -327,6 +256,10 @@ int main(int argc, char** argv)
 			gamedir = sci_strdup (optarg);
 			break;
 
+		case 'r':
+			res_version = atoi(optarg);
+			break;
+
 		case 'c':
 			conversion = 1;
 			break;
@@ -374,7 +307,7 @@ int main(int argc, char** argv)
 			exit(1);
 		}
 
-	if (!(resmgr = scir_new_resource_manager(gamedir, SCI_VERSION_AUTODETECT,
+	if (!(resmgr = scir_new_resource_manager(gamedir, res_version,
 						 0, 1024*128))) {
 		fprintf(stderr,"Could not find any resources; quitting.\n");
 		exit(1);
@@ -437,10 +370,6 @@ int main(int argc, char** argv)
 		retval = vocab_dump();
 		break;
 
-	case ACT_SAIDSPECDUMP:
-		retval = saidspec_dump();
-		break;
-
 	default:
 		fprintf(stderr,"Invalid action %d- internal error!\n", action);
 		return 1;
@@ -465,7 +394,9 @@ void unpack_resource(int stype, int snr, char *outfilename)
 	if (!outfilename) {
 		outfilename = fnamebuffer;
 		if ((stype == sci_sound) && conversion) {
+#ifdef HAVE_OBSTACK_H
 			map_MIDI_instruments(resmgr);
+#endif
 			sprintf(outfilename,"%03d.midi", snr);
 		}
 #ifdef DRAW_GRAPHICS
